@@ -1,3 +1,5 @@
+private final GL4bcProcAddressTable _pat;
+
 // Tracks glBegin/glEnd calls to determine whether it is legal to
 // query Vertex Buffer Object state
 private boolean inBeginEndPair;
@@ -17,35 +19,21 @@ public void setObjectTracker(GLObjectTracker tracker) {
 
 public GL4bcImpl(GLProfile glp, GLContextImpl context) {
   this._context = context; 
-  if(null != context) {
-      this.bufferObjectTracker  = context.getBufferObjectTracker();
-      this.bufferStateTracker = context.getBufferStateTracker();
-      this.glStateTracker     = context.getGLStateTracker();
-  } else {
-      this.bufferObjectTracker  = null;
-      this.bufferStateTracker = null;
-      this.glStateTracker     = null;
-  }
+  this._pat = (GL4bcProcAddressTable)_context.getGLProcAddressTable();
+  this.bufferObjectTracker = context.getBufferObjectTracker();
+  this.bufferStateTracker = context.getBufferStateTracker();
+  this.glStateTracker     = context.getGLStateTracker();
   this.glProfile = glp;
 }
 
 public final void finalizeInit() {
-  if(null != _context) {
-      haveARBPixelBufferObject  = isExtensionAvailable("GL_ARB_pixel_buffer_object");
-      haveEXTPixelBufferObject  = isExtensionAvailable("GL_EXT_pixel_buffer_object");
-      haveGL15                  = isExtensionAvailable("GL_VERSION_1_5");
-      haveGL21                  = isExtensionAvailable("GL_VERSION_2_1");
-      haveARBVertexBufferObject = isExtensionAvailable("GL_ARB_vertex_buffer_object");
-      haveARBVertexArrayObject  = _context.getGLVersionNumber().compareTo(GLContext.Version300) >= 0 ||
-                                  isExtensionAvailable("GL_ARB_vertex_array_object");
-   } else {
-      haveARBPixelBufferObject  = false;
-      haveEXTPixelBufferObject  = false;
-      haveGL15                  = false;
-      haveGL21                  = false;
-      haveARBVertexBufferObject = false;
-      haveARBVertexArrayObject  = false;
-   }
+  haveARBPixelBufferObject  = isExtensionAvailable("GL_ARB_pixel_buffer_object");
+  haveEXTPixelBufferObject  = isExtensionAvailable("GL_EXT_pixel_buffer_object");
+  haveGL15                  = isExtensionAvailable("GL_VERSION_1_5");
+  haveGL21                  = isExtensionAvailable("GL_VERSION_2_1");
+  haveARBVertexBufferObject = isExtensionAvailable("GL_ARB_vertex_buffer_object");
+  haveARBVertexArrayObject  = _context.getGLVersionNumber().compareTo(GLContext.Version3_0) >= 0 ||
+                              isExtensionAvailable("GL_ARB_vertex_array_object");
 }
 
 private int[] imageSizeTemp = new int[1];
@@ -129,6 +117,16 @@ public final boolean isGLES3Compatible() {
     return _context.isGLES3Compatible();
 }
 
+@Override
+public final boolean isGLES31Compatible() {
+    return _context.isGLES31Compatible();
+}
+
+@Override
+public final boolean isGLES32Compatible() {
+    return _context.isGLES32Compatible();
+}
+
 @Override
 public final boolean isGL2GL3() {
     return _context.isGL2GL3();
@@ -266,15 +264,6 @@ public final GLES3 getGLES3() throws GLException {
 public final boolean isNPOTTextureAvailable() {
   return _context.isNPOTTextureAvailable();
 }
-@Override
-public final java.nio.ByteBuffer glAllocateMemoryNV(int size, float readFrequency, float writeFrequency, float priority) {
-  return _context.glAllocateMemoryNV(size, readFrequency, writeFrequency, priority);
-}
-
-@Override
-public final void glFreeMemoryNV(java.nio.ByteBuffer pointer) {
-  _context.glFreeMemoryNV(pointer);
-}
 
 //
 // Helpers for ensuring the correct amount of texture data
@@ -381,7 +370,7 @@ private final boolean checkIndirectVBOUnbound(boolean throwException) {
   return checkBufferObject(haveGL15 || haveARBVertexBufferObject,
                            haveARBVertexArrayObject, // allowVAO
                            false, // bound
-                           GL4.GL_DRAW_INDIRECT_BUFFER,
+                           GL4ES3.GL_DRAW_INDIRECT_BUFFER,
                            "indirect vertex_buffer_object", throwException);
 }
 
@@ -389,7 +378,7 @@ private final boolean checkIndirectVBOBound(boolean throwException) {
   return checkBufferObject(haveGL15 || haveARBVertexBufferObject,
                            haveARBVertexArrayObject, // allowVAO
                            true, // bound
-                           GL4.GL_DRAW_INDIRECT_BUFFER,
+                           GL4ES3.GL_DRAW_INDIRECT_BUFFER,
                            "indirect vertex_buffer_object", throwException);
 }
 
@@ -481,145 +470,141 @@ public final void glTexCoordPointer(GLArrayData array) {
 //
 
 @Override
-public final void glBufferData(int target, long size, Buffer data, int usage)  {
-    final long glProcAddress = ((GL4bcProcAddressTable)_context.getGLProcAddressTable())._addressof_glBufferData;
-    if ( 0 == glProcAddress ) {
-      throw new GLException(String.format("Method \"%s\" not available", "glBufferData"));
-    }
-    bufferObjectTracker.createBufferStorage(bufferStateTracker, this, 
-                                            target, size, data, usage, 0 /* immutableFlags */, 
-                                            createBoundMutableStorageDispatch, glProcAddress);
+public final void glNamedBufferData(int buffer, long size, Buffer data, int usage)  {
+    bufferObjectTracker.createBufferStorage(this, 
+                                            buffer, size, data, usage, 0 /* immutableFlags */,
+                                            glNamedBufferDataDispatch);
 }
-/** FIXME Add for OpenGL 4.4
-@Override
-public final void glBufferStorage(int target, long size, Buffer data, int flags)  {
-    final long glProcAddress = ((GL4bcProcAddressTable)_context.getGLProcAddressTable())._addressof_glBufferStorage;
-    if ( 0 == glProcAddress ) {
-      throw new GLException(String.format("Method \"%s\" not available", "glBufferStorage"));
-    }
-    bufferObjectTracker.createBufferStorage(bufferStateTracker, this, 
-                                            target, size, data, 0 * mutableUsage *, flags, 
-                                            createBoundImmutableStorageDispatch, glProcAddress);
-}
-private final jogamp.opengl.GLBufferObjectTracker.CreateStorageDispatch createBoundImmutableStorageDispatch = 
+private final jogamp.opengl.GLBufferObjectTracker.CreateStorageDispatch glNamedBufferDataDispatch = 
     new jogamp.opengl.GLBufferObjectTracker.CreateStorageDispatch() {
-        public final void create(final int target, final long size, final Buffer data, final int immutableFlags, final long glProcAddress) {
-            final boolean data_is_direct = Buffers.isDirect(data);
-            dispatch_glBufferStorage(target, size, 
-                                  data_is_direct ? data : Buffers.getArray(data), 
-                                  data_is_direct ? Buffers.getDirectBufferByteOffset(data) : Buffers.getIndirectBufferByteOffset(data), 
-                                  data_is_direct, immutableFlags, glProcAddress);
+        public final void create(final int buffer, final long size, final Buffer data, final int mutableUsage) {
+            glNamedBufferDataDelegate(buffer, size, data, mutableUsage);
         }
     };
-private native void dispatch_glBufferStorage(int target, long size, Object data, int data_byte_offset, boolean data_is_direct, int flags, long procAddress);
- */
 
 @Override
 public final void glNamedBufferDataEXT(int buffer, long size, Buffer data, int usage)  {
-    final long glProcAddress = ((GL4bcProcAddressTable)_context.getGLProcAddressTable())._addressof_glNamedBufferDataEXT;
-    if ( 0 == glProcAddress ) {
-      throw new GLException(String.format("Method \"%s\" not available", "glNamedBufferDataEXT"));
-    }
     bufferObjectTracker.createBufferStorage(this, 
                                             buffer, size, data, usage, 0 /* immutableFlags */,
-                                            createNamedStorageDispatch, glProcAddress);
+                                            glNamedBufferDataEXTDispatch);
 }
-private final jogamp.opengl.GLBufferObjectTracker.CreateStorageDispatch createNamedStorageDispatch = 
+private final jogamp.opengl.GLBufferObjectTracker.CreateStorageDispatch glNamedBufferDataEXTDispatch = 
     new jogamp.opengl.GLBufferObjectTracker.CreateStorageDispatch() {
-        public final void create(final int buffer, final long size, final Buffer data, final int mutableUsage, final long glProcAddress) {
-            final boolean data_is_direct = Buffers.isDirect(data);
-            dispatch_glNamedBufferDataEXT(buffer, size,
-                                          data_is_direct ? data : Buffers.getArray(data), 
-                                          data_is_direct ? Buffers.getDirectBufferByteOffset(data) : Buffers.getIndirectBufferByteOffset(data), 
-                                          data_is_direct, mutableUsage, glProcAddress);
+        public final void create(final int buffer, final long size, final Buffer data, final int mutableUsage) {
+            glNamedBufferDataEXTDelegate(buffer, size, data, mutableUsage);
         }
     };
-private native void dispatch_glNamedBufferDataEXT(int buffer, long size, Object data, int data_byte_offset, boolean data_is_direct, int usage, long procAddress);
 
 @Override
-public boolean glUnmapBuffer(int target)  {
-    final long glProcAddress = ((GL4bcProcAddressTable)_context.getGLProcAddressTable())._addressof_glUnmapBuffer;
-    if ( 0 == glProcAddress ) {
-      throw new GLException(String.format("Method \"%s\" not available", "glUnmapBuffer"));
-    }
-    return bufferObjectTracker.unmapBuffer(bufferStateTracker, this, target, unmapBoundBufferDispatch, glProcAddress);
+public final void glBufferStorage(int target, long size, Buffer data, int flags)  {
+    bufferObjectTracker.createBufferStorage(bufferStateTracker, this, 
+                                            target, size, data, 0 /* mutableUsage */, flags, 
+                                            glBufferStorageDispatch);
 }
+private final jogamp.opengl.GLBufferObjectTracker.CreateStorageDispatch glBufferStorageDispatch = 
+    new jogamp.opengl.GLBufferObjectTracker.CreateStorageDispatch() {
+        public final void create(final int target, final long size, final Buffer data, final int immutableFlags) {
+            glBufferStorageDelegate(target, size, data, immutableFlags);
+        }
+    };
 
 @Override
-public boolean glUnmapNamedBufferEXT(int buffer)  {
-    final long glProcAddress = ((GL4bcProcAddressTable)_context.getGLProcAddressTable())._addressof_glUnmapNamedBufferEXT;
-    if ( 0 == glProcAddress ) {
-      throw new GLException(String.format("Method \"%s\" not available", "glUnmapNamedBufferEXT"));
-    }
-    return bufferObjectTracker.unmapBuffer(buffer, unmapNamedBufferDispatch, glProcAddress);
+public final void glNamedBufferStorage(int buffer, long size, Buffer data, int flags)  {
+    bufferObjectTracker.createBufferStorage(this, 
+                                            buffer, size, data, 0 /* mutableUsage */, flags,
+                                            glNamedBufferStorageDispatch);
 }
-private final jogamp.opengl.GLBufferObjectTracker.UnmapBufferDispatch unmapNamedBufferDispatch = 
+private final jogamp.opengl.GLBufferObjectTracker.CreateStorageDispatch glNamedBufferStorageDispatch = 
+    new jogamp.opengl.GLBufferObjectTracker.CreateStorageDispatch() {
+        public final void create(final int buffer, final long size, final Buffer data, final int immutableFlags) {
+            glNamedBufferStorageDelegate(buffer, size, data, immutableFlags);
+        }
+    };
+@Override
+public final void glNamedBufferStorageEXT(int buffer, long size, Buffer data, int flags)  {
+    bufferObjectTracker.createBufferStorage(this, 
+                                            buffer, size, data, 0 /* mutableUsage */, flags,
+                                            glNamedBufferStorageEXTDispatch);
+}
+private final jogamp.opengl.GLBufferObjectTracker.CreateStorageDispatch glNamedBufferStorageEXTDispatch = 
+    new jogamp.opengl.GLBufferObjectTracker.CreateStorageDispatch() {
+        public final void create(final int buffer, final long size, final Buffer data, final int immutableFlags) {
+            glNamedBufferStorageEXTDelegate(buffer, size, data, immutableFlags);
+        }
+    };
+
+@Override
+public boolean glUnmapNamedBuffer(int buffer)  {
+    return bufferObjectTracker.unmapBuffer(buffer, glUnmapNamedBufferDispatch);
+}
+private final jogamp.opengl.GLBufferObjectTracker.UnmapBufferDispatch glUnmapNamedBufferDispatch = 
     new jogamp.opengl.GLBufferObjectTracker.UnmapBufferDispatch() {
-        public final boolean unmap(final int buffer, final long glProcAddress) {
-            return dispatch_glUnmapNamedBufferEXT(buffer, glProcAddress);
+        public final boolean unmap(final int buffer) {
+            return glUnmapNamedBufferDelegate(buffer);
         }
     };
-private native boolean dispatch_glUnmapNamedBufferEXT(int buffer, long procAddress);
 
 @Override
-public final GLBufferStorage mapBuffer(final int target, final int access) {
-  final long glProcAddress = ((GL4bcProcAddressTable)_context.getGLProcAddressTable())._addressof_glMapBuffer;
-  if ( 0 == glProcAddress ) {
-    throw new GLException("Method \"glMapBuffer\" not available");
-  }
-  return bufferObjectTracker.mapBuffer(bufferStateTracker, this, target, access, mapBoundBufferAllDispatch, glProcAddress);
+public final java.nio.ByteBuffer glMapNamedBuffer(int bufferName, int access)  {
+  return mapNamedBuffer(bufferName, access).getMappedBuffer();
 }
-@Override
-public final GLBufferStorage mapBufferRange(final int target, final long offset, final long length, final int access) {
-  final long glProcAddress = ((GL4bcProcAddressTable)_context.getGLProcAddressTable())._addressof_glMapBufferRange;
-  if ( 0 == glProcAddress ) {
-    throw new GLException("Method \"glMapBufferRange\" not available");
-  }
-  return bufferObjectTracker.mapBuffer(bufferStateTracker, this, target, offset, length, access, mapBoundBufferRangeDispatch, glProcAddress);
-}
-
 @Override
 public final GLBufferStorage mapNamedBuffer(final int bufferName, final int access) {
-  final long glProcAddress = ((GL4bcProcAddressTable)_context.getGLProcAddressTable())._addressof_glMapNamedBufferEXT;
-  if ( 0 == glProcAddress ) {
-    throw new GLException("Method \"glMapNamedBufferEXT\" not available");
-  }
-  return bufferObjectTracker.mapBuffer(bufferName, access, mapNamedBufferAllDispatch, glProcAddress);
+  return bufferObjectTracker.mapBuffer(bufferName, access, glMapNamedBufferDispatch);
 }
-private final jogamp.opengl.GLBufferObjectTracker.MapBufferAllDispatch mapNamedBufferAllDispatch = 
+private final jogamp.opengl.GLBufferObjectTracker.MapBufferAllDispatch glMapNamedBufferDispatch = 
     new jogamp.opengl.GLBufferObjectTracker.MapBufferAllDispatch() {
         public final ByteBuffer allocNioByteBuffer(final long addr, final long length) { return newDirectByteBuffer(addr, length); }
-        public final long mapBuffer(final int bufferName, final int access, final long glProcAddress) {
-            return dispatch_glMapNamedBufferEXT(bufferName, access, glProcAddress);
+        public final long mapBuffer(final int bufferName, final int access) {
+            return glMapNamedBufferDelegate(bufferName, access);
         }
     };
-private native long dispatch_glMapNamedBufferEXT(int buffer, int access, long glProcAddress);
-
-@Override
-public final GLBufferStorage mapNamedBufferRange(final int bufferName, final long offset, final long length, final int access) {
-  final long glProcAddress = ((GL4bcProcAddressTable)_context.getGLProcAddressTable())._addressof_glMapNamedBufferRangeEXT;
-  if ( 0 == glProcAddress ) {
-    throw new GLException("Method \"glMapNamedBufferRangeEXT\" not available");
-  }
-  return bufferObjectTracker.mapBuffer(bufferName, offset, length, access, mapNamedBufferRangeDispatch, glProcAddress);
-}
-private final jogamp.opengl.GLBufferObjectTracker.MapBufferRangeDispatch mapNamedBufferRangeDispatch = 
-    new jogamp.opengl.GLBufferObjectTracker.MapBufferRangeDispatch() {
-        public final ByteBuffer allocNioByteBuffer(final long addr, final long length) { return newDirectByteBuffer(addr, length); }
-        public final long mapBuffer(final int bufferName, final long offset, final long length, final int access, final long glProcAddress) {
-            return dispatch_glMapNamedBufferRangeEXT(bufferName, offset, length, access, glProcAddress);
-        }
-    };
-private native long dispatch_glMapNamedBufferRangeEXT(int buffer, long offset, long length, int access, long procAddress);
 
 @Override
 public final java.nio.ByteBuffer glMapNamedBufferEXT(int bufferName, int access)  {
-  return mapNamedBuffer(bufferName, access).getMappedBuffer();
+  return mapNamedBufferEXT(bufferName, access).getMappedBuffer();
 }
+@Override
+public final GLBufferStorage mapNamedBufferEXT(final int bufferName, final int access) {
+  return bufferObjectTracker.mapBuffer(bufferName, access, glMapNamedBufferEXTDispatch);
+}
+private final jogamp.opengl.GLBufferObjectTracker.MapBufferAllDispatch glMapNamedBufferEXTDispatch = 
+    new jogamp.opengl.GLBufferObjectTracker.MapBufferAllDispatch() {
+        public final ByteBuffer allocNioByteBuffer(final long addr, final long length) { return newDirectByteBuffer(addr, length); }
+        public final long mapBuffer(final int bufferName, final int access) {
+            return glMapNamedBufferEXTDelegate(bufferName, access);
+        }
+    };
+
+@Override
+public final ByteBuffer glMapNamedBufferRange(int bufferName, long offset, long length, int access)  {
+  return mapNamedBufferRange(bufferName, offset, length, access).getMappedBuffer();
+}
+@Override
+public final GLBufferStorage mapNamedBufferRange(final int bufferName, final long offset, final long length, final int access) {
+  return bufferObjectTracker.mapBuffer(bufferName, offset, length, access, glMapNamedBufferRangeDispatch);
+}
+private final jogamp.opengl.GLBufferObjectTracker.MapBufferRangeDispatch glMapNamedBufferRangeDispatch = 
+    new jogamp.opengl.GLBufferObjectTracker.MapBufferRangeDispatch() {
+        public final ByteBuffer allocNioByteBuffer(final long addr, final long length) { return newDirectByteBuffer(addr, length); }
+        public final long mapBuffer(final int bufferName, final long offset, final long length, final int access) {
+            return glMapNamedBufferRangeDelegate(bufferName, offset, length, access);
+        }
+    };
 
 @Override
 public final ByteBuffer glMapNamedBufferRangeEXT(int bufferName, long offset, long length, int access)  {
-  return mapNamedBufferRange(bufferName, offset, length, access).getMappedBuffer();
+  return mapNamedBufferRangeEXT(bufferName, offset, length, access).getMappedBuffer();
 }
-
+@Override
+public final GLBufferStorage mapNamedBufferRangeEXT(final int bufferName, final long offset, final long length, final int access) {
+  return bufferObjectTracker.mapBuffer(bufferName, offset, length, access, glMapNamedBufferRangeEXTDispatch);
+}
+private final jogamp.opengl.GLBufferObjectTracker.MapBufferRangeDispatch glMapNamedBufferRangeEXTDispatch = 
+    new jogamp.opengl.GLBufferObjectTracker.MapBufferRangeDispatch() {
+        public final ByteBuffer allocNioByteBuffer(final long addr, final long length) { return newDirectByteBuffer(addr, length); }
+        public final long mapBuffer(final int bufferName, final long offset, final long length, final int access) {
+            return glMapNamedBufferRangeEXTDelegate(bufferName, offset, length, access);
+        }
+    };
 
