@@ -39,9 +39,12 @@
  */
 package com.jogamp.gluegen.opengl;
 
+import static java.util.logging.Level.INFO;
+
 import com.jogamp.gluegen.ConstantDefinition;
 import com.jogamp.gluegen.FunctionEmitter;
 import com.jogamp.gluegen.GlueEmitterControls;
+import com.jogamp.gluegen.GlueGen;
 import com.jogamp.gluegen.JavaConfiguration;
 import com.jogamp.gluegen.JavaEmitter;
 import com.jogamp.gluegen.JavaMethodBindingEmitter;
@@ -49,6 +52,7 @@ import com.jogamp.gluegen.JavaType;
 import com.jogamp.gluegen.MethodBinding;
 import com.jogamp.gluegen.SymbolFilter;
 import com.jogamp.gluegen.cgram.types.FunctionSymbol;
+import com.jogamp.gluegen.cgram.types.Type;
 import com.jogamp.gluegen.procaddress.ProcAddressEmitter;
 import com.jogamp.gluegen.procaddress.ProcAddressJavaMethodBindingEmitter;
 import com.jogamp.gluegen.runtime.opengl.GLNameResolver;
@@ -56,7 +60,6 @@ import com.jogamp.gluegen.runtime.opengl.GLNameResolver;
 import java.io.IOException;
 import java.io.PrintWriter;
 import java.util.ArrayList;
-import java.util.Collection;
 import java.util.IdentityHashMap;
 import java.util.Iterator;
 import java.util.LinkedHashMap;
@@ -78,53 +81,65 @@ public class GLEmitter extends ProcAddressEmitter {
 
     @Override
     public void beginEmission(final GlueEmitterControls controls) throws IOException {
-        getGLConfig().parseGLHeaders(controls);
+        getGLConfig().parseGLSemHeaders(controls);
+        if( null == getGLConfig().getGLSemInfo() ) {
+            throw new RuntimeException("No 'GLSemHeader' defined.");
+        }
+        getGLConfig().parseGLDocHeaders(controls);
+        if( null == getGLConfig().getGLDocInfo() ) {
+            throw new InternalError("XXX"); // since GLDocHeader contains all GLSemHeader ..
+        }
         renameExtensionsIntoCore();
-        if (getGLConfig().getAutoUnifyExtensions()) {
+        if ( getGLConfig().getAutoUnifyExtensions() ) {
             unifyExtensions(controls);
         }
         super.beginEmission(controls);
     }
 
     protected void renameExtensionsIntoCore() {
+        final GLConfiguration config = getGLConfig();
+        renameExtensionsIntoCore(config, config.getGLSemInfo(), true);
+        renameExtensionsIntoCore(config, config.getGLDocInfo(), false);
+    }
+    protected void renameExtensionsIntoCore(final GLConfiguration config, final BuildStaticGLInfo glInfo, final boolean isSemHeader) {
         // This method handles renaming of entire extensions into the
         // OpenGL core namespace. For example, it is used to move certain
         // OpenGL ES (OES) extensions into the core namespace which are
         // already in the core namespace in desktop OpenGL. It builds upon
         // renaming mechanisms that are built elsewhere.
 
-        final GLConfiguration config = getGLConfig();
-        final Set<String> extensionsRenamedIntoCore = config.getExtensionsRenamedIntoCore();
-        final BuildStaticGLInfo glInfo = config.getGLInfo();
-        if (null == glInfo) {
-            if (extensionsRenamedIntoCore.size() > 0) {
-                throw new RuntimeException("ExtensionRenamedIntoCore (num: " + extensionsRenamedIntoCore.size() + "), but no GLHeader");
-            }
-            return;
-        }
-        for (final String extension : extensionsRenamedIntoCore) {
-            if(JavaConfiguration.DEBUG_RENAMES) {
-                System.err.println("<RenameExtensionIntoCore: "+extension+" BEGIN");
-            }
-            final Set<String> declarations = glInfo.getDeclarations(extension);
-            if (declarations != null) {
-                for (final Iterator<String> iterator = declarations.iterator(); iterator.hasNext();) {
-                    final String decl = iterator.next();
-                    final boolean isGLFunction = GLNameResolver.isGLFunction(decl);
-                    boolean isGLEnumeration = false;
-                    if (!isGLFunction) {
-                        isGLEnumeration = GLNameResolver.isGLEnumeration(decl);
-                    }
-                    if (isGLFunction || isGLEnumeration) {
-                        final String renamed = GLNameResolver.normalize(decl, isGLFunction);
-                        if (!renamed.equals(decl)) {
-                            config.addJavaSymbolRename(decl, renamed);
+        final String headerType = isSemHeader ? "GLSemHeader" : "GLDocHeader";
+        final Set<String> extensionSet = isSemHeader ? config.getExtensionsRenamedIntoCore() : glInfo.getExtensions();
+
+        for (final String extension : extensionSet) {
+            if( isSemHeader && config.isIgnoredExtension(extension) ) {
+                LOG.log(INFO, "<RenameExtensionIntoCore: {0} IGNORED {1}>", extension, headerType);
+            } else {
+                LOG.log(INFO, "<RenameExtensionIntoCore: {0} BEGIN {1}", extension, headerType);
+                final Set<String> declarations = glInfo.getDeclarations(extension);
+                if (declarations != null) {
+                    for (final Iterator<String> iterator = declarations.iterator(); iterator.hasNext();) {
+                        final String decl = iterator.next();
+                        final boolean isGLFunction = GLNameResolver.isGLFunction(decl);
+                        boolean isGLEnumeration = false;
+                        if (!isGLFunction) {
+                            isGLEnumeration = GLNameResolver.isGLEnumeration(decl);
+                        }
+                        if (isGLFunction || isGLEnumeration) {
+                            final String renamed = GLNameResolver.normalize(decl, isGLFunction);
+                            if (!renamed.equals(decl)) {
+                                if( isSemHeader ) {
+                                    // Sem + Doc
+                                    config.addJavaSymbolRename(decl, renamed);
+                                } else {
+                                    // Doc only
+                                    config.addJavaDocSymbolRename(decl, renamed);
+                                }
+                            }
                         }
                     }
                 }
-            }
-            if(JavaConfiguration.DEBUG_RENAMES) {
-                System.err.println("RenameExtensionIntoCore: "+extension+" END>");
+                LOG.log(INFO, "RenameExtensionIntoCore: {0} END>", extension, headerType);
             }
         }
     }
@@ -134,36 +149,29 @@ public class GLEmitter extends ProcAddressEmitter {
         private List<ConstantDefinition> constants;
         private List<FunctionSymbol> functions;
 
-        @Override
-        public void filterSymbols(final List<ConstantDefinition> constants,
-                final List<FunctionSymbol> functions) {
-            this.constants = constants;
-            this.functions = functions;
-            doWork();
-        }
-
         @Override
         public List<ConstantDefinition> getConstants() {
             return constants;
         }
-
         @Override
         public List<FunctionSymbol> getFunctions() {
             return functions;
         }
 
-        private void doWork() {
-            final BuildStaticGLInfo glInfo = getGLConfig().getGLInfo();
+        @Override
+        public void filterSymbols(final List<ConstantDefinition> inConstList,
+                                  final List<FunctionSymbol> inFuncList) {
+            final BuildStaticGLInfo glInfo = getGLConfig().getGLSemInfo();
             if (glInfo == null) {
                 return;
             }
             // Try to retain a "good" ordering for these symbols
             final Map<String, ConstantDefinition> constantMap = new LinkedHashMap<String, ConstantDefinition>();
-            for (final ConstantDefinition def : constants) {
+            for (final ConstantDefinition def : inConstList) {
                 constantMap.put(def.getName(), def);
             }
             final Map<String, FunctionSymbol> functionMap = new LinkedHashMap<String, FunctionSymbol>();
-            for (final FunctionSymbol sym : functions) {
+            for (final FunctionSymbol sym : inFuncList) {
                 functionMap.put(sym.getName(), sym);
             }
 
@@ -253,6 +261,20 @@ public class GLEmitter extends ProcAddressEmitter {
         return new GLConfiguration(this);
     }
 
+    /**
+     * {@inheritDoc}
+     * <p>
+     * Implementation sets the binding's native name to it's interface name,
+     * which is the final aliased shortest name.
+     * The latter is used for the proc-address-table etc ..
+     * </p>
+     */
+    @Override
+    protected void mangleBinding(final MethodBinding binding) {
+        binding.setNativeName(binding.getInterfaceName());
+        super.mangleBinding(binding);
+    }
+
     /** In order to implement Buffer Object variants of certain
     functions we generate another MethodBinding which maps the void*
     argument to a Java long. The generation of emitters then takes
@@ -268,7 +290,7 @@ public class GLEmitter extends ProcAddressEmitter {
         final GLConfiguration glConfig = getGLConfig();
         final List<MethodBinding> bindings = super.expandMethodBinding(binding);
 
-        if ( !glConfig.isBufferObjectFunction(binding.getName()) ) {
+        if ( !glConfig.isBufferObjectFunction(binding.getCSymbol()) ) {
             return bindings;
         }
         final boolean bufferObjectOnly = glConfig.isBufferObjectOnly(binding.getName());
@@ -277,6 +299,10 @@ public class GLEmitter extends ProcAddressEmitter {
 
         // Need to expand each one of the generated bindings to take a
         // Java long instead of a Buffer for each void* argument
+        if( GlueGen.debug() ) {
+            System.err.println("expandMethodBinding: j "+binding.toString());
+            System.err.println("expandMethodBinding: c "+binding.getCSymbol());
+        }
 
         // for (MethodBinding cur : bindings) {
         int j=0;
@@ -294,24 +320,38 @@ public class GLEmitter extends ProcAddressEmitter {
             }
 
             MethodBinding result = cur;
+            int replacedCount = 0;
             for (int i = 0; i < cur.getNumArguments(); i++) {
-                if (cur.getJavaArgumentType(i).isNIOBuffer()) {
+                final JavaType jt = cur.getJavaArgumentType(i);
+                if( jt.isOpaqued() ) {
+                    replacedCount++; // already replaced, i.e. due to opaque
+                } else if ( jt.isNIOBuffer() ) {
                     result = result.replaceJavaArgumentType(i, JavaType.createForClass(Long.TYPE));
+                    replacedCount++;
+                }
+                if( GlueGen.debug() ) {
+                    final Type ct = cur.getCArgumentType(i);
+                    System.err.println("  ["+i+"]: #"+replacedCount+", "+ct.getDebugString()+", "+jt.getDebugString());
                 }
             }
 
-            if (result == cur) {
+            if ( 0 == replacedCount ) {
                 throw new RuntimeException("Error: didn't find any void* arguments for BufferObject function "
-                        + binding.getName());
+                        + binding.toString());
             }
 
-            newBindings.add(result);
             // Now need to flag this MethodBinding so that we generate the
             // correct flags in the emitters later
             bufferObjectMethodBindings.put(result, result);
 
-            if( bufferObjectOnly ) {
-                bindings.remove(j);
+            if( result != cur ) {
+                // replaced
+                newBindings.add(result);
+                if( bufferObjectOnly ) {
+                    bindings.remove(j);
+                } else {
+                    j++;
+                }
             } else {
                 j++;
             }
@@ -323,12 +363,14 @@ public class GLEmitter extends ProcAddressEmitter {
 
     @Override
     protected boolean needsModifiedEmitters(final FunctionSymbol sym) {
-        if ((!needsProcAddressWrapper(sym) && !needsBufferObjectVariant(sym))
-                || getConfig().isUnimplemented(sym.getName())) {
+        if ( ( !callThroughProcAddress(sym) && !needsBufferObjectVariant(sym) ) ||
+             getConfig().isUnimplemented(sym)
+           )
+        {
             return false;
+        } else {
+            return true;
         }
-
-        return true;
     }
 
     public boolean isBufferObjectMethodBinding(final MethodBinding binding) {
@@ -337,51 +379,45 @@ public class GLEmitter extends ProcAddressEmitter {
 
     @Override
     public void emitDefine(final ConstantDefinition def, final String optionalComment) throws Exception {
-        final BuildStaticGLInfo glInfo = getGLConfig().getGLInfo();
-        if (null == glInfo) {
-            throw new Exception("No GLInfo for: " + def);
-        }
         final String symbolRenamed = def.getName();
         final StringBuilder newComment = new StringBuilder();
-        newComment.append("Part of ");
-        if (0 == addExtensionsOfSymbols2Buffer(newComment, ", ", "; ", symbolRenamed, def.getAliasedNames())) {
+        if (0 == addExtensionsOfSymbols2Doc(newComment, ", ", ", ", symbolRenamed)) {
             if (def.isEnum()) {
                 final String enumName = def.getEnumName();
-                if (null != enumName) {
-                    newComment.append(enumName);
-                } else {
-                    newComment.append("CORE ENUM");
+                if (null == enumName) {
+                    newComment.append("Part of CORE ");
+                    newComment.append("ENUM");
                 }
             } else {
                 if (getGLConfig().getAllowNonGLExtensions()) {
-                    newComment.append("CORE DEF");
+                    newComment.append("Part of CORE ");
+                    newComment.append("DEF");
                 } else {
                     // Note: All GL defines must be contained within an extension marker !
                     // #ifndef GL_EXT_lala
                     // #define GL_EXT_lala 1
                     // ...
                     // #endif
-                    if (JavaConfiguration.DEBUG_IGNORES) {
-                        final StringBuilder sb = new StringBuilder();
-                        JavaEmitter.addStrings2Buffer(sb, ", ", symbolRenamed, def.getAliasedNames());
-                        System.err.println("Dropping marker: " + sb.toString());
-                    }
+                    final StringBuilder sb = new StringBuilder();
+                    JavaEmitter.addStrings2Buffer(sb, ", ", symbolRenamed, def.getAliasedNames());
+                    LOG.log(INFO, def.getASTLocusTag(), "Dropping marker: {0}", sb.toString());
                     return;
                 }
             }
         }
-
         if (null != optionalComment) {
-            newComment.append("<br>");
+            if( newComment.length() > 0 ) {
+                newComment.append("<br>");
+            }
             newComment.append(optionalComment);
         }
 
         super.emitDefine(def, newComment.toString());
     }
 
-    private int addExtensionListOfSymbol2Buffer(final BuildStaticGLInfo glInfo, final StringBuilder buf, final String sep1, final String name) {
+    private int addExtensionListOfSymbol2Doc(final BuildStaticGLInfo glDocInfo, final StringBuilder buf, final String sep1, final String name) {
         int num = 0;
-        final Set<String> extensionNames = glInfo.getExtension(name);
+        final Set<String> extensionNames = glDocInfo.getExtension(name);
         if(null!=extensionNames) {
             for(final Iterator<String> i=extensionNames.iterator(); i.hasNext(); ) {
                 final String extensionName = i.next();
@@ -398,49 +434,35 @@ public class GLEmitter extends ProcAddressEmitter {
         }
         return num;
     }
-    private int addExtensionListOfAliasedSymbols2Buffer(final BuildStaticGLInfo glInfo, final StringBuilder buf, final String sep1, final String sep2, final String name, final Collection<String> exclude) {
+    private int addExtensionListOfAliasedSymbols2Doc(final BuildStaticGLInfo glDocInfo, final StringBuilder buf, final String sep1, final String sep2, final String name) {
         int num = 0;
         if(null != name) {
-            num += addExtensionListOfSymbol2Buffer(glInfo, buf, sep1, name); // extensions of given name
-            boolean needsSep2 = 0<num;
-            final Set<String> origNames = cfg.getRenamedJavaSymbols(name);
-            if(null != origNames) {
-                for(final String origName : origNames) {
-                    if(!exclude.contains(origName)) {
-                        if (needsSep2) {
-                            buf.append(sep2); // diff-name seperator
-                        }
-                        final int num2 = addExtensionListOfSymbol2Buffer(glInfo, buf, sep1, origName); // extensions of orig-name
-                        needsSep2 = num<num2;
-                        num += num2;
+            num += addExtensionListOfSymbol2Doc(glDocInfo, buf, sep1, name); // extensions of given name
+            boolean needsSep2 = num > 0;
+            final Set<String> aliases = ((GLConfiguration)cfg).getRenamedJavaDocSymbols(name);
+            if(null != aliases) {
+                for(final String alias : aliases) {
+                    if (needsSep2) {
+                        buf.append(sep2);
                     }
+                    final int num2 = addExtensionListOfSymbol2Doc(glDocInfo, buf, sep1, alias); // extensions of orig-name
+                    needsSep2 = num2 > 0;
+                    num += num2;
                 }
             }
         }
         return num;
     }
 
-    public int addExtensionsOfSymbols2Buffer(StringBuilder buf, final String sep1, final String sep2, final String first, final Collection<String> col) {
-        final BuildStaticGLInfo glInfo = getGLConfig().getGLInfo();
-        if (null == glInfo) {
-            throw new RuntimeException("No GLInfo for: " + first);
+    public int addExtensionsOfSymbols2Doc(StringBuilder buf, final String sep1, final String sep2, final String first) {
+        final BuildStaticGLInfo glDocInfo = getGLConfig().getGLDocInfo();
+        if (null == glDocInfo) {
+            throw new RuntimeException("No GLDocInfo for: " + first);
         }
-        int num = 0;
         if (null == buf) {
             buf = new StringBuilder();
         }
-
-        num += addExtensionListOfAliasedSymbols2Buffer(glInfo, buf, sep1, sep2, first, col);
-        boolean needsSep2 = 0<num;
-        for(final Iterator<String> iter = col.iterator(); iter.hasNext(); ) {
-            if(needsSep2) {
-                buf.append(sep2); // diff-name seperator
-            }
-            final int num2 = addExtensionListOfAliasedSymbols2Buffer(glInfo, buf, sep1, sep2, iter.next(), col);
-            needsSep2 = num<num2;
-            num += num2;
-        }
-        return num;
+        return addExtensionListOfAliasedSymbols2Doc(glDocInfo, buf, sep1, sep2, first);
     }
 
     //----------------------------------------------------------------------
@@ -463,7 +485,7 @@ public class GLEmitter extends ProcAddressEmitter {
     }
 
     protected boolean needsBufferObjectVariant(final FunctionSymbol sym) {
-        return getGLConfig().isBufferObjectFunction(sym.getName());
+        return getGLConfig().isBufferObjectFunction(sym);
     }
 
     protected GLConfiguration getGLConfig() {
