@@ -29,13 +29,12 @@ package com.jogamp.opengl;
 
 import java.util.IdentityHashMap;
 
-import javax.media.nativewindow.AbstractGraphicsDevice;
-import javax.media.opengl.GLCapabilitiesImmutable;
+import com.jogamp.nativewindow.AbstractGraphicsDevice;
+import com.jogamp.opengl.GLCapabilitiesImmutable;
 
 import com.jogamp.common.os.Platform;
-
-import jogamp.opengl.egl.EGL;
-import jogamp.opengl.egl.EGLExt;
+import com.jogamp.opengl.egl.EGL;
+import com.jogamp.opengl.egl.EGLExt;
 
 /**
  * GLRendererQuirks contains information of known bugs of various GL renderer.
@@ -288,6 +287,7 @@ public class GLRendererQuirks {
      *   <li>EGL_VERSION     1.4</li>
      *   <li>GL_VENDOR       NVIDIA Corporation</li>
      *   <li>GL_VERSION      OpenGL ES 3.0 331.38 (probably w/ 1st NV EGL lib on x86)</li>
+     *   <li>GL_VERSION      OpenGL ES 3.1 NVIDIA 355.06 (unstable)</li>
      *   <li>Platform        X11</li>
      *   <li>CPU Family      {@link Platform.CPUFamily#X86}</li>
      * </ul>
@@ -348,7 +348,7 @@ public class GLRendererQuirks {
      * Some drivers wrongly claim to support pbuffers
      * with accumulation buffers. However, the creation of such pbuffer fails:
      * <pre>
-     *   javax.media.opengl.GLException: pbuffer creation error: Couldn't find a suitable pixel format
+     *   com.jogamp.opengl.GLException: pbuffer creation error: Couldn't find a suitable pixel format
      * </pre>
      * </p>
      * <p>
@@ -395,11 +395,60 @@ public class GLRendererQuirks {
      */
     public static final int NeedSharedObjectSync = 20;
 
-    /** @deprecated Use {@link #getCount()}, this value is no more valid! */
-    public static final int COUNT = 18;
+    /**
+     * No reliable ARB_create_context implementation,
+     * even if driver claims otherwise.
+     * <p>
+     * Some drivers wrongly claim to support ARB_create_context.
+     * However, the creation of such context fails:
+     * <pre>
+     *   com.jogamp.opengl.GLException: AWT-EventQueue-0: WindowsWGLContex.createContextImpl ctx !ARB, profile > GL2
+     *   requested (OpenGL >= 3.0.1). Requested: GLProfile[GL3bc/GL3bc.hw], current: 2.1 (Compat profile, FBO, hardware)
+     *   - 2.1.8787
+     * </pre>
+     * </p>
+     * <p>
+     * Appears on:
+     * <ul>
+     *   <li>GL_VENDOR       ATI Technologies Inc.</li>
+     *   <li>GL_RENDERER     ATI Radeon 3100 Graphics</li>
+     *   <li>GL_VERSION      2.1.8787</li>
+     *   <li>Platform        Windows</li>
+     * </ul>
+     * </p>
+     */
+    public static final int NoARBCreateContext = 21;
+
+    /**
+     * No support for ES or desktop GL >= 3.0 current context without surface,
+     * i.e. without a default framebuffer as read- and write drawables.
+     * <p>
+     * See <i>OpenGL spec 3.0, chapter 2.1 OpenGL Fundamentals, page 7</i> or<br>
+     * <i>OpenGL ES spec 3.0.2, chapter 2.1 OpenGL Fundamentals, page 6</i>:
+     * <pre>
+     * It is possible to use a GL context without a default framebuffer, in which case
+     * a framebuffer object must be used to perform all rendering. This is useful for
+     * applications neeting to perform offscreen rendering.
+     * </pre>
+     * </p>
+     * <p>
+     * The feature will be attempted at initialization and this quirk will be set if failing.
+     * </p>
+     * <p>
+     * Known drivers failing the specification:
+     * <ul>
+     *   <li>GNU/Linux X11 Nvidia proprietary driver
+     *   <ul>
+     *     <li>GL_VERSION      4.4.0 NVIDIA 340.24</li>
+     *     <li>Platform        GNU/Linux X11</li>
+     *   </ul></li>
+     * </ul>
+     * </p>
+     */
+    public static final int NoSurfacelessCtx = 22;
 
     /** Return the number of known quirks. */
-    public static final int getCount() { return 21; }
+    public static final int getCount() { return 23; }
 
     private static final String[] _names = new String[] { "NoDoubleBufferedPBuffer", "NoDoubleBufferedBitmap", "NoSetSwapInterval",
                                                           "NoOffscreenBitmap", "NoSetSwapIntervalPostRetarget", "GLSLBuggyDiscard",
@@ -408,7 +457,7 @@ public class GLRendererQuirks {
                                                           "NoFullFBOSupport", "GLSLNonCompliant", "GL4NeedsGL3Request",
                                                           "GLSharedContextBuggy", "GLES3ViaEGLES2Config", "SingletonEGLDisplayOnly",
                                                           "NoMultiSamplingBuffers", "BuggyColorRenderbuffer", "NoPBufferWithAccum",
-                                                          "NeedSharedObjectSync"
+                                                          "NeedSharedObjectSync", "NoARBCreateContext", "NoSurfacelessCtx"
                                                         };
 
     private static final IdentityHashMap<String, GLRendererQuirks> stickyDeviceQuirks = new IdentityHashMap<String, GLRendererQuirks>();
