@@ -34,10 +34,13 @@ import jogamp.opengl.FPSCounterImpl;
 import java.io.PrintStream;
 import java.util.ArrayList;
 
-import javax.media.opengl.GLAnimatorControl;
-import javax.media.opengl.GLAutoDrawable;
-import javax.media.opengl.GLException;
-import javax.media.opengl.GLProfile;
+import com.jogamp.opengl.GLAnimatorControl;
+import com.jogamp.opengl.GLAutoDrawable;
+import com.jogamp.opengl.GLException;
+import com.jogamp.opengl.GLProfile;
+
+import com.jogamp.common.ExceptionUtils;
+import com.jogamp.common.util.InterruptedRuntimeException;
 
 /**
  * Base implementation of GLAnimatorControl<br>
@@ -528,12 +531,17 @@ public abstract class AnimatorBase implements GLAnimatorControl {
     /**
      * Should be called in case of an uncaught exception
      * from within the animator thread, throws given exception if no handler has been installed.
+     * @return {@code true} if handled, otherwise {@code false}. In case of {@code false},
+     *         caller needs to propagate the exception.
      */
-    protected final synchronized void handleUncaughtException(final UncaughtAnimatorException ue) {
+    protected final synchronized boolean handleUncaughtException(final UncaughtAnimatorException ue) {
         if( null != uncaughtExceptionHandler ) {
             try {
                 uncaughtExceptionHandler.uncaughtException(this, ue.getGLAutoDrawable(), ue.getCause());
             } catch (final Throwable t) { /* ignore intentionally */ }
+            return true;
+        } else {
+            return false;
         }
     }
 
@@ -589,7 +597,9 @@ public abstract class AnimatorBase implements GLAnimatorControl {
                 notifyAll();
                 try {
                     wait(pollPeriod);
-                } catch (final InterruptedException ie) {  }
+                } catch (final InterruptedException ie) {
+                    throw new InterruptedRuntimeException(ie);
+                }
                 remaining -= System.currentTimeMillis() - t1 ;
                 nok = waitCondition.eval();
             }
@@ -620,7 +630,7 @@ public abstract class AnimatorBase implements GLAnimatorControl {
                     " - " + getThreadName());
             System.err.println(" - "+toString());
             if(nok) {
-                Thread.dumpStack();
+                ExceptionUtils.dumpStack(System.err);
             }
         }
         return res;
