@@ -46,27 +46,28 @@ import java.util.Iterator;
 import java.util.Map;
 import java.util.Set;
 
-import javax.media.nativewindow.AbstractGraphicsConfiguration;
-import javax.media.nativewindow.AbstractGraphicsDevice;
-import javax.media.nativewindow.AbstractGraphicsScreen;
-import javax.media.nativewindow.NativeSurface;
-import javax.media.nativewindow.NativeWindowFactory;
-import javax.media.nativewindow.OffscreenLayerSurface;
-import javax.media.nativewindow.ProxySurface;
-import javax.media.nativewindow.MutableSurface;
-import javax.media.nativewindow.UpstreamSurfaceHook;
-import javax.media.opengl.GLAutoDrawable;
-import javax.media.opengl.GLCapabilities;
-import javax.media.opengl.GLCapabilitiesChooser;
-import javax.media.opengl.GLCapabilitiesImmutable;
-import javax.media.opengl.GLContext;
-import javax.media.opengl.GLDrawable;
-import javax.media.opengl.GLDrawableFactory;
-import javax.media.opengl.GLException;
-import javax.media.opengl.GLFBODrawable;
-import javax.media.opengl.GLOffscreenAutoDrawable;
-import javax.media.opengl.GLProfile;
+import com.jogamp.nativewindow.AbstractGraphicsConfiguration;
+import com.jogamp.nativewindow.AbstractGraphicsDevice;
+import com.jogamp.nativewindow.AbstractGraphicsScreen;
+import com.jogamp.nativewindow.NativeSurface;
+import com.jogamp.nativewindow.NativeWindowFactory;
+import com.jogamp.nativewindow.OffscreenLayerSurface;
+import com.jogamp.nativewindow.ProxySurface;
+import com.jogamp.nativewindow.MutableSurface;
+import com.jogamp.nativewindow.UpstreamSurfaceHook;
+import com.jogamp.opengl.GLAutoDrawable;
+import com.jogamp.opengl.GLCapabilities;
+import com.jogamp.opengl.GLCapabilitiesChooser;
+import com.jogamp.opengl.GLCapabilitiesImmutable;
+import com.jogamp.opengl.GLContext;
+import com.jogamp.opengl.GLDrawable;
+import com.jogamp.opengl.GLDrawableFactory;
+import com.jogamp.opengl.GLException;
+import com.jogamp.opengl.GLFBODrawable;
+import com.jogamp.opengl.GLOffscreenAutoDrawable;
+import com.jogamp.opengl.GLProfile;
 
+import com.jogamp.common.ExceptionUtils;
 import com.jogamp.nativewindow.MutableGraphicsConfiguration;
 import com.jogamp.nativewindow.DelegatedUpstreamSurfaceHookWithSurfaceSize;
 import com.jogamp.nativewindow.UpstreamSurfaceHookMutableSize;
@@ -86,12 +87,98 @@ public abstract class GLDrawableFactoryImpl extends GLDrawableFactory {
     super();
   }
 
+  /**
+   * Returns {@code true} if context is capable of operating without a surface,
+   * otherwise returns {@code false} and sets the {@link GLRendererQuirks#NoSurfacelessCtx}.
+   * <p>
+   * Method will skip probing in case {@link GLRendererQuirks#NoSurfacelessCtx} has been already set.
+   * </p>
+   * <p>
+   * Caller shall skip probing in case surfaceless support is not possible,
+   * e.g. OpenGL ES without {@code EGL_KHR_surfaceless_context} or EGL &lt; 1.5,
+   * or desktop OpenGL context &lt; 3.0 and instead call {@link #setNoSurfacelessCtxQuirk(GLContext)}!
+   * </p>
+   *
+   * @param context the context to probe, must be current
+   * @param restoreDrawable If {@code true}, the initial drawable will be restored after probing
+   *                        and the temporary {@code zeroDrawable} will be released.
+   *                        If {@code false}, the temporary {@code zeroDrawable} will be kept (bound) to the context.
+   *                        Restoration may be skipped, if the drawable and context will be destroyed anyways.
+   *
+   * @see GLRendererQuirks#NoSurfacelessCtx
+   */
+  protected final boolean probeSurfacelessCtx(final GLContext context, final boolean restoreDrawable) {
+      final GLDrawable origDrawable = context.getGLDrawable();
+      final AbstractGraphicsDevice device = origDrawable.getNativeSurface().getGraphicsConfiguration().getScreen().getDevice();
+
+      final boolean noSurfacelessCtxQuirk = context.hasRendererQuirk(GLRendererQuirks.NoSurfacelessCtx);
+      boolean allowsSurfacelessCtx = false;
+
+      if( !noSurfacelessCtxQuirk ) {
+          GLDrawable zeroDrawable = null;
+          try {
+              final GLCapabilitiesImmutable caps = origDrawable.getRequestedGLCapabilities();
+              final ProxySurface zeroSurface = createSurfacelessImpl(device, true, caps, caps, null, 64, 64);
+              zeroDrawable = createOnscreenDrawableImpl(zeroSurface);
+              zeroDrawable.setRealized(true);
+
+              // Since context is still current,
+              // will keep context current w/ zeroDrawable or throws GLException
+              context.setGLDrawable(zeroDrawable, false);
+              allowsSurfacelessCtx = true; // if setGLDrawable is successful, i.e. no GLException
+
+              if( restoreDrawable ) {
+                  context.setGLDrawable(origDrawable, false);
+              }
+          } catch (final Throwable t) {
+              if( DEBUG  || GLContext.DEBUG ) {
+                  ExceptionUtils.dumpThrowable("", t);
+              }
+          } finally {
+              if( null != zeroDrawable && restoreDrawable ) {
+                  zeroDrawable.setRealized(false);
+              }
+          }
+      }
+      if( !noSurfacelessCtxQuirk && !allowsSurfacelessCtx ) {
+          setNoSurfacelessCtxQuirkImpl(device, context);
+      }
+      return allowsSurfacelessCtx;
+  }
+
+  /**
+   * Method will set the {@link GLRendererQuirks#NoSurfacelessCtx}
+   * if it has not been set already.
+   *
+   * @param context the context to probe, must be current
+   * @see GLRendererQuirks#NoSurfacelessCtx
+   */
+  protected final void setNoSurfacelessCtxQuirk(final GLContext context) {
+      final boolean noSurfacelessCtxQuirk = context.hasRendererQuirk(GLRendererQuirks.NoSurfacelessCtx);
+      if( !noSurfacelessCtxQuirk ) {
+          final GLDrawable origDrawable = context.getGLDrawable();
+          final AbstractGraphicsDevice device = origDrawable.getNativeSurface().getGraphicsConfiguration().getScreen().getDevice();
+          setNoSurfacelessCtxQuirkImpl(device, context);
+      }
+  }
+  private final void setNoSurfacelessCtxQuirkImpl(final AbstractGraphicsDevice device, final GLContext context) {
+      final int quirk = GLRendererQuirks.NoSurfacelessCtx;
+      if(DEBUG || GLContext.DEBUG) {
+          System.err.println("Quirk: "+GLRendererQuirks.toString(quirk)+" -> "+device+": cause: probe");
+      }
+      final GLRendererQuirks glrq = context.getRendererQuirks();
+      if( null != glrq ) {
+          glrq.addQuirk(quirk);
+      }
+      GLRendererQuirks.addStickyDeviceQuirk(device, quirk);
+  }
+
   /**
    * Returns the shared resource mapped to the <code>device</code> {@link AbstractGraphicsDevice#getConnection()},
    * either a pre-existing or newly created, or <code>null</code> if creation failed or not supported.<br>
    * Creation of the shared resource is tried only once.
    *
-   * @param device which {@link javax.media.nativewindow.AbstractGraphicsDevice#getConnection() connection} denotes the shared the target device, may be <code>null</code> for the platform's default device.
+   * @param device which {@link com.jogamp.nativewindow.AbstractGraphicsDevice#getConnection() connection} denotes the shared the target device, may be <code>null</code> for the platform's default device.
    */
   protected final SharedResourceRunner.Resource getOrCreateSharedResource(AbstractGraphicsDevice device) {
       try {
@@ -114,7 +201,7 @@ public abstract class GLDrawableFactoryImpl extends GLDrawableFactory {
    * either a pre-existing or newly created, or <code>null</code> if creation failed or <b>not supported</b>.<br>
    * Creation of the shared context is tried only once.
    *
-   * @param device which {@link javax.media.nativewindow.AbstractGraphicsDevice#getConnection() connection} denotes the shared the target device, may be <code>null</code> for the platform's default device.
+   * @param device which {@link com.jogamp.nativewindow.AbstractGraphicsDevice#getConnection() connection} denotes the shared the target device, may be <code>null</code> for the platform's default device.
    */
   public final GLContext getOrCreateSharedContext(final AbstractGraphicsDevice device) {
       final SharedResourceRunner.Resource sr = getOrCreateSharedResource( device );
@@ -128,26 +215,35 @@ public abstract class GLDrawableFactoryImpl extends GLDrawableFactory {
   protected final boolean createSharedResourceImpl(final AbstractGraphicsDevice device) {
       final SharedResourceRunner.Resource sr = getOrCreateSharedResource( device );
       if(null!=sr) {
-          return sr.isValid();
+          return sr.isAvailable();
       }
       return false;
   }
 
   @Override
-  public final GLRendererQuirks getRendererQuirks(final AbstractGraphicsDevice device) {
+  public final GLRendererQuirks getRendererQuirks(final AbstractGraphicsDevice device, final GLProfile glp) {
       final SharedResourceRunner.Resource sr = getOrCreateSharedResource( device );
       if(null!=sr) {
-          return sr.getRendererQuirks();
+          return sr.getRendererQuirks(glp);
       }
       return null;
   }
 
+  /**
+   * Method returns {@code true} if underlying {@link #createContextARBImpl(long, boolean, int, int, int) <i>ARB context creation</i>}
+   * supports {@code major} and {@code minor} version number.
+   * <p>
+   * Otherwise only the {@code major} version number is supported for context creation.
+   * </p>
+   */
+  public abstract boolean hasMajorMinorCreateContextARB();
+
   /**
    * Returns the shared device mapped to the <code>device</code> {@link AbstractGraphicsDevice#getConnection()},
    * either a preexisting or newly created, or <code>null</code> if creation failed or not supported.<br>
    * Creation of the shared context is tried only once.
    *
-   * @param device which {@link javax.media.nativewindow.AbstractGraphicsDevice#getConnection() connection} denotes the shared device to be used, may be <code>null</code> for the platform's default device.
+   * @param device which {@link com.jogamp.nativewindow.AbstractGraphicsDevice#getConnection() connection} denotes the shared device to be used, may be <code>null</code> for the platform's default device.
    */
   protected final AbstractGraphicsDevice getOrCreateSharedDevice(final AbstractGraphicsDevice device) {
       final SharedResourceRunner.Resource sr = getOrCreateSharedResource( device );
@@ -158,11 +254,11 @@ public abstract class GLDrawableFactoryImpl extends GLDrawableFactory {
   }
 
   /**
-   * Returns the GLDynamicLookupHelper
-   * @param profile if EGL/ES, profile <code>1</code> refers to ES1 and <code>2</code> to ES2,
-   *        otherwise the profile is ignored.
+   * Returns the GLDynamicLookupHelper if installed, otherwise {@code null}.
+   * @param majorVersion the major OpenGL profile version
+   * @param contextOptions the context profile options
    */
-  public abstract GLDynamicLookupHelper getGLDynamicLookupHelper(int profile);
+  public abstract GLDynamicLookupHelper getGLDynamicLookupHelper(final int majorVersion, final int contextOptions);
 
   //---------------------------------------------------------------------------
   // Dispatching GLDrawable construction in respect to the NativeSurface Capabilities
@@ -195,7 +291,7 @@ public abstract class GLDrawableFactoryImpl extends GLDrawableFactory {
                 System.err.println("chosenCapsMod: "+chosenCapsMod);
                 System.err.println("OffscreenLayerSurface: **** "+ols);
                 System.err.println("Target: **** "+target);
-                Thread.dumpStack();
+                ExceptionUtils.dumpStack(System.err);
             }
             if( ! ( target instanceof MutableSurface ) ) {
                 throw new IllegalArgumentException("Passed NativeSurface must implement SurfaceChangeable for offscreen layered surface: "+target);
@@ -271,9 +367,9 @@ public abstract class GLDrawableFactoryImpl extends GLDrawableFactory {
 
   @Override
   public final GLOffscreenAutoDrawable createOffscreenAutoDrawable(final AbstractGraphicsDevice deviceReq,
-                                                             final GLCapabilitiesImmutable capsRequested,
-                                                             final GLCapabilitiesChooser chooser,
-                                                             final int width, final int height) {
+                                                                   final GLCapabilitiesImmutable capsRequested,
+                                                                   final GLCapabilitiesChooser chooser,
+                                                                   final int width, final int height) {
     final GLDrawable drawable = createOffscreenDrawable( deviceReq, capsRequested, chooser, width, height );
     try {
         drawable.setRealized(true);
@@ -306,24 +402,32 @@ public abstract class GLDrawableFactoryImpl extends GLDrawableFactory {
 
   @Override
   public final GLDrawable createOffscreenDrawable(final AbstractGraphicsDevice deviceReq,
-                                            final GLCapabilitiesImmutable capsRequested,
-                                            final GLCapabilitiesChooser chooser,
-                                            final int width, final int height) {
+                                                  final GLCapabilitiesImmutable capsRequested,
+                                                  final GLCapabilitiesChooser chooser,
+                                                  final int width, final int height) {
     if(width<=0 || height<=0) {
         throw new GLException("initial size must be positive (were (" + width + " x " + height + "))");
     }
-    final AbstractGraphicsDevice device = getOrCreateSharedDevice(deviceReq);
-    if(null == device) {
+    final SharedResourceRunner.Resource sr = getOrCreateSharedResource( deviceReq );
+    if( null == sr ) {
         throw new GLException("No shared device for requested: "+deviceReq);
     }
-
+    final AbstractGraphicsDevice device = sr.getDevice();
     final GLCapabilitiesImmutable capsChosen = GLGraphicsConfigurationUtil.fixOffscreenGLCapabilities(capsRequested, this, device);
 
     if( capsChosen.isFBO() ) {
         // Use minimum GLCapabilities for the dummy surface w/ same profile
-        final ProxySurface dummySurface = createDummySurfaceImpl(device, true, new GLCapabilities(capsChosen.getGLProfile()), capsRequested, null, width, height);
-        final GLDrawableImpl dummyDrawable = createOnscreenDrawableImpl(dummySurface);
-        return new GLFBODrawableImpl.ResizeableImpl(this, dummyDrawable, dummySurface, capsChosen, 0);
+        final GLProfile glp = capsChosen.getGLProfile();
+        final GLCapabilitiesImmutable glCapsMin = new GLCapabilities(glp);
+        final GLRendererQuirks glrq = sr.getRendererQuirks(glp);
+        final ProxySurface surface;
+        if( null != glrq && !glrq.exist(GLRendererQuirks.NoSurfacelessCtx) ) {
+            surface = createSurfacelessImpl(device, true, glCapsMin, capsRequested, null, width, height);
+        } else {
+            surface = createDummySurfaceImpl(device, true, glCapsMin, capsRequested, null, width, height);
+        }
+        final GLDrawableImpl drawable = createOnscreenDrawableImpl(surface);
+        return new GLFBODrawableImpl.ResizeableImpl(this, drawable, surface, capsChosen, 0);
     }
     return createOffscreenDrawableImpl( createMutableSurfaceImpl(device, true, capsChosen, capsRequested, chooser,
                                                                  new UpstreamSurfaceHookMutableSize(width, height) ) );
@@ -380,8 +484,7 @@ public abstract class GLDrawableFactoryImpl extends GLDrawableFactory {
                                                            GLCapabilitiesChooser chooser, UpstreamSurfaceHook upstreamHook);
 
   /**
-   * A dummy surface is not visible on screen and will not be used to render directly to,
-   * it maybe on- or offscreen.
+   * A dummy surface is not visible on screen, it may be on- or offscreen.
    * <p>
    * It is used to allow the creation of a {@link GLDrawable} and {@link GLContext} to query information.
    * It also allows creation of framebuffer objects which are used for rendering or using a shared GLContext w/o actually rendering to a usable framebuffer.
@@ -389,7 +492,7 @@ public abstract class GLDrawableFactoryImpl extends GLDrawableFactory {
    * <p>
    * Creates a new independent device instance using <code>deviceReq</code> details.
    * </p>
-   * @param deviceReq which {@link javax.media.nativewindow.AbstractGraphicsDevice#getConnection() connection} denotes the shared device to be used, may be <code>null</code> for the platform's default device.
+   * @param deviceReq which {@link com.jogamp.nativewindow.AbstractGraphicsDevice#getConnection() connection} denotes the shared device to be used, may be <code>null</code> for the platform's default device.
    * @param requestedCaps
    * @param chooser the custom chooser, may be null for default
    * @param width the initial width as returned by {@link NativeSurface#getSurfaceWidth()}, not the actual dummy surface width.
@@ -409,8 +512,7 @@ public abstract class GLDrawableFactoryImpl extends GLDrawableFactory {
   }
 
   /**
-   * A dummy surface is not visible on screen and will not be used to render directly to,
-   * it maybe on- or offscreen.
+   * A dummy surface is not visible on screen and may be on- or offscreen.
    * <p>
    * It is used to allow the creation of a {@link GLDrawable} and {@link GLContext} to query information.
    * It also allows creation of framebuffer objects which are used for rendering or using a shared GLContext w/o actually rendering to a usable framebuffer.
@@ -430,6 +532,27 @@ public abstract class GLDrawableFactoryImpl extends GLDrawableFactory {
   public abstract ProxySurface createDummySurfaceImpl(AbstractGraphicsDevice device, boolean createNewDevice,
                                                       GLCapabilitiesImmutable chosenCaps, GLCapabilitiesImmutable requestedCaps, GLCapabilitiesChooser chooser, int width, int height);
 
+  /**
+   * A surfaceless {@link ProxySurface} is a non-existing surface and will not be used as a render target.
+   * <p>
+   * It is used to allow the creation of a {@link GLDrawable} and {@link GLContext} w/o default framebuffer to query information.
+   * It also allows creation of framebuffer objects which are used for rendering or using a shared GLContext w/o actually rendering to a usable framebuffer.
+   * </p>
+   * @param device a valid platform dependent target device.
+   * @param createNewDevice if <code>true</code> a new device instance is created using <code>device</code> details,
+   *                        otherwise <code>device</code> instance is used as-is.
+   * @param chosenCaps
+   * @param requestedCaps
+   * @param chooser the custom chooser, may be null for default
+   * @param width the initial width as returned by {@link NativeSurface#getSurfaceWidth()}, not the actual dummy surface width.
+   *        The latter is platform specific and small
+   * @param height the initial height as returned by {@link NativeSurface#getSurfaceHeight()}, not the actual dummy surface height,
+   *        The latter is platform specific and small
+   * @return the created {@link ProxySurface} instance w/o defined surface handle but platform specific {@link UpstreamSurfaceHook}.
+   */
+  public abstract ProxySurface createSurfacelessImpl(AbstractGraphicsDevice device, boolean createNewDevice,
+                                                     GLCapabilitiesImmutable chosenCaps, GLCapabilitiesImmutable requestedCaps, GLCapabilitiesChooser chooser, int width, int height);
+
   //---------------------------------------------------------------------------
   //
   // ProxySurface (Wrapped pre-existing native surface) construction
