@@ -1,16 +1,17 @@
 package jogamp.opengl;
 
-import javax.media.nativewindow.NativeSurface;
-import javax.media.nativewindow.NativeWindowException;
-import javax.media.nativewindow.ProxySurface;
-import javax.media.nativewindow.UpstreamSurfaceHook;
-import javax.media.opengl.GL;
-import javax.media.opengl.GLCapabilities;
-import javax.media.opengl.GLCapabilitiesImmutable;
-import javax.media.opengl.GLContext;
-import javax.media.opengl.GLException;
-import javax.media.opengl.GLFBODrawable;
+import com.jogamp.nativewindow.NativeSurface;
+import com.jogamp.nativewindow.NativeWindowException;
+import com.jogamp.nativewindow.ProxySurface;
+import com.jogamp.nativewindow.UpstreamSurfaceHook;
+import com.jogamp.opengl.GL;
+import com.jogamp.opengl.GLCapabilities;
+import com.jogamp.opengl.GLCapabilitiesImmutable;
+import com.jogamp.opengl.GLContext;
+import com.jogamp.opengl.GLException;
+import com.jogamp.opengl.GLFBODrawable;
 
+import com.jogamp.common.ExceptionUtils;
 import com.jogamp.common.util.PropertyAccess;
 import com.jogamp.common.util.VersionUtil;
 import com.jogamp.nativewindow.MutableGraphicsConfiguration;
@@ -52,6 +53,7 @@ public class GLFBODrawableImpl extends GLDrawableImpl implements GLFBODrawable {
     private GLCapabilitiesImmutable origParentChosenCaps;
 
     private boolean initialized;
+    private int maxSamples;
     private int fboModeBits;
     private int texUnit;
     private int samples;
@@ -103,7 +105,7 @@ public class GLFBODrawableImpl extends GLDrawableImpl implements GLFBODrawable {
 
     private final void setupFBO(final GL gl, final int idx, final int width, final int height, final int samples,
                                 final boolean useAlpha, final int depthBits, final int stencilBits,
-                                final boolean useTexture, final boolean realUnbind) {
+                                final boolean useTexture, final boolean setupViewportScissors, final boolean realUnbind) {
         final FBObject fbo = new FBObject();
         fbos[idx] = fbo;
 
@@ -154,6 +156,11 @@ public class GLFBODrawableImpl extends GLDrawableImpl implements GLFBODrawable {
         // Also remedy for Bug 1020, i.e. OSX/Nvidia's FBO needs to be cleared before blitting,
         // otherwise first MSAA frame lacks antialiasing.
         fbo.bind(gl);
+        if( setupViewportScissors ) {
+            // Surfaceless: Set initial viewport/scissors
+            gl.glViewport(0, 0, width, height);
+            gl.glScissor(0, 0, width, height);
+        }
         if( useDepth ) {
             gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
         } else {
@@ -170,7 +177,7 @@ public class GLFBODrawableImpl extends GLDrawableImpl implements GLFBODrawable {
         if( !initialized && !realize ) {
             if( DEBUG ) {
                 System.err.println("GLFBODrawableImpl.initialize(): WARNING - Already unrealized!");
-                Thread.dumpStack();
+                ExceptionUtils.dumpStack(System.err);
             }
             return; // NOP, no exception for de-init twice or no init!
         }
@@ -180,7 +187,7 @@ public class GLFBODrawableImpl extends GLDrawableImpl implements GLFBODrawable {
         if(realize) {
             final GLCapabilities chosenFBOCaps = (GLCapabilities) getChosenGLCapabilities(); // cloned at setRealized(true)
 
-            final int maxSamples = gl.getMaxRenderbufferSamples();
+            maxSamples = gl.getMaxRenderbufferSamples(); // if > 0 implies fullFBOSupport
             {
                 final int newSamples = samples <= maxSamples ? samples : maxSamples;
                 if(DEBUG) {
@@ -215,7 +222,9 @@ public class GLFBODrawableImpl extends GLDrawableImpl implements GLFBODrawable {
 
             for(int i=0; i<fbosN; i++) {
                 setupFBO(gl, i, width, height, samples, useAlpha,
-                         chosenFBOCaps.getDepthBits(), chosenFBOCaps.getStencilBits(), useTexture, fbosN-1==i);
+                         chosenFBOCaps.getDepthBits(), chosenFBOCaps.getStencilBits(), useTexture,
+                         0==i && 0 == parent.getHandle() /* setupViewportScissors for surfaceless */,
+                         fbosN-1==i /* unbind */);
             }
             fbos[0].formatToGLCapabilities(chosenFBOCaps);
             chosenFBOCaps.setDoubleBuffered( chosenFBOCaps.getDoubleBuffered() || samples > 0 );
@@ -232,7 +241,7 @@ public class GLFBODrawableImpl extends GLDrawableImpl implements GLFBODrawable {
 
         if(DEBUG) {
             System.err.println("GLFBODrawableImpl.initialize("+realize+"): "+this);
-            Thread.dumpStack();
+            ExceptionUtils.dumpStack(System.err);
         }
     }
 
@@ -269,7 +278,7 @@ public class GLFBODrawableImpl extends GLDrawableImpl implements GLFBODrawable {
         // resetQuirk fallback
         fbos[idx].destroy(gl);
         final boolean useTexture = 0 != ( FBOMODE_USE_TEXTURE & fboModeBits );
-        setupFBO(gl, idx, width, height, samples, useAlpha, depthBits, stencilBits, useTexture, true);
+        setupFBO(gl, idx, width, height, samples, useAlpha, depthBits, stencilBits, useTexture, false, true);
     }
 
     private final void reset(final GL gl, int newSamples) throws GLException {
@@ -283,7 +292,7 @@ public class GLFBODrawableImpl extends GLDrawableImpl implements GLFBODrawable {
         final boolean ctxSwitch = null != curContext && curContext != ourContext;
         if(DEBUG) {
             System.err.println("GLFBODrawableImpl.reset(newSamples "+newSamples+"): BEGIN - ctxSwitch "+ctxSwitch+", "+this);
-            Thread.dumpStack();
+            ExceptionUtils.dumpStack(System.err);
         }
         Throwable tFBO = null;
         Throwable tGL = null;
@@ -292,7 +301,6 @@ public class GLFBODrawableImpl extends GLDrawableImpl implements GLFBODrawable {
         fboBound = false; // clear bound-flag immediatly, caused by contextMadeCurrent(..) - otherwise we would swap @ release
         fboSwapped = false;
         try {
-            final int maxSamples = gl.getMaxRenderbufferSamples();
             newSamples = newSamples <= maxSamples ? newSamples : maxSamples;
 
             if(0==samples && 0<newSamples || 0<samples && 0==newSamples) {
@@ -358,11 +366,6 @@ public class GLFBODrawableImpl extends GLDrawableImpl implements GLFBODrawable {
     // GLDrawableImpl
     //
 
-    @Override
-    public final GLDynamicLookupHelper getGLDynamicLookupHelper() {
-        return parent.getGLDynamicLookupHelper();
-    }
-
     @Override
     protected final int getDefaultDrawFramebuffer() { return initialized ? fbos[fboIBack].getWriteFramebuffer() : 0; }
 
@@ -525,7 +528,7 @@ public class GLFBODrawableImpl extends GLDrawableImpl implements GLFBODrawable {
     }
 
     @Override
-    public final int setNumBuffers(final int bufferCount) throws /* IllegalStateException, */ GLException {
+    public final int setNumBuffers(final int bufferCount) throws IllegalStateException, GLException {
         if( isInitialized() ) {
             throw new IllegalStateException("Already initialized: "+this);
         }
