@@ -118,7 +118,8 @@ package jogamp.opengl;
 import java.nio.FloatBuffer;
 import java.nio.IntBuffer;
 
-import javax.media.opengl.fixedfunc.GLMatrixFunc;
+import com.jogamp.opengl.GLException;
+import com.jogamp.opengl.fixedfunc.GLMatrixFunc;
 
 import com.jogamp.opengl.math.FloatUtil;
 
@@ -169,8 +170,10 @@ public class ProjectFloat {
    * @param aspect
    * @param zNear
    * @param zFar
+   * @throws GLException if {@code zNear <= 0} or {@code zFar <= zNear}
+   * @see FloatUtil#makePerspective(float[], int, boolean, float, float, float, float)
    */
-  public void gluPerspective(final GLMatrixFunc gl, final float fovy_deg, final float aspect, final float zNear, final float zFar) {
+  public void gluPerspective(final GLMatrixFunc gl, final float fovy_deg, final float aspect, final float zNear, final float zFar) throws GLException {
     gl.glMultMatrixf(FloatUtil.makePerspective(mat4Tmp1, 0, true, fovy_deg * FloatUtil.PI / 180.0f, aspect, zNear, zFar), 0);
   }
 
