@@ -30,10 +30,9 @@ package jogamp.opengl.android.av;
 import java.io.IOException;
 import java.util.List;
 
-import javax.media.opengl.GL;
-import javax.media.opengl.GLES2;
-import javax.media.opengl.GLException;
-
+import com.jogamp.opengl.GL;
+import com.jogamp.opengl.GLES2;
+import com.jogamp.opengl.GLException;
 import com.jogamp.common.os.AndroidVersion;
 import com.jogamp.common.os.Platform;
 import com.jogamp.opengl.util.TimeFrameI;
@@ -254,7 +253,7 @@ public class AndroidGLMediaPlayerAPI14 extends GLMediaPlayerImpl {
 
     @Override
     protected final void initStreamImpl(final int vid, final int aid) throws IOException {
-        if( null == getURI() ) {
+        if( null == getUri() ) {
             return;
         }
         if( null == mp && null == cam ) {
@@ -263,7 +262,7 @@ public class AndroidGLMediaPlayerAPI14 extends GLMediaPlayerImpl {
             } else {
                 int cameraId = 0;
                 try {
-                    cameraId = Integer.parseInt(cameraPath);
+                    cameraId = Integer.parseInt(cameraPath.decode());
                 } catch (final NumberFormatException nfe) {}
                 if( 0 <= cameraId && cameraId < Camera.getNumberOfCameras() ) {
                     cam = Camera.open(cameraId);
@@ -280,7 +279,7 @@ public class AndroidGLMediaPlayerAPI14 extends GLMediaPlayerImpl {
             } // else FIXME: Select aid !
             // Note: Both FIXMEs seem to be n/a via Android's MediaPlayer -> Switch to API level 16 MediaCodec/MediaExtractor ..
             try {
-                final Uri _uri = Uri.parse(getURI().toString());
+                final Uri _uri = Uri.parse(getUri().toString());
                 mp.setDataSource(StaticContext.getContext(), _uri);
             } catch (final IllegalArgumentException e) {
                 throw new RuntimeException(e);
@@ -293,7 +292,7 @@ public class AndroidGLMediaPlayerAPI14 extends GLMediaPlayerImpl {
             try {
                 mp.prepare();
             } catch (final IOException ioe) {
-                throw new IOException("MediaPlayer failed to process stream <"+getURI().toString()+">: "+ioe.getMessage(), ioe);
+                throw new IOException("MediaPlayer failed to process stream <"+getUri().toString()+">: "+ioe.getMessage(), ioe);
             }
             final int r_aid = GLMediaPlayer.STREAM_ID_NONE == aid ? GLMediaPlayer.STREAM_ID_NONE : 1 /* fake */;
             final String icodec = "android";
@@ -364,7 +363,7 @@ public class AndroidGLMediaPlayerAPI14 extends GLMediaPlayerImpl {
     }
 
     @Override
-    protected final int getNextTextureImpl(final GL gl, final TextureFrame nextFrame) {
+    protected final int getNextTextureImpl(final GL gl, final TextureFrame nextFrame) throws InterruptedException {
         int pts = TimeFrameI.INVALID_PTS;
         if(null != mp || null != cam) {
             final SurfaceTextureFrame sTexFrame = null != nextFrame ? (SurfaceTextureFrame) nextFrame : singleSTexFrame;
@@ -381,7 +380,7 @@ public class AndroidGLMediaPlayerAPI14 extends GLMediaPlayerImpl {
                         cam.setPreviewTexture(sTexFrame.surfaceTex);
                         cam.startPreview();
                     } catch (final IOException ioe) {
-                        throw new RuntimeException("MediaPlayer failed to process stream <"+getURI().toString()+">: "+ioe.getMessage(), ioe);
+                        throw new RuntimeException("MediaPlayer failed to process stream <"+getUri().toString()+">: "+ioe.getMessage(), ioe);
                     }
                 }
                 if( null != surface ) {
@@ -398,12 +397,8 @@ public class AndroidGLMediaPlayerAPI14 extends GLMediaPlayerImpl {
                 boolean update = updateSurface;
                 if( !update ) {
                     synchronized(updateSurfaceLock) {
-                        if(!updateSurface) { // volatile OK.
-                            try {
-                                updateSurfaceLock.wait();
-                            } catch (final InterruptedException e) {
-                                e.printStackTrace();
-                            }
+                        while(!updateSurface) { // volatile OK.
+                            updateSurfaceLock.wait(); // propagates InterruptedException
                         }
                         update = updateSurface;
                         updateSurface = false;
