@@ -45,11 +45,12 @@ import java.util.Set;
 import java.util.Map.Entry;
 
 import javax.imageio.ImageIO;
-import javax.media.nativewindow.util.DimensionImmutable;
-import javax.media.opengl.GL;
-import javax.media.opengl.GLAutoDrawable;
-import javax.media.opengl.GLCapabilitiesImmutable;
-import javax.media.opengl.GLEventListener;
+import com.jogamp.nativewindow.util.DimensionImmutable;
+import com.jogamp.nativewindow.util.PixelFormat;
+import com.jogamp.opengl.GL;
+import com.jogamp.opengl.GLAutoDrawable;
+import com.jogamp.opengl.GLCapabilitiesImmutable;
+import com.jogamp.opengl.GLEventListener;
 
 import jogamp.opengl.Debug;
 
@@ -288,8 +289,9 @@ public class AWTTilePainter {
                 final int tWidth = renderer.getParam(TileRenderer.TR_TILE_WIDTH);
                 final int tHeight = renderer.getParam(TileRenderer.TR_TILE_HEIGHT);
                 final AWTGLPixelBufferProvider printBufferProvider = new AWTGLPixelBufferProvider( true /* allowRowStride */ );
-                final GLPixelAttributes pixelAttribs = printBufferProvider.getAttributes(gl, componentCount);
-                tBuffer = printBufferProvider.allocate(gl, pixelAttribs, tWidth, tHeight, 1, true, 0);
+                final PixelFormat.Composition hostPixelComp = printBufferProvider.getHostPixelComp(gl.getGLProfile(), componentCount);
+                final GLPixelAttributes pixelAttribs = printBufferProvider.getAttributes(gl, componentCount, true);
+                tBuffer = printBufferProvider.allocate(gl, hostPixelComp, pixelAttribs, true, tWidth, tHeight, 1, 0);
                 renderer.setTileBuffer(tBuffer);
                 if( flipVertical ) {
                     vFlipImage = new BufferedImage(tBuffer.width, tBuffer.height, tBuffer.image.getType());
