@@ -31,16 +31,19 @@ package jogamp.opengl.egl;
 import java.nio.IntBuffer;
 import java.util.Iterator;
 
-import javax.media.nativewindow.AbstractGraphicsDevice;
-import javax.media.nativewindow.NativeSurface;
-import javax.media.nativewindow.NativeWindowFactory;
-import javax.media.nativewindow.ToolkitLock;
-import javax.media.opengl.GLException;
+import com.jogamp.nativewindow.AbstractGraphicsDevice;
+import com.jogamp.nativewindow.NativeSurface;
+import com.jogamp.nativewindow.NativeWindowFactory;
+import com.jogamp.nativewindow.ToolkitLock;
+import com.jogamp.opengl.GLException;
 
 import jogamp.opengl.Debug;
 
+import com.jogamp.common.ExceptionUtils;
+import com.jogamp.common.nio.Buffers;
 import com.jogamp.common.util.LongObjectHashMap;
 import com.jogamp.nativewindow.egl.EGLGraphicsDevice;
+import com.jogamp.opengl.egl.EGL;
 
 /**
  * This implementation provides recursive calls to
@@ -77,10 +80,17 @@ public class EGLDisplayUtil {
         static EGLDisplayRef getOrCreateOpened(final long eglDisplay, final IntBuffer major, final IntBuffer minor) {
             final EGLDisplayRef o = (EGLDisplayRef) openEGLDisplays.get(eglDisplay);
             if( null == o ) {
-                if( EGL.eglInitialize(eglDisplay, major, minor) ) {
+                final boolean ok = EGL.eglInitialize(eglDisplay, major, minor);
+                if( DEBUG ) {
+                    System.err.println("EGLDisplayUtil.EGL.eglInitialize 0x"+Long.toHexString(eglDisplay)+" -> "+ok);
+                }
+                if( ok ) {
                     final EGLDisplayRef n = new EGLDisplayRef(eglDisplay);
                     openEGLDisplays.put(eglDisplay, n);
                     n.initRefCount++;
+                    if( DEBUG ) {
+                        System.err.println("EGLDisplayUtil.EGL.eglInitialize "+n);
+                    }
                     if( null == singletonEGLDisplay ) {
                         singletonEGLDisplay = n;
                     }
@@ -110,7 +120,12 @@ public class EGLDisplayUtil {
                 if( 0 < o.initRefCount ) { // no negative refCount
                     o.initRefCount--;
                     if( 0 == o.initRefCount ) {
-                        res[0] = EGL.eglTerminate(eglDisplay);
+                        final boolean ok = EGL.eglTerminate(eglDisplay);
+                        if( DEBUG ) {
+                            System.err.println("EGLDisplayUtil.EGL.eglTerminate 0x"+Long.toHexString(eglDisplay)+" -> "+ok);
+                            System.err.println("EGLDisplayUtil.EGL.eglTerminate "+o);
+                        }
+                        res[0] = ok;
                         if( o == singletonEGLDisplay ) {
                             singletonEGLDisplay = null;
                         }
@@ -148,7 +163,7 @@ public class EGLDisplayUtil {
         if(DEBUG || verbose || openEGLDisplays.size() > 0 ) {
             System.err.println("EGLDisplayUtil.EGLDisplays: Shutdown (open: "+openEGLDisplays.size()+")");
             if(DEBUG) {
-                Thread.dumpStack();
+                ExceptionUtils.dumpStack(System.err);
             }
             if( openEGLDisplays.size() > 0) {
                 dumpOpenDisplayConnections();
@@ -198,17 +213,30 @@ public class EGLDisplayUtil {
      *
      * @see EGL#eglInitialize(long, IntBuffer, IntBuffer)
      */
-    private static synchronized boolean eglInitialize(final long eglDisplay, final IntBuffer major, final IntBuffer minor)  {
+    private static synchronized boolean eglInitialize(final long eglDisplay, final int[] major, final int[] minor)  {
         if( EGL.EGL_NO_DISPLAY == eglDisplay) {
             return false;
         }
-        final EGLDisplayRef d = EGLDisplayRef.getOrCreateOpened(eglDisplay, major, minor);
+        final EGLDisplayRef d = EGLDisplayRef.getOrCreateOpened(eglDisplay, _eglMajorVersion, _eglMinorVersion);
+        final int _major = _eglMajorVersion.get(0);
+        final int _minor = _eglMinorVersion.get(0);
+        if( null != major && null != minor ) {
+            if( null != d ) {
+                major[0] = _major;
+                minor[0] = _minor;
+            } else {
+                major[0] = 0;
+                minor[0] = 0;
+            }
+        }
         if(DEBUG) {
-            System.err.println("EGLDisplayUtil.eglInitialize("+EGLContext.toHexString(eglDisplay)+" ...): "+d+" = "+(null != d)+", singletonEGLDisplay "+singletonEGLDisplay+" (use "+useSingletonEGLDisplay+")");
+            System.err.println("EGLDisplayUtil.eglInitialize("+EGLContext.toHexString(eglDisplay)+" ...): "+d+" = "+(null != d)+", eglVersion "+_major+"."+_minor+", singletonEGLDisplay "+singletonEGLDisplay+" (use "+useSingletonEGLDisplay+")");
             // Thread.dumpStack();
         }
         return null != d;
     }
+    private static final IntBuffer _eglMajorVersion = Buffers.newDirectIntBuffer(1);
+    private static final IntBuffer _eglMinorVersion = Buffers.newDirectIntBuffer(1);
 
     /**
      * @param nativeDisplayID
@@ -222,7 +250,7 @@ public class EGLDisplayUtil {
      * @see #eglGetDisplay(long)
      * @see #eglInitialize(long, IntBuffer, IntBuffer)
      */
-    private static synchronized int eglGetDisplayAndInitialize(final long nativeDisplayID, final long[] eglDisplay, final int[] eglErr, final IntBuffer major, final IntBuffer minor) {
+    private static synchronized int eglGetDisplayAndInitialize(final long nativeDisplayID, final long[] eglDisplay, final int[] eglErr, final int[] major, final int[] minor) {
         eglDisplay[0] = EGL.EGL_NO_DISPLAY;
         final long _eglDisplay = eglGetDisplay( nativeDisplayID );
         if ( EGL.EGL_NO_DISPLAY == _eglDisplay ) {
@@ -244,13 +272,15 @@ public class EGLDisplayUtil {
      *
      * @throws GLException if {@link EGL#eglGetDisplay(long)} or {@link EGL#eglInitialize(long, int[], int, int[], int)} fails incl fallback
      * @param nativeDisplayID in/out array of size 1, passing the requested nativeVisualID, may return a different revised nativeVisualID handle
+     * @param major
+     * @param minor
      * @return the initialized EGL display ID
      * @throws GLException if not successful
      */
-    private static synchronized long eglGetDisplayAndInitialize(final long[] nativeDisplayID) {
+    private static synchronized long eglGetDisplayAndInitialize(final long[] nativeDisplayID, final int[] major, final int[] minor) {
         final long[] eglDisplay = new long[1];
         final int[] eglError = new int[1];
-        int eglRes = EGLDisplayUtil.eglGetDisplayAndInitialize(nativeDisplayID[0], eglDisplay, eglError, null, null);
+        int eglRes = EGLDisplayUtil.eglGetDisplayAndInitialize(nativeDisplayID[0], eglDisplay, eglError, major, minor);
         if( EGL.EGL_SUCCESS == eglRes ) {
             return eglDisplay[0];
         }
@@ -258,7 +288,7 @@ public class EGLDisplayUtil {
             if(DEBUG) {
                 System.err.println("EGLDisplayUtil.eglGetAndInitDisplay failed with native "+EGLContext.toHexString(nativeDisplayID[0])+", error "+EGLContext.toHexString(eglRes)+"/"+EGLContext.toHexString(eglError[0])+" - fallback!");
             }
-            eglRes = EGLDisplayUtil.eglGetDisplayAndInitialize(EGL.EGL_DEFAULT_DISPLAY, eglDisplay, eglError, null, null);
+            eglRes = EGLDisplayUtil.eglGetDisplayAndInitialize(EGL.EGL_DEFAULT_DISPLAY, eglDisplay, eglError, major, minor);
             if( EGL.EGL_SUCCESS == eglRes ) {
                 nativeDisplayID[0] = EGL.EGL_DEFAULT_DISPLAY;
                 return eglDisplay[0];
@@ -286,8 +316,8 @@ public class EGLDisplayUtil {
 
     private static final EGLGraphicsDevice.EGLDisplayLifecycleCallback eglLifecycleCallback = new EGLGraphicsDevice.EGLDisplayLifecycleCallback() {
         @Override
-        public long eglGetAndInitDisplay(final long[] nativeDisplayID) {
-            return eglGetDisplayAndInitialize(nativeDisplayID);
+        public long eglGetAndInitDisplay(final long[] nativeDisplayID, final int[] major, final int[] minor) {
+            return eglGetDisplayAndInitialize(nativeDisplayID, major, minor);
         }
         @Override
         public void eglTerminate(final long eglDisplayHandle) {
@@ -313,6 +343,22 @@ public class EGLDisplayUtil {
         return new EGLGraphicsDevice(nativeDisplayID, EGL.EGL_NO_DISPLAY, connection, unitID, eglLifecycleCallback);
     }
 
+    /**
+     * Returns an uninitialized {@link EGLGraphicsDevice}. User needs to issue {@link EGLGraphicsDevice#open()} before usage.
+     * <p>
+     * Using {@link #eglGetDisplayAndInitialize(long[])} for the {@link EGLGraphicsDevice#open()} implementation
+     * and {@link #eglTerminate(long)} for {@link EGLGraphicsDevice#close()}.
+     * </p>
+     * <p>
+     * Using the default {@link ToolkitLock}, via {@link NativeWindowFactory#getDefaultToolkitLock(String, long)}.
+     * </p>
+     * @param adevice
+     * @return an uninitialized {@link EGLGraphicsDevice}
+     */
+    public static EGLGraphicsDevice eglCreateEGLGraphicsDevice(final AbstractGraphicsDevice aDevice)  {
+        return new EGLGraphicsDevice(aDevice, EGL.EGL_NO_DISPLAY, eglLifecycleCallback);
+    }
+
     /**
      * Returns an uninitialized {@link EGLGraphicsDevice}. User needs to issue {@link EGLGraphicsDevice#open()} before usage.
      * <p>
