@@ -53,8 +53,8 @@
 package jogamp.opengl.glu.tessellator;
 
 import jogamp.opengl.glu.tessellator.*;
-import javax.media.opengl.*;
-import javax.media.opengl.glu.*;
+import com.jogamp.opengl.*;
+import com.jogamp.opengl.glu.*;
 
 public class GLUtessellatorImpl implements GLUtessellator {
     public static final int TESS_MAX_CACHE = 100;
