@@ -28,28 +28,32 @@
 package jogamp.opengl.util.av;
 
 import java.io.IOException;
-import java.net.URI;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.Map;
 
-import javax.media.nativewindow.AbstractGraphicsDevice;
-import javax.media.opengl.GL;
-import javax.media.opengl.GL2GL3;
-import javax.media.opengl.GLContext;
-import javax.media.opengl.GLDrawable;
-import javax.media.opengl.GLDrawableFactory;
-import javax.media.opengl.GLES2;
-import javax.media.opengl.GLException;
-import javax.media.opengl.GLProfile;
+import com.jogamp.nativewindow.AbstractGraphicsDevice;
+import com.jogamp.opengl.GL;
+import com.jogamp.opengl.GL2GL3;
+import com.jogamp.opengl.GLContext;
+import com.jogamp.opengl.GLDrawable;
+import com.jogamp.opengl.GLDrawableFactory;
+import com.jogamp.opengl.GLES2;
+import com.jogamp.opengl.GLException;
+import com.jogamp.opengl.GLProfile;
 
 import jogamp.opengl.Debug;
 
-import com.jogamp.common.net.URIQueryProps;
+import com.jogamp.common.net.UriQueryProps;
+import com.jogamp.common.ExceptionUtils;
+import com.jogamp.common.net.Uri;
 import com.jogamp.common.os.Platform;
+import com.jogamp.common.util.InterruptSource;
+import com.jogamp.common.util.InterruptedRuntimeException;
 import com.jogamp.common.util.LFRingbuffer;
 import com.jogamp.common.util.Ringbuffer;
+import com.jogamp.common.util.SourcedInterruptedException;
 import com.jogamp.opengl.GLExtensions;
 import com.jogamp.opengl.util.TimeFrameI;
 import com.jogamp.opengl.util.av.AudioSink;
@@ -92,15 +96,15 @@ public abstract class GLMediaPlayerImpl implements GLMediaPlayer {
     private final int[] texWrapST = { GL.GL_CLAMP_TO_EDGE, GL.GL_CLAMP_TO_EDGE };
 
     /** User requested URI stream location. */
-    private URI streamLoc = null;
+    private Uri streamLoc = null;
 
     /**
      * In case {@link #streamLoc} is a {@link GLMediaPlayer#CameraInputScheme},
      * {@link #cameraPath} holds the URI's path portion
-     * as parsed in {@link #initStream(URI, int, int, int)}.
+     * as parsed in {@link #initStream(Uri, int, int, int)}.
      * @see #cameraProps
      */
-    protected String cameraPath = null;
+    protected Uri.Encoded cameraPath = null;
     /** Optional camera properties, see {@link #cameraPath}. */
     protected Map<String, String> cameraProps = null;
 
@@ -365,7 +369,7 @@ public abstract class GLMediaPlayerImpl implements GLMediaPlayer {
                 event_mask = addStateEventMask(event_mask, GLMediaPlayer.State.Paused);
                 setState( State.Paused );
                 if( null != streamWorker ) {
-                    streamWorker.doPause();
+                    streamWorker.doPause(true);
                 }
                 if( flush ) {
                     resetAVPTSAndFlush();
@@ -414,7 +418,7 @@ public abstract class GLMediaPlayerImpl implements GLMediaPlayer {
                     final State _state = state;
                     setState( State.Paused );
                     if( null != streamWorker ) {
-                        streamWorker.doPause();
+                        streamWorker.doPause(true);
                     }
                     // Adjust target ..
                     if( msec >= duration ) {
@@ -530,7 +534,7 @@ public abstract class GLMediaPlayerImpl implements GLMediaPlayer {
     }
 
     @Override
-    public final void initStream(final URI streamLoc, final int vid, final int aid, final int reqTextureCount) throws IllegalStateException, IllegalArgumentException {
+    public final void initStream(final Uri streamLoc, final int vid, final int aid, final int reqTextureCount) throws IllegalStateException, IllegalArgumentException {
         synchronized( stateLock ) {
             if(State.Uninitialized != state) {
                 throw new IllegalStateException("Instance not in state unintialized: "+this);
@@ -556,13 +560,13 @@ public abstract class GLMediaPlayerImpl implements GLMediaPlayer {
             // Pre-parse for camera-input scheme
             cameraPath = null;
             cameraProps = null;
-            final String streamLocScheme = streamLoc.getScheme();
+            final Uri.Encoded streamLocScheme = streamLoc.scheme;
             if( null != streamLocScheme && streamLocScheme.equals(CameraInputScheme) ) {
-                final String rawPath = streamLoc.getRawPath();
+                final Uri.Encoded rawPath = streamLoc.path;
                 if( null != rawPath && rawPath.length() > 0 ) {
                     // cut-off root fwd-slash
                     cameraPath = rawPath.substring(1);
-                    final URIQueryProps props = URIQueryProps.create(streamLoc, ';');
+                    final UriQueryProps props = UriQueryProps.create(streamLoc, ';');
                     cameraProps = props.getProperties();
                 } else {
                     throw new IllegalArgumentException("Camera path is empty: "+streamLoc.toString());
@@ -571,7 +575,7 @@ public abstract class GLMediaPlayerImpl implements GLMediaPlayer {
 
             this.vid = vid;
             this.aid = aid;
-            new Thread() {
+            new InterruptSource.Thread() {
                 public void run() {
                     try {
                         // StreamWorker may be used, see API-doc of StreamWorker
@@ -968,8 +972,9 @@ public abstract class GLMediaPlayerImpl implements GLMediaPlayer {
      *                  shall be <code>null</code> for audio only.
      * @return the last processed video PTS value, maybe {@link TimeFrameI#INVALID_PTS} if video frame is invalid or n/a.
      *         Will be {@link TimeFrameI#END_OF_STREAM_PTS} if end of stream reached.
+     * @throws InterruptedException if waiting for next frame fails
      */
-    protected abstract int getNextTextureImpl(GL gl, TextureFrame nextFrame);
+    protected abstract int getNextTextureImpl(GL gl, TextureFrame nextFrame) throws InterruptedException;
 
     protected final int getNextSingleThreaded(final GL gl, final TextureFrame nextFrame, final boolean[] gotVFrame) throws InterruptedException {
         final int pts;
@@ -1064,7 +1069,7 @@ public abstract class GLMediaPlayerImpl implements GLMediaPlayer {
      * {@link GLMediaPlayerImpl#updateAttributes(int, int, int, int, int, int, int, float, int, int, int, String, String) updateAttributes(..)},
      * the latter decides whether StreamWorker is being used.
      */
-    class StreamWorker extends Thread {
+    class StreamWorker extends InterruptSource.Thread {
         private volatile boolean isRunning = false;
         private volatile boolean isActive = false;
         private volatile boolean isBlocked = false;
@@ -1086,13 +1091,13 @@ public abstract class GLMediaPlayerImpl implements GLMediaPlayer {
             setDaemon(true);
             synchronized(this) {
                 start();
-                while( !isRunning ) {
+                try {
                     this.notifyAll();  // wake-up startup-block
-                    try {
+                    while( !isRunning && !shallStop ) {
                         this.wait();  // wait until started
-                    } catch (final InterruptedException e) {
-                        e.printStackTrace();
                     }
+                } catch (final InterruptedException e) {
+                    throw new InterruptedRuntimeException(e);
                 }
             }
         }
@@ -1140,18 +1145,20 @@ public abstract class GLMediaPlayerImpl implements GLMediaPlayer {
                 sharedGLCtx.release();
             }
         }
-        public final synchronized void doPause() {
+        public final synchronized void doPause(final boolean waitUntilDone) {
             if( isActive ) {
                 shallPause = true;
-                if( Thread.currentThread() != this ) {
+                if( java.lang.Thread.currentThread() != this ) {
                     if( isBlocked && isActive ) {
                         this.interrupt();
                     }
-                    while( isActive && isRunning ) {
+                    if( waitUntilDone ) {
                         try {
-                            this.wait(); // wait until paused
+                            while( isActive && isRunning ) {
+                                this.wait(); // wait until paused
+                            }
                         } catch (final InterruptedException e) {
-                            e.printStackTrace();
+                            throw new InterruptedRuntimeException(e);
                         }
                     }
                 }
@@ -1160,14 +1167,16 @@ public abstract class GLMediaPlayerImpl implements GLMediaPlayer {
         public final synchronized void doResume() {
             if( isRunning && !isActive ) {
                 shallPause = false;
-                if( Thread.currentThread() != this ) {
-                    while( !isActive && !shallPause && isRunning ) {
+                if( java.lang.Thread.currentThread() != this ) {
+                    try {
                         this.notifyAll();  // wake-up pause-block
-                        try {
+                        while( !isActive && !shallPause && isRunning ) {
                             this.wait(); // wait until resumed
-                        } catch (final InterruptedException e) {
-                            e.printStackTrace();
                         }
+                    } catch (final InterruptedException e) {
+                        final InterruptedException e2 = SourcedInterruptedException.wrap(e);
+                        doPause(false);
+                        throw new InterruptedRuntimeException(e2);
                     }
                 }
             }
@@ -1175,17 +1184,17 @@ public abstract class GLMediaPlayerImpl implements GLMediaPlayer {
         public final synchronized void doStop() {
             if( isRunning ) {
                 shallStop = true;
-                if( Thread.currentThread() != this ) {
+                if( java.lang.Thread.currentThread() != this ) {
                     if( isBlocked && isRunning ) {
                         this.interrupt();
                     }
-                    while( isRunning ) {
+                    try {
                         this.notifyAll();  // wake-up pause-block (opt)
-                        try {
+                        while( isRunning ) {
                             this.wait();  // wait until stopped
-                        } catch (final InterruptedException e) {
-                            e.printStackTrace();
                         }
+                    } catch (final InterruptedException e) {
+                        throw new InterruptedRuntimeException(e);
                     }
                 }
             }
@@ -1203,48 +1212,48 @@ public abstract class GLMediaPlayerImpl implements GLMediaPlayer {
                 this.notifyAll(); // wake-up ctor()
             }
 
-            while( !shallStop ){
-                if( shallPause ) {
-                    synchronized ( this ) {
-                        if( sharedGLCtxCurrent ) {
-                            postNextTextureImpl(sharedGLCtx.getGL());
-                            sharedGLCtx.release();
-                        }
-                        while( shallPause && !shallStop ) {
-                            isActive = false;
-                            this.notifyAll();   // wake-up doPause()
-                            try {
-                                this.wait(); // wait until resumed
-                            } catch (final InterruptedException e) {
-                                if( !shallPause ) {
-                                    e.printStackTrace();
+            while( !shallStop ) {
+                TextureFrame nextFrame = null;
+                try {
+                    if( shallPause ) {
+                        synchronized ( this ) {
+                            if( sharedGLCtxCurrent ) {
+                                postNextTextureImpl(sharedGLCtx.getGL());
+                                sharedGLCtx.release();
+                            }
+                            while( shallPause && !shallStop ) {
+                                isActive = false;
+                                this.notifyAll(); // wake-up doPause()
+                                try {
+                                    this.wait();  // wait until resumed
+                                } catch (final InterruptedException e) {
+                                    if( !shallPause ) {
+                                        throw SourcedInterruptedException.wrap(e);
+                                    }
                                 }
                             }
+                            if( sharedGLCtxCurrent ) {
+                                makeCurrent(sharedGLCtx);
+                                preNextTextureImpl(sharedGLCtx.getGL());
+                            }
+                            isActive = true;
+                            this.notifyAll(); // wake-up doResume()
                         }
-                        if( sharedGLCtxCurrent ) {
-                            makeCurrent(sharedGLCtx);
-                            preNextTextureImpl(sharedGLCtx.getGL());
-                        }
-                        isActive = true;
-                        this.notifyAll(); // wake-up doResume()
                     }
-                }
-                if( !sharedGLCtxCurrent && null != sharedGLCtx ) {
-                    synchronized ( this ) {
-                        if( null != sharedGLCtx ) {
-                            makeCurrent( sharedGLCtx );
-                            preNextTextureImpl(sharedGLCtx.getGL());
-                            sharedGLCtxCurrent = true;
-                        }
-                        if( null == videoFramesFree ) {
-                            throw new InternalError("XXX videoFramesFree is null");
+                    if( !sharedGLCtxCurrent && null != sharedGLCtx ) {
+                        synchronized ( this ) {
+                            if( null != sharedGLCtx ) {
+                                makeCurrent( sharedGLCtx );
+                                preNextTextureImpl(sharedGLCtx.getGL());
+                                sharedGLCtxCurrent = true;
+                            }
+                            if( null == videoFramesFree ) {
+                                throw new InternalError("XXX videoFramesFree is null");
+                            }
                         }
                     }
-                }
 
-                if( !shallStop ) {
-                    TextureFrame nextFrame = null;
-                    try {
+                    if( !shallStop ) {
                         isBlocked = true;
                         final GL gl;
                         if( STREAM_ID_NONE != vid ) {
@@ -1260,7 +1269,7 @@ public abstract class GLMediaPlayerImpl implements GLMediaPlayer {
                         if( TimeFrameI.INVALID_PTS != vPTS ) {
                             if( null != nextFrame ) {
                                 if( STREAM_WORKER_DELAY > 0 ) {
-                                    Thread.sleep(STREAM_WORKER_DELAY);
+                                    java.lang.Thread.sleep(STREAM_WORKER_DELAY);
                                 }
                                 if( !videoFramesDecoded.put(nextFrame) ) {
                                     throw new InternalError("XXX: free "+videoFramesFree+", decoded "+videoFramesDecoded+", "+GLMediaPlayerImpl.this);
@@ -1294,31 +1303,30 @@ public abstract class GLMediaPlayerImpl implements GLMediaPlayer {
                             }
                             pauseImpl(true, GLMediaEventListener.EVENT_CHANGE_EOS);
                         }
-                    } catch (final InterruptedException e) {
-                        isBlocked = false;
-                        if( !shallStop && !shallPause ) {
-                            streamErr = new StreamException("InterruptedException while decoding: "+GLMediaPlayerImpl.this.toString(), e);
+                    }
+                } catch (final InterruptedException e) {
+                    if( !isBlocked ) { // !shallStop && !shallPause
+                        streamErr = new StreamException("InterruptedException while decoding: "+GLMediaPlayerImpl.this.toString(),
+                                                        SourcedInterruptedException.wrap(e));
+                    }
+                    isBlocked = false;
+                } catch (final Throwable t) {
+                    streamErr = new StreamException(t.getClass().getSimpleName()+" while decoding: "+GLMediaPlayerImpl.this.toString(), t);
+                } finally {
+                    if( null != nextFrame ) { // put back
+                        videoFramesFree.put(nextFrame);
+                    }
+                    if( null != streamErr ) {
+                        if( DEBUG ) {
+                            ExceptionUtils.dumpThrowable("handled", streamErr);
                         }
-                    } catch (final Throwable t) {
-                        streamErr = new StreamException(t.getClass().getSimpleName()+" while decoding: "+GLMediaPlayerImpl.this.toString(), t);
-                    } finally {
-                        if( null != nextFrame ) { // put back
-                            videoFramesFree.put(nextFrame);
-                        }
-                        if( null != streamErr ) {
-                            if( DEBUG ) {
-                                final Throwable t = null != streamErr.getCause() ? streamErr.getCause() : streamErr;
-                                System.err.println("Caught StreamException: "+t.getMessage());
-                                t.printStackTrace();
-                            }
-                            // state transition incl. notification
-                            synchronized ( this ) {
-                                shallPause = true;
-                                isActive = false;
-                                this.notifyAll(); // wake-up potential do*()
-                            }
-                            pauseImpl(true, GLMediaEventListener.EVENT_CHANGE_ERR);
+                        // state transition incl. notification
+                        synchronized ( this ) {
+                            shallPause = true;
+                            isActive = false;
+                            this.notifyAll(); // wake-up potential do*()
                         }
+                        pauseImpl(true, GLMediaEventListener.EVENT_CHANGE_ERR);
                     }
                 }
             }
@@ -1379,6 +1387,16 @@ public abstract class GLMediaPlayerImpl implements GLMediaPlayer {
         }
     }
 
+    /**
+     * Called initially by {@link #initStreamImpl(int, int)}, which
+     * is called off-thread by {@link #initStream(Uri, int, int, int)}.
+     * <p>
+     * The latter catches an occurring exception and set the state delivers the error events.
+     * </p>
+     * <p>
+     * Further calls are issues off-thread by the decoder implementation.
+     * </p>
+     */
     protected final void updateAttributes(int vid, final int aid, final int width, final int height, final int bps_stream,
                                           final int bps_video, final int bps_audio, final float fps,
                                           final int videoFrames, final int audioFrames, final int duration, final String vcodec, final String acodec) {
@@ -1413,7 +1431,12 @@ public abstract class GLMediaPlayerImpl implements GLMediaPlayer {
             this.fps = fps;
             if( 0 != fps ) {
                 this.frame_duration = 1000f / fps;
-                this.maxNullFrameCountUntilEOS = MAX_FRAMELESS_MS_UNTIL_EOS / (int)this.frame_duration;
+                final int fdurI = (int)this.frame_duration;
+                if( 0 < fdurI ) {
+                    this.maxNullFrameCountUntilEOS = MAX_FRAMELESS_MS_UNTIL_EOS / fdurI;
+                } else {
+                    this.maxNullFrameCountUntilEOS = MAX_FRAMELESS_UNTIL_EOS_DEFAULT;
+                }
             } else {
                 this.frame_duration = 0;
                 this.maxNullFrameCountUntilEOS = MAX_FRAMELESS_UNTIL_EOS_DEFAULT;
@@ -1472,7 +1495,7 @@ public abstract class GLMediaPlayerImpl implements GLMediaPlayer {
     }
 
     @Override
-    public final URI getURI() { return streamLoc; }
+    public final Uri getUri() { return streamLoc; }
 
     @Override
     public final int getVID() { return vid; }
@@ -1524,7 +1547,7 @@ public abstract class GLMediaPlayerImpl implements GLMediaPlayer {
         final int decVideoFrames = null != videoFramesDecoded ? videoFramesDecoded.size() : 0;
         final int video_scr = video_scr_pts + (int) ( ( Platform.currentTimeMillis() - video_scr_t0 ) * playSpeed );
         final String camPath = null != cameraPath ? ", camera: "+cameraPath : "";
-        return "GLMediaPlayer["+state+", vSCR "+video_scr+", frames[p "+presentedFrameCount+", d "+decodedFrameCount+", t "+videoFrames+" ("+tt+" s), z "+nullFrameCount+" / "+maxNullFrameCountUntilEOS+"], "+
+        return getClass().getSimpleName()+"["+state+", vSCR "+video_scr+", frames[p "+presentedFrameCount+", d "+decodedFrameCount+", t "+videoFrames+" ("+tt+" s), z "+nullFrameCount+" / "+maxNullFrameCountUntilEOS+"], "+
                "speed "+playSpeed+", "+bps_stream+" bps, hasSW "+(null!=streamWorker)+
                ", Texture[count "+textureCount+", free "+freeVideoFrames+", dec "+decVideoFrames+", tagt "+toHexString(textureTarget)+", ifmt "+toHexString(textureInternalFormat)+", fmt "+toHexString(textureFormat)+", type "+toHexString(textureType)+"], "+
                "Video[id "+vid+", <"+vcodec+">, "+width+"x"+height+", glOrient "+isInGLOrientation+", "+fps+" fps, "+frame_duration+" fdur, "+bps_video+" bps], "+
