@@ -36,8 +36,8 @@ import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
 
-import javax.media.opengl.GLProfile;
-
+import com.jogamp.opengl.GLProfile;
+import com.jogamp.common.ExceptionUtils;
 import com.jogamp.common.os.DynamicLibraryBundle;
 import com.jogamp.common.os.DynamicLibraryBundleInfo;
 import com.jogamp.common.util.RunnableExecutor;
@@ -115,7 +115,7 @@ class FFMPEGDynamicLibraryBundleInfo implements DynamicLibraryBundleInfo  {
 /* 54 */ "avformat_find_stream_info", // 53.3.0    (opt)
 
          // libavdevice
-/* 55 */ "avdevice_register_all",     // ???
+/* 55 */ "avdevice_register_all",     // supported in all version <= 56
 
          // libavresample
          "avresample_alloc_context",  //  1.0.1
@@ -154,7 +154,7 @@ class FFMPEGDynamicLibraryBundleInfo implements DynamicLibraryBundleInfo  {
          "avresample_free",
          "avresample_convert",
 
-         // libavresample
+         // libswresample
          "av_opt_set_sample_fmt",     // actually lavu .. but exist only w/ swresample!
          "swresample_version",        //  0
          "swr_alloc",
@@ -163,12 +163,11 @@ class FFMPEGDynamicLibraryBundleInfo implements DynamicLibraryBundleInfo  {
          "swr_convert",
     };
 
+    /** util, format, codec, device, avresample, swresample */
+    private static final boolean[] libLoaded = new boolean[6];
     private static final long[] symbolAddr = new long[symbolCount];
     private static final boolean ready;
     private static final boolean libsUFCLoaded;
-    private static final boolean avresampleLoaded;  // optional
-    private static final boolean swresampleLoaded;  // optional
-    private static final boolean avdeviceLoaded; // optional
     static final VersionNumber avCodecVersion;
     static final VersionNumber avFormatVersion;
     static final VersionNumber avUtilVersion;
@@ -183,23 +182,85 @@ class FFMPEGDynamicLibraryBundleInfo implements DynamicLibraryBundleInfo  {
     private static final int LIB_IDX_AVR = 4;
     private static final int LIB_IDX_SWR = 5;
 
+    private static final PrivilegedAction<DynamicLibraryBundle> privInitSymbolsAction = new PrivilegedAction<DynamicLibraryBundle>() {
+        @Override
+        public DynamicLibraryBundle run() {
+            final DynamicLibraryBundle dl = new DynamicLibraryBundle(new FFMPEGDynamicLibraryBundleInfo());
+            for(int i=0; i<6; i++) {
+                libLoaded[i] = dl.isToolLibLoaded(i);
+            }
+            if( !libLoaded[LIB_IDX_UTI] || !libLoaded[LIB_IDX_FMT] || !libLoaded[LIB_IDX_COD] ) {
+                System.err.println("FFMPEG Tool library incomplete: [ avutil "+libLoaded[LIB_IDX_UTI]+", avformat "+libLoaded[LIB_IDX_FMT]+", avcodec "+libLoaded[LIB_IDX_COD]+"]");
+                return null;
+            }
+            dl.claimAllLinkPermission();
+            try {
+                for(int i = 0; i<symbolCount; i++) {
+                    symbolAddr[i] = dl.dynamicLookupFunction(symbolNames[i]);
+                }
+            } finally {
+                dl.releaseAllLinkPermission();
+            }
+            return dl;
+        } };
+
+    /**
+     * @param loaded 6: util, format, codec, device, avresample, swresample
+     * @param versions 5: util, format, codec, avresample, swresample
+     * @return
+     */
+    private static final boolean initSymbols(final VersionNumber[] versions) {
+        for(int i=0; i<6; i++) {
+            libLoaded[i] = false;
+        }
+        if(symbolNames.length != symbolCount) {
+            throw new InternalError("XXX0 "+symbolNames.length+" != "+symbolCount);
+        }
+
+        final DynamicLibraryBundle dl = AccessController.doPrivileged(privInitSymbolsAction);
+        if( null == dl ) {
+            return false;
+        }
+
+        // optional symbol name set
+        final Set<String> optionalSymbolNameSet = new HashSet<String>();
+        optionalSymbolNameSet.addAll(Arrays.asList(optionalSymbolNames));
+
+        // validate results
+        boolean res = true;
+        for(int i = 0; i<symbolCount; i++) {
+            if( 0 == symbolAddr[i] ) {
+                // no symbol, check optional and alternative symbols
+                final String symbol = symbolNames[i];
+                if ( !optionalSymbolNameSet.contains(symbol) ) {
+                    System.err.println("Fail: Could not resolve symbol <"+symbolNames[i]+">: not optional, no alternatives.");
+                    res = false;
+                } else if(DEBUG) {
+                    System.err.println("OK: Unresolved optional symbol <"+symbolNames[i]+">");
+                }
+            }
+        }
+        versions[0] = FFMPEGStaticNatives.getAVVersion(FFMPEGStaticNatives.getAvVersion0(symbolAddr[0]));
+        versions[1] = FFMPEGStaticNatives.getAVVersion(FFMPEGStaticNatives.getAvVersion0(symbolAddr[1]));
+        versions[2] = FFMPEGStaticNatives.getAVVersion(FFMPEGStaticNatives.getAvVersion0(symbolAddr[2]));
+        versions[3] = FFMPEGStaticNatives.getAVVersion(FFMPEGStaticNatives.getAvVersion0(symbolAddr[3]));
+        versions[4] = FFMPEGStaticNatives.getAVVersion(FFMPEGStaticNatives.getAvVersion0(symbolAddr[4]));
+
+        return res;
+    }
+
     static {
         // native ffmpeg media player implementation is included in jogl_desktop and jogl_mobile
         GLProfile.initSingleton();
         boolean _ready = false;
-        /** util, format, codec, device, avresample, swresample */
-        final boolean[] _loaded= new boolean[6];
         /** util, format, codec, avresample, swresample */
         final VersionNumber[] _versions = new VersionNumber[5];
         try {
-            _ready = initSymbols(_loaded, _versions);
+            _ready = initSymbols(_versions);
         } catch (final Throwable t) {
-            t.printStackTrace();
+            ExceptionUtils.dumpThrowable("", t);
         }
-        libsUFCLoaded = _loaded[LIB_IDX_UTI] && _loaded[LIB_IDX_FMT] && _loaded[LIB_IDX_COD];
-        avdeviceLoaded = _loaded[LIB_IDX_DEV];
-        avresampleLoaded = _loaded[LIB_IDX_AVR];
-        swresampleLoaded = _loaded[LIB_IDX_SWR];
+        libsUFCLoaded = libLoaded[LIB_IDX_UTI] && libLoaded[LIB_IDX_FMT] && libLoaded[LIB_IDX_COD];
         avUtilVersion = _versions[0];
         avFormatVersion = _versions[1];
         avCodecVersion = _versions[2];
@@ -226,6 +287,9 @@ class FFMPEGDynamicLibraryBundleInfo implements DynamicLibraryBundleInfo  {
             } else if( avCodecMajor == 55 && avFormatMajor == 55 && ( avUtilMajor == 52 || avUtilMajor == 53 ) ) {
                 // lavc55.lavf55.lavu52.lavr01 (ffmpeg) or lavc55.lavf55.lavu53.lavr01 (libav)
                 natives = new FFMPEGv10Natives();
+            } else if( avCodecMajor == 56 && avFormatMajor == 56 && avUtilMajor == 54 ) {
+                // lavc56.lavf56.lavu54.lavr02
+                natives = new FFMPEGv11Natives();
             } else {
                 System.err.println("LIB_AV No Version/Native-Impl Match");
                 natives = null;
@@ -239,74 +303,12 @@ class FFMPEGDynamicLibraryBundleInfo implements DynamicLibraryBundleInfo  {
     }
 
     static boolean libsLoaded() { return libsUFCLoaded; }
-    static boolean avDeviceLoaded() { return avdeviceLoaded; }
-    static boolean avResampleLoaded() { return avresampleLoaded; }
-    static boolean swResampleLoaded() { return swresampleLoaded; }
+    static boolean avDeviceLoaded() { return libLoaded[LIB_IDX_DEV]; }
+    static boolean avResampleLoaded() { return libLoaded[LIB_IDX_AVR]; }
+    static boolean swResampleLoaded() { return libLoaded[LIB_IDX_SWR]; }
     static FFMPEGNatives getNatives() { return natives; }
     static boolean initSingleton() { return ready; }
 
-    /**
-     * @param loaded 6: util, format, codec, device, avresample, swresample
-     * @param versions 5: util, format, codec, avresample, swresample
-     * @return
-     */
-    private static final boolean initSymbols(final boolean[] loaded, final VersionNumber[] versions) {
-        for(int i=0; i<6; i++) {
-            loaded[i] = false;
-        }
-        final DynamicLibraryBundle dl = AccessController.doPrivileged(new PrivilegedAction<DynamicLibraryBundle>() {
-                                          @Override
-                                          public DynamicLibraryBundle run() {
-                                              return new DynamicLibraryBundle(new FFMPEGDynamicLibraryBundleInfo());
-                                          } } );
-        dl.toString();
-        for(int i=0; i<6; i++) {
-            loaded[i] = dl.isToolLibLoaded(i);
-        }
-        if( !loaded[LIB_IDX_UTI] || !loaded[LIB_IDX_FMT] || !loaded[LIB_IDX_COD] ) {
-            throw new RuntimeException("FFMPEG Tool library incomplete: [ avutil "+loaded[LIB_IDX_UTI]+", avformat "+loaded[LIB_IDX_FMT]+", avcodec "+loaded[LIB_IDX_COD]+"]");
-        }
-        if(symbolNames.length != symbolCount) {
-            throw new InternalError("XXX0 "+symbolNames.length+" != "+symbolCount);
-        }
-
-        // optional symbol name set
-        final Set<String> optionalSymbolNameSet = new HashSet<String>();
-        optionalSymbolNameSet.addAll(Arrays.asList(optionalSymbolNames));
-
-        // lookup
-        AccessController.doPrivileged(new PrivilegedAction<Object>() {
-            @Override
-            public Object run() {
-                for(int i = 0; i<symbolCount; i++) {
-                    symbolAddr[i] = dl.dynamicLookupFunction(symbolNames[i]);
-                }
-                return null;
-            } } );
-
-        // validate results
-        boolean res = true;
-        for(int i = 0; i<symbolCount; i++) {
-            if( 0 == symbolAddr[i] ) {
-                // no symbol, check optional and alternative symbols
-                final String symbol = symbolNames[i];
-                if ( !optionalSymbolNameSet.contains(symbol) ) {
-                    System.err.println("Fail: Could not resolve symbol <"+symbolNames[i]+">: not optional, no alternatives.");
-                    res = false;
-                } else if(DEBUG) {
-                    System.err.println("OK: Unresolved optional symbol <"+symbolNames[i]+">");
-                }
-            }
-        }
-        versions[0] = FFMPEGStaticNatives.getAVVersion(FFMPEGStaticNatives.getAvVersion0(symbolAddr[0]));
-        versions[1] = FFMPEGStaticNatives.getAVVersion(FFMPEGStaticNatives.getAvVersion0(symbolAddr[1]));
-        versions[2] = FFMPEGStaticNatives.getAVVersion(FFMPEGStaticNatives.getAvVersion0(symbolAddr[2]));
-        versions[3] = FFMPEGStaticNatives.getAVVersion(FFMPEGStaticNatives.getAvVersion0(symbolAddr[3]));
-        versions[4] = FFMPEGStaticNatives.getAVVersion(FFMPEGStaticNatives.getAvVersion0(symbolAddr[4]));
-
-        return res;
-    }
-
     protected FFMPEGDynamicLibraryBundleInfo() {
     }
 
@@ -338,13 +340,17 @@ class FFMPEGDynamicLibraryBundleInfo implements DynamicLibraryBundleInfo  {
         final List<String> avutil = new ArrayList<String>();
         avutil.add("avutil");        // default
 
-        avutil.add("libavutil.so.53");     // dummy future proof
-        avutil.add("libavutil.so.52");     // ffmpeg 1.2 + 2 / libav 9 + 10
+        avutil.add("libavutil.so.55");     // dummy future proof
+        avutil.add("libavutil.so.54");     // ffmpeg 2.[4-x] / libav 11
+        avutil.add("libavutil.so.53");     // ffmpeg 2.[0-3] / libav 10
+        avutil.add("libavutil.so.52");     // ffmpeg 1.2 + 2.[0-3] / libav 9
         avutil.add("libavutil.so.51");     // 0.8
         avutil.add("libavutil.so.50");     // 0.7
 
-        avutil.add("avutil-53");     // dummy future proof
-        avutil.add("avutil-52");     // ffmpeg 1.2 + 2 / libav 9 + 10
+        avutil.add("avutil-55");     // dummy future proof
+        avutil.add("avutil-54");     // ffmpeg 2.[4-x] / libav 11
+        avutil.add("avutil-53");     // ffmpeg 2.[0-3] / libav 10
+        avutil.add("avutil-52");     // ffmpeg 1.2 + 2.[0-3] / libav 9
         avutil.add("avutil-51");     // 0.8
         avutil.add("avutil-50");     // 0.7
         libsList.add(avutil);
@@ -352,14 +358,16 @@ class FFMPEGDynamicLibraryBundleInfo implements DynamicLibraryBundleInfo  {
         final List<String> avformat = new ArrayList<String>();
         avformat.add("avformat");    // default
 
-        avformat.add("libavformat.so.56"); // dummy future proof
-        avformat.add("libavformat.so.55"); // ffmpeg 2 / libav 10
+        avformat.add("libavformat.so.57"); // dummy future proof
+        avformat.add("libavformat.so.56"); // ffmpeg 2.[4-x] / libav 11
+        avformat.add("libavformat.so.55"); // ffmpeg 2.[0-3] / libav 10
         avformat.add("libavformat.so.54"); // ffmpeg 1.2 / libav 9
         avformat.add("libavformat.so.53"); // 0.8
         avformat.add("libavformat.so.52"); // 0.7
 
-        avformat.add("avformat-56"); // dummy future proof
-        avformat.add("avformat-55"); // ffmpeg 2 / libav 10
+        avformat.add("avformat-57"); // dummy future proof
+        avformat.add("avformat-56"); // ffmpeg 2.[4-x] / libav 11
+        avformat.add("avformat-55"); // ffmpeg 2.[0-3] / libav 10
         avformat.add("avformat-54"); // ffmpeg 1.2 / libav 9
         avformat.add("avformat-53"); // 0.8
         avformat.add("avformat-52"); // 0.7
@@ -368,14 +376,16 @@ class FFMPEGDynamicLibraryBundleInfo implements DynamicLibraryBundleInfo  {
         final List<String> avcodec = new ArrayList<String>();
         avcodec.add("avcodec");      // default
 
-        avcodec.add("libavcodec.so.56");   // dummy future proof
-        avcodec.add("libavcodec.so.55");   // ffmpeg 2/ libav 10
+        avcodec.add("libavcodec.so.57");   // dummy future proof
+        avcodec.add("libavcodec.so.56");   // ffmpeg 2.[4-x] / libav 11
+        avcodec.add("libavcodec.so.55");   // ffmpeg 2.[0-3] / libav 10
         avcodec.add("libavcodec.so.54");   // ffmpeg 1.2 / libav 9
         avcodec.add("libavcodec.so.53");   // 0.8
         avcodec.add("libavcodec.so.52");   // 0.7
 
-        avcodec.add("avcodec-56");   // dummy future proof
-        avcodec.add("avcodec-55");   // ffmpeg 2/ libav 10
+        avcodec.add("avcodec-57");   // dummy future proof
+        avcodec.add("avcodec-56");   // ffmpeg 2.[4-x] / libav 11
+        avcodec.add("avcodec-55");   // ffmpeg 2.[0-3] / libav 10
         avcodec.add("avcodec-54");   // ffmpeg 1.2 / libav 9
         avcodec.add("avcodec-53");   // 0.8
         avcodec.add("avcodec-52");   // 0.7
@@ -384,13 +394,15 @@ class FFMPEGDynamicLibraryBundleInfo implements DynamicLibraryBundleInfo  {
         final List<String> avdevice = new ArrayList<String>();
         avdevice.add("avdevice");        // default
 
-        avdevice.add("libavdevice.so.56");     // dummy future proof
-        avdevice.add("libavdevice.so.55");     // ffmpeg 2
+        avdevice.add("libavdevice.so.57");     // dummy future proof
+        avdevice.add("libavdevice.so.56");     // ffmpeg 2.[4-x]
+        avdevice.add("libavdevice.so.55");     // ffmpeg 2.[0-3] / libav 11
         avdevice.add("libavdevice.so.54");     // ffmpeg 1.2 / libav 10
         avdevice.add("libavdevice.so.53");     // 0.8 && libav 9
 
-        avdevice.add("avdevice-56");     // dummy future proof
-        avdevice.add("avdevice-55");     // ffmpeg 2
+        avdevice.add("avdevice-57");     // dummy future proof
+        avdevice.add("avdevice-56");     // ffmpeg 2.[4-x]
+        avdevice.add("avdevice-55");     // ffmpeg 2.[0-3] / libav 11
         avdevice.add("avdevice-54");     // ffmpeg 1.2 / libav 10
         avdevice.add("avdevice-53");     // 0.8 && libav 9
         libsList.add(avdevice);
@@ -398,21 +410,25 @@ class FFMPEGDynamicLibraryBundleInfo implements DynamicLibraryBundleInfo  {
         final List<String> avresample = new ArrayList<String>();
         avresample.add("avresample");        // default
 
-        avresample.add("libavresample.so.2");     // dummy future proof
+        avresample.add("libavresample.so.3");     // dummy future proof
+        avresample.add("libavresample.so.2");     // libav 11
         avresample.add("libavresample.so.1");     // libav 9 + 10
 
-        avresample.add("avresample-2");     // dummy future proof
+        avresample.add("avresample-3");     // dummy future proof
+        avresample.add("avresample-2");     // libav 11
         avresample.add("avresample-1");     // libav 9 + 10
         libsList.add(avresample);
 
         final List<String> swresample = new ArrayList<String>();
         swresample.add("swresample");        // default
 
-        swresample.add("libswresample.so.1");     // dummy future proof
-        swresample.add("libswresample.so.0");     // ffmpeg 1.2 + 2.x
+        swresample.add("libswresample.so.2");     // dummy future proof
+        swresample.add("libswresample.so.1");     // ffmpeg 2.[4-x]
+        swresample.add("libswresample.so.0");     // ffmpeg 1.2 + 2.[0-3]
 
-        swresample.add("swresample-1");     // dummy future proof
-        swresample.add("swresample-0");     // ffmpeg 1.2 + 2.x
+        swresample.add("swresample-2");     // dummy future proof
+        swresample.add("swresample-1");     // ffmpeg 2.[4-x]
+        swresample.add("swresample-0");     // ffmpeg 1.2 + 2.[0-3]
         libsList.add(swresample);
 
         return libsList;
