@@ -33,11 +33,10 @@ import java.nio.ByteBuffer;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
 
-import javax.media.opengl.GL;
-import javax.media.opengl.GL2ES2;
-import javax.media.opengl.GLException;
+import com.jogamp.opengl.GL;
+import com.jogamp.opengl.GL2ES2;
+import com.jogamp.opengl.GLException;
 
-import com.jogamp.common.os.Platform;
 import com.jogamp.common.util.IOUtil;
 import com.jogamp.common.util.VersionNumber;
 import com.jogamp.gluegen.runtime.ProcAddressTable;
@@ -107,14 +106,25 @@ import jogamp.opengl.util.av.VideoPixelFormat;
  * <p>
  * Currently we are binary compatible w/:
  * <table border="1">
- * <tr><th>libav / ffmpeg</th><th>lavc</th><th>lavf</th><th>lavu</th><th>lavr</th>    <th>FFMPEG* class</th></tr>
+ * <tr><th>libav / ffmpeg</th><th>lavc</th><th>lavf</th><th>lavu</th><th>lavr/lswr</th>  <th>FFMPEG* class</th></tr>
  * <tr><td>0.8</td>           <td>53</td>  <td>53</td>  <td>51</td>     <td></td>        <td>FFMPEGv08</td></tr>
  * <tr><td>9.0 / 1.2</td>     <td>54</td>  <td>54</td>  <td>52</td>     <td>01/00</td>   <td>FFMPEGv09</td></tr>
- * <tr><td>10 / 2</td>        <td>55</td>  <td>55</td>  <td>53/52</td>  <td>01/00</td>   <td>FFMPEGv10</td></tr>
+ * <tr><td>10 / 2.[0-3]</td>  <td>55</td>  <td>55</td>  <td>53/52</td>  <td>01/00</td>   <td>FFMPEGv10</td></tr>
+ * <tr><td>11 / 2.[4-8]</td>  <td>56</td>  <td>56</td>  <td>54</td>     <td>02/01</td>   <td>FFMPEGv11</td></tr>
+ * <tr><td>12 / 2.[9-x]</td>  <td>57</td>  <td>57</td>  <td>55</td>     <td>02/01</td>   <td>TODO</td></tr>
  * </table>
  * </p>
  * <p>
- * See http://upstream-tracker.org/versions/libav.html
+ * See FFmpeg:
+ * <ul>
+ *  <li>http://ffmpeg.org/documentation.html</li>
+ *  <li>http://git.videolan.org/?p=ffmpeg.git;a=blob;f=doc/APIchanges;hb=HEAD</li>
+ * </ul>
+ * See libav:
+ * <ul>
+ *  <li>https://libav.org/documentation.html</li>
+ *  <li>http://upstream-tracker.org/versions/libav.html</li>
+ * </ul>
  * </p>
  * <p>
  * Check tag 'FIXME: Add more planar formats !'
@@ -136,13 +146,21 @@ import jogamp.opengl.util.av.VideoPixelFormat;
  *   <li>GNU/Linux: ffmpeg or libav are deployed in most distributions.</li>
  *   <li>Windows:
  *   <ul>
+ *     <li>https://ffmpeg.org/download.html#build-windows</li>
  *     <li>http://ffmpeg.zeranoe.com/builds/ (ffmpeg) <i>recommended, works w/ dshow</i></li>
  *     <li>http://win32.libav.org/releases/  (libav)</li>
  *   </ul></li>
- *   <li>MacOSX using Homebrew
+ *   <li>MacOSX
  *   <ul>
- *   <li>https://github.com/Homebrew/homebrew/wiki/Installation</li>
- *   <li>https://trac.ffmpeg.org/wiki/CompilationGuide/MacOSX</li>
+ *     <li>Builds
+ *     <ul>
+ *       <li>https://ffmpeg.org/download.html#build-mac</li>
+ *     </ul></li>
+ *     <li>Building using Homebrew *
+ *     <ul>
+ *       <li>https://github.com/Homebrew/homebrew/wiki/Installation</li>
+ *       <li>https://trac.ffmpeg.org/wiki/CompilationGuide/MacOSX</li>
+ *     </ul></li>
  *   </ul></li>
  *   <li>OpenIndiana/Solaris:<pre>
  *       pkg set-publisher -p http://pkg.openindiana.org/sfe-encumbered.
@@ -164,6 +182,8 @@ public class FFMPEGMediaPlayer extends GLMediaPlayerImpl {
     private static final int avResampleMajorVersionCC;
     private static final int swResampleMajorVersionCC;
     private static final boolean available;
+    private static final boolean enableAvResample;
+    private static final boolean enableSwResample;
 
     static {
         final boolean libAVGood = FFMPEGDynamicLibraryBundleInfo.initSingleton();
@@ -206,9 +226,13 @@ public class FFMPEGMediaPlayer extends GLMediaPlayerImpl {
                                avFormatMajorVersionCC == avFormatMajor &&
                                ( avUtilMajorVersionCC == avUtilMajor ||
                                  55 == avCodecMajorVersionCC && 53 == avUtilMajorVersionCC && 52 == avUtilMajor /* ffmpeg 2.x */
-                               ) &&
-                               ( !avResampleLoaded || avResampleMajorVersionCC < 0 || avResampleMajorVersionCC  == avResampleVersion.getMajor() ) &&
-                               ( !swResampleLoaded || swResampleMajorVersionCC < 0 || swResampleMajorVersionCC  == swResampleVersion.getMajor() ) ;
+                               );
+            enableAvResample = avResampleLoaded && avResampleMajorVersionCC  == avResampleVersion.getMajor();
+            enableSwResample = swResampleLoaded && swResampleMajorVersionCC  == swResampleVersion.getMajor();
+            if( DEBUG ) {
+                System.err.println("LIB_AV Resample: enabled "+enableAvResample);
+                System.err.println("LIB_SW Resample: enabled "+enableSwResample);
+            }
             if( !libAVVersionGood ) {
                 System.err.println("LIB_AV Not Matching Compile-Time / Runtime Major-Version");
             }
@@ -220,6 +244,8 @@ public class FFMPEGMediaPlayer extends GLMediaPlayerImpl {
             avResampleMajorVersionCC = 0;
             swResampleMajorVersionCC = 0;
             libAVVersionGood = false;
+            enableAvResample = false;
+            enableSwResample = false;
         }
         available = libAVGood && libAVVersionGood && null != natives;
     }
@@ -257,7 +283,7 @@ public class FFMPEGMediaPlayer extends GLMediaPlayerImpl {
         if(!available) {
             throw new RuntimeException("FFMPEGMediaPlayer not available");
         }
-        moviePtr = natives.createInstance0(this, DEBUG_NATIVE);
+        moviePtr = natives.createInstance0(this, enableAvResample, enableSwResample, DEBUG_NATIVE);
         if(0==moviePtr) {
             throw new GLException("Couldn't create FFMPEGInstance");
         }
@@ -292,7 +318,7 @@ public class FFMPEGMediaPlayer extends GLMediaPlayerImpl {
             System.err.println("initStream: p1 "+this);
         }
 
-        final String streamLocS = IOUtil.decodeURIIfFilePath(getURI());
+        final String streamLocS = IOUtil.getUriFilePathOrASCII(getUri());
         destroyAudioSink();
         if( GLMediaPlayer.STREAM_ID_NONE == aid ) {
             audioSink = AudioSinkFactory.createNull();
@@ -317,15 +343,13 @@ public class FFMPEGMediaPlayer extends GLMediaPlayerImpl {
                 case HPUX:
                 case LINUX:
                 case SUNOS:
-                    resStreamLocS = dev_video_linux + cameraPath;
+                    resStreamLocS = dev_video_linux + cameraPath.decode();
                     break;
                 case WINDOWS:
-                    resStreamLocS = cameraPath;
-                    break;
                 case MACOS:
                 case OPENKODE:
                 default:
-                    resStreamLocS = streamLocS; // FIXME: ??
+                    resStreamLocS = cameraPath.decode();
                     break;
             }
             if( null != cameraProps ) {
@@ -345,7 +369,7 @@ public class FFMPEGMediaPlayer extends GLMediaPlayerImpl {
          // setStream(..) issues updateAttributes*(..), and defines avChosenAudioFormat, vid, aid, .. etc
         if(DEBUG) {
             System.err.println("initStream: p3 cameraPath "+cameraPath+", isCameraInput "+isCameraInput);
-            System.err.println("initStream: p3 stream "+getURI()+" -> "+streamLocS+" -> "+resStreamLocS);
+            System.err.println("initStream: p3 stream "+getUri()+" -> "+streamLocS+" -> "+resStreamLocS);
             System.err.println("initStream: p3 vid "+vid+", sizes "+sizes+", reqVideo "+rw+"x"+rh+"@"+rr+", aid "+aid+", aMaxChannelCount "+aMaxChannelCount+", aPrefSampleRate "+aPrefSampleRate);
         }
         natives.setStream0(moviePtr, resStreamLocS, isCameraInput, vid, sizes, rw, rh, rr, aid, aMaxChannelCount, aPrefSampleRate);
@@ -419,9 +443,10 @@ public class FFMPEGMediaPlayer extends GLMediaPlayerImpl {
                     }
                     break;
 
-                case 2: if( vPixelFmt == VideoPixelFormat.YUYV422 ) {
+                case 2: if( vPixelFmt == VideoPixelFormat.YUYV422 || vPixelFmt == VideoPixelFormat.UYVY422 ) {
                             // YUYV422: // < packed YUV 4:2:2, 2x 16bpp, Y0 Cb Y1 Cr
-                            // Stuffed into RGBA half width texture
+                            // UYVY422: // < packed YUV 4:2:2, 2x 16bpp, Cb Y0 Cr Y1
+                            // Both stuffed into RGBA half width texture
                             tf = GL.GL_RGBA; tif=GL.GL_RGBA; break;
                         } else {
                             tf = GL2ES2.GL_RG;   tif=GL2ES2.GL_RG; break;
@@ -589,6 +614,7 @@ public class FFMPEGMediaPlayer extends GLMediaPlayerImpl {
                     texWidth = vTexWidth[0] + vTexWidth[1] + vTexWidth[2]; texHeight = vH;
                     break;
                 case YUYV422: // < packed YUV 4:2:2, 2x 16bpp, Y0 Cb Y1 Cr - stuffed into RGBA half width texture
+                case UYVY422: // < packed YUV 4:2:2, 2x 16bpp, Cb Y0 Cr Y1 - stuffed into RGBA half width texture
                 case BGR24:
                     usesTexLookupShader = true;
                     texWidth = vTexWidth[0]; texHeight = vH;
@@ -747,6 +773,29 @@ public class FFMPEGMediaPlayer extends GLMediaPlayerImpl {
               "  return vec4(r, g, b, 1);\n"+
               "}\n"
           ;
+        case UYVY422: // < packed YUV 4:2:2, 2 x 16bpp, Cb Y0 Cr Y1
+                      // Stuffed into RGBA half width texture
+          return
+              "vec4 "+texLookupFuncName+"(in "+getTextureSampler2DType()+" image, in vec2 texCoord) {\n"+
+              "  "+
+              "  float y1,u,y2,v,y,r,g,b;\n"+
+              "  vec2 tc_halfw = vec2(texCoord.x*0.5, texCoord.y);\n"+
+              "  vec4 uyvy = texture2D(image, tc_halfw).rgba;\n"+
+              "  u  = uyvy.r;\n"+
+              "  y1 = uyvy.g;\n"+
+              "  v  = uyvy.b;\n"+
+              "  y2 = uyvy.a;\n"+
+              "  y = mix( y1, y2, mod(gl_FragCoord.x, 2) ); /* avoid branching! */\n"+
+              "  y = 1.1643*(y-0.0625);\n"+
+              "  u = u-0.5;\n"+
+              "  v = v-0.5;\n"+
+              "  r = y+1.5958*v;\n"+
+              "  g = y-0.39173*u-0.81290*v;\n"+
+              "  b = y+2.017*u;\n"+
+              "  return vec4(r, g, b, 1);\n"+
+              "}\n"
+          ;
+
         case BGR24:
           return
               "vec4 "+texLookupFuncName+"(in "+getTextureSampler2DType()+" image, in vec2 texCoord) {\n"+
