@@ -40,31 +40,31 @@
 
 package jogamp.opengl.windows.wgl;
 
-import java.nio.ByteBuffer;
 import java.nio.IntBuffer;
 import java.util.HashMap;
 import java.util.Map;
 
-import javax.media.nativewindow.AbstractGraphicsConfiguration;
-import javax.media.nativewindow.AbstractGraphicsDevice;
-import javax.media.nativewindow.NativeSurface;
-import javax.media.opengl.GLContext;
-import javax.media.opengl.GLException;
-import javax.media.opengl.GLCapabilitiesImmutable;
-import javax.media.opengl.GLProfile;
-
+import com.jogamp.nativewindow.AbstractGraphicsConfiguration;
+import com.jogamp.nativewindow.AbstractGraphicsDevice;
+import com.jogamp.nativewindow.NativeSurface;
+import com.jogamp.opengl.GLContext;
+import com.jogamp.opengl.GLException;
+import com.jogamp.opengl.GLCapabilitiesImmutable;
+import com.jogamp.common.ExceptionUtils;
 import com.jogamp.common.nio.Buffers;
 import com.jogamp.gluegen.runtime.ProcAddressTable;
 import com.jogamp.gluegen.runtime.opengl.GLProcAddressResolver;
 import com.jogamp.opengl.GLExtensions;
+import com.jogamp.opengl.GLProfile;
 import com.jogamp.opengl.GLRendererQuirks;
 
 import jogamp.nativewindow.windows.GDI;
 import jogamp.opengl.GLContextImpl;
 import jogamp.opengl.GLDrawableImpl;
+import jogamp.opengl.GLDynamicLookupHelper;
+import jogamp.opengl.GLXExtensions;
 
 public class WindowsWGLContext extends GLContextImpl {
-  static final Map<String, String> functionNameMap;
   static final Map<String, String> extensionNameMap;
   private boolean wglGetExtensionsStringEXTInitialized;
   private boolean wglGetExtensionsStringEXTAvailable;
@@ -74,14 +74,11 @@ public class WindowsWGLContext extends GLContextImpl {
   // Table that holds the addresses of the native C-language entry points for
   // WGL extension functions.
   private WGLExtProcAddressTable wglExtProcAddressTable;
-  private int hasSwapIntervalSGI = 0;
+  /** 2 WGL_EXT_swap_control_tear, 1 WGL_EXT_swap_control, 0 undefined, -1 none */
+  private int hasSwapInterval = 0;
   private int hasSwapGroupNV = 0;
 
   static {
-    functionNameMap = new HashMap<String, String>();
-    functionNameMap.put("glAllocateMemoryNV", "wglAllocateMemoryNV");
-    functionNameMap.put("glFreeMemoryNV", "wglFreeMemoryNV");
-
     extensionNameMap = new HashMap<String, String>();
     extensionNameMap.put(GLExtensions.ARB_pbuffer, WindowsWGLDrawableFactory.WGL_ARB_pbuffer);
     extensionNameMap.put(GLExtensions.ARB_pixel_format, WindowsWGLDrawableFactory.WGL_ARB_pixel_format);
@@ -101,7 +98,7 @@ public class WindowsWGLContext extends GLContextImpl {
     wglGLReadDrawableAvailable=false;
     // no inner state _wglExt=null;
     wglExtProcAddressTable=null;
-    hasSwapIntervalSGI = 0;
+    hasSwapInterval = 0;
     hasSwapGroupNV = 0;
     super.resetStates(isInit);
   }
@@ -152,17 +149,33 @@ public class WindowsWGLContext extends GLContextImpl {
         // should not happen due to 'isGLReadDrawableAvailable()' query in GLContextImpl
         throw new InternalError("Given readDrawable but no driver support");
     }
-    final int werr = ( !ok ) ? GDI.GetLastError() : GDI.ERROR_SUCCESS;
     if(DEBUG && !ok) {
-        final Throwable t = new Throwable ("Info: wglMakeContextCurrent draw "+
+        final Throwable t = new Throwable ("Info: wglMakeContextCurrent NOK: draw "+
                 GLContext.toHexString(hDrawDC) + ", read " + GLContext.toHexString(hReadDC) +
-                ", ctx " + GLContext.toHexString(ctx) + ", werr " + werr);
+                ", ctx " + GLContext.toHexString(ctx) + ", werr " + GDI.GetLastError());
         t.printStackTrace();
     }
-    if(!ok && 0==hDrawDC && 0==hReadDC) {
+    return ok;
+  }
+
+  private final boolean wglReleaseContext() {
+    boolean ok = false;
+    if(wglGLReadDrawableAvailable) {
+        // needs initilized WGL ProcAddress table
+        ok = getWGLExt().wglMakeContextCurrent(0, 0, 0);
+    } else {
+        ok = WGL.wglMakeCurrent(0, 0);
+    }
+    if( !ok ) {
+        final int werr = GDI.GetLastError();
+        final boolean ok2 = werr == GDI.ERROR_SUCCESS;
+        if(DEBUG) {
+            final Throwable t = new Throwable ("Info: wglReleaseContext NOK: werr " + werr + " -> ok "+ok2);
+            t.printStackTrace();
+        }
         // Some GPU's falsely fails with a zero error code (success),
         // in case this is a release context request we tolerate this
-        return werr == GDI.ERROR_SUCCESS ;
+        return ok2 ;
     }
     return ok;
   }
@@ -177,7 +190,7 @@ public class WindowsWGLContext extends GLContextImpl {
   }
 
   @Override
-  protected Map<String, String> getFunctionNameMap() { return functionNameMap; }
+  protected Map<String, String> getFunctionNameMap() { return null; }
 
   @Override
   protected Map<String, String> getExtensionNameMap() { return extensionNameMap; }
@@ -190,19 +203,36 @@ public class WindowsWGLContext extends GLContextImpl {
 
   @Override
   protected long createContextARBImpl(final long share, final boolean direct, final int ctp, final int major, final int minor) {
-    if( null == getWGLExtProcAddressTable()) {
-        updateGLXProcAddressTable();
-    }
-    final WGLExt _wglExt = getWGLExt();
     if(DEBUG) {
-      System.err.println(getThreadName()+" - WindowWGLContext.createContextARBImpl: "+getGLVersion(major, minor, ctp, "@creation") +
-                         ", handle "+toHexString(drawable.getHandle()) + ", share "+toHexString(share)+", direct "+direct+
-                         ", wglCreateContextAttribsARB: "+toHexString(wglExtProcAddressTable._addressof_wglCreateContextAttribsARB));
+        System.err.println(getThreadName()+" - WindowWGLContext.createContextARBImpl: "+getGLVersion(major, minor, ctp, "@creation") +
+                ", handle "+toHexString(drawable.getHandle()) + ", share "+toHexString(share)+", direct "+direct);
     }
-
+    final boolean ctDesktopGL = 0 == ( CTX_PROFILE_ES & ctp );
     final boolean ctBwdCompat = 0 != ( CTX_PROFILE_COMPAT & ctp ) ;
     final boolean ctFwdCompat = 0 != ( CTX_OPTION_FORWARD & ctp ) ;
     final boolean ctDebug     = 0 != ( CTX_OPTION_DEBUG & ctp ) ;
+    if( !ctDesktopGL ) {
+        if(DEBUG) {
+            System.err.println(getThreadName() + ": WindowWGLContext.createContextARBImpl: GL ES not avail "+getGLVersion(major, minor, ctp, "@creation"));
+        }
+        return 0; // n/a
+    }
+    if( null == getWGLExtProcAddressTable()) {
+        final GLDynamicLookupHelper dlh = getGLDynamicLookupHelper(major, ctp);
+        if( null == dlh ) {
+            if(DEBUG) {
+                System.err.println(getThreadName()+" - WindowWGLContext.createContextARBImpl: Null GLDynamicLookupHelper");
+            }
+            return 0;
+        } else {
+            updateGLXProcAddressTable(null, dlh);
+        }
+    }
+    final WGLExt _wglExt = getWGLExt();
+    if(DEBUG) {
+      System.err.println(getThreadName()+" - WindowWGLContext.createContextARBImpl: "+
+                         ", wglCreateContextAttribsARB: "+toHexString(wglExtProcAddressTable._addressof_wglCreateContextAttribsARB));
+    }
 
     long ctx=0;
 
@@ -269,7 +299,7 @@ public class WindowsWGLContext extends GLContextImpl {
    * called by {@link #makeCurrentImpl()}.
    */
   @Override
-  protected boolean createImpl(long shareWithHandle) {
+  protected boolean createImpl(long shareWithHandle) throws GLException {
     final AbstractGraphicsConfiguration config = drawable.getNativeSurface().getGraphicsConfiguration();
     final AbstractGraphicsDevice device = config.getScreen().getDevice();
     final WindowsWGLDrawableFactory factory = (WindowsWGLDrawableFactory)drawable.getFactoryImpl();
@@ -278,14 +308,23 @@ public class WindowsWGLContext extends GLContextImpl {
 
     isGLReadDrawableAvailable(); // trigger setup wglGLReadDrawableAvailable
 
-    if (DEBUG) {
-        System.err.println(getThreadName() + ": createImpl: START "+glCaps+", share "+toHexString(shareWithHandle));
+    final boolean createContextARBAvailable = isCreateContextARBAvail(device) && !glCaps.isBitmap();
+    final boolean sharedCreatedWithARB = null != sharedContext && sharedContext.isCreatedWithARBMethod();
+    if(DEBUG) {
+        System.err.println(getThreadName() + ": WindowsWGLContext.createImpl: START "+glCaps+", share "+toHexString(shareWithHandle));
+        System.err.println(getThreadName() + ": Use ARB[avail["+getCreateContextARBAvailStr(device)+
+                "], bitmap "+glCaps.isBitmap()+" -> "+createContextARBAvailable+
+                "], shared "+sharedCreatedWithARB+"]");
+    }
+    final GLProfile glp = glCaps.getGLProfile();
+    if( glp.isGLES() ) {
+        throw new GLException(getThreadName()+": Unable to create OpenGL ES context on desktopDevice "+device+
+                ", config "+config+", "+glp+", shareWith "+toHexString(shareWithHandle));
     }
-
     boolean createContextARBTried = false;
 
-    // utilize the shared context's GLXExt in case it was using the ARB method and it already exists ; exclude BITMAP
-    if( null != sharedContext && sharedContext.isCreatedWithARBMethod() && !glCaps.isBitmap() ) {
+    // utilize the shared context's GLXExt in case it was using the ARB method and it already exists
+    if( createContextARBAvailable && sharedCreatedWithARB ) {
         if ( sharedContext.getRendererQuirks().exist( GLRendererQuirks.NeedCurrCtx4ARBCreateContext ) ) {
             if(GLContext.CONTEXT_NOT_CURRENT == sharedContext.makeCurrent()) {
                 throw new GLException("Could not make Shared Context current: "+sharedContext);
@@ -316,37 +355,31 @@ public class WindowsWGLContext extends GLContextImpl {
             throw new GLException("Error making temp context current: 0x" + toHexString(temp_ctx) + ", werr: "+GDI.GetLastError());
         }
         if( !setGLFunctionAvailability(true, 0, 0, CTX_PROFILE_COMPAT, false /* strictMatch */, null == sharedContext /* withinGLVersionsMapping */) ) { // use GL_VERSION
-            throw new InternalError("setGLFunctionAvailability !strictMatch failed");
+            WGL.wglMakeCurrent(0, 0); // release temp context
+            WGL.wglDeleteContext(temp_ctx);
+            throw new GLException("setGLFunctionAvailability !strictMatch failed");
         }
         WGL.wglMakeCurrent(0, 0); // release temp context
 
-        if( !createContextARBTried ) {
+        if( createContextARBAvailable && !createContextARBTried ) {
             // is*Available calls are valid since setGLFunctionAvailability(..) was called
-            final boolean isProcCreateContextAttribsARBAvailable;
-            final boolean isExtARBCreateContextAvailable;
-            if( !glCaps.isBitmap() ) { // exclude ARB if BITMAP
-                isProcCreateContextAttribsARBAvailable = isFunctionAvailable("wglCreateContextAttribsARB");
-                isExtARBCreateContextAvailable = isExtensionAvailable("WGL_ARB_create_context");
-            } else {
-                isProcCreateContextAttribsARBAvailable = false;
-                isExtARBCreateContextAvailable = false;
-            }
-            if ( isProcCreateContextAttribsARBAvailable && isExtARBCreateContextAvailable && !GLProfile.disableOpenGLARBContext ) {
+            final boolean isProcCreateContextAttribsARBAvailable = isFunctionAvailable("wglCreateContextAttribsARB");
+            final boolean isExtARBCreateContextAvailable = isExtensionAvailable("WGL_ARB_create_context");
+            if ( isProcCreateContextAttribsARBAvailable && isExtARBCreateContextAvailable ) {
                 // initial ARB context creation
                 contextHandle = createContextARB(shareWithHandle, true);
                 createContextARBTried=true;
                 if (DEBUG) {
                     if( 0 != contextHandle ) {
-                        System.err.println(getThreadName() + ": createContextImpl: OK (ARB, initial) share "+toHexString(shareWithHandle));
+                        System.err.println(getThreadName() + ": createImpl: OK (ARB, initial) share "+toHexString(shareWithHandle));
                     } else {
-                        System.err.println(getThreadName() + ": createContextImpl: NOT OK (ARB, initial) - creation failed - share "+toHexString(shareWithHandle));
+                        System.err.println(getThreadName() + ": createImpl: NOT OK (ARB, initial) - creation failed - share "+toHexString(shareWithHandle));
                     }
                 }
             } else if (DEBUG) {
-                System.err.println(getThreadName() + ": createContextImpl: NOT OK (ARB, initial) - extension not available - share "+toHexString(shareWithHandle)+
+                System.err.println(getThreadName() + ": createImpl: NOT OK (ARB, initial) - extension not available - share "+toHexString(shareWithHandle)+
                                    ", isProcCreateContextAttribsARBAvailable "+isProcCreateContextAttribsARBAvailable+
-                                   ", isExtGLXARBCreateContextAvailable "+isExtARBCreateContextAvailable+
-                                   ", disableOpenGLARBContext "+GLProfile.disableOpenGLARBContext);
+                                   ", isExtGLXARBCreateContextAvailable "+isExtARBCreateContextAvailable);
             }
         }
     } else {
@@ -368,10 +401,11 @@ public class WindowsWGLContext extends GLContextImpl {
             // otherwise context of similar profile but different creation method may not be share-able.
             WGL.wglMakeCurrent(0, 0);
             WGL.wglDeleteContext(temp_ctx);
-            throw new GLException(getThreadName()+": WindowsWGLContex.createContextImpl ctx !ARB but ARB is used, profile > GL2 requested (OpenGL >= 3.0.1). Requested: "+glCaps.getGLProfile()+", current: "+getGLVersion());
+            throw new GLException(getThreadName()+": createImpl ctx !ARB but ARB is used, profile > GL2 requested (OpenGL >= 3.1). Requested: "+glCaps.getGLProfile()+", current: "+getGLVersion());
         }
         if(DEBUG) {
-            System.err.println("WindowsWGLContext.createContext ARB not used, fall back to !ARB context "+getGLVersion());
+            System.err.println(getThreadName()+": createImpl ARB not used[avail "+createContextARBAvailable+
+                               ", tried "+createContextARBTried+"], fall back to !ARB context "+getGLVersion());
         }
 
         // continue with temp context
@@ -400,20 +434,18 @@ public class WindowsWGLContext extends GLContextImpl {
 
   @Override
   protected void  makeCurrentImpl() throws GLException {
-    if (WGL.wglGetCurrentContext() != contextHandle) {
-      if (!wglMakeContextCurrent(drawable.getHandle(), drawableRead.getHandle(), contextHandle)) {
+      if ( !wglMakeContextCurrent(drawable.getHandle(), drawableRead.getHandle(), contextHandle) ) {
         throw new GLException("Error making context " + toHexString(contextHandle) +
                               " current on Thread " + getThreadName() +
                               ", drawableWrite " + toHexString(drawable.getHandle()) +
                               ", drawableRead "+ toHexString(drawableRead.getHandle()) +
                               ", werr: " + GDI.GetLastError() + ", " + this);
       }
-    }
   }
 
   @Override
   protected void releaseImpl() throws GLException {
-    if (!wglMakeContextCurrent(0, 0, 0)) {
+    if ( !wglReleaseContext() ) {
         throw new GLException("Error freeing OpenGL context, werr: " + GDI.GetLastError());
     }
   }
@@ -433,8 +465,17 @@ public class WindowsWGLContext extends GLContextImpl {
     }
   }
 
+  /**
+   * {@inheritDoc}
+   * <p>
+   * Ignoring {@code contextFQN}, using {@code WGL}-{@link AbstractGraphicsDevice#getUniqueID()}.
+   * </p>
+   */
   @Override
-  protected final void updateGLXProcAddressTable() {
+  protected final void updateGLXProcAddressTable(final String contextFQN, final GLDynamicLookupHelper dlh) {
+    if( null == dlh ) {
+        throw new GLException("No GLDynamicLookupHelper for "+this);
+    }
     final AbstractGraphicsConfiguration aconfig = drawable.getNativeSurface().getGraphicsConfiguration();
     final AbstractGraphicsDevice adevice = aconfig.getScreen().getDevice();
     final String key = "WGL-"+adevice.getUniqueID();
@@ -457,7 +498,7 @@ public class WindowsWGLContext extends GLContextImpl {
         }
     } else {
         wglExtProcAddressTable = new WGLExtProcAddressTable(new GLProcAddressResolver());
-        resetProcAddressTable(wglExtProcAddressTable);
+        resetProcAddressTable(wglExtProcAddressTable, dlh);
         synchronized(mappedContextTypeObjectLock) {
             mappedGLXProcAddress.put(key, getWGLExtProcAddressTable());
             if(DEBUG) {
@@ -482,19 +523,42 @@ public class WindowsWGLContext extends GLContextImpl {
   }
 
   @Override
-  protected boolean setSwapIntervalImpl(final int interval) {
-    final WGLExt wglExt = getWGLExt();
-    if(0==hasSwapIntervalSGI) {
-        try {
-            hasSwapIntervalSGI = wglExt.isExtensionAvailable("WGL_EXT_swap_control")?1:-1;
-        } catch (final Throwable t) { hasSwapIntervalSGI=1; }
+  protected final Integer setSwapIntervalImpl2(final int interval) {
+    if( !drawable.getChosenGLCapabilities().isOnscreen() ) {
+        return null;
     }
-    if (hasSwapIntervalSGI>0) {
+    if( 0 == hasSwapInterval ) {
         try {
-            return wglExt.wglSwapIntervalEXT(interval);
-        } catch (final Throwable t) { hasSwapIntervalSGI=-1; }
+            if ( isExtensionAvailable(GLXExtensions.WGL_EXT_swap_control) ) {
+                hasSwapInterval = 1;
+                if ( isExtensionAvailable(GLXExtensions.WGL_EXT_swap_control_tear) ) {
+                    hasSwapInterval =  2;
+                    if(DEBUG) { System.err.println("WGLContext.setSwapInterval.2 using: "+GLXExtensions.WGL_EXT_swap_control_tear + ", " + GLXExtensions.WGL_EXT_swap_control_tear); }
+                } else {
+                    hasSwapInterval =  1;
+                    if(DEBUG) { System.err.println("WGLContext.setSwapInterval.1 using: "+GLXExtensions.WGL_EXT_swap_control); }
+                }
+            } else {
+                hasSwapInterval = -1;
+                if(DEBUG) { System.err.println("WGLContext.setSwapInterval.0 N/A"); }
+            }
+        } catch (final Throwable t) { hasSwapInterval=-1; if(DEBUG) { ExceptionUtils.dumpThrowable("", t); } }
     }
-    return false;
+    if ( 0 < hasSwapInterval ) { // 2 || 1
+        final int useInterval;
+        if( 1 == hasSwapInterval && 0 > interval ) {
+            useInterval = Math.abs(interval);
+        } else {
+            useInterval = interval;
+        }
+        try {
+            final WGLExt wglExt = getWGLExt();
+            if( wglExt.wglSwapIntervalEXT(useInterval) ) {
+                return Integer.valueOf(useInterval);
+            }
+        } catch (final Throwable t) { hasSwapInterval=-1; if(DEBUG) { ExceptionUtils.dumpThrowable("", t); } }
+    }
+    return null;
   }
 
   private final int initSwapGroupImpl(final WGLExt wglExt) {
@@ -558,15 +622,4 @@ public class WindowsWGLContext extends GLContextImpl {
       }
       return res;
   }
-
-  @Override
-  public final ByteBuffer glAllocateMemoryNV(final int size, final float readFrequency, final float writeFrequency, final float priority) {
-    return getWGLExt().wglAllocateMemoryNV(size, readFrequency, writeFrequency, priority);
-  }
-
-  @Override
-  public final void glFreeMemoryNV(final ByteBuffer pointer) {
-    getWGLExt().wglFreeMemoryNV(pointer);
-  }
-
 }
