@@ -38,16 +38,17 @@ import java.nio.IntBuffer;
 import java.util.ArrayList;
 import java.util.List;
 
-import javax.media.nativewindow.AbstractGraphicsScreen;
-import javax.media.nativewindow.NativeSurface;
-import javax.media.nativewindow.AbstractGraphicsDevice;
-import javax.media.opengl.GL;
-import javax.media.opengl.GLCapabilitiesImmutable;
-import javax.media.opengl.GLCapabilitiesChooser;
-import javax.media.opengl.GLDrawableFactory;
-import javax.media.opengl.GLException;
-import javax.media.opengl.GLProfile;
+import com.jogamp.nativewindow.AbstractGraphicsScreen;
+import com.jogamp.nativewindow.NativeSurface;
+import com.jogamp.nativewindow.AbstractGraphicsDevice;
+import com.jogamp.opengl.GL;
+import com.jogamp.opengl.GLCapabilitiesImmutable;
+import com.jogamp.opengl.GLCapabilitiesChooser;
+import com.jogamp.opengl.GLDrawableFactory;
+import com.jogamp.opengl.GLException;
+import com.jogamp.opengl.GLProfile;
 
+import com.jogamp.common.ExceptionUtils;
 import com.jogamp.common.nio.Buffers;
 import com.jogamp.nativewindow.MutableGraphicsConfiguration;
 import com.jogamp.opengl.GLRendererQuirks;
@@ -170,24 +171,7 @@ public class WindowsWGLGraphicsConfiguration extends MutableGraphicsConfiguratio
                                   ": error code " + GDI.GetLastError());
         }
         if( !caps.isBackgroundOpaque() ) {
-            final long hwnd = GDI.WindowFromDC(hdc);
-            final DWM_BLURBEHIND bb = DWM_BLURBEHIND.create();
-            bb.setDwFlags(GDI.DWM_BB_ENABLE| GDI.DWM_BB_TRANSITIONONMAXIMIZED);
-            bb.setFEnable( 1 );
-            boolean ok = GDI.DwmEnableBlurBehindWindow(hwnd, bb);
-            if( ok ) {
-                final MARGINS m = MARGINS.create();
-                m.setCxLeftWidth(-1);
-                m.setCxRightWidth(-1);
-                m.setCyBottomHeight(-1);
-                m.setCyTopHeight(-1);
-                ok = GDI.DwmExtendFrameIntoClientArea(hwnd, m);
-            }
-            if(DEBUG) {
-                final boolean isUndecorated = GDIUtil.IsUndecorated(hwnd);
-                final boolean isChild = GDIUtil.IsChild(hwnd);
-                System.err.println("translucency enabled on wnd: 0x"+Long.toHexString(hwnd)+" - isUndecorated "+isUndecorated+", isChild "+isChild+", ok: "+ok);
-            }
+            GDIUtil.DwmSetupTranslucency(GDI.WindowFromDC(hdc), true);
         }
         if (DEBUG) {
             System.err.println("setPixelFormat: hdc "+toHexString(hdc) +", "+caps);
@@ -226,6 +210,7 @@ public class WindowsWGLGraphicsConfiguration extends MutableGraphicsConfiguratio
     public final boolean isDetermined() { return isDetermined; }
 
     public final PIXELFORMATDESCRIPTOR getPixelFormat()   { return isDetermined ? ((WGLGLCapabilities)capabilitiesChosen).getPFD() : null; }
+    /** FIXME: NPE ..*/
     public final int getPixelFormatID() { return isDetermined ? ((WGLGLCapabilities)capabilitiesChosen).getPFDID() : 0; }
     public final boolean isChoosenByARB() { return isDetermined ? ((WGLGLCapabilities)capabilitiesChosen).isSetByARB() : false; }
 
@@ -281,7 +266,7 @@ public class WindowsWGLGraphicsConfiguration extends MutableGraphicsConfiguratio
                 System.err.println("GetPixelFormatAttribivARB: Failed - HDC 0x" + Long.toHexString(hdc) +
                                   ", value "+iresults.get(0)+
                                   ", LastError: " + GDI.GetLastError());
-                Thread.dumpStack();
+                ExceptionUtils.dumpStack(System.err);
             }
             return 0;
         }
@@ -290,7 +275,7 @@ public class WindowsWGLGraphicsConfiguration extends MutableGraphicsConfiguratio
             if(DEBUG) {
                 System.err.println("GetPixelFormatAttribivARB: No formats - HDC 0x" + Long.toHexString(hdc) +
                                   ", LastError: " + GDI.GetLastError());
-                Thread.dumpStack();
+                ExceptionUtils.dumpStack(System.err);
             }
         }
         return pfdIDCount;
@@ -351,7 +336,7 @@ public class WindowsWGLGraphicsConfiguration extends MutableGraphicsConfiguratio
                                                                          iattributes, accelerationMode, null) ) {
             if (DEBUG) {
                 System.err.println("wglChoosePixelFormatARB: GLCapabilities2AttribList failed: " + GDI.GetLastError());
-                Thread.dumpStack();
+                ExceptionUtils.dumpStack(System.err);
             }
             return null;
         }
@@ -365,7 +350,7 @@ public class WindowsWGLGraphicsConfiguration extends MutableGraphicsConfiguratio
                                              pformatsTmp, numFormatsTmp) ) {
             if (DEBUG) {
                 System.err.println("wglChoosePixelFormatARB: wglChoosePixelFormatARB failed: " + GDI.GetLastError());
-                Thread.dumpStack();
+                ExceptionUtils.dumpStack(System.err);
             }
             return null;
         }
@@ -508,7 +493,7 @@ public class WindowsWGLGraphicsConfiguration extends MutableGraphicsConfiguratio
             caps.getAccumGreenBits() > 0 ||
             caps.getAccumBlueBits()  > 0 ||
             caps.getAccumAlphaBits() > 0 ) {
-            final GLRendererQuirks sharedQuirks = sharedResource.getRendererQuirks();
+            final GLRendererQuirks sharedQuirks = sharedResource.getRendererQuirks(null);
             if ( !usePBuffer || null==sharedQuirks || !sharedQuirks.exist(GLRendererQuirks.NoPBufferWithAccum) ) {
               iattributes.put(niattribs++, WGLExt.WGL_ACCUM_BITS_ARB);
               iattributes.put(niattribs++, ( caps.getAccumRedBits() +
