@@ -40,25 +40,26 @@
 
 package jogamp.opengl.x11.glx;
 
-import java.nio.ByteBuffer;
 import java.nio.IntBuffer;
 import java.util.HashMap;
 import java.util.Map;
 
-import javax.media.nativewindow.AbstractGraphicsConfiguration;
-import javax.media.nativewindow.AbstractGraphicsDevice;
-import javax.media.nativewindow.NativeSurface;
-import javax.media.opengl.GLCapabilitiesImmutable;
-import javax.media.opengl.GLContext;
-import javax.media.opengl.GLException;
-import javax.media.opengl.GLProfile;
+import com.jogamp.nativewindow.AbstractGraphicsConfiguration;
+import com.jogamp.nativewindow.AbstractGraphicsDevice;
+import com.jogamp.nativewindow.NativeSurface;
+import com.jogamp.opengl.GLCapabilitiesImmutable;
+import com.jogamp.opengl.GLContext;
+import com.jogamp.opengl.GLException;
+import com.jogamp.opengl.GLProfile;
 
 import jogamp.nativewindow.x11.X11Lib;
 import jogamp.nativewindow.x11.X11Util;
 import jogamp.opengl.GLContextImpl;
 import jogamp.opengl.GLDrawableImpl;
+import jogamp.opengl.GLDynamicLookupHelper;
 import jogamp.opengl.GLXExtensions;
 
+import com.jogamp.common.ExceptionUtils;
 import com.jogamp.common.nio.Buffers;
 import com.jogamp.common.util.VersionNumber;
 import com.jogamp.gluegen.runtime.ProcAddressTable;
@@ -67,13 +68,12 @@ import com.jogamp.nativewindow.x11.X11GraphicsDevice;
 import com.jogamp.opengl.GLExtensions;
 
 public class X11GLXContext extends GLContextImpl {
-  private static final Map<String, String> functionNameMap;
   private static final Map<String, String> extensionNameMap;
   private GLXExt _glXExt;
   // Table that holds the addresses of the native C-language entry points for
   // GLX extension functions.
   private GLXExtProcAddressTable glXExtProcAddressTable;
-  /** 1 MESA, 2 SGI, 0 undefined, -1 none */
+  /** 3 SGI, 2 GLX_EXT_swap_control_tear, 1 GLX_EXT_swap_control, 0 undefined, -1 none */
   private int hasSwapInterval = 0;
   private int hasSwapGroupNV = 0;
 
@@ -85,10 +85,6 @@ public class X11GLXContext extends GLContextImpl {
   protected volatile boolean isGLXVersionGreaterEqualOneThree;
 
   static {
-    functionNameMap = new HashMap<String, String>();
-    functionNameMap.put("glAllocateMemoryNV", "glXAllocateMemoryNV");
-    functionNameMap.put("glFreeMemoryNV", "glXFreeMemoryNV");
-
     extensionNameMap = new HashMap<String, String>();
     extensionNameMap.put(GLExtensions.ARB_pbuffer,      X11GLXDrawableFactory.GLX_SGIX_pbuffer);
     extensionNameMap.put(GLExtensions.ARB_pixel_format, X11GLXDrawableFactory.GLX_SGIX_pbuffer); // good enough
@@ -133,7 +129,7 @@ public class X11GLXContext extends GLContextImpl {
   }
 
   @Override
-  protected Map<String, String> getFunctionNameMap() { return functionNameMap; }
+  protected Map<String, String> getFunctionNameMap() { return null; }
 
   @Override
   protected Map<String, String> getExtensionNameMap() { return extensionNameMap; }
@@ -186,11 +182,32 @@ public class X11GLXContext extends GLContextImpl {
     return res;
   }
 
+  private final boolean glXReleaseContext(final long dpy) {
+    boolean res = false;
+
+    try {
+        if ( isGLXVersionGreaterEqualOneThree() ) {
+            // System.err.println(getThreadName() +": X11GLXContext.releaseCurrent: obj " + toHexString(hashCode()) + " / ctx "+toHexString(contextHandle)+": ctx "+toHexString(ctx)+" - switch");
+            res = GLX.glXMakeContextCurrent(dpy, 0, 0, 0);
+        } else {
+            // System.err.println(getThreadName() +": X11GLXContext.releaseCurrent: obj " + toHexString(hashCode()) + " / ctx "+toHexString(contextHandle)+": ctx "+toHexString(ctx)+" - switch");
+            res = GLX.glXMakeCurrent(dpy, 0, 0);
+        }
+    } catch (final RuntimeException re) {
+        if( DEBUG_TRACE_SWITCH ) {
+          System.err.println(getThreadName()+": Warning: X11GLXContext.glXReleaseContext failed: "+re+", with "+
+            "dpy "+toHexString(dpy));
+          re.printStackTrace();
+        }
+    }
+    return res;
+  }
+
   @Override
   protected void destroyContextARBImpl(final long ctx) {
     final long display = drawable.getNativeSurface().getDisplayHandle();
 
-    glXMakeContextCurrent(display, 0, 0, 0);
+    glXReleaseContext(display);
     GLX.glXDestroyContext(display, ctx);
   }
   private static final int ctx_arb_attribs_idx_major = 0;
@@ -208,19 +225,34 @@ public class X11GLXContext extends GLContextImpl {
 
   @Override
   protected long createContextARBImpl(final long share, final boolean direct, final int ctp, final int major, final int minor) {
-    updateGLXProcAddressTable();
-    final GLXExt _glXExt = getGLXExt();
     if(DEBUG) {
       System.err.println(getThreadName()+": X11GLXContext.createContextARBImpl: "+getGLVersion(major, minor, ctp, "@creation") +
-                         ", handle "+toHexString(drawable.getHandle()) + ", share "+toHexString(share)+", direct "+direct+
-                         ", glXCreateContextAttribsARB: "+toHexString(glXExtProcAddressTable._addressof_glXCreateContextAttribsARB));
+                         ", handle "+toHexString(drawable.getHandle()) + ", share "+toHexString(share)+", direct "+direct);
     }
-
+    final boolean ctDesktopGL = 0 == ( CTX_PROFILE_ES & ctp );
     final boolean ctBwdCompat = 0 != ( CTX_PROFILE_COMPAT & ctp ) ;
     final boolean ctFwdCompat = 0 != ( CTX_OPTION_FORWARD & ctp ) ;
     final boolean ctDebug     = 0 != ( CTX_OPTION_DEBUG & ctp ) ;
-
-    long ctx=0;
+    if( !ctDesktopGL ) {
+        if(DEBUG) {
+            System.err.println(getThreadName() + ": X11GLXContext.createContextARBImpl: GL ES not avail "+getGLVersion(major, minor, ctp, "@creation"));
+        }
+        return 0; // n/a
+    }
+    final GLDynamicLookupHelper dlh = getGLDynamicLookupHelper(major, ctp);
+    if( null == dlh ) {
+        if(DEBUG) {
+            System.err.println(getThreadName()+" - X11GLXContext.createContextARBImpl: Null GLDynamicLookupHelper");
+        }
+        return 0;
+    } else {
+        updateGLXProcAddressTable(null, dlh);
+    }
+    final GLXExt _glXExt = getGLXExt();
+    if(DEBUG) {
+      System.err.println(getThreadName()+": X11GLXContext.createContextARBImpl: "+
+                         ", glXCreateContextAttribsARB: "+toHexString(glXExtProcAddressTable._addressof_glXCreateContextAttribsARB));
+    }
 
     final IntBuffer attribs = Buffers.newDirectIntBuffer(ctx_arb_attribs_rom);
     attribs.put(ctx_arb_attribs_idx_major + 1, major);
@@ -249,6 +281,7 @@ public class X11GLXContext extends GLContextImpl {
     final X11GLXGraphicsConfiguration config = (X11GLXGraphicsConfiguration)drawable.getNativeSurface().getGraphicsConfiguration();
     final AbstractGraphicsDevice device = config.getScreen().getDevice();
     final long display = device.getHandle();
+    long ctx=0;
 
     try {
         // critical path, a remote display might not support this command,
@@ -258,8 +291,8 @@ public class X11GLXContext extends GLContextImpl {
         ctx = _glXExt.glXCreateContextAttribsARB(display, config.getFBConfig(), share, direct, attribs);
     } catch (final RuntimeException re) {
         if(DEBUG) {
-          final Throwable t = new Throwable(getThreadName()+": Info: X11GLXContext.createContextARBImpl glXCreateContextAttribsARB failed with "+getGLVersion(major, minor, ctp, "@creation"), re);
-          t.printStackTrace();
+          System.err.println(getThreadName()+": Info: X11GLXContext.createContextARBImpl glXCreateContextAttribsARB failed with "+getGLVersion(major, minor, ctp, "@creation"));
+          ExceptionUtils.dumpThrowable("", re);
         }
     }
 
@@ -269,7 +302,7 @@ public class X11GLXContext extends GLContextImpl {
               System.err.println(getThreadName()+": X11GLXContext.createContextARBImpl couldn't make current "+getGLVersion(major, minor, ctp, "@creation"));
             }
             // release & destroy
-            glXMakeContextCurrent(display, 0, 0, 0);
+            glXReleaseContext(display);
             GLX.glXDestroyContext(display, ctx);
             ctx = 0;
         } else if (DEBUG) {
@@ -283,7 +316,7 @@ public class X11GLXContext extends GLContextImpl {
   }
 
   @Override
-  protected boolean createImpl(final long shareWithHandle) {
+  protected boolean createImpl(final long shareWithHandle) throws GLException {
     boolean direct = true; // try direct always
     isDirect = false; // fall back
 
@@ -299,6 +332,18 @@ public class X11GLXContext extends GLContextImpl {
 
     final GLCapabilitiesImmutable glCaps = (GLCapabilitiesImmutable) config.getChosenCapabilities();
     final GLProfile glp = glCaps.getGLProfile();
+    final boolean createContextARBAvailable = isCreateContextARBAvail(device) && config.hasFBConfig();
+    final boolean sharedCreatedWithARB = null != sharedContext && sharedContext.isCreatedWithARBMethod();
+    if(DEBUG) {
+        System.err.println(getThreadName() + ": X11GLXContext.createImpl: START "+glCaps+", share "+toHexString(shareWithHandle));
+        System.err.println(getThreadName() + ": Use ARB[avail["+getCreateContextARBAvailStr(device)+
+                "], fbCfg "+config.hasFBConfig()+" -> "+createContextARBAvailable+
+                "], shared "+sharedCreatedWithARB+"]");
+    }
+    if( glp.isGLES() ) {
+        throw new GLException(getThreadName()+": Unable to create OpenGL ES context on desktopDevice "+device+
+                ", config "+config+", "+glp+", shareWith "+toHexString(shareWithHandle));
+    }
 
     if( !config.hasFBConfig() ) {
         // not able to use FBConfig -> GLX 1.1
@@ -314,23 +359,25 @@ public class X11GLXContext extends GLContextImpl {
           throw new GLException(getThreadName()+": Error making temp context(0) current: display "+toHexString(display)+", context "+toHexString(contextHandle)+", drawable "+drawable);
         }
         if( !setGLFunctionAvailability(true, 0, 0, CTX_PROFILE_COMPAT, false /* strictMatch */, null == sharedContext /* withinGLVersionsMapping */) ) { // use GL_VERSION
-            throw new InternalError("setGLFunctionAvailability !strictMatch failed");
+            glXReleaseContext(display); // release temp context
+            GLX.glXDestroyContext(display, contextHandle);
+            contextHandle = 0;
+            throw new GLException("setGLFunctionAvailability !strictMatch failed.1");
         }
         isDirect = GLX.glXIsDirect(display, contextHandle);
         if (DEBUG) {
-            System.err.println(getThreadName() + ": createContextImpl: OK (old-1) share "+toHexString(shareWithHandle)+", direct "+isDirect+"/"+direct);
+            System.err.println(getThreadName() + ": createImpl: OK (old-1) share "+toHexString(shareWithHandle)+", direct "+isDirect+"/"+direct);
         }
         return true;
     }
-
     boolean createContextARBTried = false;
 
     // utilize the shared context's GLXExt in case it was using the ARB method and it already exists
-    if( null != sharedContext && sharedContext.isCreatedWithARBMethod() ) {
+    if( createContextARBAvailable && sharedCreatedWithARB ) {
         contextHandle = createContextARB(shareWithHandle, direct);
         createContextARBTried = true;
         if ( DEBUG && 0 != contextHandle ) {
-            System.err.println(getThreadName() + ": createContextImpl: OK (ARB, using sharedContext) share "+toHexString(shareWithHandle));
+            System.err.println(getThreadName() + ": createImpl: OK (ARB, using sharedContext) share "+toHexString(shareWithHandle));
         }
     }
 
@@ -345,28 +392,31 @@ public class X11GLXContext extends GLContextImpl {
         if ( !glXMakeContextCurrent(display, drawable.getHandle(), drawableRead.getHandle(), temp_ctx) ) {
             throw new GLException(getThreadName()+": Error making temp context(1) current: display "+toHexString(display)+", context "+toHexString(temp_ctx)+", drawable "+drawable);
         }
-        setGLFunctionAvailability(true, 0, 0, CTX_PROFILE_COMPAT, false /* strictMatch */, null == sharedContext /* withinGLVersionsMapping */); // use GL_VERSION
-        glXMakeContextCurrent(display, 0, 0, 0); // release temp context
-        if( !createContextARBTried ) {
+        if( !setGLFunctionAvailability(true, 0, 0, CTX_PROFILE_COMPAT, false /* strictMatch */, null == sharedContext /* withinGLVersionsMapping */) ) { // use GL_VERSION
+            glXReleaseContext(display); // release temp context
+            GLX.glXDestroyContext(display, temp_ctx);
+            throw new GLException("setGLFunctionAvailability !strictMatch failed.2");
+        }
+        glXReleaseContext(display); // release temp context
+        if( createContextARBAvailable && !createContextARBTried ) {
             // is*Available calls are valid since setGLFunctionAvailability(..) was called
             final boolean isProcCreateContextAttribsARBAvailable = isFunctionAvailable("glXCreateContextAttribsARB");
             final boolean isExtARBCreateContextAvailable = isExtensionAvailable("GLX_ARB_create_context");
-            if ( isProcCreateContextAttribsARBAvailable && isExtARBCreateContextAvailable && !GLProfile.disableOpenGLARBContext ) {
+            if ( isProcCreateContextAttribsARBAvailable && isExtARBCreateContextAvailable ) {
                 // initial ARB context creation
                 contextHandle = createContextARB(shareWithHandle, direct);
                 createContextARBTried=true;
                 if (DEBUG) {
                     if( 0 != contextHandle ) {
-                        System.err.println(getThreadName() + ": createContextImpl: OK (ARB, initial) share "+toHexString(shareWithHandle));
+                        System.err.println(getThreadName() + ": createImpl: OK (ARB, initial) share "+toHexString(shareWithHandle));
                     } else {
-                        System.err.println(getThreadName() + ": createContextImpl: NOT OK (ARB, initial) - creation failed - share "+toHexString(shareWithHandle));
+                        System.err.println(getThreadName() + ": createImpl: NOT OK (ARB, initial) - creation failed - share "+toHexString(shareWithHandle));
                     }
                 }
             } else if( DEBUG ) {
-                System.err.println(getThreadName() + ": createContextImpl: NOT OK (ARB, initial) - extension not available - share "+toHexString(shareWithHandle)+
+                System.err.println(getThreadName() + ": createImpl: NOT OK (ARB, initial) - extension not available - share "+toHexString(shareWithHandle)+
                                    ", isProcCreateContextAttribsARBAvailable "+isProcCreateContextAttribsARBAvailable+
-                                   ", isExtGLXARBCreateContextAvailable "+isExtARBCreateContextAvailable+
-                                   ", disableOpenGLARBContext "+GLProfile.disableOpenGLARBContext);
+                                   ", isExtGLXARBCreateContextAvailable "+isExtARBCreateContextAvailable);
             }
         }
     } else {
@@ -375,7 +425,7 @@ public class X11GLXContext extends GLContextImpl {
 
     if( 0 != contextHandle ) {
         if( 0 != temp_ctx ) {
-            glXMakeContextCurrent(display, 0, 0, 0);
+            glXReleaseContext(display);
             GLX.glXDestroyContext(display, temp_ctx);
             if ( !glXMakeContextCurrent(display, drawable.getHandle(), drawableRead.getHandle(), contextHandle) ) {
                 throw new GLException(getThreadName()+": Cannot make previous verified context current");
@@ -385,29 +435,29 @@ public class X11GLXContext extends GLContextImpl {
         if( glp.isGL3() && createContextARBTried ) {
             // We shall not allow context creation >= GL3 w/ non ARB methods if ARB is used,
             // otherwise context of similar profile but different creation method may not be share-able.
-            glXMakeContextCurrent(display, 0, 0, 0);
+            glXReleaseContext(display);
             GLX.glXDestroyContext(display, temp_ctx);
-            throw new GLException(getThreadName()+": X11GLXContext.createContextImpl ctx !ARB but ARB is used, profile > GL2 requested (OpenGL >= 3.0.1). Requested: "+glp+", current: "+getGLVersion());
+            throw new GLException(getThreadName()+": createImpl ARB n/a but required, profile > GL2 requested (OpenGL >= 3.1). Requested: "+glp+", current: "+getGLVersion());
         }
-
         if(DEBUG) {
-            System.err.println(getThreadName()+": X11GLXContext.createContextImpl ARB not used, fall back to !ARB context "+getGLVersion());
+            System.err.println(getThreadName()+": createImpl ARB not used[avail "+createContextARBAvailable+
+                               ", tried "+createContextARBTried+"], fall back to !ARB context "+getGLVersion());
         }
 
         // continue with temp context
         contextHandle = temp_ctx;
         if ( !glXMakeContextCurrent(display, drawable.getHandle(), drawableRead.getHandle(), contextHandle) ) {
-            glXMakeContextCurrent(display, 0, 0, 0);
+            glXReleaseContext(display);
             GLX.glXDestroyContext(display, temp_ctx);
             throw new GLException(getThreadName()+": Error making context(1) current: display "+toHexString(display)+", context "+toHexString(contextHandle)+", drawable "+drawable);
         }
         if (DEBUG) {
-            System.err.println(getThreadName() + ": createContextImpl: OK (old-2) share "+toHexString(shareWithHandle));
+            System.err.println(getThreadName() + ": createImpl: OK (old-2) share "+toHexString(shareWithHandle));
         }
     }
     isDirect = GLX.glXIsDirect(display, contextHandle);
     if (DEBUG) {
-        System.err.println(getThreadName() + ": createContextImpl: OK direct "+isDirect+"/"+direct);
+        System.err.println(getThreadName() + ": createImpl: OK direct "+isDirect+"/"+direct);
     }
 
     return true;
@@ -416,23 +466,20 @@ public class X11GLXContext extends GLContextImpl {
   @Override
   protected void makeCurrentImpl() throws GLException {
     final long dpy = drawable.getNativeSurface().getDisplayHandle();
-
-    if (GLX.glXGetCurrentContext() != contextHandle) {
-        if (!glXMakeContextCurrent(dpy, drawable.getHandle(), drawableRead.getHandle(), contextHandle)) {
-            throw new GLException("Error making context " + toHexString(contextHandle) +
-                                  " current on Thread " + getThreadName() +
-                                  " with display " + toHexString(dpy) +
-                                  ", drawableWrite " + toHexString(drawable.getHandle()) +
-                                  ", drawableRead "+ toHexString(drawableRead.getHandle()) +
-                                  " - " + this);
-        }
+    if ( !glXMakeContextCurrent(dpy, drawable.getHandle(), drawableRead.getHandle(), contextHandle) ) {
+        throw new GLException("Error making context " + toHexString(contextHandle) +
+                " current on Thread " + getThreadName() +
+                " with display " + toHexString(dpy) +
+                ", drawableWrite " + toHexString(drawable.getHandle()) +
+                ", drawableRead "+ toHexString(drawableRead.getHandle()) +
+                " - " + this);
     }
   }
 
   @Override
   protected void releaseImpl() throws GLException {
     final long display = drawable.getNativeSurface().getDisplayHandle();
-    if (!glXMakeContextCurrent(display, 0, 0, 0)) {
+    if ( !glXReleaseContext(display) ) {
         throw new GLException(getThreadName()+": Error freeing OpenGL context");
     }
   }
@@ -454,8 +501,17 @@ public class X11GLXContext extends GLContextImpl {
     // Should check for X errors and raise GLException
   }
 
+  /**
+   * {@inheritDoc}
+   * <p>
+   * Ignoring {@code contextFQN}, using {@code GLX}+{@link AbstractGraphicsDevice#getUniqueID()}.
+   * </p>
+   */
   @Override
-  protected final void updateGLXProcAddressTable() {
+  protected final void updateGLXProcAddressTable(final String contextFQN, final GLDynamicLookupHelper dlh) {
+    if( null == dlh ) {
+        throw new GLException("No GLDynamicLookupHelper for "+this);
+    }
     final AbstractGraphicsConfiguration aconfig = drawable.getNativeSurface().getGraphicsConfiguration();
     final AbstractGraphicsDevice adevice = aconfig.getScreen().getDevice();
     final String key = "GLX-"+adevice.getUniqueID();
@@ -473,7 +529,7 @@ public class X11GLXContext extends GLContextImpl {
         }
     } else {
         glXExtProcAddressTable = new GLXExtProcAddressTable(new GLProcAddressResolver());
-        resetProcAddressTable(getGLXExtProcAddressTable());
+        resetProcAddressTable(getGLXExtProcAddressTable(), dlh);
         synchronized(mappedContextTypeObjectLock) {
             mappedGLXProcAddress.put(key, getGLXExtProcAddressTable());
             if(DEBUG) {
@@ -491,7 +547,7 @@ public class X11GLXContext extends GLContextImpl {
     x11Device.lock();
     try{
         if (DEBUG) {
-          System.err.println("GLX Version client version "+ GLXUtil.getClientVersionNumber()+
+          System.err.println("GLX Version client "+ GLXUtil.getClientVersionNumber()+
                              ", server: "+ GLXUtil.getGLXServerVersionNumber(x11Device));
         }
         if(((X11GLXDrawableFactory)drawable.getFactoryImpl()).isGLXVersionGreaterEqualOneOne(x11Device)) {
@@ -524,39 +580,67 @@ public class X11GLXContext extends GLContextImpl {
   }
 
   @Override
-  protected boolean setSwapIntervalImpl(final int interval) {
-    if( !drawable.getChosenGLCapabilities().isOnscreen() ) { return false; }
-
-    final GLXExt glXExt = getGLXExt();
-    if(0==hasSwapInterval) {
+  protected final Integer setSwapIntervalImpl2(final int interval) {
+    if( !drawable.getChosenGLCapabilities().isOnscreen() ) {
+        return null;
+    }
+    final long displayHandle = drawable.getNativeSurface().getDisplayHandle();
+    if( 0 == hasSwapInterval ) {
         try {
-            /** Same impl. ..
-            if( glXExt.isExtensionAvailable(GLXExtensions.GLX_MESA_swap_control) ) {
-                if(DEBUG) { System.err.println("X11GLXContext.setSwapInterval using: "+GLXExtensions.GLX_MESA_swap_control); }
-                hasSwapInterval =  1;
-            } else */
-            if ( glXExt.isExtensionAvailable(GLXExtensions.GLX_SGI_swap_control) ) {
-                if(DEBUG) { System.err.println("X11GLXContext.setSwapInterval using: "+GLXExtensions.GLX_SGI_swap_control); }
-                hasSwapInterval =  2;
+            if ( isExtensionAvailable(GLXExtensions.GLX_EXT_swap_control) ) {
+                hasSwapInterval = 1;
+                if ( isExtensionAvailable(GLXExtensions.GLX_EXT_swap_control_tear) ) {
+                    try {
+                        final IntBuffer val = Buffers.newDirectIntBuffer(1);
+                        GLX.glXQueryDrawable(displayHandle, drawable.getHandle(), GLX.GLX_LATE_SWAPS_TEAR_EXT, val);
+                        if( 1 == val.get(0) ) {
+                            hasSwapInterval =  2;
+                            if(DEBUG) { System.err.println("X11GLXContext.setSwapInterval.2 using: "+GLXExtensions.GLX_EXT_swap_control_tear + ", " + GLXExtensions.GLX_EXT_swap_control_tear); }
+                        } else if(DEBUG) {
+                            System.err.println("X11GLXContext.setSwapInterval.2 n/a: "+GLXExtensions.GLX_EXT_swap_control_tear+", query: "+val.get(0));
+                        }
+                    } catch (final Throwable t) { if(DEBUG) { ExceptionUtils.dumpThrowable("", t); } }
+                }
+                if(DEBUG) {
+                    if( 1 == hasSwapInterval ) {
+                        System.err.println("X11GLXContext.setSwapInterval.1 using: "+GLXExtensions.GLX_EXT_swap_control);
+                    }
+                }
+            } else if ( isExtensionAvailable(GLXExtensions.GLX_SGI_swap_control) ) {
+                hasSwapInterval =  3;
+                if(DEBUG) { System.err.println("X11GLXContext.setSwapInterval.3 using: "+GLXExtensions.GLX_SGI_swap_control); }
             } else {
                 hasSwapInterval = -1;
+                if(DEBUG) { System.err.println("X11GLXContext.setSwapInterval.0 N/A"); }
             }
-        } catch (final Throwable t) { hasSwapInterval=-1; }
+        } catch (final Throwable t) { hasSwapInterval=-1; if(DEBUG) { ExceptionUtils.dumpThrowable("", t); } }
     }
-    /* try {
-        switch( hasSwapInterval ) {
-            case 1:
-                return 0 == glXExt.glXSwapIntervalMESA(interval);
-            case 2:
-                return 0 == glXExt.glXSwapIntervalSGI(interval);
+    if (3 == hasSwapInterval) {
+        final int useInterval;
+        if( 0 > interval ) {
+            useInterval = Math.abs(interval);
+        } else {
+            useInterval = interval;
         }
-    } catch (Throwable t) { hasSwapInterval = -1; } */
-    if (2 == hasSwapInterval) {
         try {
-            return 0 == glXExt.glXSwapIntervalSGI(interval);
-        } catch (final Throwable t) { hasSwapInterval=-1; }
+            final GLXExt glXExt = getGLXExt();
+            if( 0 == glXExt.glXSwapIntervalSGI(useInterval) ) {
+                return Integer.valueOf(useInterval);
+            }
+        } catch (final Throwable t) { hasSwapInterval=-1; if(DEBUG) { ExceptionUtils.dumpThrowable("", t); } }
+    } else if ( 0 < hasSwapInterval ) { // 2 || 1
+        final int useInterval;
+        if( 1 == hasSwapInterval && 0 > interval ) {
+            useInterval = Math.abs(interval);
+        } else {
+            useInterval = interval;
+        }
+        try {
+            GLX.glXSwapIntervalEXT(displayHandle, drawable.getHandle(), useInterval);
+            return Integer.valueOf(useInterval);
+        } catch (final Throwable t) { hasSwapInterval=-1; if(DEBUG) { ExceptionUtils.dumpThrowable("", t); } }
     }
-    return false;
+    return null;
   }
 
   private final int initSwapGroupImpl(final GLXExt glXExt) {
@@ -622,16 +706,6 @@ public class X11GLXContext extends GLContextImpl {
       return res;
   }
 
-  @Override
-  public final ByteBuffer glAllocateMemoryNV(final int size, final float readFrequency, final float writeFrequency, final float priority) {
-    return getGLXExt().glXAllocateMemoryNV(size, readFrequency, writeFrequency, priority);
-  }
-
-  @Override
-  public final void glFreeMemoryNV(final ByteBuffer pointer) {
-    getGLXExt().glXFreeMemoryNV(pointer);
-  }
-
   @Override
   public String toString() {
     final StringBuilder sb = new StringBuilder();
