@@ -41,24 +41,22 @@ import java.nio.Buffer;
 import java.nio.ShortBuffer;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
-import java.util.Collection;
-import java.util.HashMap;
 import java.util.List;
 
-import javax.media.nativewindow.AbstractGraphicsConfiguration;
-import javax.media.nativewindow.AbstractGraphicsDevice;
-import javax.media.nativewindow.AbstractGraphicsScreen;
-import javax.media.nativewindow.NativeSurface;
-import javax.media.nativewindow.ProxySurface;
-import javax.media.nativewindow.UpstreamSurfaceHook;
-import javax.media.nativewindow.VisualIDHolder;
-import javax.media.opengl.GLCapabilities;
-import javax.media.opengl.GLCapabilitiesChooser;
-import javax.media.opengl.GLCapabilitiesImmutable;
-import javax.media.opengl.GLContext;
-import javax.media.opengl.GLDrawable;
-import javax.media.opengl.GLException;
-import javax.media.opengl.GLProfile;
+import com.jogamp.nativewindow.AbstractGraphicsConfiguration;
+import com.jogamp.nativewindow.AbstractGraphicsDevice;
+import com.jogamp.nativewindow.AbstractGraphicsScreen;
+import com.jogamp.nativewindow.NativeSurface;
+import com.jogamp.nativewindow.ProxySurface;
+import com.jogamp.nativewindow.UpstreamSurfaceHook;
+import com.jogamp.nativewindow.VisualIDHolder;
+import com.jogamp.opengl.GLCapabilities;
+import com.jogamp.opengl.GLCapabilitiesChooser;
+import com.jogamp.opengl.GLCapabilitiesImmutable;
+import com.jogamp.opengl.GLContext;
+import com.jogamp.opengl.GLDrawable;
+import com.jogamp.opengl.GLException;
+import com.jogamp.opengl.GLProfile;
 
 import jogamp.nativewindow.WrappedSurface;
 import jogamp.nativewindow.x11.X11DummyUpstreamSurfaceHook;
@@ -72,13 +70,16 @@ import jogamp.opengl.GLDynamicLookupHelper;
 import jogamp.opengl.GLGraphicsConfigurationUtil;
 import jogamp.opengl.SharedResourceRunner;
 
+import com.jogamp.common.ExceptionUtils;
 import com.jogamp.common.nio.Buffers;
 import com.jogamp.common.util.VersionNumber;
+import com.jogamp.nativewindow.GenericUpstreamSurfacelessHook;
 import com.jogamp.nativewindow.x11.X11GraphicsDevice;
 import com.jogamp.nativewindow.x11.X11GraphicsScreen;
 import com.jogamp.opengl.GLRendererQuirks;
 
 public class X11GLXDrawableFactory extends GLDrawableFactoryImpl {
+  private static final boolean DEBUG_SHAREDCTX = DEBUG  || GLContext.DEBUG;
 
   public static final VersionNumber versionOneZero = new VersionNumber(1, 0, 0);
   public static final VersionNumber versionOneOne = new VersionNumber(1, 1, 0);
@@ -123,11 +124,10 @@ public class X11GLXDrawableFactory extends GLDrawableFactoryImpl {
         // The act of constructing them causes them to be registered
         X11GLXGraphicsConfigurationFactory.registerFactory();
 
-        sharedMap = new HashMap<String, SharedResourceRunner.Resource>();
-
         // Init shared resources off thread
         // Will be released via ShutdownHook
-        sharedResourceRunner = new SharedResourceRunner(new SharedResourceImplementation());
+        sharedResourceImplementation = new SharedResourceImplementation();
+        sharedResourceRunner = new SharedResourceRunner(sharedResourceImplementation);
         sharedResourceRunner.start();
     }
   }
@@ -146,9 +146,9 @@ public class X11GLXDrawableFactory extends GLDrawableFactoryImpl {
         sharedResourceRunner.stop();
         sharedResourceRunner = null;
     }
-    if(null != sharedMap) {
-        sharedMap.clear();
-        sharedMap = null;
+    if(null != sharedResourceImplementation) {
+        sharedResourceImplementation.clear();
+        sharedResourceImplementation = null;
     }
     defaultDevice = null;
     /**
@@ -160,13 +160,13 @@ public class X11GLXDrawableFactory extends GLDrawableFactoryImpl {
   }
 
   @Override
-  public final GLDynamicLookupHelper getGLDynamicLookupHelper(final int profile) {
+  public final GLDynamicLookupHelper getGLDynamicLookupHelper(final int majorVersion, final int contextOptions) {
       return x11GLXDynamicLookupHelper;
   }
 
   private X11GraphicsDevice defaultDevice;
+  private SharedResourceImplementation sharedResourceImplementation;
   private SharedResourceRunner sharedResourceRunner;
-  private HashMap<String /* connection */, SharedResourceRunner.Resource> sharedMap;
 
   static class SharedResource implements SharedResourceRunner.Resource {
       private final String glXServerVendorName;
@@ -183,7 +183,8 @@ public class X11GLXDrawableFactory extends GLDrawableFactoryImpl {
 
       SharedResource(final X11GraphicsDevice dev, final X11GraphicsScreen scrn,
                      final GLDrawableImpl draw, final GLContextImpl ctx,
-                     final VersionNumber glXServerVer, final String glXServerVendor, final boolean glXServerMultisampleAvail) {
+                     final VersionNumber glXServerVer, final String glXServerVendor,
+                     final boolean glXServerMultisampleAvail) {
           device = dev;
           screen = scrn;
           drawable = draw;
@@ -197,7 +198,7 @@ public class X11GLXDrawableFactory extends GLDrawableFactoryImpl {
           glXMultisampleAvailable = glXServerMultisampleAvail;
       }
       @Override
-      public final boolean isValid() {
+      public final boolean isAvailable() {
           return null != context;
       }
       @Override
@@ -209,7 +210,7 @@ public class X11GLXDrawableFactory extends GLDrawableFactoryImpl {
       @Override
       final public GLContextImpl getContext() { return context; }
       @Override
-      public GLRendererQuirks getRendererQuirks() {
+      public GLRendererQuirks getRendererQuirks(final GLProfile glp) {
           return null != context ? context.getRendererQuirks() : null;
       }
 
@@ -222,28 +223,11 @@ public class X11GLXDrawableFactory extends GLDrawableFactoryImpl {
       final boolean isGLXMultisampleAvailable() { return glXMultisampleAvailable; }
   }
 
-  class SharedResourceImplementation implements SharedResourceRunner.Implementation {
+  class SharedResourceImplementation extends SharedResourceRunner.AImplementation {
         @Override
-        public void clear() {
-            sharedMap.clear();
-        }
-        @Override
-        public SharedResourceRunner.Resource mapPut(final String connection, final SharedResourceRunner.Resource resource) {
-            return sharedMap.put(connection, resource);
-        }
-        @Override
-        public SharedResourceRunner.Resource mapGet(final String connection) {
-            return sharedMap.get(connection);
-        }
-        @Override
-        public Collection<SharedResourceRunner.Resource> mapValues() {
-            return sharedMap.values();
-        }
-
-        @Override
-        public boolean isDeviceSupported(final String connection) {
+        public boolean isDeviceSupported(final AbstractGraphicsDevice device) {
             final boolean res;
-            final X11GraphicsDevice x11Device = new X11GraphicsDevice(X11Util.openDisplay(connection), AbstractGraphicsDevice.DEFAULT_UNIT, true /* owner */);
+            final X11GraphicsDevice x11Device = new X11GraphicsDevice(X11Util.openDisplay(device.getConnection()), device.getUnitID(), true /* owner */);
             x11Device.lock();
             try {
                 res = GLXUtil.isGLXAvailableOnServer(x11Device);
@@ -258,51 +242,56 @@ public class X11GLXDrawableFactory extends GLDrawableFactoryImpl {
         }
 
         @Override
-        public SharedResourceRunner.Resource createSharedResource(final String connection) {
-            final X11GraphicsDevice sharedDevice = new X11GraphicsDevice(X11Util.openDisplay(connection), AbstractGraphicsDevice.DEFAULT_UNIT, true /* owner */);
-            sharedDevice.lock();
+        public SharedResourceRunner.Resource createSharedResource(final AbstractGraphicsDevice adevice) {
+            final X11GraphicsDevice device = new X11GraphicsDevice(X11Util.openDisplay(adevice.getConnection()), adevice.getUnitID(), true /* owner */);
+            GLContextImpl context = null;
+            boolean contextIsCurrent = false;
+            device.lock();
             try {
-                final X11GraphicsScreen sharedScreen = new X11GraphicsScreen(sharedDevice, sharedDevice.getDefaultScreen());
+                final X11GraphicsScreen screen = new X11GraphicsScreen(device, device.getDefaultScreen());
 
-                GLXUtil.initGLXClientDataSingleton(sharedDevice);
-                final String glXServerVendorName = GLX.glXQueryServerString(sharedDevice.getHandle(), 0, GLX.GLX_VENDOR);
-                final boolean glXServerMultisampleAvailable = GLXUtil.isMultisampleAvailable(GLX.glXQueryServerString(sharedDevice.getHandle(), 0, GLX.GLX_EXTENSIONS));
+                GLXUtil.initGLXClientDataSingleton(device);
+                final String glXServerVendorName = GLX.glXQueryServerString(device.getHandle(), 0, GLX.GLX_VENDOR);
+                final boolean glXServerMultisampleAvailable = GLXUtil.isMultisampleAvailable(GLX.glXQueryServerString(device.getHandle(), 0, GLX.GLX_EXTENSIONS));
 
-                final GLProfile glp = GLProfile.get(sharedDevice, GLProfile.GL_PROFILE_LIST_MIN_DESKTOP, false);
+                final GLProfile glp = GLProfile.get(device, GLProfile.GL_PROFILE_LIST_MIN_DESKTOP, false);
                 if (null == glp) {
-                    throw new GLException("Couldn't get default GLProfile for device: "+sharedDevice);
+                    throw new GLException("Couldn't get default GLProfile for device: "+device);
                 }
 
                 final GLCapabilitiesImmutable caps = new GLCapabilities(glp);
-                final GLDrawableImpl sharedDrawable = createOnscreenDrawableImpl(createDummySurfaceImpl(sharedDevice, false, caps, caps, null, 64, 64));
-                sharedDrawable.setRealized(true);
-                final X11GLCapabilities chosenCaps =  (X11GLCapabilities) sharedDrawable.getChosenGLCapabilities();
+                final GLDrawableImpl drawable = createOnscreenDrawableImpl(createDummySurfaceImpl(device, false, caps, caps, null, 64, 64));
+                drawable.setRealized(true);
+                final X11GLCapabilities chosenCaps =  (X11GLCapabilities) drawable.getChosenGLCapabilities();
                 final boolean glxForcedOneOne = !chosenCaps.hasFBConfig();
                 final VersionNumber glXServerVersion;
                 if( glxForcedOneOne ) {
                     glXServerVersion = versionOneOne;
                 } else {
-                    glXServerVersion = GLXUtil.getGLXServerVersionNumber(sharedDevice);
+                    glXServerVersion = GLXUtil.getGLXServerVersionNumber(device);
                 }
-                final GLContextImpl sharedContext = (GLContextImpl) sharedDrawable.createContext(null);
-                if (null == sharedContext) {
-                    throw new GLException("Couldn't create shared context for drawable: "+sharedDrawable);
+                context = (GLContextImpl) drawable.createContext(null);
+                if (null == context) {
+                    throw new GLException("Couldn't create shared context for drawable: "+drawable);
+                }
+                contextIsCurrent = GLContext.CONTEXT_NOT_CURRENT != context.makeCurrent();
+
+                final boolean allowsSurfacelessCtx;
+                if( contextIsCurrent && context.getGLVersionNumber().compareTo(GLContext.Version3_0) >= 0 ) {
+                    allowsSurfacelessCtx = probeSurfacelessCtx(context, true /* restoreDrawable */);
+                } else {
+                    setNoSurfacelessCtxQuirk(context);
+                    allowsSurfacelessCtx = false;
                 }
 
-                boolean madeCurrent = false;
-                sharedContext.makeCurrent();
-                try {
-                    madeCurrent = sharedContext.isCurrent();
-                } finally {
-                    sharedContext.release();
-                }
-                if( sharedContext.hasRendererQuirk( GLRendererQuirks.DontCloseX11Display ) ) {
+                if( context.hasRendererQuirk( GLRendererQuirks.DontCloseX11Display ) ) {
                     X11Util.markAllDisplaysUnclosable();
                 }
-                if (DEBUG) {
-                    System.err.println("SharedDevice:  " + sharedDevice);
-                    System.err.println("SharedScreen:  " + sharedScreen);
-                    System.err.println("SharedContext: " + sharedContext + ", madeCurrent " + madeCurrent);
+                if ( DEBUG_SHAREDCTX ) {
+                    System.err.println("SharedDevice:  " + device);
+                    System.err.println("SharedScreen:  " + screen);
+                    System.err.println("SharedContext: " + context + ", madeCurrent " + contextIsCurrent);
+                    System.err.println("  allowsSurfacelessCtx "+allowsSurfacelessCtx);
                     System.err.println("GLX Server Vendor:      " + glXServerVendorName);
                     System.err.println("GLX Server Version:     " + glXServerVersion + ", forced "+glxForcedOneOne);
                     System.err.println("GLX Server Multisample: " + glXServerMultisampleAvailable);
@@ -310,26 +299,29 @@ public class X11GLXDrawableFactory extends GLDrawableFactoryImpl {
                     System.err.println("GLX Client Version:     " + GLXUtil.getClientVersionNumber());
                     System.err.println("GLX Client Multisample: " + GLXUtil.isClientMultisampleAvailable());
                 }
-                return new SharedResource(sharedDevice, sharedScreen, sharedDrawable, sharedContext,
+                return new SharedResource(device, screen, drawable, context,
                                           glXServerVersion, glXServerVendorName,
                                           glXServerMultisampleAvailable && GLXUtil.isClientMultisampleAvailable());
             } catch (final Throwable t) {
-                throw new GLException("X11GLXDrawableFactory - Could not initialize shared resources for "+connection, t);
+                throw new GLException("X11GLXDrawableFactory - Could not initialize shared resources for "+adevice, t);
             } finally {
-                sharedDevice.unlock();
+                if ( contextIsCurrent ) {
+                    context.release();
+                }
+                device.unlock();
             }
         }
 
         @Override
         public void releaseSharedResource(final SharedResourceRunner.Resource shared) {
             final SharedResource sr = (SharedResource) shared;
-            if (DEBUG) {
+            if ( DEBUG_SHAREDCTX ) {
                 System.err.println("Shutdown Shared:");
                 System.err.println("Device  : " + sr.device);
                 System.err.println("Screen  : " + sr.screen);
                 System.err.println("Drawable: " + sr.drawable);
                 System.err.println("CTX     : " + sr.context);
-                Thread.dumpStack();
+                ExceptionUtils.dumpStack(System.err);
             }
 
             if (null != sr.context) {
@@ -387,6 +379,33 @@ public class X11GLXDrawableFactory extends GLDrawableFactoryImpl {
     return 0;
   }
 
+  /**
+   * {@inheritDoc}
+   * <p>
+   * This factory always supports native desktop OpenGL profiles.
+   * </p>
+   */
+  @Override
+  public final boolean hasOpenGLDesktopSupport() { return true; }
+
+  /**
+   * {@inheritDoc}
+   * <p>
+   * This factory never supports native GLES profiles.
+   * </p>
+   */
+  @Override
+  public final boolean hasOpenGLESSupport() { return false; }
+
+  /**
+   * {@inheritDoc}
+   * <p>
+   * Always returns true.
+   * </p>
+   */
+  @Override
+  public final boolean hasMajorMinorCreateContextARB() { return true; }
+
   @Override
   protected List<GLCapabilitiesImmutable> getAvailableCapabilitiesImpl(final AbstractGraphicsDevice device) {
     return X11GLXGraphicsConfigurationFactory.getAvailableCapabilities(this, device);
@@ -523,6 +542,13 @@ public class X11GLXDrawableFactory extends GLDrawableFactoryImpl {
     return createMutableSurfaceImpl(deviceReq, createNewDevice, chosenCaps, requestedCaps, chooser, new X11DummyUpstreamSurfaceHook(width, height));
   }
 
+  @Override
+  public final ProxySurface createSurfacelessImpl(final AbstractGraphicsDevice deviceReq, final boolean createNewDevice,
+                                                  GLCapabilitiesImmutable chosenCaps, final GLCapabilitiesImmutable requestedCaps, final GLCapabilitiesChooser chooser, final int width, final int height) {
+    chosenCaps = GLGraphicsConfigurationUtil.fixOnscreenGLCapabilities(chosenCaps);
+    return createMutableSurfaceImpl(deviceReq, createNewDevice, chosenCaps, requestedCaps, chooser, new GenericUpstreamSurfacelessHook(width, height));
+  }
+
   @Override
   protected final ProxySurface createProxySurfaceImpl(final AbstractGraphicsDevice deviceReq, final int screenIdx, final long windowHandle, final GLCapabilitiesImmutable capsRequested, final GLCapabilitiesChooser chooser, final UpstreamSurfaceHook upstream) {
     final X11GraphicsDevice device = new X11GraphicsDevice(X11Util.openDisplay(deviceReq.getConnection()), deviceReq.getUnitID(), true /* owner */);
