@@ -535,7 +535,8 @@ JNIEXPORT jint JNICALL FF_FUNC(getSwResampleMajorVersionCC0)
 }
 
 JNIEXPORT jlong JNICALL FF_FUNC(createInstance0)
-  (JNIEnv *env, jobject instance, jobject ffmpegMediaPlayer, jboolean verbose)
+  (JNIEnv *env, jobject instance, jobject ffmpegMediaPlayer,
+   jboolean enableAvResample, jboolean enableSwResample, jboolean verbose)
 {
     FFMPEGToolBasicAV_t * pAV = calloc(1, sizeof(FFMPEGToolBasicAV_t));
     if(NULL==pAV) {
@@ -545,12 +546,12 @@ JNIEXPORT jlong JNICALL FF_FUNC(createInstance0)
     pAV->avcodecVersion = sp_avcodec_version();
     pAV->avformatVersion = sp_avformat_version(); 
     pAV->avutilVersion = sp_avutil_version();
-    if(HAS_FUNC(sp_avresample_version)) {
+    if(HAS_FUNC(sp_avresample_version) && enableAvResample) {
         pAV->avresampleVersion = sp_avresample_version();
     } else {
         pAV->avresampleVersion = 0;
     }
-    if(HAS_FUNC(sp_swresample_version)) {
+    if(HAS_FUNC(sp_swresample_version) && enableSwResample) {
         pAV->swresampleVersion = sp_swresample_version();
     } else {
         pAV->swresampleVersion = 0;
@@ -621,6 +622,7 @@ static const char * inFmtNames[] = {
     "video4linux",  // linux (old)
     "dshow",        // windows
     "vfwcap",       // windows (old)
+    "avfoundation", // osx
     "mpg",
     "yuv2",
     "mjpeg",
@@ -731,6 +733,9 @@ JNIEXPORT void JNICALL FF_FUNC(setStream0)
                 fprintf(stderr, "Camera %d not found\n", devIdx);
             }
         }
+        if(pAV->verbose) {
+            fprintf(stderr, "Camera: Filename: %s\n", filename);
+        }
 
         const char *sizeS = NULL != jSizeS ? (*env)->GetStringUTFChars(env, jSizeS, &iscopy) : NULL;
         int hasSize = 0;
@@ -755,6 +760,7 @@ JNIEXPORT void JNICALL FF_FUNC(setStream0)
             }
             sp_av_dict_set(&inOpts, "framerate", buffer, 0);
         }
+        // FIXME pre-select: sp_av_dict_set(&inOpts, "pixel_format", "yuyv422", 0); 
     }
 
     MY_MUTEX_LOCK(env, mutex_avcodec_openclose);
@@ -1147,8 +1153,10 @@ JNIEXPORT void JNICALL FF_FUNC(setStream0)
                 #endif
             } else {
                 vLinesize[0] = pAV->pVFrame->linesize[0];
-                if( pAV->vPixFmt == PIX_FMT_YUYV422 ) {
-                    // Stuff 2x 16bpp (YUYV) into one RGBA pixel!
+                if( pAV->vPixFmt == PIX_FMT_YUYV422 || 
+                    pAV->vPixFmt == PIX_FMT_UYVY422 ) 
+                {
+                    // Stuff 2x 16bpp (YUYV, UYVY) into one RGBA pixel!
                     pAV->vTexWidth[0] = pAV->pVCodecCtx->width / 2;
                 } else {
                     pAV->vTexWidth[0] = pAV->pVCodecCtx->width;
