@@ -37,6 +37,7 @@
 
 package com.jogamp.nativewindow.awt;
 
+import com.jogamp.common.ExceptionUtils;
 import com.jogamp.common.os.Platform;
 import com.jogamp.common.util.awt.AWTEDTExecutor;
 import com.jogamp.common.util.locks.LockFactory;
@@ -46,30 +47,31 @@ import com.jogamp.nativewindow.MutableGraphicsConfiguration;
 import java.awt.Component;
 import java.awt.Container;
 import java.awt.Cursor;
-import java.awt.Window;
+import java.awt.EventQueue;
+import java.awt.GraphicsConfiguration;
 import java.awt.event.ComponentEvent;
 import java.awt.event.ComponentListener;
 import java.awt.event.HierarchyEvent;
 import java.awt.event.HierarchyListener;
 import java.applet.Applet;
 
-import javax.media.nativewindow.AbstractGraphicsConfiguration;
-import javax.media.nativewindow.AbstractGraphicsDevice;
-import javax.media.nativewindow.CapabilitiesImmutable;
-import javax.media.nativewindow.NativeSurface;
-import javax.media.nativewindow.NativeWindow;
-import javax.media.nativewindow.NativeWindowException;
-import javax.media.nativewindow.OffscreenLayerOption;
-import javax.media.nativewindow.OffscreenLayerSurface;
-import javax.media.nativewindow.ScalableSurface;
-import javax.media.nativewindow.SurfaceUpdatedListener;
-import javax.media.nativewindow.util.Insets;
-import javax.media.nativewindow.util.InsetsImmutable;
-import javax.media.nativewindow.util.PixelRectangle;
-import javax.media.nativewindow.util.Point;
-import javax.media.nativewindow.util.PointImmutable;
-import javax.media.nativewindow.util.Rectangle;
-import javax.media.nativewindow.util.RectangleImmutable;
+import com.jogamp.nativewindow.AbstractGraphicsConfiguration;
+import com.jogamp.nativewindow.AbstractGraphicsDevice;
+import com.jogamp.nativewindow.CapabilitiesImmutable;
+import com.jogamp.nativewindow.NativeSurface;
+import com.jogamp.nativewindow.NativeWindow;
+import com.jogamp.nativewindow.NativeWindowException;
+import com.jogamp.nativewindow.OffscreenLayerOption;
+import com.jogamp.nativewindow.OffscreenLayerSurface;
+import com.jogamp.nativewindow.ScalableSurface;
+import com.jogamp.nativewindow.SurfaceUpdatedListener;
+import com.jogamp.nativewindow.util.Insets;
+import com.jogamp.nativewindow.util.InsetsImmutable;
+import com.jogamp.nativewindow.util.PixelRectangle;
+import com.jogamp.nativewindow.util.Point;
+import com.jogamp.nativewindow.util.PointImmutable;
+import com.jogamp.nativewindow.util.Rectangle;
+import com.jogamp.nativewindow.util.RectangleImmutable;
 
 import jogamp.common.os.PlatformPropsImpl;
 import jogamp.nativewindow.SurfaceScaleUtils;
@@ -88,10 +90,10 @@ public abstract class JAWTWindow implements NativeWindow, OffscreenLayerSurface,
   // lifetime: forever
   protected final Component component;
   private final AppContextInfo appContextInfo;
-  private final AWTGraphicsConfiguration config; // control access due to delegation
   private final SurfaceUpdatedHelper surfaceUpdatedHelper = new SurfaceUpdatedHelper();
   private final RecursiveLock surfaceLock = LockFactory.createRecursiveLock();
   private final JAWTComponentListener jawtComponentListener;
+  private volatile AWTGraphicsConfiguration awtConfig; // control access through delegation
 
   // lifetime: valid after lock but may change with each 1st lock, purges after invalidate
   private boolean isApplet;
@@ -102,10 +104,11 @@ public abstract class JAWTWindow implements NativeWindow, OffscreenLayerSurface,
   protected Insets insets;
   private volatile long offscreenSurfaceLayer;
 
-  private final int[] nativePixelScale = new int[] { ScalableSurface.IDENTITY_PIXELSCALE, ScalableSurface.IDENTITY_PIXELSCALE };
-  private final int[] hasPixelScale = new int[] { ScalableSurface.IDENTITY_PIXELSCALE, ScalableSurface.IDENTITY_PIXELSCALE };
-  protected final int[] reqPixelScale = new int[] { ScalableSurface.AUTOMAX_PIXELSCALE, ScalableSurface.AUTOMAX_PIXELSCALE };
-
+  private final float[] minPixelScale = new float[] { ScalableSurface.IDENTITY_PIXELSCALE, ScalableSurface.IDENTITY_PIXELSCALE };
+  private final float[] maxPixelScale = new float[] { ScalableSurface.IDENTITY_PIXELSCALE, ScalableSurface.IDENTITY_PIXELSCALE };
+  private final float[] hasPixelScale = new float[] { ScalableSurface.IDENTITY_PIXELSCALE, ScalableSurface.IDENTITY_PIXELSCALE };
+  private final float[] reqPixelScale = new float[] { ScalableSurface.AUTOMAX_PIXELSCALE, ScalableSurface.AUTOMAX_PIXELSCALE };
+  private volatile boolean hasPixelScaleChanged = false;
   private long drawable_old;
 
   /**
@@ -117,24 +120,28 @@ public abstract class JAWTWindow implements NativeWindow, OffscreenLayerSurface,
    */
   protected JAWTWindow(final Object comp, final AbstractGraphicsConfiguration config) {
     if (config == null) {
-        throw new NativeWindowException("Error: AbstractGraphicsConfiguration is null");
+        throw new IllegalArgumentException("Error: AbstractGraphicsConfiguration is null");
     }
     if(! ( config instanceof AWTGraphicsConfiguration ) ) {
         throw new NativeWindowException("Error: AbstractGraphicsConfiguration is not an AWTGraphicsConfiguration: "+config);
     }
     appContextInfo = new AppContextInfo("<init>");
     this.component = (Component)comp;
-    this.config = (AWTGraphicsConfiguration) config;
     this.jawtComponentListener = new JAWTComponentListener();
     invalidate();
+    this.awtConfig = (AWTGraphicsConfiguration) config;
     this.isApplet = false;
     this.offscreenSurfaceLayer = 0;
+    if(DEBUG) {
+        System.err.println(jawtStr2("ctor"));
+    }
   }
   private static String id(final Object obj) { return ( null!=obj ? toHexString(obj.hashCode()) : "nil" ); }
-  private String jawtStr() { return "JAWTWindow["+id(JAWTWindow.this)+"]"; }
+  private String jawtStr1() { return "JAWTWindow["+id(JAWTWindow.this)+"]"; }
+  private String jawtStr2(final String sub) { return jawtStr1()+"."+sub+" @ Thread "+getThreadName(); }
 
   private class JAWTComponentListener implements ComponentListener, HierarchyListener {
-        private boolean isShowing;
+        private volatile boolean isShowing;
 
         private String str(final Object obj) {
             if( null == obj ) {
@@ -175,8 +182,9 @@ public abstract class JAWTWindow implements NativeWindow, OffscreenLayerSurface,
             AWTEDTExecutor.singleton.invoke(false, new Runnable() { // Bug 952: Avoid deadlock via AWTTreeLock acquisition ..
                 @Override
                 public void run() {
+                    isShowing = component.isShowing(); // Bug 1161: Runnable might be deferred, hence need to update
                     if(DEBUG) {
-                        System.err.println(jawtStr()+".attach @ Thread "+getThreadName()+": "+JAWTComponentListener.this.toString());
+                        System.err.println(jawtStr2("attach")+": "+JAWTComponentListener.this.toString());
                     }
                     component.addComponentListener(JAWTComponentListener.this);
                     component.addHierarchyListener(JAWTComponentListener.this);
@@ -188,7 +196,7 @@ public abstract class JAWTWindow implements NativeWindow, OffscreenLayerSurface,
                 @Override
                 public void run() {
                     if(DEBUG) {
-                        System.err.println(jawtStr()+".detach @ Thread "+getThreadName()+": "+JAWTComponentListener.this.toString());
+                        System.err.println(jawtStr2("detach")+": "+JAWTComponentListener.this.toString());
                     }
                     component.removeComponentListener(JAWTComponentListener.this);
                     component.removeHierarchyListener(JAWTComponentListener.this);
@@ -198,7 +206,7 @@ public abstract class JAWTWindow implements NativeWindow, OffscreenLayerSurface,
         @Override
         public final void componentResized(final ComponentEvent e) {
             if(DEBUG) {
-                System.err.println(jawtStr()+".componentResized: "+s(e));
+                System.err.println(jawtStr2("componentResized")+": "+s(e));
             }
             layoutSurfaceLayerIfEnabled(isShowing);
         }
@@ -206,7 +214,7 @@ public abstract class JAWTWindow implements NativeWindow, OffscreenLayerSurface,
         @Override
         public final void componentMoved(final ComponentEvent e) {
             if(DEBUG) {
-                System.err.println(jawtStr()+".componentMoved: "+s(e));
+                System.err.println(jawtStr2("componentMoved")+": "+s(e));
             }
             layoutSurfaceLayerIfEnabled(isShowing);
         }
@@ -214,7 +222,7 @@ public abstract class JAWTWindow implements NativeWindow, OffscreenLayerSurface,
         @Override
         public final void componentShown(final ComponentEvent e) {
             if(DEBUG) {
-                System.err.println(jawtStr()+".componentShown: "+s(e));
+                System.err.println(jawtStr2("componentShown")+": "+s(e));
             }
             layoutSurfaceLayerIfEnabled(isShowing);
         }
@@ -222,7 +230,7 @@ public abstract class JAWTWindow implements NativeWindow, OffscreenLayerSurface,
         @Override
         public final void componentHidden(final ComponentEvent e) {
             if(DEBUG) {
-                System.err.println(jawtStr()+".componentHidden: "+s(e));
+                System.err.println(jawtStr2("componentHidden")+": "+s(e));
             }
             layoutSurfaceLayerIfEnabled(isShowing);
         }
@@ -244,7 +252,7 @@ public abstract class JAWTWindow implements NativeWindow, OffscreenLayerSurface,
                 final java.awt.Component changed = e.getChanged();
                 final boolean displayable = changed.isDisplayable();
                 final boolean showing = changed.isShowing();
-                System.err.println(jawtStr()+".hierarchyChanged: action "+action+", displayable "+displayable+", showing [changed "+showing+", comp "+isShowing+"], "+s(e));
+                System.err.println(jawtStr2("hierarchyChanged")+": action "+action+", displayable "+displayable+", showing [changed "+showing+", comp "+wasShowing+" -> "+isShowing+"], "+s(e));
             }
         }
   }
@@ -253,7 +261,7 @@ public abstract class JAWTWindow implements NativeWindow, OffscreenLayerSurface,
 
   protected synchronized void invalidate() {
     if(DEBUG) {
-        System.err.println(jawtStr()+".invalidate() - "+jawtComponentListener.toString());
+        System.err.println(jawtStr2("invalidate")+" - "+jawtComponentListener.toString());
         if( isSurfaceLayerAttached() ) {
             System.err.println("OffscreenSurfaceLayer still attached: 0x"+Long.toHexString(offscreenSurfaceLayer));
         }
@@ -261,6 +269,7 @@ public abstract class JAWTWindow implements NativeWindow, OffscreenLayerSurface,
     }
     invalidateNative();
     jawt = null;
+    awtConfig = null;
     isOffscreenLayerSurface = false;
     drawable= 0;
     drawable_old = 0;
@@ -268,46 +277,81 @@ public abstract class JAWTWindow implements NativeWindow, OffscreenLayerSurface,
     insets = new Insets();
     hasPixelScale[0] = ScalableSurface.IDENTITY_PIXELSCALE;
     hasPixelScale[1] = ScalableSurface.IDENTITY_PIXELSCALE;
-    nativePixelScale[0] = ScalableSurface.IDENTITY_PIXELSCALE;
-    nativePixelScale[1] = ScalableSurface.IDENTITY_PIXELSCALE;
+    minPixelScale[0] = ScalableSurface.IDENTITY_PIXELSCALE;
+    minPixelScale[1] = ScalableSurface.IDENTITY_PIXELSCALE;
+    maxPixelScale[0] = ScalableSurface.IDENTITY_PIXELSCALE;
+    maxPixelScale[1] = ScalableSurface.IDENTITY_PIXELSCALE;
+    hasPixelScaleChanged = false;
   }
   protected abstract void invalidateNative();
 
   /**
-   * {@inheritDoc}
+   * Set a new {@link AWTGraphicsConfiguration} instance,
+   * as required if {@link #getAWTComponent() upstream component}'s {@link GraphicsConfiguration} has been changed
+   * due to reconfiguration, e.g. moving to a different monitor or changed capabilities.
    * <p>
-   * Per default impl. only works for not yet {@link #isRealized() realized} instances,
-   * current exception OSX.
+   * {@link #getAWTComponent() Upstream component} shall override {@link Component#getGraphicsConfiguration()},
+   * which shall call this method if detecting a reconfiguration.
+   * See JOGL's GLCanvas and NewtCanvasAWT.
    * </p>
+   * @param config the new {@link AWTGraphicsConfiguration}
+   * @see #getAWTGraphicsConfiguration()
    */
-  @Override
-  public void setSurfaceScale(final int[] pixelScale) {
-      SurfaceScaleUtils.validateReqPixelScale(reqPixelScale, pixelScale, DEBUG ? getClass().getSimpleName() : null);
+  public final void setAWTGraphicsConfiguration(final AWTGraphicsConfiguration config) {
+    if(DEBUG) {
+        System.err.println(jawtStr2("setAWTGraphicsConfiguration")+": "+this.awtConfig+" -> "+config);
+        // Thread.dumpStack();
+    }
+    if( null == awtConfig ) {
+        throw new IllegalArgumentException(jawtStr2("")+": null config");
+    }
+    this.awtConfig = config;
+  }
+  /**
+   * Return the current {@link AWTGraphicsConfiguration} instance,
+   * which also holds its {@link #getAWTComponent() upstream component}'s {@link GraphicsConfiguration}
+   * @see #setAWTGraphicsConfiguration(AWTGraphicsConfiguration)
+   */
+  public final AWTGraphicsConfiguration getAWTGraphicsConfiguration() {
+    return awtConfig;
   }
 
   @Override
-  public final int[] getRequestedSurfaceScale(final int[] result) {
+  public boolean setSurfaceScale(final float[] pixelScale) {
+      System.arraycopy(pixelScale, 0, reqPixelScale, 0, 2);
+      return false;
+  }
+
+  @Override
+  public final float[] getRequestedSurfaceScale(final float[] result) {
       System.arraycopy(reqPixelScale, 0, result, 0, 2);
       return result;
   }
 
   @Override
-  public final int[] getCurrentSurfaceScale(final int[] result) {
+  public final float[] getCurrentSurfaceScale(final float[] result) {
       System.arraycopy(hasPixelScale, 0, result, 0, 2);
       return result;
   }
 
   @Override
-  public final int[] getNativeSurfaceScale(final int[] result) {
-      System.arraycopy(nativePixelScale, 0, result, 0, 2);
+  public float[] getMinimumSurfaceScale(final float[] result) {
+      System.arraycopy(minPixelScale, 0, result, 0, 2);
+      return result;
+  }
+
+  @Override
+  public final float[] getMaximumSurfaceScale(final float[] result) {
+      System.arraycopy(maxPixelScale, 0, result, 0, 2);
       return result;
   }
 
   /**
    * Updates bounds and pixelScale
+   * @param gc GraphicsConfiguration for {@link #updatePixelScale(GraphicsConfiguration, boolean)}
    * @return true if bounds or pixelScale has changed, otherwise false
    */
-  protected final boolean updateLockedData(final JAWT_Rectangle jawtBounds) {
+  protected final boolean updateLockedData(final JAWT_Rectangle jawtBounds, final GraphicsConfiguration gc) {
     final Rectangle jb = new Rectangle(jawtBounds.getX(), jawtBounds.getY(), jawtBounds.getWidth(), jawtBounds.getHeight());
     final boolean changedBounds = !bounds.equals(jb);
 
@@ -322,31 +366,97 @@ public abstract class JAWTWindow implements NativeWindow, OffscreenLayerSurface,
             insets.set(contInsets.left, contInsets.right, contInsets.top, contInsets.bottom);
         }
     }
-    {
-        final int ps = JAWTUtil.getPixelScale(config.getAWTGraphicsConfiguration());
-        nativePixelScale[0] = ps;
-        nativePixelScale[1] = ps;
-    }
 
-    return updatePixelScale() || changedBounds;
+    updatePixelScale(gc, false);
+    return hasPixelScaleChanged || changedBounds;
   }
 
   /**
-   * Update pixelScale
+   * Updates the minimum and maximum pixel-scale values
+   * and returns {@code true} if they were updated.
+   * @param gc pre-fetched {@link GraphicsConfiguration} instance of {@link #getAWTComponent() upstream component},
+   *           caller may use cached {@link #getAWTGraphicsConfiguration()}'s {@link AWTGraphicsConfiguration#getAWTGraphicsConfiguration() GC}
+   *           or a {@link Component#getGraphicsConfiguration()}.
+   * @param clearFlag if {@code true}, the {@code hasPixelScaleChanged} flag will be cleared
+   * @return {@code true} if values were updated, otherwise {@code false}.
+   * @see #hasPixelScaleChanged()
+   * @see #getAWTGraphicsConfiguration()
+   * @see Component#getGraphicsConfiguration()
+   */
+  public final boolean updatePixelScale(final GraphicsConfiguration gc, final boolean clearFlag) {
+      if( JAWTUtil.getPixelScale(gc, minPixelScale, maxPixelScale) ) {
+          hasPixelScaleChanged = true;
+          if( DEBUG ) {
+              System.err.println("JAWTWindow.updatePixelScale: updated req["+
+                      reqPixelScale[0]+", "+reqPixelScale[1]+"], min["+
+                      minPixelScale[0]+", "+minPixelScale[1]+"], max["+
+                      maxPixelScale[0]+", "+maxPixelScale[1]+"], has["+
+                      hasPixelScale[0]+", "+hasPixelScale[1]+"]");
+          }
+      }
+      if( clearFlag ) {
+          final boolean r = hasPixelScaleChanged;
+          hasPixelScaleChanged = false;
+          return r;
+      } else {
+          return hasPixelScaleChanged;
+      }
+  }
+  /**
+   * @deprecated Use {@link #updatePixelScale(GraphicsConfiguration, boolean)}.
+   */
+  public final boolean updatePixelScale(final boolean clearFlag) {
+      return updatePixelScale(awtConfig.getAWTGraphicsConfiguration(), clearFlag);
+  }
+
+  /**
+   * @deprecated Use {@link #updateLockedData(JAWT_Rectangle, GraphicsConfiguration)}.
+   */
+  protected final boolean updateLockedData(final JAWT_Rectangle jawtBounds) {
+      throw new RuntimeException("Invalid API entry");
+  }
+  /**
+   * @deprecated Use {@link #lockSurfaceImpl(GraphicsConfiguration)}
+   */
+  protected int lockSurfaceImpl() throws NativeWindowException {
+      throw new RuntimeException("Invalid API entry");
+  }
+
+
+  /**
+   * Returns and clears the {@code hasPixelScaleChanged} flag, as set via {@link #lockSurface()}.
+   * <p>
+   * {@code hasPixelScaleChanged} is {@code true},
+   * if the {@link #getMinimumSurfaceScale(float[]) minimum} or {@link #getMaximumSurfaceScale(float[]) maximum}
+   * pixel scale has changed.
+   * User needs to {@link #setSurfaceScale(float[]) set the current pixel scale} in this case
+   * using the {@link #getRequestedSurfaceScale(float[]) requested pixel scale}
+   * to update the surface pixel scale.
+   * </p>
+   */
+  public final boolean hasPixelScaleChanged() {
+      final boolean v = hasPixelScaleChanged;
+      hasPixelScaleChanged = false;
+      return v;
+  }
+
+  /**
+   * set requested pixelScale
    * @return true if pixelScale has changed, otherwise false
    */
-  protected final boolean updatePixelScale() {
-    return SurfaceScaleUtils.computePixelScale(hasPixelScale, hasPixelScale, reqPixelScale, nativePixelScale, DEBUG ? getClass().getSimpleName() : null);
+  protected final boolean setReqPixelScale() {
+    updatePixelScale(awtConfig.getAWTGraphicsConfiguration(), true);
+    return SurfaceScaleUtils.setNewPixelScale(hasPixelScale, hasPixelScale, reqPixelScale, minPixelScale, maxPixelScale, DEBUG ? getClass().getSimpleName() : null);
   }
 
   /** @return the JAWT_DrawingSurfaceInfo's (JAWT_Rectangle) bounds, updated with lock */
   public final RectangleImmutable getBounds() { return bounds; }
 
   /** @return the safe pixelScale value for x-direction, i.e. never negative or zero. Updated with lock. */
-  protected final int getPixelScaleX() { return hasPixelScale[0]; }
+  protected final float getPixelScaleX() { return hasPixelScale[0]; }
 
   /** @return the safe pixelScale value for y-direction, i.e. never negative or zero. Updated with lock. */
-  protected final int getPixelScaleY() { return hasPixelScale[1]; }
+  protected final float getPixelScaleY() { return hasPixelScale[1]; }
 
   @Override
   public final InsetsImmutable getInsets() { return insets; }
@@ -476,7 +586,7 @@ public abstract class JAWTWindow implements NativeWindow, OffscreenLayerSurface,
   @Override
   public final void setChosenCapabilities(final CapabilitiesImmutable caps) {
       ((MutableGraphicsConfiguration)getGraphicsConfiguration()).setChosenCapabilities(caps);
-      config.setChosenCapabilities(caps);
+      awtConfig.setChosenCapabilities(caps);
   }
 
   @Override
@@ -540,7 +650,7 @@ public abstract class JAWTWindow implements NativeWindow, OffscreenLayerSurface,
    * @throws NativeWindowException
    */
   protected abstract JAWT fetchJAWTImpl() throws NativeWindowException;
-  protected abstract int lockSurfaceImpl() throws NativeWindowException;
+  protected abstract int lockSurfaceImpl(GraphicsConfiguration gc) throws NativeWindowException;
 
   protected void dumpJAWTInfo() {
       System.err.println(jawt2String(null).toString());
@@ -558,9 +668,22 @@ public abstract class JAWTWindow implements NativeWindow, OffscreenLayerSurface,
             surfaceLock.unlock();
             if(DEBUG) {
                 System.err.println("JAWTWindow: Can't lock surface, component peer n/a. Component displayable "+component.isDisplayable()+", "+component);
-                Thread.dumpStack();
+                ExceptionUtils.dumpStack(System.err);
             }
         } else {
+            final GraphicsConfiguration gc;
+            if( EventQueue.isDispatchThread() || Thread.holdsLock(component.getTreeLock()) ) {
+                /**
+                 * Trigger detection of possible reconfiguration before 'sun.awt.SunToolkit.awtLock()',
+                 * which maybe triggered via adevice.lock() below (X11).
+                 * See setAWTGraphicsConfiguration(..).
+                 */
+                gc = component.getGraphicsConfiguration();
+            } else {
+                // Reuse cached instance
+                gc = awtConfig.getAWTGraphicsConfiguration();
+            }
+
             determineIfApplet();
             try {
                 final AbstractGraphicsDevice adevice = getGraphicsConfiguration().getScreen().getDevice();
@@ -570,7 +693,7 @@ public abstract class JAWTWindow implements NativeWindow, OffscreenLayerSurface,
                         jawt = fetchJAWTImpl();
                         isOffscreenLayerSurface = JAWTUtil.isJAWTUsingOffscreenLayer(jawt);
                     }
-                    res = lockSurfaceImpl();
+                    res = lockSurfaceImpl(gc);
                     if(LOCK_SUCCESS == res && drawable_old != drawable) {
                         res = LOCK_SURFACE_CHANGED;
                         if(DEBUG) {
@@ -655,7 +778,10 @@ public abstract class JAWTWindow implements NativeWindow, OffscreenLayerSurface,
 
   @Override
   public final AbstractGraphicsConfiguration getGraphicsConfiguration() {
-    return config.getNativeGraphicsConfiguration();
+    if( null == awtConfig ) {
+        throw new NativeWindowException(jawtStr2("")+": null awtConfig, invalidated");
+    }
+    return awtConfig.getNativeGraphicsConfiguration();
   }
 
   @Override
@@ -670,26 +796,22 @@ public abstract class JAWTWindow implements NativeWindow, OffscreenLayerSurface,
 
   @Override
   public final int getSurfaceWidth() {
-    return getWidth() * getPixelScaleX();
+    return SurfaceScaleUtils.scale(getWidth(), getPixelScaleX());
   }
 
   @Override
   public final int getSurfaceHeight() {
-    return getHeight() * getPixelScaleY();
+    return SurfaceScaleUtils.scale(getHeight(), getPixelScaleY());
   }
 
   @Override
   public final int[] convertToWindowUnits(final int[] pixelUnitsAndResult) {
-      pixelUnitsAndResult[0] /= getPixelScaleX();
-      pixelUnitsAndResult[1] /= getPixelScaleY();
-      return pixelUnitsAndResult;
+      return SurfaceScaleUtils.scaleInv(pixelUnitsAndResult, pixelUnitsAndResult, hasPixelScale);
   }
 
   @Override
   public final int[] convertToPixelUnits(final int[] windowUnitsAndResult) {
-      windowUnitsAndResult[0] *= getPixelScaleX();
-      windowUnitsAndResult[1] *= getPixelScaleY();
-      return windowUnitsAndResult;
+      return SurfaceScaleUtils.scale(windowUnitsAndResult, windowUnitsAndResult, hasPixelScale);
   }
 
   @Override
@@ -714,7 +836,7 @@ public abstract class JAWTWindow implements NativeWindow, OffscreenLayerSurface,
     surfaceLock.lock();
     try {
         if(DEBUG) {
-            System.err.println(jawtStr()+".destroy @ Thread "+getThreadName());
+            System.err.println(jawtStr2("destroy"));
         }
         jawtComponentListener.detach();
         invalidate();
@@ -758,27 +880,10 @@ public abstract class JAWTWindow implements NativeWindow, OffscreenLayerSurface,
    * </p>
    */
   @Override
-  public Point getLocationOnScreen(Point storage) {
+  public Point getLocationOnScreen(final Point storage) {
       Point los = getLocationOnScreenNative(storage);
       if(null == los) {
-          if(!Thread.holdsLock(component.getTreeLock())) {
-              // avoid deadlock ..
-              if(DEBUG) {
-                  System.err.println("Warning: JAWT Lock hold, but not the AWT tree lock: "+this);
-                  Thread.dumpStack();
-              }
-              if( null == storage ) {
-                  storage = new Point();
-              }
-              getLocationOnScreenNonBlocking(storage, component);
-              return storage;
-          }
-          final java.awt.Point awtLOS = component.getLocationOnScreen();
-          if(null!=storage) {
-              los = storage.translate(awtLOS.x, awtLOS.y);
-          } else {
-              los = new Point(awtLOS.x, awtLOS.y);
-          }
+          los = AWTMisc.getLocationOnScreenSafe(storage, component, DEBUG);
       }
       return los;
   }
@@ -788,7 +893,7 @@ public abstract class JAWTWindow implements NativeWindow, OffscreenLayerSurface,
       if(LOCK_SURFACE_NOT_READY == lockRes) {
           if(DEBUG) {
               System.err.println("Warning: JAWT Lock couldn't be acquired: "+this);
-              Thread.dumpStack();
+              ExceptionUtils.dumpStack(System.err);
           }
           return null;
       }
@@ -807,35 +912,6 @@ public abstract class JAWTWindow implements NativeWindow, OffscreenLayerSurface,
   }
   protected abstract Point getLocationOnScreenNativeImpl(int x, int y);
 
-  protected static Component getLocationOnScreenNonBlocking(final Point storage, Component comp) {
-      final java.awt.Insets insets = new java.awt.Insets(0, 0, 0, 0); // DEBUG
-      Component last = null;
-      while(null != comp) {
-          final int dx = comp.getX();
-          final int dy = comp.getY();
-          if( DEBUG ) {
-              final java.awt.Insets ins = AWTMisc.getInsets(comp, false);
-              if( null != ins ) {
-                  insets.bottom += ins.bottom;
-                  insets.top += ins.top;
-                  insets.left += ins.left;
-                  insets.right += ins.right;
-              }
-              System.err.print("LOS: "+storage+" + "+comp.getClass().getName()+"["+dx+"/"+dy+", vis "+comp.isVisible()+", ins "+ins+" -> "+insets+"] -> ");
-          }
-          storage.translate(dx, dy);
-          if( DEBUG ) {
-              System.err.println(storage);
-          }
-          last = comp;
-          if( comp instanceof Window ) { // top-level heavy-weight ?
-              break;
-          }
-          comp = comp.getParent();
-      }
-      return last;
-  }
-
   @Override
   public boolean hasFocus() {
       return component.hasFocus();
@@ -864,7 +940,7 @@ public abstract class JAWTWindow implements NativeWindow, OffscreenLayerSurface,
   public String toString() {
     final StringBuilder sb = new StringBuilder();
 
-    sb.append(jawtStr()+"[");
+    sb.append(jawtStr1()+"[");
     jawt2String(sb);
     sb.append(  ", shallUseOffscreenLayer "+shallUseOffscreenLayer+", isOffscreenLayerSurface "+isOffscreenLayerSurface+
                 ", attachedSurfaceLayer "+toHexString(getAttachedSurfaceLayer())+
@@ -873,10 +949,10 @@ public abstract class JAWTWindow implements NativeWindow, OffscreenLayerSurface,
                 ", bounds "+bounds+", insets "+insets
                 );
     sb.append(", window ["+getX()+"/"+getY()+" "+getWidth()+"x"+getHeight()+
-             "], pixels[s "+getPixelScaleX()+"x"+getPixelScaleY()+" -> "+getSurfaceWidth()+"x"+getSurfaceHeight()+"]"+
+             "], pixels[scale "+getPixelScaleX()+", "+getPixelScaleY()+" -> "+getSurfaceWidth()+"x"+getSurfaceHeight()+"]"+
               ", visible "+component.isVisible());
     sb.append(", lockedExt "+isSurfaceLockedByOtherThread()+
-              ",\n\tconfig "+config+
+              ",\n\tconfig "+awtConfig+
               ",\n\tawtComponent "+getAWTComponent()+
               ",\n\tsurfaceLock "+surfaceLock+"]");
 
