@@ -37,11 +37,11 @@ import org.eclipse.swt.graphics.GCData;
 import org.eclipse.swt.graphics.Rectangle;
 import org.eclipse.swt.widgets.Control;
 
-import javax.media.nativewindow.AbstractGraphicsScreen;
-import javax.media.nativewindow.NativeWindowException;
-import javax.media.nativewindow.AbstractGraphicsDevice;
-import javax.media.nativewindow.NativeWindowFactory;
-import javax.media.nativewindow.VisualIDHolder;
+import com.jogamp.nativewindow.AbstractGraphicsScreen;
+import com.jogamp.nativewindow.NativeWindowException;
+import com.jogamp.nativewindow.AbstractGraphicsDevice;
+import com.jogamp.nativewindow.NativeWindowFactory;
+import com.jogamp.nativewindow.VisualIDHolder;
 
 import com.jogamp.common.util.ReflectionUtil;
 import com.jogamp.common.util.VersionNumber;
@@ -114,9 +114,9 @@ public class SWTAccessor {
 
     private static VersionNumber GTK_VERSION(final int version) {
         // return (major << 16) + (minor << 8) + micro;
-        final int micro = ( version       ) & 0x0f;
-        final int minor = ( version >>  8 ) & 0x0f;
-        final int major = ( version >> 16 ) & 0x0f;
+        final int micro = ( version       ) & 0xff;
+        final int minor = ( version >>  8 ) & 0xff;
+        final int major = ( version >> 16 ) & 0xff;
         return new VersionNumber(major, minor, micro);
     }
 
@@ -386,7 +386,7 @@ public class SWTAccessor {
             return new X11GraphicsDevice(xdisplay0, AbstractGraphicsDevice.DEFAULT_UNIT, false /* owner */);
         }
         if( isWindows ) {
-            return new WindowsGraphicsDevice(AbstractGraphicsDevice.DEFAULT_CONNECTION, AbstractGraphicsDevice.DEFAULT_UNIT);
+            return new WindowsGraphicsDevice(AbstractGraphicsDevice.DEFAULT_UNIT);
         }
         if( isOSX ) {
             return new MacOSXGraphicsDevice(AbstractGraphicsDevice.DEFAULT_UNIT);
