@@ -41,17 +41,17 @@
 package jogamp.nativewindow.jawt.macosx;
 
 import java.awt.Component;
+import java.awt.GraphicsConfiguration;
 import java.nio.Buffer;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
 
-import javax.media.nativewindow.AbstractGraphicsConfiguration;
-import javax.media.nativewindow.Capabilities;
-import javax.media.nativewindow.NativeSurface;
-import javax.media.nativewindow.NativeWindow;
-import javax.media.nativewindow.NativeWindowException;
-import javax.media.nativewindow.MutableSurface;
-import javax.media.nativewindow.util.Point;
+import com.jogamp.nativewindow.AbstractGraphicsConfiguration;
+import com.jogamp.nativewindow.Capabilities;
+import com.jogamp.nativewindow.NativeSurface;
+import com.jogamp.nativewindow.NativeWindowException;
+import com.jogamp.nativewindow.MutableSurface;
+import com.jogamp.nativewindow.util.Point;
 
 import com.jogamp.common.util.PropertyAccess;
 import com.jogamp.nativewindow.awt.JAWTWindow;
@@ -101,14 +101,15 @@ public class MacOSXJAWTWindow extends JAWTWindow implements MutableSurface {
           OSXUtil.RunOnMainThread(false, true /* kickNSApp */, new Runnable() {
               @Override
               public void run() {
+                  if( 0 != jawtSurfaceLayersHandle) {
+                      // null rootSurfaceLayer OK
+                      UnsetJAWTRootSurfaceLayer0(jawtSurfaceLayersHandle, rootSurfaceLayer);
+                  }
+                  jawtSurfaceLayersHandle = 0;
                   if( 0 != rootSurfaceLayer ) {
-                      if( 0 != jawtSurfaceLayersHandle) {
-                          UnsetJAWTRootSurfaceLayer0(jawtSurfaceLayersHandle, rootSurfaceLayer);
-                      }
                       OSXUtil.DestroyCALayer(rootSurfaceLayer);
                       rootSurfaceLayer = 0;
                   }
-                  jawtSurfaceLayersHandle = 0;
               }
           });
       }
@@ -116,13 +117,10 @@ public class MacOSXJAWTWindow extends JAWTWindow implements MutableSurface {
   }
 
   @Override
-  public void setSurfaceScale(final int[] pixelScale) {
+  public boolean setSurfaceScale(final float[] pixelScale) {
       super.setSurfaceScale(pixelScale);
-      if( 0 != getWindowHandle() ) { // locked at least once !
-          final int hadPixelScaleX = getPixelScaleX();
-          updatePixelScale();
-
-          if( hadPixelScaleX != getPixelScaleX() && 0 != getAttachedSurfaceLayer() ) {
+      if( 0 != getWindowHandle() && setReqPixelScale() ) { // locked at least once _and_ updated pixel-scale
+          if( 0 != getAttachedSurfaceLayer() ) {
               OSXUtil.RunOnMainThread(false, false, new Runnable() {
                   @Override
                   public void run() {
@@ -133,6 +131,9 @@ public class MacOSXJAWTWindow extends JAWTWindow implements MutableSurface {
                   }
               });
           }
+          return true;
+      } else {
+          return false;
       }
   }
 
@@ -146,7 +147,7 @@ public class MacOSXJAWTWindow extends JAWTWindow implements MutableSurface {
                   // CALayer position will be determined in native code.
                   // See detailed description in {@link JAWTUtil#JAWT_OSX_CALAYER_QUIRK_LAYOUT}
                   final Point p0 = new Point();
-                  final Component outterComp = getLocationOnScreenNonBlocking(p0, component);
+                  final Component outterComp = AWTMisc.getLocationOnScreenNonBlocking(p0, component, DEBUG);
                   final java.awt.Insets outterInsets = AWTMisc.getInsets(outterComp, true);
                   final Point p1 = (Point)p0.cloneMutable();
                   p1.translate(-outterComp.getX(), -outterComp.getY());
@@ -179,7 +180,7 @@ public class MacOSXJAWTWindow extends JAWTWindow implements MutableSurface {
       // CALayer position will be determined in native code.
       // See detailed description in {@link JAWTUtil#JAWT_OSX_CALAYER_QUIRK_LAYOUT}
       final Point p0 = new Point();
-      final Component outterComp = getLocationOnScreenNonBlocking(p0, component);
+      final Component outterComp = AWTMisc.getLocationOnScreenNonBlocking(p0, component, DEBUG);
       final java.awt.Insets outterInsets = AWTMisc.getInsets(outterComp, true);
       final Point p1 = (Point)p0.cloneMutable();
       p1.translate(-outterComp.getX(), -outterComp.getY());
@@ -243,7 +244,7 @@ public class MacOSXJAWTWindow extends JAWTWindow implements MutableSurface {
   }
 
   @Override
-  protected int lockSurfaceImpl() throws NativeWindowException {
+  protected int lockSurfaceImpl(final GraphicsConfiguration gc) throws NativeWindowException {
     int ret = NativeSurface.LOCK_SURFACE_NOT_READY;
     ds = getJAWT().GetDrawingSurface(component);
     if (ds == null) {
@@ -280,7 +281,7 @@ public class MacOSXJAWTWindow extends JAWTWindow implements MutableSurface {
       unlockSurfaceImpl();
       return NativeSurface.LOCK_SURFACE_NOT_READY;
     }
-    updateLockedData(dsi.getBounds());
+    updateLockedData(dsi.getBounds(), gc);
     if (DEBUG && firstLock ) {
       dumpInfo();
     }
@@ -406,7 +407,7 @@ public class MacOSXJAWTWindow extends JAWTWindow implements MutableSurface {
       if( null == storage ) {
           storage = new Point();
       }
-      getLocationOnScreenNonBlocking(storage, component);
+      AWTMisc.getLocationOnScreenNonBlocking(storage, component, DEBUG);
       return storage;
   }
   @Override
