@@ -27,13 +27,14 @@
  */
 package jogamp.nativewindow.macosx;
 
-import javax.media.nativewindow.NativeWindowException;
-import javax.media.nativewindow.NativeWindowFactory;
-import javax.media.nativewindow.util.Insets;
-import javax.media.nativewindow.util.Point;
+import com.jogamp.nativewindow.NativeWindowException;
+import com.jogamp.nativewindow.NativeWindowFactory;
+import com.jogamp.nativewindow.util.Insets;
+import com.jogamp.nativewindow.util.Point;
 
 import com.jogamp.common.util.Function;
 import com.jogamp.common.util.FunctionTask;
+import com.jogamp.common.util.InterruptedRuntimeException;
 import com.jogamp.common.util.RunnableTask;
 
 import jogamp.nativewindow.Debug;
@@ -107,12 +108,19 @@ public class OSXUtil implements ToolkitProperties {
       return (Insets) GetInsets0(windowOrView);
     }
 
-    public static double GetPixelScale(final int screenIndex) {
-      return GetPixelScale0(screenIndex);
+    public static double GetPixelScaleByDisplayID(final int displayID) {
+      if( 0 != displayID ) {
+          return GetPixelScale1(displayID);
+      } else {
+          return 1.0; // default
+      }
     }
-
     public static double GetPixelScale(final long windowOrView) {
-      return GetPixelScale1(windowOrView);
+      if( 0 != windowOrView ) {
+          return GetPixelScale2(windowOrView);
+      } else {
+          return 1.0; // default
+      }
     }
 
     public static long CreateNSWindow(final int x, final int y, final int width, final int height) {
@@ -260,22 +268,21 @@ public class OSXUtil implements ToolkitProperties {
         } else {
             // Utilize Java side lock/wait and simply pass the Runnable async to OSX main thread,
             // otherwise we may freeze the OSX main thread.
-            Throwable throwable = null;
             final Object sync = new Object();
             final RunnableTask rt = new RunnableTask( runnable, waitUntilDone ? sync : null, true, waitUntilDone ? null : System.err );
             synchronized(sync) {
                 RunOnMainThread0(kickNSApp, rt);
                 if( waitUntilDone ) {
-                    try {
-                        sync.wait();
-                    } catch (final InterruptedException ie) {
-                        throwable = ie;
-                    }
-                    if(null==throwable) {
-                        throwable = rt.getThrowable();
-                    }
-                    if(null!=throwable) {
-                        throw new RuntimeException(throwable);
+                    while( rt.isInQueue() ) {
+                        try {
+                            sync.wait();
+                        } catch (final InterruptedException ie) {
+                            throw new InterruptedRuntimeException(ie);
+                        }
+                        final Throwable throwable = rt.getThrowable();
+                        if(null!=throwable) {
+                            throw new RuntimeException(throwable);
+                        }
                     }
                 }
             }
@@ -334,23 +341,22 @@ public class OSXUtil implements ToolkitProperties {
         } else {
             // Utilize Java side lock/wait and simply pass the Runnable async to OSX main thread,
             // otherwise we may freeze the OSX main thread.
-            Throwable throwable = null;
             final Object sync = new Object();
             final FunctionTask<R,A> rt = new FunctionTask<R,A>( func, waitUntilDone ? sync : null, true, waitUntilDone ? null : System.err );
             synchronized(sync) {
                 rt.setArgs(args);
                 RunOnMainThread0(kickNSApp, rt);
                 if( waitUntilDone ) {
-                    try {
-                        sync.wait();
-                    } catch (final InterruptedException ie) {
-                        throwable = ie;
-                    }
-                    if(null==throwable) {
-                        throwable = rt.getThrowable();
-                    }
-                    if(null!=throwable) {
-                        throw new RuntimeException(throwable);
+                    while( rt.isInQueue() ) {
+                        try {
+                            sync.wait();
+                        } catch (final InterruptedException ie) {
+                            throw new InterruptedRuntimeException(ie);
+                        }
+                        final Throwable throwable = rt.getThrowable();
+                        if(null!=throwable) {
+                            throw new RuntimeException(throwable);
+                        }
                     }
                 }
             }
@@ -393,8 +399,8 @@ public class OSXUtil implements ToolkitProperties {
     private static native boolean isNSWindow0(long object);
     private static native Object GetLocationOnScreen0(long windowOrView, int src_x, int src_y);
     private static native Object GetInsets0(long windowOrView);
-    private static native double GetPixelScale0(int screenIndex);
-    private static native double GetPixelScale1(long windowOrView);
+    private static native double GetPixelScale1(int displayID);
+    private static native double GetPixelScale2(long windowOrView);
     private static native long CreateNSWindow0(int x, int y, int width, int height);
     private static native void DestroyNSWindow0(long nsWindow);
     private static native long GetNSView0(long nsWindow);
