@@ -37,14 +37,15 @@ import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
 
-import javax.media.nativewindow.AbstractGraphicsDevice;
-import javax.media.nativewindow.NativeWindowException;
-import javax.media.nativewindow.NativeWindowFactory;
+import com.jogamp.nativewindow.AbstractGraphicsDevice;
+import com.jogamp.nativewindow.NativeWindowException;
+import com.jogamp.nativewindow.NativeWindowFactory;
 
 import jogamp.nativewindow.Debug;
 import jogamp.nativewindow.NWJNILibLoader;
 import jogamp.nativewindow.ToolkitProperties;
 
+import com.jogamp.common.ExceptionUtils;
 import com.jogamp.common.util.LongObjectHashMap;
 import com.jogamp.common.util.PropertyAccess;
 import com.jogamp.nativewindow.x11.X11GraphicsDevice;
@@ -196,7 +197,7 @@ public class X11Util implements ToolkitProperties {
                                            ", pending (open in creation order): "+pendingDisplayList.size()+
                                            ")");
                         if(DEBUG) {
-                            Thread.dumpStack();
+                            ExceptionUtils.dumpStack(System.err);
                         }
                         if( openDisplayList.size() > 0) {
                             X11Util.dumpOpenDisplayConnections();
@@ -615,7 +616,7 @@ public class X11Util implements ToolkitProperties {
     }
 
     private static final String getCurrentThreadName() { return Thread.currentThread().getName(); } // Callback for JNI
-    private static final void dumpStack() { Thread.dumpStack(); } // Callback for JNI
+    private static final void dumpStack() { ExceptionUtils.dumpStack(System.err); } // Callback for JNI
 
     private static native boolean initialize0(boolean debug);
     private static native void shutdown0();
