@@ -65,12 +65,12 @@ static jmethodID runnableRunID = NULL;
 
 static const char * const ClazzAnyCstrName = "<init>";
 
-static const char * const ClazzNamePoint = "javax/media/nativewindow/util/Point";
+static const char * const ClazzNamePoint = "com/jogamp/nativewindow/util/Point";
 static const char * const ClazzNamePointCstrSignature = "(II)V";
 static jclass pointClz = NULL;
 static jmethodID pointCstr = NULL;
 
-static const char * const ClazzNameInsets = "javax/media/nativewindow/util/Insets";
+static const char * const ClazzNameInsets = "com/jogamp/nativewindow/util/Insets";
 static const char * const ClazzNameInsetsCstrSignature = "(IIII)V";
 static jclass insetsClz = NULL;
 static jmethodID insetsCstr = NULL;
@@ -138,17 +138,32 @@ Java_jogamp_nativewindow_macosx_OSXUtil_isNSWindow0(JNIEnv *env, jclass _unused,
 }
 
 static CGDirectDisplayID OSXUtil_getCGDirectDisplayIDByNSScreen(NSScreen *screen) {
-    // Mind: typedef uint32_t CGDirectDisplayID; - however, we assume it's 64bit on 64bit ?!
+    if( NULL == screen ) {
+        return (CGDirectDisplayID)0;
+    }
+    // Mind: typedef uint32_t CGDirectDisplayID;
     NSDictionary * dict = [screen deviceDescription];
     NSNumber * val = (NSNumber *) [dict objectForKey: @"NSScreenNumber"];
     // [NSNumber integerValue] returns NSInteger which is 32 or 64 bit native size
     return (CGDirectDisplayID) [val integerValue];
 }
+static NSScreen * OSXUtil_getNSScreenByCGDirectDisplayID(CGDirectDisplayID displayID) {
+    NSArray *screens = [NSScreen screens];
+    int i;
+    for(i=[screens count]-1; i>=0; i--) {
+        NSScreen * screen = (NSScreen *) [screens objectAtIndex: i];
+        CGDirectDisplayID dID = OSXUtil_getCGDirectDisplayIDByNSScreen(screen);
+        if( dID == displayID ) {
+            return screen;
+        }
+    }
+    return NULL;
+}
 
 /*
  * Class:     Java_jogamp_nativewindow_macosx_OSXUtil
  * Method:    getLocationOnScreen0
- * Signature: (JII)Ljavax/media/nativewindow/util/Point;
+ * Signature: (JII)Lcom/jogamp/nativewindow/util/Point;
  */
 JNIEXPORT jobject JNICALL Java_jogamp_nativewindow_macosx_OSXUtil_GetLocationOnScreen0
   (JNIEnv *env, jclass unused, jlong winOrView, jint src_x, jint src_y)
@@ -209,7 +224,7 @@ JNIEXPORT jobject JNICALL Java_jogamp_nativewindow_macosx_OSXUtil_GetLocationOnS
 /*
  * Class:     Java_jogamp_nativewindow_macosx_OSXUtil
  * Method:    getInsets0
- * Signature: (J)Ljavax/media/nativewindow/util/Insets;
+ * Signature: (J)Lcom/jogamp/nativewindow/util/Insets;
  */
 JNIEXPORT jobject JNICALL Java_jogamp_nativewindow_macosx_OSXUtil_GetInsets0
   (JNIEnv *env, jclass unused, jlong winOrView)
@@ -251,23 +266,17 @@ JNIEXPORT jobject JNICALL Java_jogamp_nativewindow_macosx_OSXUtil_GetInsets0
 
 /*
  * Class:     Java_jogamp_nativewindow_macosx_OSXUtil
- * Method:    GetPixelScale0
+ * Method:    GetPixelScale1
  * Signature: (I)D
  */
-JNIEXPORT jdouble JNICALL Java_jogamp_nativewindow_macosx_OSXUtil_GetPixelScale0
-  (JNIEnv *env, jclass unused, jint screen_idx)
+JNIEXPORT jdouble JNICALL Java_jogamp_nativewindow_macosx_OSXUtil_GetPixelScale1
+  (JNIEnv *env, jclass unused, jint displayID)
 {
     NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
 
-    CGFloat pixelScale;
-    NSScreen *screen;
-    NSArray *screens = [NSScreen screens];
-    if( screen_idx<0 || screen_idx>=[screens count] ) {
-        screen = NULL;
-        pixelScale = 0.0;
-    } else {
-        screen = (NSScreen *) [screens objectAtIndex: screen_idx];
-        pixelScale = 1.0; // default
+    CGFloat pixelScale = 1.0; // default
+    NSScreen *screen =  OSXUtil_getNSScreenByCGDirectDisplayID((CGDirectDisplayID)displayID);
+    if( NULL != screen ) {
 NS_DURING
         // Available >= 10.7
         pixelScale = [screen backingScaleFactor]; // HiDPI scaling
@@ -284,7 +293,7 @@ NS_ENDHANDLER
  * Method:    GetPixelScale1
  * Signature: (J)D
  */
-JNIEXPORT jdouble JNICALL Java_jogamp_nativewindow_macosx_OSXUtil_GetPixelScale1
+JNIEXPORT jdouble JNICALL Java_jogamp_nativewindow_macosx_OSXUtil_GetPixelScale2
   (JNIEnv *env, jclass unused, jlong winOrView)
 {
     NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
@@ -852,8 +861,12 @@ JNIEXPORT jlong JNICALL Java_jogamp_nativewindow_jawt_macosx_MacOSXJAWTWindow_Ge
         NativewindowCommon_throwNewRuntimeException(env, "Argument \"jawtDrawingSurfaceInfoBuffer\" was not a direct buffer");
         return 0;
     }
-    id <JAWT_SurfaceLayers> surfaceLayers = (id <JAWT_SurfaceLayers>)dsi->platformInfo;
-    return (jlong) ((intptr_t) surfaceLayers);
+    NSObject *surfaceLayersObj = (NSObject*) dsi->platformInfo;
+    [surfaceLayersObj retain]; // Pairs w/ Unset
+    DBG_PRINT("CALayer::GetJAWTSurfaceLayersHandle: surfaceLayers %p (refcnt %d)\n", surfaceLayersObj, (int)[surfaceLayersObj retainCount]);
+
+    id <JAWT_SurfaceLayers> surfaceLayers = (id <JAWT_SurfaceLayers>)surfaceLayersObj;
+    return (jlong) (intptr_t) surfaceLayers;
 }
 
 /*
@@ -869,9 +882,11 @@ JNIEXPORT void JNICALL Java_jogamp_nativewindow_jawt_macosx_MacOSXJAWTWindow_Set
     [CATransaction begin];
     [CATransaction setValue:(id)kCFBooleanTrue forKey:kCATransactionDisableActions];
 
-    id <JAWT_SurfaceLayers> surfaceLayers = (id <JAWT_SurfaceLayers>)(intptr_t)jawtSurfaceLayersHandle;
+    NSObject *surfaceLayersObj = (NSObject*) (intptr_t) jawtSurfaceLayersHandle;
+    id <JAWT_SurfaceLayers> surfaceLayers = (id <JAWT_SurfaceLayers>)surfaceLayersObj;
     MyCALayer* layer = (MyCALayer*) (intptr_t) caLayer;
     DBG_PRINT("CALayer::SetJAWTRootSurfaceLayer.0: pre %p -> root %p (refcnt %d)\n", [surfaceLayers layer], layer, (int)[layer retainCount]);
+
     [surfaceLayers setLayer: [layer retain]]; // Pairs w/ Unset
 
     [CATransaction commit];
@@ -893,20 +908,26 @@ JNIEXPORT void JNICALL Java_jogamp_nativewindow_jawt_macosx_MacOSXJAWTWindow_Uns
     [CATransaction begin];
     [CATransaction setValue:(id)kCFBooleanTrue forKey:kCATransactionDisableActions];
 
-    id <JAWT_SurfaceLayers> surfaceLayers = (id <JAWT_SurfaceLayers>)(intptr_t)jawtSurfaceLayersHandle;
+    NSObject *surfaceLayersObj = (NSObject*) (intptr_t) jawtSurfaceLayersHandle;
+    id <JAWT_SurfaceLayers> surfaceLayers = (id <JAWT_SurfaceLayers>)surfaceLayersObj;
+    DBG_PRINT("CALayer::UnsetJAWTRootSurfaceLayer.0: surfaceLayers %p (refcnt %d)\n", surfaceLayersObj, (int)[surfaceLayersObj retainCount]);
+
     MyCALayer* layer = (MyCALayer*) (intptr_t) caLayer;
-    if(layer != [surfaceLayers layer]) {
-        NativewindowCommon_throwNewRuntimeException(env, "Attached layer %p doesn't match given layer %p\n", surfaceLayers.layer, layer);
-        return;
+    if(NULL != layer) {
+        if(layer != [surfaceLayers layer]) {
+            NativewindowCommon_throwNewRuntimeException(env, "Attached layer %p doesn't match given layer %p\n", surfaceLayers.layer, layer);
+            return;
+        }
+        DBG_PRINT("CALayer::UnsetJAWTRootSurfaceLayer.1: root %p (refcnt %d) -> nil\n", layer, (int)[layer retainCount]);
+        [layer release]; // Pairs w/ Set
+        [surfaceLayers setLayer: NULL]; // Pairs w/ Set
     }
-    DBG_PRINT("CALayer::UnsetJAWTRootSurfaceLayer.0: root %p (refcnt %d) -> nil\n", layer, (int)[layer retainCount]);
-    [layer release]; // Pairs w/ Set
-    [surfaceLayers setLayer: NULL];
+    [surfaceLayersObj release]; // Pairs w/ Get
 
     [CATransaction commit];
 
     [pool release];
-    DBG_PRINT("CALayer::UnsetJAWTRootSurfaceLayer.X: root %p (refcnt %d) -> nil\n", layer, (int)[layer retainCount]);
+    DBG_PRINT("CALayer::UnsetJAWTRootSurfaceLayer.X\n");
 }
 
 @interface MainRunnable : NSObject
