@@ -38,12 +38,12 @@ import java.security.AccessController;
 import java.security.PrivilegedAction;
 import java.util.Arrays;
 
-import javax.media.nativewindow.AbstractGraphicsConfiguration;
-import javax.media.nativewindow.AbstractGraphicsDevice;
-import javax.media.nativewindow.AbstractGraphicsScreen;
-import javax.media.nativewindow.CapabilitiesImmutable;
-import javax.media.nativewindow.NativeWindow;
-import javax.media.nativewindow.NativeWindowFactory;
+import com.jogamp.nativewindow.AbstractGraphicsConfiguration;
+import com.jogamp.nativewindow.AbstractGraphicsDevice;
+import com.jogamp.nativewindow.AbstractGraphicsScreen;
+import com.jogamp.nativewindow.CapabilitiesImmutable;
+import com.jogamp.nativewindow.NativeWindow;
+import com.jogamp.nativewindow.NativeWindowFactory;
 
 import com.jogamp.common.util.IOUtil;
 import com.jogamp.common.util.PropertyAccess;
@@ -59,6 +59,7 @@ public class NewtFactory {
     public static final String DRIVER_DEFAULT_ROOT_PACKAGE = "jogamp.newt.driver";
 
     private static IOUtil.ClassResources defaultWindowIcons;
+    private static String sysPaths = "newt/data/jogamp-16x16.png newt/data/jogamp-32x32.png";
 
     static {
         AccessController.doPrivileged(new PrivilegedAction<Object>() {
@@ -67,12 +68,11 @@ public class NewtFactory {
                 NativeWindowFactory.initSingleton(); // last resort ..
                 {
                     /** See API Doc in {@link Window} ! */
-                    final String[] paths = PropertyAccess.getProperty("newt.window.icons", true, "newt/data/jogamp-16x16.png newt/data/jogamp-32x32.png").split("\\s");
+                    final String[] paths = PropertyAccess.getProperty("newt.window.icons", true, sysPaths).split("[\\s,]");
                     if( paths.length < 2 ) {
                         throw new IllegalArgumentException("Property 'newt.window.icons' did not specify at least two PNG icons, but "+Arrays.toString(paths));
                     }
-                    final Class<?> clazz = NewtFactory.class;
-                    defaultWindowIcons = new IOUtil.ClassResources(clazz, paths);
+                    defaultWindowIcons = new IOUtil.ClassResources(paths, NewtFactory.class.getClassLoader(), null);
                 }
                 return null;
             } } );
@@ -81,7 +81,7 @@ public class NewtFactory {
     /**
      * Returns the application window icon resources to be used.
      * <p>
-     * Property <code>newt.window.icons</code> may define a list of PNG icons separated by a whitespace character.
+     * Property <code>newt.window.icons</code> may define a list of PNG icons separated by one whitespace or one comma character.
      * Shall reference at least two PNG icons, from lower (16x16) to higher (>= 32x32) resolution.
      * </p>
      * <p>
@@ -260,7 +260,7 @@ public class NewtFactory {
      * you have to add an appropriate {@link com.jogamp.newt.event.WindowListener} for this use case.<br>
      * The parents visibility is passed to the new Window<br></p>
      * <p>
-     * In case <code>parentWindowObject</code> is a different {@link javax.media.nativewindow.NativeWindow} implementation,<br>
+     * In case <code>parentWindowObject</code> is a different {@link com.jogamp.nativewindow.NativeWindow} implementation,<br>
      * you have to handle all events appropriate.<br></p>
      * <p>
      * <p>
