@@ -42,14 +42,14 @@ import com.jogamp.newt.event.MouseListener;
 import jogamp.newt.Debug;
 import jogamp.newt.WindowImpl;
 
-import javax.media.nativewindow.CapabilitiesChooser;
-import javax.media.nativewindow.CapabilitiesImmutable;
-import javax.media.nativewindow.NativeWindow;
-import javax.media.nativewindow.ScalableSurface;
-import javax.media.nativewindow.WindowClosingProtocol;
-import javax.media.nativewindow.util.Rectangle;
-import javax.media.nativewindow.util.RectangleImmutable;
-import javax.media.nativewindow.util.SurfaceSize;
+import com.jogamp.nativewindow.CapabilitiesChooser;
+import com.jogamp.nativewindow.CapabilitiesImmutable;
+import com.jogamp.nativewindow.NativeWindow;
+import com.jogamp.nativewindow.ScalableSurface;
+import com.jogamp.nativewindow.WindowClosingProtocol;
+import com.jogamp.nativewindow.util.Rectangle;
+import com.jogamp.nativewindow.util.RectangleImmutable;
+import com.jogamp.nativewindow.util.SurfaceSize;
 
 /**
  * Specifying NEWT's Window functionality:
@@ -83,17 +83,34 @@ import javax.media.nativewindow.util.SurfaceSize;
  * <a name="customwindowicons"><h5>Custom Window Icons</h5></a>
  * <p>
  * Custom window icons can be defined via system property <code>newt.window.icons</code>,
- * which shall contain a space separated list of PNG icon locations from low- to high-resolution.
+ * which shall contain a list of PNG icon locations from low- to high-resolution,
+ * separated by one whitespace or one comma character.
  * The location must be resolvable via classpath, i.e. shall reference a location within the jar file.
  * Example (our default):
  * <pre>
- *   -Dnewt.window.icons="newt/data/jogamp-16x16.png newt/data/jogamp-32x32.png"
- *   -Djnlp.newt.window.icons="newt/data/jogamp-16x16.png newt/data/jogamp-32x32.png"
+ *   -Dnewt.window.icons="newt/data/jogamp-16x16.png,newt/data/jogamp-32x32.png"
+ *   -Djnlp.newt.window.icons="newt/data/jogamp-16x16.png,newt/data/jogamp-32x32.png"
  * </pre>
  * The property can also be set programmatically, which must happen before any NEWT classes are <i>touched</i>:
  * <pre>
- *   System.setProperty("newt.window.icons", "newt/data/jogamp-16x16.png newt/data/jogamp-32x32.png");
+ *   System.setProperty("newt.window.icons", "newt/data/jogamp-16x16.png, newt/data/jogamp-32x32.png");
  * </pre>
+ * To disable even Jogamp's own window icons in favor of system icons,
+ * simply set a non-existing location, e.g.:
+ * <pre>
+ *   -Dnewt.window.icons="null,null"
+ * </pre>
+ * </p>
+ *
+ * <a name="lifecycleHeavy"><h5>Use of Lifecycle Heavy functions</h5></a>
+ * <p>
+ * Some of the methods specified here are lifecycle-heavy. That is, they are able
+ * to destroy and/or reattach resources to/from the window. Because of this, the methods
+ * are <i>not safe</i> to be called from EDT related threads. For example, it is not
+ * safe for a method in an attached {@link KeyListener} to call {@link #setFullscreen(boolean)}
+ * on a {@link Window} directly. It is safe, however, for that method to spawn a background
+ * worker thread which calls the method directly. The documentation for individual methods
+ * indicates whether or not they are lifecycle-heavy.
  * </p>
  */
 public interface Window extends NativeWindow, WindowClosingProtocol, ScalableSurface {
@@ -104,6 +121,293 @@ public interface Window extends NativeWindow, WindowClosingProtocol, ScalableSur
     /** A 1s timeout while waiting for a native action response, ie {@link #setVisible(boolean)}. */
     public static final long TIMEOUT_NATIVEWINDOW = 1000;
 
+    //
+    // States (keep in sync w/ src/newt/native/Window.h)
+    //
+    /**
+     * Visibility of this instance.
+     * <p>Native instance gets created at first visibility, following NEWT's lazy creation pattern.</p>
+     * <p>Changing this state is <a href="#lifecycleHeavy">lifecycle heavy</a>.</p>
+     * <p>Bit number {@value}.</p>
+     * <p>Defaults to {@code false}.</p>
+     * @see #getStateMask()
+     * @since 2.3.2
+     */
+    public static final int STATE_BIT_VISIBLE = 0; // reconfig-flag
+    /**
+     * Hinting that no custom position has been set before first {@link #STATE_BIT_VISIBLE visibility} of this instance.
+     * <p>If kept {@code false} at creation, this allows the WM to choose the top-level window position,
+     * otherwise the custom position is being enforced.</p>
+     * <p>Bit number {@value}.</p>
+     * <p>Defaults to {@code true}.</p>
+     * @see #getStateMask()
+     * @since 2.3.2
+     */
+    public static final int STATE_BIT_AUTOPOSITION = 1;
+    /**
+     * Set if window is a <i>child window</i>, i.e. has been {@link #reparentWindow(NativeWindow, int, int, int) reparented}.
+     * <p>
+     * Otherwise bit is cleared, i.e. window is <i>top-level</i>.
+     * </p>
+     * <p>Changing this state is <a href="#lifecycleHeavy">lifecycle heavy</a>.</p>
+     * <p>Bit number {@value}.</p>
+     * <p>Defaults to {@code false}.</p>
+     * @see #getStateMask()
+     * @since 2.3.2
+     */
+    public static final int STATE_BIT_CHILDWIN = 2;       // reconfig-flag
+    /**
+     * Set if window has <i>the input focus</i>, otherwise cleared.
+     * <p>Bit number {@value}.</p>
+     * <p>Defaults to {@code false}.</p>
+     * @see #getStateMask()
+     * @since 2.3.2
+     */
+    public static final int STATE_BIT_FOCUSED = 3;
+    /**
+     * Set if window has <i>window decorations</i>, otherwise cleared.
+     * <p>Bit number {@value}.</p>
+     * <p>Defaults to {@code false}.</p>
+     * @see #getStateMask()
+     * @since 2.3.2
+     */
+    public static final int STATE_BIT_UNDECORATED = 4;    // reconfig-flag
+    /**
+     * Set if window is <i>always on top</i>, otherwise cleared.
+     * <p>Bit number {@value}.</p>
+     * <p>Defaults to {@code false}.</p>
+     * @see #getStateMask()
+     * @since 2.3.2
+     */
+    public static final int STATE_BIT_ALWAYSONTOP = 5;    // reconfig-flag
+    /**
+     * Set if window is <i>always on bottom</i>, otherwise cleared.
+     * <p>Bit number {@value}.</p>
+     * <p>Defaults to {@code false}.</p>
+     * @see #getStateMask()
+     * @since 2.3.2
+     */
+    public static final int STATE_BIT_ALWAYSONBOTTOM = 6; // reconfig-flag
+    /**
+     * Set if window is <i>sticky</i>, i.e. visible <i>on all virtual desktop</i>, otherwise cleared.
+     * <p>Bit number {@value}.</p>
+     * <p>Defaults to {@code false}.</p>
+     * @see #getStateMask()
+     * @since 2.3.2
+     */
+    public static final int STATE_BIT_STICKY = 7;    // reconfig-flag
+    /**
+     * Set if window is <i>resizable</i>, otherwise cleared.
+     * <p>Bit number {@value}.</p>
+     * <p>Defaults to {@code true}.</p>
+     * @see #getStateMask()
+     * @since 2.3.2
+     */
+    public static final int STATE_BIT_RESIZABLE = 8; // reconfig-flag
+    /**
+     * Set if window is <i>maximized vertically</i>, otherwise cleared.
+     * <p>Bit number {@value}.</p>
+     * <p>Defaults to {@code false}.</p>
+     * @see #getStateMask()
+     * @since 2.3.2
+     */
+    public static final int STATE_BIT_MAXIMIZED_VERT = 9; // reconfig-flag
+    /**
+     * Set if window is <i>maximized horizontally</i>, otherwise cleared.
+     * <p>Bit number {@value}.</p>
+     * <p>Defaults to {@code false}.</p>
+     * @see #getStateMask()
+     * @since 2.3.2
+     */
+    public static final int STATE_BIT_MAXIMIZED_HORZ = 10; // reconfig-flag
+    /**
+     * Set if window is in <i>fullscreen mode</i>, otherwise cleared.
+     * <p>
+     * Usually fullscreen mode implies {@link #STATE_BIT_UNDECORATED},
+     * however, an implementation is allowed to ignore this if unavailable.
+     * </p>
+     * <p>Bit number {@value}.</p>
+     * <p>Defaults to {@code false}.</p>
+     * @see #getStateMask()
+     * @since 2.3.2
+     */
+    public static final int STATE_BIT_FULLSCREEN = 11;    // reconfig-flag
+
+    /**
+     * Set if the <i>pointer is visible</i> when inside the window, otherwise cleared.
+     * <p>Bit number {@value}.</p>
+     * <p>Defaults to {@code true}.</p>
+     * @see #getStateMask()
+     * @since 2.3.2
+     */
+    public static final int STATE_BIT_POINTERVISIBLE = 12;
+    /**
+     * Set if the <i>pointer is confined</i> to the window, otherwise cleared.
+     * <p>Bit number {@value}.</p>
+     * <p>Defaults to {@code false}.</p>
+     * @see #getStateMask()
+     * @since 2.3.2
+     */
+    public static final int STATE_BIT_POINTERCONFINED = 13;
+
+    /**
+     * Bitmask for {@link #STATE_BIT_VISIBLE}, {@value}.
+     * @since 2.3.2
+     */
+    public static final int STATE_MASK_VISIBLE = 1 << STATE_BIT_VISIBLE;
+    /**
+     * Bitmask for {@link #STATE_BIT_AUTOPOSITION}, {@value}.
+     * @since 2.3.2
+     */
+    public static final int STATE_MASK_AUTOPOSITION = 1 << STATE_BIT_AUTOPOSITION;
+    /**
+     * Bitmask for {@link #STATE_BIT_CHILDWIN}, {@value}.
+     * @since 2.3.2
+     */
+    public static final int STATE_MASK_CHILDWIN = 1 << STATE_BIT_CHILDWIN;
+    /**
+     * Bitmask for {@link #STATE_BIT_FOCUSED}, {@value}.
+     * @since 2.3.2
+     */
+    public static final int STATE_MASK_FOCUSED = 1 << STATE_BIT_FOCUSED;
+    /**
+     * Bitmask for {@link #STATE_BIT_UNDECORATED}, {@value}.
+     * @since 2.3.2
+     */
+    public static final int STATE_MASK_UNDECORATED = 1 << STATE_BIT_UNDECORATED;
+    /**
+     * Bitmask for {@link #STATE_BIT_ALWAYSONTOP}, {@value}.
+     * @since 2.3.2
+     */
+    public static final int STATE_MASK_ALWAYSONTOP = 1 << STATE_BIT_ALWAYSONTOP;
+    /**
+     * Bitmask for {@link #STATE_BIT_ALWAYSONBOTTOM}, {@value}.
+     * @since 2.3.2
+     */
+    public static final int STATE_MASK_ALWAYSONBOTTOM = 1 << STATE_BIT_ALWAYSONBOTTOM;
+    /**
+     * Bitmask for {@link #STATE_BIT_STICKY}, {@value}.
+     * @since 2.3.2
+     */
+    public static final int STATE_MASK_STICKY = 1 << STATE_BIT_STICKY;
+    /**
+     * Bitmask for {@link #STATE_BIT_RESIZABLE}, {@value}.
+     * @since 2.3.2
+     */
+    public static final int STATE_MASK_RESIZABLE = 1 << STATE_BIT_RESIZABLE;
+    /**
+     * Bitmask for {@link #STATE_BIT_MAXIMIZED_VERT}, {@value}.
+     * @since 2.3.2
+     */
+    public static final int STATE_MASK_MAXIMIZED_VERT = 1 << STATE_BIT_MAXIMIZED_VERT;
+    /**
+     * Bitmask for {@link #STATE_BIT_MAXIMIZED_HORZ}, {@value}.
+     * @since 2.3.2
+     */
+    public static final int STATE_MASK_MAXIMIZED_HORZ = 1 << STATE_BIT_MAXIMIZED_HORZ;
+    /**
+     * Bitmask for {@link #STATE_BIT_FULLSCREEN}, {@value}.
+     * @since 2.3.2
+     */
+    public static final int STATE_MASK_FULLSCREEN = 1 << STATE_BIT_FULLSCREEN;
+    /**
+     * Bitmask for {@link #STATE_BIT_POINTERVISIBLE}, {@value}.
+     * @since 2.3.2
+     */
+    public static final int STATE_MASK_POINTERVISIBLE = 1 << STATE_BIT_POINTERVISIBLE;
+    /**
+     * Bitmask for {@link #STATE_BIT_POINTERCONFINED}, {@value}.
+     * @since 2.3.2
+     */
+    public static final int STATE_MASK_POINTERCONFINED = 1 << STATE_BIT_POINTERCONFINED;
+
+    /**
+     * Number of all public state bits.
+     * @see #getStateMask()
+     * @since 2.3.2
+     */
+    public int getStatePublicBitCount();
+
+    /**
+     * Bitmask covering all public state bits.
+     * @see #getStateMask()
+     * @since 2.3.2
+     */
+    public int getStatePublicBitmask();
+
+    /**
+     * Returns the current status mask of this instance.
+     * @see #getSupportedStateMask()
+     * @see #STATE_MASK_VISIBLE
+     * @see #STATE_MASK_AUTOPOSITION
+     * @see #STATE_MASK_CHILDWIN
+     * @see #STATE_MASK_FOCUSED
+     * @see #STATE_MASK_UNDECORATED
+     * @see #STATE_MASK_ALWAYSONTOP
+     * @see #STATE_MASK_ALWAYSONBOTTOM
+     * @see #STATE_MASK_STICKY
+     * @see #STATE_MASK_RESIZABLE
+     * @see #STATE_MASK_MAXIMIZED_VERT
+     * @see #STATE_MASK_MAXIMIZED_HORZ
+     * @see #STATE_MASK_FULLSCREEN
+     * @see #STATE_MASK_POINTERVISIBLE
+     * @see #STATE_MASK_POINTERCONFINED
+     * @since 2.3.2
+     */
+    int getStateMask();
+
+    /**
+     * Returns a string representation of the {@link #getStateMask() current state mask}.
+     * @since 2.3.2
+     */
+    String getStateMaskString();
+
+    /**
+     * Returns the supported {@link #getStateMask() state mask} of the implementation.
+     * <p>
+     * Implementation provides supported {@link #getStateMask() state mask} values at runtime
+     * <i>after</i> native window creation, i.e. first visibility.
+     * </p>
+     * <p>
+     * Please note that a window's size shall also be allowed to change, i.e. {@link #setSize(int, int)}.
+     * </p>
+     * <p>
+     * Default value is {@link #STATE_MASK_VISIBLE} | {@link #STATE_MASK_FOCUSED} | {@link #STATE_MASK_FULLSCREEN},
+     * i.e. the <b>minimum requirement</b> for all implementations.
+     * </p>
+     * <p>
+     * Before native window creation {@link #getStatePublicBitmask()} is returned,
+     * i.e. it is assumed all features are supported.
+     * </p>
+     * <p>
+     * Semantic of the supported state-mask bits (after native creation, i.e. 1st visibility):
+     * <ul>
+     * <li>{@link #STATE_MASK_VISIBLE}: {@link #setVisible(boolean) Visibility} can be toggled. <b>Minimum requirement</b>.</li>
+     * <li>{@link #STATE_MASK_CHILDWIN}: {@link #reparentWindow(NativeWindow, int, int, int) Native window parenting} is supported.</li>
+     * <li>{@link #STATE_MASK_FOCUSED}: Window {@link #requestFocus() focus management} is supported.  <b>Minimum requirement</b>.</li>
+     * <li>{@link #STATE_MASK_UNDECORATED}: {@link #setUndecorated(boolean) Window decoration} can be toggled.</li>
+     * <li>{@link #STATE_MASK_ALWAYSONTOP}: Window can be set {@link #setAlwaysOnTop(boolean) always-on-top}. </li>
+     * <li>{@link #STATE_MASK_ALWAYSONBOTTOM}: Window can be set {@link #setAlwaysOnBottom(boolean) always-on-bottom}. </li>
+     * <li>{@link #STATE_MASK_STICKY}: Window can be set {@link #setSticky(boolean) sticky}.</li>
+     * <li>{@link #STATE_MASK_RESIZABLE}: Window {@link #setResizable(boolean) resizability} can be toggled.</li>
+     * <li>{@link #STATE_MASK_MAXIMIZED_VERT}: Window can be {@link #setMaximized(boolean, boolean) maximized-vertically}. </li>
+     * <li>{@link #STATE_MASK_MAXIMIZED_HORZ}: Window can be {@link #setMaximized(boolean, boolean) maximized-horizontally}. </li>
+     * <li>{@link #STATE_MASK_FULLSCREEN}: Window {@link #setFullscreen(boolean) fullscreen} can be toggled. </li>
+     * <li>{@link #STATE_MASK_POINTERVISIBLE}: Window {@link #setPointerVisible(boolean) pointer visibility} can be toggled. </li>
+     * <li>{@link #STATE_MASK_POINTERCONFINED}: Window {@link #confinePointer(boolean) pointer can be confined}. </li>
+     * </ul>
+     * </p>
+     * @see #getStateMask()
+     * @since 2.3.2
+     */
+    int getSupportedStateMask();
+
+    /**
+     * Returns a string representation of the {@link #getSupportedStateMask() supported state mask}.
+     * @since 2.3.2
+     */
+    String getSupportedStateMaskString();
+
     //
     // Lifecycle
     //
@@ -174,6 +478,7 @@ public interface Window extends NativeWindow, WindowClosingProtocol, ScalableSur
      * <p>
      * The Window can be recreate via {@link #setVisible(boolean) setVisible(true)}.
      * </p>
+     * <p>This method is <a href="#lifecycleHeavy">lifecycle heavy</a>.</p>
      * @see #destroy()
      * @see #setVisible(boolean)
      */
@@ -193,16 +498,16 @@ public interface Window extends NativeWindow, WindowClosingProtocol, ScalableSur
     /**
      * Calls {@link #setVisible(boolean, boolean) setVisible(true, visible)},
      * i.e. blocks until the window becomes visible.
+     * <p>This method is <a href="#lifecycleHeavy">lifecycle heavy</a>.</p>
      * @see #setVisible(boolean, boolean)
+     * @see #STATE_BIT_VISIBLE
      */
     void setVisible(boolean visible);
 
     /**
      * <code>setVisible(..)</code> makes the window and children visible if <code>visible</code> is true,
      * otherwise the window and children becomes invisible.
-     * <p>
-     * <code>setVisible(wait, true)</code> is responsible to actual create the native window.
-     * </p>
+     * <p>Native instance gets created at first visibility, following NEWT's lazy creation pattern.</p>
      * <p>
      * If <code>wait</code> is true, method blocks until window is {@link #isVisible() visible} and {@link #isNativeValid() valid},
      * otherwise method returns immediately.
@@ -221,14 +526,20 @@ public interface Window extends NativeWindow, WindowClosingProtocol, ScalableSur
      * }
      * </pre></p>
      * <p>
-     * In case this window is a child window and has a {@link javax.media.nativewindow.NativeWindow} parent,<br>
+     * In case this window is {@link #isChildWindow() a child window} and has a {@link com.jogamp.nativewindow.NativeWindow} parent,<br>
      * <code>setVisible(wait, true)</code> has no effect as long the parent's is not valid yet,
-     * i.e. {@link javax.media.nativewindow.NativeWindow#getWindowHandle()} returns <code>null</code>.<br>
+     * i.e. {@link com.jogamp.nativewindow.NativeWindow#getWindowHandle()} returns <code>null</code>.<br>
      * <code>setVisible(wait, true)</code> shall be repeated when the parent becomes valid.
      * </p>
+     * <p>This method is <a href="#lifecycleHeavy">lifecycle heavy</a>.</p>
+     * @see #STATE_BIT_VISIBLE
      */
     void setVisible(boolean wait, boolean visible);
 
+    /**
+     * @see #STATE_BIT_VISIBLE
+     * @see #setVisible(boolean, boolean)
+     */
     boolean isVisible();
 
     /**
@@ -259,7 +570,7 @@ public interface Window extends NativeWindow, WindowClosingProtocol, ScalableSur
      * according to the {@link #getMainMonitor() main monitor}'s <i>current</i> {@link MonitorMode mode}'s
      * {@link SurfaceSize#getResolution() surface resolution}.
      * <p>
-     * Method takes the {@link #getCurrentSurfaceScale(int[]) current surface-scale} and {@link #getNativeSurfaceScale(int[]) native surface-scale}
+     * Method takes the {@link #getCurrentSurfaceScale(float[]) current surface-scale} and {@link #getMaximumSurfaceScale(float[]) native surface-scale}
      * into account, i.e.:
      * <pre>
      *    surfacePpMM = monitorPpMM * currentSurfaceScale / nativeSurfaceScale,
@@ -374,14 +685,93 @@ public interface Window extends NativeWindow, WindowClosingProtocol, ScalableSur
      */
     void setTopLevelPosition(int x, int y);
 
+    /**
+     * @see {@link #STATE_BIT_UNDECORATED}
+     * @see {@link #STATE_MASK_UNDECORATED}
+     */
     void setUndecorated(boolean value);
-
+    /**
+     * @see {@link #STATE_BIT_UNDECORATED}
+     * @see {@link #STATE_MASK_UNDECORATED}
+     */
     boolean isUndecorated();
 
+    /**
+     * <p>Operation is ignored if this instance {@link #isChildWindow() is a child window}.</p>
+     * @see {@link #STATE_BIT_ALWAYSONTOP}
+     * @see {@link #STATE_MASK_ALWAYSONTOP}
+     */
     void setAlwaysOnTop(boolean value);
-
+    /**
+     * @see {@link #STATE_BIT_ALWAYSONTOP}
+     * @see {@link #STATE_MASK_ALWAYSONTOP}
+     */
     boolean isAlwaysOnTop();
 
+    /**
+     * <p>Operation is ignored if this instance {@link #isChildWindow() is a child window}.</p>
+     * @see {@link #STATE_BIT_ALWAYSONBOTTOM}
+     * @see {@link #STATE_MASK_ALWAYSONBOTTOM}
+     * @since 2.3.2
+     */
+    void setAlwaysOnBottom(boolean value);
+    /**
+     * @see {@link #STATE_BIT_ALWAYSONBOTTOM}
+     * @see {@link #STATE_MASK_ALWAYSONBOTTOM}
+     * @since 2.3.2
+     */
+    boolean isAlwaysOnBottom();
+
+    /**
+     * <p>Operation is ignored if this instance {@link #isChildWindow() is a child window}.</p>
+     * @see {@link #STATE_BIT_RESIZABLE}
+     * @see {@link #STATE_MASK_RESIZABLE}
+     * @since 2.3.2
+     */
+    void setResizable(final boolean value);
+    /**
+     * @see {@link #STATE_BIT_RESIZABLE}
+     * @see {@link #STATE_MASK_RESIZABLE}
+     * @since 2.3.2
+     */
+    boolean isResizable();
+
+    /**
+     * <p>Operation is ignored if this instance {@link #isChildWindow() is a child window}.</p>
+     * @see {@link #STATE_BIT_STICKY}
+     * @see {@link #STATE_MASK_STICKY}
+     * @since 2.3.2
+     */
+    void setSticky(final boolean value);
+    /**
+     * @see {@link #STATE_BIT_STICKY}
+     * @see {@link #STATE_MASK_STICKY}
+     * @since 2.3.2
+     */
+    boolean isSticky();
+
+    /**
+     * <p>Operation is ignored if this instance {@link #isChildWindow() is a child window}.</p>
+     * @see {@link #STATE_BIT_MAXIMIZED_HORZ}
+     * @see {@link #STATE_BIT_MAXIMIZED_VERT}
+     * @see {@link #STATE_MASK_MAXIMIZED_HORZ}
+     * @see {@link #STATE_MASK_MAXIMIZED_VERT}
+     * @since 2.3.2
+     */
+    void setMaximized(final boolean horz, final boolean vert);
+    /**
+     * @see {@link #STATE_BIT_MAXIMIZED_VERT}
+     * @see {@link #STATE_MASK_MAXIMIZED_VERT}
+     * @since 2.3.2
+     */
+    boolean isMaximizedVert();
+    /**
+     * @see {@link #STATE_BIT_MAXIMIZED_HORZ}
+     * @see {@link #STATE_MASK_MAXIMIZED_HORZ}
+     * @since 2.3.2
+     */
+    boolean isMaximizedHorz();
+
     void setTitle(String title);
 
     String getTitle();
@@ -470,6 +860,7 @@ public interface Window extends NativeWindow, WindowClosingProtocol, ScalableSur
      * this window is removed from it's list of children.<br>
      * In case the new parent is not null and a Window,
      * this window is added to it's list of children.<br></P>
+     * <p>This method is <a href="#lifecycleHeavy">lifecycle heavy</a>.</p>
      *
      * @param newParent The new parent NativeWindow. If null, this Window becomes a top level window.
      * @param x new top-level position in window units, use -1 for default position.
@@ -480,11 +871,21 @@ public interface Window extends NativeWindow, WindowClosingProtocol, ScalableSur
      */
     ReparentOperation reparentWindow(NativeWindow newParent, int x, int y, int hints);
 
+    /**
+     * Returns {@code true} if this window is a child window,
+     * i.e. has been {@link #reparentWindow(NativeWindow, int, int, int) reparented}.
+     * <p>
+     * Otherwise return {@code false}, i.e. this window is a top-level window.
+     * </p>
+     */
+    boolean isChildWindow();
+
     /**
      * Enable or disable fullscreen mode for this window.
      * <p>
      * Fullscreen mode is established on the {@link #getMainMonitor() main monitor}.
      * </p>
+     * <p>This method is <a href="#lifecycleHeavy">lifecycle heavy</a>.</p>
      * @param fullscreen enable or disable fullscreen mode
      * @return success
      * @see #setFullscreen(List)
@@ -498,6 +899,7 @@ public interface Window extends NativeWindow, WindowClosingProtocol, ScalableSur
      * <p>
      * Disable fullscreen via {@link #setFullscreen(boolean)}.
      * </p>
+     * <p>This method is <a href="#lifecycleHeavy">lifecycle heavy</a>.</p>
      * @param monitors if <code>null</code> fullscreen will be spanned across all {@link MonitorDevice}s,
      *                 otherwise across the given list of {@link MonitorDevice}.
      * @return success
