@@ -37,6 +37,7 @@ import java.awt.EventQueue;
 import java.awt.Graphics;
 import java.awt.Graphics2D;
 import java.awt.GraphicsConfiguration;
+import java.awt.GraphicsDevice;
 import java.awt.KeyboardFocusManager;
 import java.awt.geom.NoninvertibleTransformException;
 import java.beans.Beans;
@@ -47,17 +48,18 @@ import java.security.AccessController;
 import java.security.PrivilegedAction;
 import java.util.Set;
 
-import javax.media.nativewindow.NativeWindow;
-import javax.media.nativewindow.OffscreenLayerOption;
-import javax.media.nativewindow.WindowClosingProtocol;
-import javax.media.opengl.GLAnimatorControl;
-import javax.media.opengl.GLAutoDrawable;
-import javax.media.opengl.GLCapabilities;
-import javax.media.opengl.GLCapabilitiesImmutable;
-import javax.media.opengl.GLDrawable;
-import javax.media.opengl.GLDrawableFactory;
-import javax.media.opengl.GLException;
-import javax.media.opengl.GLOffscreenAutoDrawable;
+import com.jogamp.nativewindow.CapabilitiesImmutable;
+import com.jogamp.nativewindow.NativeWindow;
+import com.jogamp.nativewindow.OffscreenLayerOption;
+import com.jogamp.nativewindow.WindowClosingProtocol;
+import com.jogamp.opengl.GLAnimatorControl;
+import com.jogamp.opengl.GLAutoDrawable;
+import com.jogamp.opengl.GLCapabilities;
+import com.jogamp.opengl.GLCapabilitiesImmutable;
+import com.jogamp.opengl.GLDrawable;
+import com.jogamp.opengl.GLDrawableFactory;
+import com.jogamp.opengl.GLException;
+import com.jogamp.opengl.GLOffscreenAutoDrawable;
 import javax.swing.MenuSelectionManager;
 
 import jogamp.nativewindow.awt.AWTMisc;
@@ -69,7 +71,9 @@ import jogamp.newt.awt.event.AWTParentWindowAdapter;
 import jogamp.newt.driver.DriverClearFocus;
 import jogamp.opengl.awt.AWTTilePainter;
 
+import com.jogamp.common.ExceptionUtils;
 import com.jogamp.common.util.awt.AWTEDTExecutor;
+import com.jogamp.nativewindow.awt.AWTGraphicsConfiguration;
 import com.jogamp.nativewindow.awt.AWTPrintLifecycle;
 import com.jogamp.nativewindow.awt.AWTWindowClosingProtocol;
 import com.jogamp.nativewindow.awt.JAWTWindow;
@@ -100,7 +104,7 @@ public class NewtCanvasAWT extends java.awt.Canvas implements WindowClosingProto
     public static final boolean DEBUG = Debug.debug("Window");
 
     private final Object sync = new Object();
-    private JAWTWindow jawtWindow = null;
+    private volatile JAWTWindow jawtWindow = null; // the JAWTWindow presentation of this AWT Canvas, bound to the 'drawable' lifecycle
     private boolean isApplet = false;
     private boolean shallUseOffscreenLayer = false;
     private Window newtChild = null;
@@ -111,6 +115,8 @@ public class NewtCanvasAWT extends java.awt.Canvas implements WindowClosingProto
     private final AWTAdapter awtMouseAdapter;
     private final AWTAdapter awtKeyAdapter;
 
+    private volatile AWTGraphicsConfiguration awtConfig;
+
     /** Mitigates Bug 910 (IcedTea-Web), i.e. crash via removeNotify() invoked before Applet.destroy(). */
     private boolean destroyJAWTPending = false;
     /** Mitigates Bug 910 (IcedTea-Web), i.e. crash via removeNotify() invoked before Applet.destroy(). */
@@ -451,14 +457,142 @@ public class NewtCanvasAWT extends java.awt.Canvas implements WindowClosingProto
         }
     }
 
+    private void setAWTGraphicsConfiguration(final AWTGraphicsConfiguration config) {
+        // Cache awtConfig
+        awtConfig = config;
+        if( null != jawtWindow ) {
+            // Notify JAWTWindow ..
+            jawtWindow.setAWTGraphicsConfiguration(config);
+        }
+    }
+
+    /**
+     * {@inheritDoc}
+     * <p>
+     * Overridden to choose a {@link GraphicsConfiguration} from a parent container's
+     * {@link GraphicsDevice}.
+     * </p>
+     * <p>
+     * Method also intercepts {@link GraphicsConfiguration} changes regarding to
+     * its capabilities and its {@link GraphicsDevice}. This may happen in case
+     * the display changes its configuration or the component is moved to another screen.
+     * </p>
+     */
+    @Override
+    public GraphicsConfiguration getGraphicsConfiguration() {
+        /**
+         * parentGC will be null unless:
+         *   - A native peer has assigned it. This means we have a native
+         *     peer, and are already committed to a graphics configuration.
+         *   - This canvas has been added to a component hierarchy and has
+         *     an ancestor with a non-null GC, but the native peer has not
+         *     yet been created. This means we can still choose the GC on
+         *     all platforms since the peer hasn't been created.
+         */
+        final GraphicsConfiguration parentGC = super.getGraphicsConfiguration();
+
+        if( Beans.isDesignTime() ) {
+            return parentGC;
+        }
+        final GraphicsConfiguration oldGC =  null != awtConfig ? awtConfig.getAWTGraphicsConfiguration() : null;
+
+        if ( null != parentGC && null != oldGC && !oldGC.equals(parentGC) ) {
+            // Previous oldGC != parentGC of native peer
+
+            if ( !oldGC.getDevice().getIDstring().equals(parentGC.getDevice().getIDstring()) ) {
+                // Previous oldGC's GraphicsDevice != parentGC's GraphicsDevice of native peer
+
+                /**
+                 * Here we select a GraphicsConfiguration on the alternate device.
+                 * In case the new configuration differs (-> !equalCaps),
+                 * we might need a reconfiguration,
+                 */
+                final AWTGraphicsConfiguration newConfig = AWTGraphicsConfiguration.create(parentGC,
+                        awtConfig.getChosenCapabilities(),
+                        awtConfig.getRequestedCapabilities());
+                final GraphicsConfiguration newGC = newConfig.getAWTGraphicsConfiguration();
+                final boolean equalCaps = newConfig.getChosenCapabilities().equals(awtConfig.getChosenCapabilities());
+                if(DEBUG) {
+                    System.err.println(getThreadName()+": getGraphicsConfiguration() Info: Changed GC and GD");
+                    System.err.println("Created Config (n): Old     GC "+oldGC);
+                    System.err.println("Created Config (n): Old     GD "+oldGC.getDevice().getIDstring());
+                    System.err.println("Created Config (n): Parent  GC "+parentGC);
+                    System.err.println("Created Config (n): Parent  GD "+parentGC.getDevice().getIDstring());
+                    System.err.println("Created Config (n): New     GC "+newGC);
+                    System.err.println("Created Config (n): Old     CF "+awtConfig);
+                    System.err.println("Created Config (n): New     CF "+newConfig);
+                    System.err.println("Created Config (n): EQUALS CAPS "+equalCaps);
+                    // Thread.dumpStack();
+                }
+                if ( null != newGC ) {
+                    setAWTGraphicsConfiguration(newConfig);
+                    /**
+                     * Return the newGC, which covers the desired capabilities and is compatible
+                     * with the available GC's of its devices.
+                     */
+                    if(DEBUG) {
+                        System.err.println(getThreadName()+": Info: getGraphicsConfiguration - end.01: newGC "+newGC);
+                    }
+                    return newGC;
+                } else {
+                    if(DEBUG) {
+                        System.err.println(getThreadName()+": Info: getGraphicsConfiguration - end.00: oldGC "+oldGC);
+                    }
+                }
+            }
+            /**
+             * If a new GC was _not_ found/defined above,
+             * method returns oldGC as selected in the constructor or first addNotify().
+             * This may cause an exception in Component.checkGD when adding to a
+             * container, and is the desired behavior.
+             */
+            return oldGC;
+        } else if (null == parentGC) {
+            /**
+             * The parentGC is null, which means we have no native peer, and are not
+             * part of a (realized) component hierarchy. So we return the
+             * desired visual that was selected in the constructor (possibly
+             * null).
+             */
+            return oldGC;
+        } else {
+            /**
+             * Otherwise we have not explicitly selected a GC in the constructor, so
+             * just return what Canvas would have.
+             */
+            return parentGC;
+        }
+    }
+    private static String getThreadName() { return Thread.currentThread().getName(); }
+
     @Override
     public void addNotify() {
         if( Beans.isDesignTime() ) {
             super.addNotify();
         } else {
+            /**
+             * 'super.addNotify()' determines the GraphicsConfiguration,
+             * while calling this class's overridden 'getGraphicsConfiguration()' method
+             * after which it creates the native peer.
+             * Hence we have to set the 'awtConfig' before since it's GraphicsConfiguration
+             * is being used in getGraphicsConfiguration().
+             * This code order also allows recreation, ie re-adding the GLCanvas.
+             */
             // before native peer is valid: X11
             disableBackgroundErase();
 
+            // Query AWT GraphicsDevice from parent tree, default
+            final GraphicsConfiguration gc = super.getGraphicsConfiguration();
+            if(null==gc) {
+                throw new GLException("Error: NULL AWT GraphicsConfiguration");
+            }
+            final CapabilitiesImmutable capsReq = null != newtChild ? newtChild.getRequestedCapabilities() : null;
+            final AWTGraphicsConfiguration awtConfig = AWTGraphicsConfiguration.create(gc, null, capsReq);
+            if(null==awtConfig) {
+                throw new GLException("Error: NULL AWTGraphicsConfiguration");
+            }
+            setAWTGraphicsConfiguration(awtConfig);
+
             // creates the native peer
             super.addNotify();
 
@@ -469,10 +603,17 @@ public class NewtCanvasAWT extends java.awt.Canvas implements WindowClosingProto
                 determineIfApplet();
                 if(DEBUG) {
                     System.err.println("NewtCanvasAWT.addNotify.0 - isApplet "+isApplet+", addedOnAWTEDT "+EventQueue.isDispatchThread()+" @ "+currentThreadName());
-                    Thread.dumpStack();
+                    ExceptionUtils.dumpStack(System.err);
                 }
-                jawtWindow = NewtFactoryAWT.getNativeWindow(NewtCanvasAWT.this, null != newtChild ? newtChild.getRequestedCapabilities() : null);
+                jawtWindow = NewtFactoryAWT.getNativeWindow(NewtCanvasAWT.this, awtConfig);
                 jawtWindow.setShallUseOffscreenLayer(shallUseOffscreenLayer);
+                // enforce initial lock on AWT-EDT, allowing acquisition of pixel-scale
+                jawtWindow.lockSurface();
+                try {
+                    // attachNewtChild sets surface scale!
+                } finally {
+                    jawtWindow.unlockSurface();
+                }
                 awtWindowClosingProtocol.addClosingListener();
                 componentAdded = true; // Bug 910
                 if(DEBUG) {
@@ -485,6 +626,25 @@ public class NewtCanvasAWT extends java.awt.Canvas implements WindowClosingProto
             }
         }
     }
+    private final boolean updatePixelScale(final GraphicsConfiguration gc) {
+        if( jawtWindow.updatePixelScale(gc, true) ) {
+            final Window cWin = newtChild;
+            final Window dWin = cWin.getDelegatedWindow();
+            if( dWin instanceof WindowImpl ) {
+                final float[] maxPixelScale = jawtWindow.getMaximumSurfaceScale(new float[2]);
+                final float[] minPixelScale = jawtWindow.getMinimumSurfaceScale(new float[2]);
+                ((WindowImpl)dWin).pixelScaleChangeNotify(minPixelScale, maxPixelScale, true);
+                // ((WindowImpl)dWin).sizeChangedNotify(true /* defer */, getWidth(), getHeight(), true /* force */);
+            } else {
+                final float[] reqPixelScale = jawtWindow.getRequestedSurfaceScale(new float[2]);
+                if( jawtWindow.setSurfaceScale(reqPixelScale) ) {
+                    // jawtWindow.getRequestedSurfaceScale(reqPixelScale);
+                }
+            }
+            return true;
+        }
+        return false;
+    }
 
     @Override
     public void removeNotify() {
@@ -493,7 +653,7 @@ public class NewtCanvasAWT extends java.awt.Canvas implements WindowClosingProto
         } else {
             if(DEBUG) {
                 System.err.println("NewtCanvasAWT.removeNotify.0 - isApplet "+isApplet+" @ "+currentThreadName());
-                Thread.dumpStack();
+                ExceptionUtils.dumpStack(System.err);
             }
             componentAdded = false; // Bug 910
             awtWindowClosingProtocol.removeClosingListener();
@@ -518,7 +678,7 @@ public class NewtCanvasAWT extends java.awt.Canvas implements WindowClosingProto
     public final void destroy() {
         if(DEBUG) {
             System.err.println("NewtCanvasAWT.destroy() @ "+currentThreadName());
-            Thread.dumpStack();
+            ExceptionUtils.dumpStack(System.err);
         }
         AWTEDTExecutor.singleton.invoke(true, new Runnable() {
             @Override
@@ -558,6 +718,7 @@ public class NewtCanvasAWT extends java.awt.Canvas implements WindowClosingProto
                 } else {
                     NewtFactoryAWT.destroyNativeWindow(jawtWindow);
                     jawtWindow=null;
+                    awtConfig=null;
                     destroyJAWTPending = false;
                 }
             }
@@ -587,7 +748,11 @@ public class NewtCanvasAWT extends java.awt.Canvas implements WindowClosingProto
                     System.err.println("NewtCanvasAWT.reshape: "+x+"/"+y+" "+width+"x"+height);
                 }
                 if( validateComponent(true) ) {
-                    // newtChild.setSize(width, height);
+                    if( !printActive && updatePixelScale(getGraphicsConfiguration()) ) {
+                        // NOP
+                    } else {
+                        // newtChild.setSize(width, height);
+                    }
                 }
             }
         }
@@ -876,18 +1041,12 @@ public class NewtCanvasAWT extends java.awt.Canvas implements WindowClosingProto
       }
       final int w = getWidth();
       final int h = getHeight();
-      final boolean isNValid = newtChild.isNativeValid();
       if(DEBUG) {
-          System.err.println("NewtCanvasAWT.attachNewtChild.2: size "+w+"x"+h+", isNValid "+isNValid);
+          System.err.println("NewtCanvasAWT.attachNewtChild.2: size "+w+"x"+h+", isNValid "+newtChild.isNativeValid());
       }
       newtChild.setVisible(false);
       newtChild.setSize(w, h);
-      final int[] reqSurfaceScale = new int[2];
-      if( isNValid ) {
-          newtChild.getCurrentSurfaceScale(reqSurfaceScale);
-      } else {
-          newtChild.getRequestedSurfaceScale(reqSurfaceScale);
-      }
+      final float[] reqSurfaceScale = newtChild.getRequestedSurfaceScale(new float[2]);
       jawtWindow.setSurfaceScale(reqSurfaceScale);
       newtChild.reparentWindow(jawtWindow, -1, -1, Window.REPARENT_HINT_BECOMES_VISIBLE);
       newtChild.addSurfaceUpdatedListener(jawtWindow);
