@@ -38,34 +38,34 @@ import java.security.AccessController;
 import java.security.PrivilegedAction;
 import java.util.List;
 
-import javax.media.nativewindow.AbstractGraphicsConfiguration;
-import javax.media.nativewindow.CapabilitiesChooser;
-import javax.media.nativewindow.CapabilitiesImmutable;
-import javax.media.nativewindow.NativeSurface;
-import javax.media.nativewindow.NativeWindow;
-import javax.media.nativewindow.NativeWindowException;
-import javax.media.nativewindow.SurfaceUpdatedListener;
-import javax.media.nativewindow.util.InsetsImmutable;
-import javax.media.nativewindow.util.Point;
-import javax.media.nativewindow.util.Rectangle;
-import javax.media.opengl.FPSCounter;
-import javax.media.opengl.GL;
-import javax.media.opengl.GL3;
-import javax.media.opengl.GL4ES3;
-import javax.media.opengl.GLAnimatorControl;
-import javax.media.opengl.GLAutoDrawable;
-import javax.media.opengl.GLCapabilities;
-import javax.media.opengl.GLCapabilitiesImmutable;
-import javax.media.opengl.GLContext;
-import javax.media.opengl.GLDrawable;
-import javax.media.opengl.GLDrawableFactory;
-import javax.media.opengl.GLES2;
-import javax.media.opengl.GLES3;
-import javax.media.opengl.GLEventListener;
-import javax.media.opengl.GLException;
-import javax.media.opengl.GLProfile;
-import javax.media.opengl.GLRunnable;
-import javax.media.opengl.GLSharedContextSetter;
+import com.jogamp.nativewindow.AbstractGraphicsConfiguration;
+import com.jogamp.nativewindow.CapabilitiesChooser;
+import com.jogamp.nativewindow.CapabilitiesImmutable;
+import com.jogamp.nativewindow.NativeSurface;
+import com.jogamp.nativewindow.NativeWindow;
+import com.jogamp.nativewindow.NativeWindowException;
+import com.jogamp.nativewindow.SurfaceUpdatedListener;
+import com.jogamp.nativewindow.util.InsetsImmutable;
+import com.jogamp.nativewindow.util.Point;
+import com.jogamp.nativewindow.util.Rectangle;
+import com.jogamp.opengl.FPSCounter;
+import com.jogamp.opengl.GL;
+import com.jogamp.opengl.GL3;
+import com.jogamp.opengl.GL4ES3;
+import com.jogamp.opengl.GLAnimatorControl;
+import com.jogamp.opengl.GLAutoDrawable;
+import com.jogamp.opengl.GLCapabilities;
+import com.jogamp.opengl.GLCapabilitiesImmutable;
+import com.jogamp.opengl.GLContext;
+import com.jogamp.opengl.GLDrawable;
+import com.jogamp.opengl.GLDrawableFactory;
+import com.jogamp.opengl.GLES2;
+import com.jogamp.opengl.GLES3;
+import com.jogamp.opengl.GLEventListener;
+import com.jogamp.opengl.GLException;
+import com.jogamp.opengl.GLProfile;
+import com.jogamp.opengl.GLRunnable;
+import com.jogamp.opengl.GLSharedContextSetter;
 
 import jogamp.newt.WindowImpl;
 import jogamp.opengl.GLAutoDrawableBase;
@@ -225,6 +225,36 @@ public class GLWindow extends GLAutoDrawableBase implements GLAutoDrawable, Wind
     // Window Access
     //
 
+    @Override
+    public final int getStatePublicBitCount() {
+        return window.getStatePublicBitCount();
+    }
+
+    @Override
+    public final int getStatePublicBitmask() {
+        return window.getStatePublicBitmask();
+    }
+
+    @Override
+    public final int getStateMask() {
+        return window.getStateMask();
+    }
+
+    @Override
+    public final String getStateMaskString() {
+        return window.getStateMaskString();
+    }
+
+    @Override
+    public final int getSupportedStateMask() {
+        return window.getSupportedStateMask();
+    }
+
+    @Override
+    public final String getSupportedStateMaskString() {
+        return window.getSupportedStateMaskString();
+    }
+
     @Override
     public CapabilitiesChooser setCapabilitiesChooser(final CapabilitiesChooser chooser) {
         return window.setCapabilitiesChooser(chooser);
@@ -325,6 +355,51 @@ public class GLWindow extends GLAutoDrawableBase implements GLAutoDrawable, Wind
         return window.isAlwaysOnTop();
     }
 
+    @Override
+    public final void setAlwaysOnBottom(final boolean value) {
+        window.setAlwaysOnBottom(value);
+    }
+
+    @Override
+    public final boolean isAlwaysOnBottom() {
+        return window.isAlwaysOnBottom();
+    }
+
+    @Override
+    public final void setResizable(final boolean value) {
+        window.setResizable(value);
+    }
+
+    @Override
+    public final boolean isResizable() {
+        return window.isResizable();
+    }
+
+    @Override
+    public final void setSticky(final boolean value) {
+        window.setSticky(value);
+    }
+
+    @Override
+    public final boolean isSticky() {
+        return window.isSticky();
+    }
+
+    @Override
+    public final void setMaximized(final boolean horz, final boolean vert) {
+        window.setMaximized(horz, vert);
+    }
+
+    @Override
+    public final boolean isMaximizedVert() {
+        return window.isMaximizedVert();
+    }
+
+    @Override
+    public final boolean isMaximizedHorz() {
+        return window.isMaximizedHorz();
+    }
+
     @Override
     public final void setFocusAction(final FocusRunnable focusAction) {
         window.setFocusAction(focusAction);
@@ -401,23 +476,28 @@ public class GLWindow extends GLAutoDrawableBase implements GLAutoDrawable, Wind
     }
 
     @Override
-    public final void setSurfaceScale(final int[] pixelScale) {
-        window.setSurfaceScale(pixelScale);
+    public final boolean setSurfaceScale(final float[] pixelScale) {
+        return window.setSurfaceScale(pixelScale);
     }
 
     @Override
-    public final int[] getRequestedSurfaceScale(final int[] result) {
+    public final float[] getRequestedSurfaceScale(final float[] result) {
         return window.getRequestedSurfaceScale(result);
     }
 
     @Override
-    public final int[] getCurrentSurfaceScale(final int[] result) {
+    public final float[] getCurrentSurfaceScale(final float[] result) {
         return window.getCurrentSurfaceScale(result);
     }
 
     @Override
-    public final int[] getNativeSurfaceScale(final int[] result) {
-        return window.getNativeSurfaceScale(result);
+    public final float[] getMinimumSurfaceScale(final float[] result) {
+        return window.getMinimumSurfaceScale(result);
+    }
+
+    @Override
+    public final float[] getMaximumSurfaceScale(final float[] result) {
+        return window.getMaximumSurfaceScale(result);
     }
 
     @Override
@@ -464,6 +544,10 @@ public class GLWindow extends GLAutoDrawableBase implements GLAutoDrawable, Wind
     public final ReparentOperation reparentWindow(final NativeWindow newParent, final int x, final int y, final int hints) {
         return window.reparentWindow(newParent, x, y, hints);
     }
+    @Override
+    public final boolean isChildWindow() {
+        return window.isChildWindow();
+    }
 
     @Override
     public final boolean removeChild(final NativeWindow win) {
@@ -999,6 +1083,11 @@ public class GLWindow extends GLAutoDrawableBase implements GLAutoDrawable, Wind
         glWindow.addGLEventListener(new GLEventListener() {
             @Override
             public void init(final GLAutoDrawable drawable) {
+                final MonitorDevice monitor = glWindow.getMainMonitor();
+                System.err.println("Main Monitor: "+monitor);
+                final float[] pixelPerMM = monitor.getPixelsPerMM(new float[2]);
+                System.err.println("    pixel/mm ["+pixelPerMM[0]+", "+pixelPerMM[1]+"]");
+                System.err.println("    pixel/in ["+pixelPerMM[0]*25.4f+", "+pixelPerMM[1]*25.4f+"]");
                 final GL gl = drawable.getGL();
                 System.err.println(JoglVersion.getGLInfo(gl, null));
                 System.err.println("Requested: "+drawable.getNativeSurface().getGraphicsConfiguration().getRequestedCapabilities());
