@@ -31,12 +31,17 @@ package jogamp.newt;
 import com.jogamp.common.util.ArrayHashSet;
 import com.jogamp.newt.MonitorDevice;
 import com.jogamp.newt.MonitorMode;
+import com.jogamp.newt.Screen;
 
+import java.util.ArrayList;
 import java.util.List;
-import javax.media.nativewindow.util.Dimension;
-import javax.media.nativewindow.util.DimensionImmutable;
-import javax.media.nativewindow.util.Rectangle;
-import javax.media.nativewindow.util.SurfaceSize;
+
+import com.jogamp.nativewindow.ScalableSurface;
+import com.jogamp.nativewindow.util.Dimension;
+import com.jogamp.nativewindow.util.DimensionImmutable;
+import com.jogamp.nativewindow.util.Rectangle;
+import com.jogamp.nativewindow.util.SurfaceSize;
+import com.jogamp.opengl.math.FloatUtil;
 
 import jogamp.newt.MonitorDeviceImpl;
 import jogamp.newt.ScreenImpl;
@@ -127,6 +132,8 @@ public class MonitorModeProps {
      * <ul>
      *   <li>count</li>
      *   <li>id</li>
+     *   <li>IsClone</li>
+     *   <li>IsPrimary</li>
      *   <li>ScreenSizeMM[width, height] (2 elements)</li>
      *   <li>Rotated Viewport pixel-units (4 elements)</li>
      *   <li>Rotated Viewport window-units  (4 elements)</li>
@@ -135,25 +142,31 @@ public class MonitorModeProps {
      *   <li>SupportedModeId+</li>
      * </ul>
      * <p>
-     * Viewport := [x, y, width, height] (4 elements)
+     * with Viewport := [x, y, width, height] (4 elements)
      * </p>
      * <p>
      * WARNING: must be synchronized with ScreenMode.h, native implementation
      * </p>
      */
-    public static final int MIN_MONITOR_DEVICE_PROPERTIES = 15;
+    public static final int MIN_MONITOR_DEVICE_PROPERTIES = 17;
 
     public static final int IDX_MONITOR_DEVICE_VIEWPORT =   1 // count
                                                           + 1 // native mode
+                                                          + 1 // isClone
+                                                          + 1 // isPrimary
                                                           + MonitorModeProps.NUM_RESOLUTION_PROPERTIES // sizeMM
                                                           ;
 
     public static class Cache {
-        public final ArrayHashSet<DimensionImmutable>       resolutions  = new ArrayHashSet<DimensionImmutable>();
-        public final ArrayHashSet<SurfaceSize>              surfaceSizes = new ArrayHashSet<SurfaceSize>();
-        public final ArrayHashSet<MonitorMode.SizeAndRRate> sizeAndRates = new ArrayHashSet<MonitorMode.SizeAndRRate>();
-        public final ArrayHashSet<MonitorMode>              monitorModes = new ArrayHashSet<MonitorMode>();
-        public final ArrayHashSet<MonitorDevice>            monitorDevices = new ArrayHashSet<MonitorDevice>();
+        public final ArrayHashSet<DimensionImmutable>       resolutions  = new ArrayHashSet<DimensionImmutable>(false, ArrayHashSet.DEFAULT_INITIAL_CAPACITY, ArrayHashSet.DEFAULT_LOAD_FACTOR);
+        public final ArrayHashSet<SurfaceSize>              surfaceSizes = new ArrayHashSet<SurfaceSize>(false, ArrayHashSet.DEFAULT_INITIAL_CAPACITY, ArrayHashSet.DEFAULT_LOAD_FACTOR);
+        public final ArrayHashSet<MonitorMode.SizeAndRRate> sizeAndRates = new ArrayHashSet<MonitorMode.SizeAndRRate>(false, ArrayHashSet.DEFAULT_INITIAL_CAPACITY, ArrayHashSet.DEFAULT_LOAD_FACTOR);
+        public final ArrayHashSet<MonitorMode>              monitorModes = new ArrayHashSet<MonitorMode>(false, ArrayHashSet.DEFAULT_INITIAL_CAPACITY, ArrayHashSet.DEFAULT_LOAD_FACTOR);
+        public final ArrayHashSet<MonitorDevice>            monitorDevices = new ArrayHashSet<MonitorDevice>(false, ArrayHashSet.DEFAULT_INITIAL_CAPACITY, ArrayHashSet.DEFAULT_LOAD_FACTOR);
+
+        public final void setPrimary(final MonitorDevice p) { primary = p; }
+        public final MonitorDevice getPrimary() { return primary;}
+        private MonitorDevice primary = null;
     }
 
     /** WARNING: must be synchronized with ScreenMode.h, native implementation */
@@ -256,14 +269,19 @@ public class MonitorModeProps {
      * <p>
      * Note: This variant only works for impl. w/ a unique mode key pair <i>modeId, rotation</i>.
      * </p>
-     * @param mode_idx if not null, returns the index of resulting {@link MonitorDevice} within {@link Cache#monitorDevices}.
      * @param cache hash arrays of unique {@link MonitorMode} components and {@link MonitorDevice}s, allowing to avoid duplicates
-     * @param modeProperties the input data
+     * @param screen the associated {@link ScreenImpl}
+     * @param pixelScale pre-fetched current pixel-scale, maybe {@code null} for {@link ScalableSurface#IDENTITY_PIXELSCALE}.
+     * @param monitorProperties the input data inclusive supported modes.
      * @param offset the offset to the input data
+     * @param monitor_idx if not null, returns the index of resulting {@link MonitorDevice} within {@link Cache#monitorDevices}.
      * @return {@link MonitorDevice} of the identical (old or new) element in {@link Cache#monitorDevices},
      *         matching the input <code>modeProperties</code>, or null if input could not be processed.
      */
-    public static MonitorDevice streamInMonitorDevice(final int[] monitor_idx, final Cache cache, final ScreenImpl screen, final int[] monitorProperties, int offset) {
+    public static MonitorDevice streamInMonitorDevice(final Cache cache, final ScreenImpl screen,
+                                                      final float[] pixelScale,
+                                                      final int[] monitorProperties, int offset,
+                                                      final int[] monitor_idx) {
         // min 11: count, id, ScreenSizeMM[width, height], Viewport[x, y, width, height], currentMonitorModeId, rotation, supportedModeId+
         final int count = monitorProperties[offset];
         if(MIN_MONITOR_DEVICE_PROPERTIES > count) {
@@ -279,6 +297,8 @@ public class MonitorModeProps {
         offset++;
         final List<MonitorMode> allMonitorModes = cache.monitorModes.getData();
         final int id = monitorProperties[offset++];
+        final boolean isClone = 0 == monitorProperties[offset++] ? false : true;
+        final boolean isPrimary = 0 == monitorProperties[offset++] ? false : true;
         final DimensionImmutable sizeMM = streamInResolution(monitorProperties, offset); offset+=NUM_RESOLUTION_PROPERTIES;
         final Rectangle viewportPU = new Rectangle(monitorProperties[offset++], monitorProperties[offset++], monitorProperties[offset++], monitorProperties[offset++]);
         final Rectangle viewportWU = new Rectangle(monitorProperties[offset++], monitorProperties[offset++], monitorProperties[offset++], monitorProperties[offset++]);
@@ -288,7 +308,7 @@ public class MonitorModeProps {
             final int rotation = monitorProperties[offset++];
             currentMode = getByNativeIdAndRotation(allMonitorModes, modeId, rotation);
         }
-        final ArrayHashSet<MonitorMode> supportedModes = new ArrayHashSet<MonitorMode>();
+        final ArrayHashSet<MonitorMode> supportedModes = new ArrayHashSet<MonitorMode>(false, ArrayHashSet.DEFAULT_INITIAL_CAPACITY, ArrayHashSet.DEFAULT_LOAD_FACTOR);
         while( offset < limit ) {
             final int modeId = monitorProperties[offset++];
             for (int i=0; i<allMonitorModes.size(); i++) {
@@ -298,9 +318,14 @@ public class MonitorModeProps {
                 }
             }
         }
-        MonitorDevice monitorDevice = new MonitorDeviceImpl(screen, id, sizeMM, viewportPU, viewportWU, currentMode, supportedModes);
+        MonitorDevice monitorDevice = new MonitorDeviceImpl(screen, id, isClone, isPrimary,
+                                                            sizeMM, currentMode, pixelScale,
+                                                            viewportPU, viewportWU, supportedModes);
         if(null!=cache) {
             monitorDevice = cache.monitorDevices.getOrAdd(monitorDevice);
+            if( monitorDevice.isPrimary() ) {
+                cache.setPrimary(monitorDevice);
+            }
         }
         if( null != monitor_idx ) {
             final int _monitorIdx  = cache.monitorDevices.indexOf(monitorDevice);
@@ -329,17 +354,23 @@ public class MonitorModeProps {
      * This variant expects <code>count</code> to be <code>{@link MIN_MONITOR_DEVICE_PROPERTIES} - 1 - {@link NUM_MONITOR_MODE_PROPERTIES}</code>,
      * due to lack of supported mode and current mode.
      * </p>
-     *
-     * @param mode_idx if not null, returns the index of resulting {@link MonitorDevice} within {@link Cache#monitorDevices}.
      * @param cache hash arrays of unique {@link MonitorMode} components and {@link MonitorDevice}s, allowing to avoid duplicates
-     * @param supportedModes pre-assembled list of supported {@link MonitorMode}s from cache.
+     * @param screen the associated {@link ScreenImpl}
      * @param currentMode pre-fetched current {@link MonitorMode}s from cache.
-     * @param modeProperties the input data minus supported modes!
+     * @param pixelScale pre-fetched current pixel-scale, maybe {@code null} for {@link ScalableSurface#IDENTITY_PIXELSCALE}.
+     * @param supportedModes pre-assembled list of supported {@link MonitorMode}s from cache.
+     * @param monitorProperties the input data minus supported modes!
      * @param offset the offset to the input data
+     * @param monitor_idx if not null, returns the index of resulting {@link MonitorDevice} within {@link Cache#monitorDevices}.
      * @return {@link MonitorDevice} of the identical (old or new) element in {@link Cache#monitorDevices},
      *         matching the input <code>modeProperties</code>, or null if input could not be processed.
      */
-    public static MonitorDevice streamInMonitorDevice(final int[] monitor_idx, final Cache cache, final ScreenImpl screen, final ArrayHashSet<MonitorMode> supportedModes, final MonitorMode currentMode, final int[] monitorProperties, int offset) {
+    public static MonitorDevice streamInMonitorDevice(final Cache cache, final ScreenImpl screen,
+                                                      final MonitorMode currentMode,
+                                                      final float[] pixelScale,
+                                                      final ArrayHashSet<MonitorMode> supportedModes,
+                                                      final int[] monitorProperties, int offset,
+                                                      final int[] monitor_idx) {
         // min 11: count, id, ScreenSizeMM[width, height], Viewport[x, y, width, height], currentMonitorModeId, rotation, supportedModeId+
         final int count = monitorProperties[offset];
         if(MIN_MONITOR_DEVICE_PROPERTIES - 1 - NUM_MONITOR_MODE_PROPERTIES != count) {
@@ -353,12 +384,19 @@ public class MonitorModeProps {
         }
         offset++;
         final int id = monitorProperties[offset++];
+        final boolean isClone = 0 == monitorProperties[offset++] ? false : true;
+        final boolean isPrimary = 0 == monitorProperties[offset++] ? false : true;
         final DimensionImmutable sizeMM = streamInResolution(monitorProperties, offset); offset+=NUM_RESOLUTION_PROPERTIES;
         final Rectangle viewportPU = new Rectangle(monitorProperties[offset++], monitorProperties[offset++], monitorProperties[offset++], monitorProperties[offset++]);
         final Rectangle viewportWU = new Rectangle(monitorProperties[offset++], monitorProperties[offset++], monitorProperties[offset++], monitorProperties[offset++]);
-        MonitorDevice monitorDevice = new MonitorDeviceImpl(screen, id, sizeMM, viewportPU, viewportWU, currentMode, supportedModes);
+        MonitorDevice monitorDevice = new MonitorDeviceImpl(screen, id, isClone, isPrimary,
+                                                            sizeMM, currentMode, pixelScale,
+                                                            viewportPU, viewportWU, supportedModes);
         if(null!=cache) {
             monitorDevice = cache.monitorDevices.getOrAdd(monitorDevice);
+            if( monitorDevice.isPrimary() ) {
+                cache.setPrimary(monitorDevice);
+            }
         }
         if( null != monitor_idx ) {
             final int _monitorIdx  = cache.monitorDevices.indexOf(monitorDevice);
@@ -381,6 +419,8 @@ public class MonitorModeProps {
         int idx=0;
         data[idx++] = data.length;
         data[idx++] = monitorDevice.getId();
+        data[idx++] = monitorDevice.isClone() ? 1 : 0;
+        data[idx++] = monitorDevice.isPrimary() ? 1 : 0;
         data[idx++] = monitorDevice.getSizeMM().getWidth();
         data[idx++] = monitorDevice.getSizeMM().getHeight();
         data[idx++] = monitorDevice.getViewport().getX();
@@ -403,7 +443,41 @@ public class MonitorModeProps {
         return data;
     }
 
-    public final void swapRotatePair(final int rotation, final int[] pairs, int offset, final int numPairs) {
+    /**
+     * Identify monitor devices:
+     * <ul>
+     *   <li><i>cloned</i> mode, i.e. consecutive devices being 100% covered by preceding devices.</li>
+     * </ul>
+     */
+    /* pp */ static void identifyMonitorDevices(final MonitorModeProps.Cache cache) {
+        final ArrayList<MonitorDevice> monitors = cache.monitorDevices.toArrayList();
+        final int monitorCount = monitors.size();
+        for(int i=0; i<monitorCount; i++) {
+            final MonitorDevice a = monitors.get(i);
+            if( !a.isClone() ) {
+                for(int j=i+1; j<monitorCount; j++) {
+                    final MonitorDevice b = monitors.get(j);
+                    if( !b.isClone() ) {
+                        final float coverage = b.getViewport().coverage( a.getViewport() );
+                        if( FloatUtil.isZero( 1f - coverage, FloatUtil.EPSILON ) ) {
+                            ((MonitorDeviceImpl)b).setIsClone(true);
+                            if( Screen.DEBUG ) {
+                                System.err.printf("MonitorCloneTest[%d of %d]: %f -> _is_ covered%n", j, i, coverage);
+                                System.err.printf("  Monitor[%d] %s%n", j, b.toString());
+                                System.err.printf("  Monitor[%d] %s%n", i, a.toString());
+                            }
+                        } else if( Screen.DEBUG ) {
+                            System.err.printf("MonitorDevice-CloneTest[%d of %d]: %f -> not covered%n", j, i, coverage);
+                            System.err.printf("  Monitor[%d] %s%n", j, b.toString());
+                            System.err.printf("  Monitor[%d] %s%n", i, a.toString());
+                        }
+                    }
+                }
+            }
+        }
+    }
+
+    public static final void swapRotatePair(final int rotation, final int[] pairs, int offset, final int numPairs) {
         if( MonitorMode.ROTATE_0 == rotation || MonitorMode.ROTATE_180 == rotation ) {
             // nop
             return;
