@@ -38,11 +38,11 @@ import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.List;
 
-import javax.media.nativewindow.AbstractGraphicsScreen;
-import javax.media.nativewindow.NativeWindowException;
-import javax.media.nativewindow.util.Dimension;
-import javax.media.nativewindow.util.Rectangle;
-import javax.media.nativewindow.util.RectangleImmutable;
+import com.jogamp.nativewindow.AbstractGraphicsScreen;
+import com.jogamp.nativewindow.NativeWindowException;
+import com.jogamp.nativewindow.util.Dimension;
+import com.jogamp.nativewindow.util.Rectangle;
+import com.jogamp.nativewindow.util.RectangleImmutable;
 
 import com.jogamp.common.util.ArrayHashSet;
 import com.jogamp.common.util.PropertyAccess;
@@ -364,8 +364,8 @@ public abstract class ScreenImpl extends Screen implements MonitorModeListener {
      *   <li>{@link MonitorModeProps#MIN_MONITOR_DEVICE_PROPERTIES}</li>
      * </ul>, i.e.
      * <ul>
-     *   <li>{@link MonitorModeProps#streamInMonitorDevice(int[], jogamp.newt.MonitorModeProps.Cache, ScreenImpl, int[], int)}</li>
-     *   <li>{@link MonitorModeProps#streamInMonitorDevice(int[], jogamp.newt.MonitorModeProps.Cache, ScreenImpl, ArrayHashSet, int[], int)}</li>
+     *   <li>{@link MonitorModeProps#streamInMonitorDevice(jogamp.newt.MonitorModeProps.Cache, ScreenImpl, double[], int[], int, int[])}</li>
+     *   <li>{@link MonitorModeProps#streamInMonitorDevice(int[], jogamp.newt.MonitorModeProps.Cache, ArrayHashSet, int[], int, ScreenImpl)}</li>
      *   <li>{@link MonitorModeProps#streamInMonitorMode(int[], jogamp.newt.MonitorModeProps.Cache, int[], int)}</li>
      * </ul>
      * @param cache memory pool caching the result
@@ -373,6 +373,7 @@ public abstract class ScreenImpl extends Screen implements MonitorModeListener {
     protected abstract void collectNativeMonitorModesAndDevicesImpl(MonitorModeProps.Cache cache);
 
     protected boolean updateNativeMonitorDeviceViewportImpl(final MonitorDevice monitor,
+                                                            final float[] pixelScale,
                                                             final Rectangle viewportPU, final Rectangle viewportWU) {
         return false;
     }
@@ -405,6 +406,12 @@ public abstract class ScreenImpl extends Screen implements MonitorModeListener {
         return null != sms ? sms.getMonitorDevices().getData() : null;
     }
 
+    @Override
+    public MonitorDevice getPrimaryMonitor() {
+        final ScreenMonitorState sms = getScreenMonitorStatus(false);
+        return null != sms ? sms.getPrimaryMonitorDevice() : null;
+    }
+
     final ScreenMonitorState getScreenMonitorStatus(final boolean throwException) {
         final String key = this.getFQName();
         final ScreenMonitorState res = ScreenMonitorState.getScreenMonitorState(key);
@@ -428,9 +435,14 @@ public abstract class ScreenImpl extends Screen implements MonitorModeListener {
         final List<MonitorDevice> monitors = getMonitorDevices();
         for(int i=monitors.size()-1; i>=0; i--) {
             final MonitorDeviceImpl monitor = (MonitorDeviceImpl) monitors.get(i);
-            final boolean viewportUpdated = updateNativeMonitorDeviceViewportImpl(monitor, monitor.getMutuableViewportPU(), monitor.getMutuableViewportWU());
+            final float[] pixelScale = monitor.getPixelScale(new float[2]);
+            final boolean viewportUpdated = updateNativeMonitorDeviceViewportImpl(monitor, pixelScale,
+                                                                                  monitor.getMutuableViewportPU(),
+                                                                                  monitor.getMutuableViewportWU());
             if( DEBUG ) {
-                System.err.println("Screen.updateMonitorViewport["+i+"] @  "+Thread.currentThread().getName()+": updated: "+viewportUpdated+", PU "+monitor.getViewport()+", WU "+monitor.getViewportInWindowUnits());
+                System.err.println("Screen.updateMonitorViewport["+Display.toHexString(monitor.getId())+"] @  "+Thread.currentThread().getName()+": updated: "+viewportUpdated+
+                                   ", PU "+monitor.getViewport()+", WU "+monitor.getViewportInWindowUnits()+
+                                   ", pixelScale ["+pixelScale[0]+", "+pixelScale[1]+"]");
             }
         }
     }
@@ -494,6 +506,8 @@ public abstract class ScreenImpl extends Screen implements MonitorModeListener {
         int i = 0;
         props[i++] = MonitorModeProps.MIN_MONITOR_DEVICE_PROPERTIES;
         props[i++] = monitorId;
+        props[i++] = 0; // is-clone
+        props[i++] = 0 == monitorId ? 1 : 0; // is-primary
         props[i++] = default_sm_widthmm;
         props[i++] = default_sm_heightmm;
         props[i++] = 0; // rotated viewport x pixel-units
@@ -510,7 +524,7 @@ public abstract class ScreenImpl extends Screen implements MonitorModeListener {
         if( MonitorModeProps.MIN_MONITOR_DEVICE_PROPERTIES != i ) {
             throw new InternalError("XX");
         }
-        return MonitorModeProps.streamInMonitorDevice(null, cache, this, props, 0);
+        return MonitorModeProps.streamInMonitorDevice(cache, this, null, props, 0, null);
     }
 
     /**
@@ -555,7 +569,18 @@ public abstract class ScreenImpl extends Screen implements MonitorModeListener {
                     final MonitorMode mode = getVirtualMonitorMode(cache, 0);
                     cache.monitorModes.getOrAdd(mode);
                     final MonitorDevice monitor = getVirtualMonitorDevice(cache, 0, mode);
+                    ((MonitorDeviceImpl)monitor).setIsPrimary(true);
                     cache.monitorDevices.getOrAdd(monitor);
+                    cache.setPrimary(monitor);
+                }
+                if( null == cache.getPrimary() ) {
+                    // Fallback ..
+                    final MonitorDevice p = cache.monitorDevices.get(0);
+                    ((MonitorDeviceImpl)p).setIsPrimary(true);
+                    cache.setPrimary(p);
+                    if( DEBUG ) {
+                        System.err.println("WARNING: Fallback primary: "+p);
+                    }
                 }
                 // Sort MonitorModes (all and per device) in descending order - default!
                 MonitorModeUtil.sort(cache.monitorModes.getData(), false ); // descending order
@@ -576,8 +601,9 @@ public abstract class ScreenImpl extends Screen implements MonitorModeListener {
                             System.err.println("["+i+"]["+j+"]: "+iMode.next());
                         }
                     }
+                    System.err.println("Primary: "+cache.getPrimary());
                 }
-                sms = new ScreenMonitorState(cache.monitorDevices, cache.monitorModes);
+                sms = new ScreenMonitorState(cache.monitorDevices, cache.monitorModes, cache.getPrimary());
                 ScreenMonitorState.mapScreenMonitorState(this.getFQName(), sms);
             }
         } finally {
@@ -602,6 +628,7 @@ public abstract class ScreenImpl extends Screen implements MonitorModeListener {
     private final int collectNativeMonitorModes(final MonitorModeProps.Cache cache) {
         if(!DEBUG_TEST_SCREENMODE_DISABLED) {
             collectNativeMonitorModesAndDevicesImpl(cache);
+            MonitorModeProps.identifyMonitorDevices(cache);
         }
         // filter out insufficient modes
         for(int i=cache.monitorModes.size()-1; i>=0; i--) {
