@@ -29,30 +29,28 @@
 package jogamp.newt.driver.android;
 
 import jogamp.common.os.android.StaticContext;
-import jogamp.newt.WindowImpl;
 import jogamp.newt.driver.android.event.AndroidNewtEventFactory;
 import jogamp.newt.driver.android.event.AndroidNewtEventTranslator;
 
-import javax.media.nativewindow.AbstractGraphicsScreen;
-import javax.media.nativewindow.Capabilities;
-import javax.media.nativewindow.CapabilitiesImmutable;
-import javax.media.nativewindow.DefaultGraphicsScreen;
-import javax.media.nativewindow.NativeWindowException;
-import javax.media.nativewindow.VisualIDHolder;
-import javax.media.nativewindow.util.Insets;
-import javax.media.nativewindow.util.Point;
-import javax.media.nativewindow.util.Rectangle;
-import javax.media.nativewindow.util.RectangleImmutable;
-import javax.media.opengl.GLCapabilitiesChooser;
-import javax.media.opengl.GLCapabilitiesImmutable;
-import javax.media.opengl.GLException;
+import com.jogamp.nativewindow.AbstractGraphicsScreen;
+import com.jogamp.nativewindow.Capabilities;
+import com.jogamp.nativewindow.CapabilitiesImmutable;
+import com.jogamp.nativewindow.DefaultGraphicsScreen;
+import com.jogamp.nativewindow.NativeWindowException;
+import com.jogamp.nativewindow.VisualIDHolder;
+import com.jogamp.nativewindow.util.Point;
+import com.jogamp.nativewindow.util.RectangleImmutable;
+import com.jogamp.opengl.GLCapabilitiesChooser;
+import com.jogamp.opengl.GLCapabilitiesImmutable;
+import com.jogamp.opengl.GLException;
 
+import com.jogamp.common.ExceptionUtils;
 import com.jogamp.common.os.AndroidVersion;
 import com.jogamp.nativewindow.egl.EGLGraphicsDevice;
 import com.jogamp.newt.MonitorDevice;
 import com.jogamp.newt.Window;
+import com.jogamp.opengl.egl.EGL;
 
-import jogamp.opengl.egl.EGL;
 import jogamp.opengl.egl.EGLDisplayUtil;
 import jogamp.opengl.egl.EGLGraphicsConfiguration;
 import jogamp.opengl.egl.EGLGraphicsConfigurationFactory;
@@ -265,8 +263,8 @@ public class WindowDriver extends jogamp.newt.WindowImpl implements Callback2 {
     public final SurfaceView getAndroidView() { return androidView; }
 
     @Override
-    protected final void instantiationFinished() {
-        Log.d(MD.TAG, "instantiationFinished() - "+Thread.currentThread().getName());
+    protected final void instantiationFinishedImpl() {
+        Log.d(MD.TAG, "instantiationFinishedImpl() - "+Thread.currentThread().getName());
 
         final Context ctx = StaticContext.getContext();
         if(null == ctx) {
@@ -282,7 +280,7 @@ public class WindowDriver extends jogamp.newt.WindowImpl implements Callback2 {
     protected final boolean canCreateNativeImpl() {
         Log.d(MD.TAG, "canCreateNativeImpl.0: surfaceHandle ready "+(0!=surfaceHandle)+" - on thread "+Thread.currentThread().getName());
         if(Window.DEBUG_IMPLEMENTATION) {
-            Thread.dumpStack();
+            ExceptionUtils.dumpStack(System.err);
         }
 
         if( isFullscreen() ) {
@@ -386,7 +384,7 @@ public class WindowDriver extends jogamp.newt.WindowImpl implements Callback2 {
                     "], pixel["+getSurfaceWidth()+"x"+getSurfaceHeight()+"],"+
                     " - on thread "+Thread.currentThread().getName());
         if(Window.DEBUG_IMPLEMENTATION) {
-            Thread.dumpStack();
+            ExceptionUtils.dumpStack(System.err);
         }
 
         setupInputListener(false);
@@ -455,11 +453,16 @@ public class WindowDriver extends jogamp.newt.WindowImpl implements Callback2 {
         }
     }
 
+    @Override
+    protected final int getSupportedReconfigMaskImpl() {
+        return minimumReconfigStateMask;
+    }
+
     @Override
     protected final boolean reconfigureWindowImpl(final int x, final int y, final int width, final int height, final int flags) {
         boolean res = true;
 
-        if( 0 != ( FLAG_CHANGE_FULLSCREEN & flags) ) {
+        if( 0 != ( CHANGE_MASK_FULLSCREEN & flags) ) {
             Log.d(MD.TAG, "reconfigureWindowImpl.setFullscreen post creation (setContentView()) n/a");
             return false;
         }
@@ -479,8 +482,8 @@ public class WindowDriver extends jogamp.newt.WindowImpl implements Callback2 {
                 definePosition(x, y);
             }
         }
-        if( 0 != ( FLAG_CHANGE_VISIBILITY & flags) ) {
-            visibleChanged(false, 0 != ( FLAG_IS_VISIBLE & flags));
+        if( 0 != ( CHANGE_MASK_VISIBILITY & flags) ) {
+            visibleChanged(false, 0 != ( STATE_MASK_VISIBLE & flags));
         }
         return res;
     }
@@ -490,11 +493,6 @@ public class WindowDriver extends jogamp.newt.WindowImpl implements Callback2 {
         return new Point(x,y);
     }
 
-    @Override
-    protected final void updateInsetsImpl(final Insets insets) {
-        // nop ..
-    }
-
     //----------------------------------------------------------------------
     // Virtual On-Screen Keyboard / SoftInput
     //
@@ -557,7 +555,7 @@ public class WindowDriver extends jogamp.newt.WindowImpl implements Callback2 {
     public final void surfaceChanged(final SurfaceHolder aHolder, final int aFormat, final int aWidth, final int aHeight) {
         Log.d(MD.TAG, "surfaceChanged: f "+nativeFormat+" -> "+aFormat+", "+aWidth+"x"+aHeight+", current surfaceHandle: 0x"+Long.toHexString(surfaceHandle)+" - on thread "+Thread.currentThread().getName());
         if(Window.DEBUG_IMPLEMENTATION) {
-            Thread.dumpStack();
+            ExceptionUtils.dumpStack(System.err);
         }
         if(0!=surfaceHandle && androidFormat != aFormat ) {
             // re-create
@@ -611,7 +609,7 @@ public class WindowDriver extends jogamp.newt.WindowImpl implements Callback2 {
     public final void surfaceDestroyed(final SurfaceHolder holder) {
         Log.d(MD.TAG, "surfaceDestroyed - on thread "+Thread.currentThread().getName());
         windowDestroyNotify(true); // actually too late .. however ..
-        Thread.dumpStack();
+        ExceptionUtils.dumpStack(System.err);
     }
 
     @Override
