@@ -34,16 +34,16 @@
 
 package jogamp.newt.driver.macosx;
 
-import javax.media.nativewindow.AbstractGraphicsConfiguration;
-import javax.media.nativewindow.GraphicsConfigurationFactory;
-import javax.media.nativewindow.NativeWindow;
-import javax.media.nativewindow.NativeWindowException;
-import javax.media.nativewindow.MutableSurface;
-import javax.media.nativewindow.ScalableSurface;
-import javax.media.nativewindow.VisualIDHolder;
-import javax.media.nativewindow.util.Insets;
-import javax.media.nativewindow.util.Point;
-import javax.media.nativewindow.util.PointImmutable;
+import com.jogamp.common.util.InterruptSource;
+import com.jogamp.nativewindow.AbstractGraphicsConfiguration;
+import com.jogamp.nativewindow.GraphicsConfigurationFactory;
+import com.jogamp.nativewindow.NativeWindow;
+import com.jogamp.nativewindow.NativeWindowException;
+import com.jogamp.nativewindow.MutableSurface;
+import com.jogamp.nativewindow.ScalableSurface;
+import com.jogamp.nativewindow.VisualIDHolder;
+import com.jogamp.nativewindow.util.Point;
+import com.jogamp.nativewindow.util.PointImmutable;
 
 import jogamp.nativewindow.SurfaceScaleUtils;
 import jogamp.nativewindow.macosx.OSXUtil;
@@ -67,20 +67,26 @@ public class WindowDriver extends WindowImpl implements MutableSurface, DriverCl
     public WindowDriver() {
     }
 
-    private boolean updatePixelScale(final boolean sendEvent, final boolean defer, final float newPixelScaleRaw, final float nativePixelScaleRaw) {
-        final int[] newPixelScale = new int[2];
+    private boolean updatePixelScale(final boolean sendEvent, final boolean defer, final boolean deferOffThread,
+                                     final float newPixelScaleRaw, final float maxPixelScaleRaw) {
+        final float[] newPixelScale = new float[2];
         {
-            final int _newPixelScale = FloatUtil.isZero(newPixelScaleRaw, FloatUtil.EPSILON) ? ScalableSurface.IDENTITY_PIXELSCALE : (int) newPixelScaleRaw;
+            final float _newPixelScale = FloatUtil.isZero(newPixelScaleRaw, FloatUtil.EPSILON) ? ScalableSurface.IDENTITY_PIXELSCALE : newPixelScaleRaw;
             newPixelScale[0]= _newPixelScale;
             newPixelScale[1]= _newPixelScale;
-            final int _nativePixelScale = FloatUtil.isZero(nativePixelScaleRaw, FloatUtil.EPSILON) ? ScalableSurface.IDENTITY_PIXELSCALE : (int) nativePixelScaleRaw;
-            nativePixelScale[0]= _nativePixelScale;
-            nativePixelScale[1]= _nativePixelScale;
+            final float _maxPixelScale = FloatUtil.isZero(maxPixelScaleRaw, FloatUtil.EPSILON) ? ScalableSurface.IDENTITY_PIXELSCALE : maxPixelScaleRaw;
+            maxPixelScale[0]= _maxPixelScale;
+            maxPixelScale[1]= _maxPixelScale;
         }
+        // We keep minPixelScale at [1f, 1f]!
 
-        if( SurfaceScaleUtils.computePixelScale(hasPixelScale, hasPixelScale, reqPixelScale, newPixelScale, DEBUG_IMPLEMENTATION ? getClass().getName() : null) ) {
+        if( SurfaceScaleUtils.setNewPixelScale(hasPixelScale, hasPixelScale, newPixelScale, minPixelScale, maxPixelScale, DEBUG_IMPLEMENTATION ? getClass().getName() : null) ) {
             if( sendEvent ) {
-                super.sizeChanged(defer, getWidth(), getHeight(), true);
+                if( deferOffThread ) {
+                    superSizeChangedOffThread(defer, getWidth(), getHeight(), true);
+                } else {
+                    super.sizeChanged(defer, getWidth(), getHeight(), true);
+                }
             } else {
                 defineSize(getWidth(), getHeight());
             }
@@ -90,47 +96,46 @@ public class WindowDriver extends WindowImpl implements MutableSurface, DriverCl
         }
     }
 
-    private boolean updatePixelScaleByScreenIdx(final boolean sendEvent) {
-        final float nativePixelScaleRaw = (float) OSXUtil.GetPixelScale(getScreen().getIndex());
+    private boolean updatePixelScaleByDisplayID(final boolean sendEvent) {
+        final float maxPixelScaleRaw = (float) OSXUtil.GetPixelScaleByDisplayID(getDisplayID());
         if( DEBUG_IMPLEMENTATION ) {
-            System.err.println("WindowDriver.updatePixelScale.1: "+hasPixelScale[0]+", "+nativePixelScaleRaw+" (native)");
+            System.err.println("WindowDriver.updatePixelScale.1: "+hasPixelScale[0]+", "+maxPixelScaleRaw+" (max)");
         }
-        return updatePixelScale(sendEvent, true /* defer */, nativePixelScaleRaw, nativePixelScaleRaw);
+        return updatePixelScale(sendEvent, true /* defer */, false /*offthread */, maxPixelScaleRaw, maxPixelScaleRaw);
     }
 
     private boolean updatePixelScaleByWindowHandle(final boolean sendEvent) {
         final long handle = getWindowHandle();
         if( 0 != handle ) {
-            final float nativePixelScaleRaw = (float)OSXUtil.GetPixelScale(handle);
+            final float maxPixelScaleRaw = (float)OSXUtil.GetPixelScale(handle);
             if( DEBUG_IMPLEMENTATION ) {
-                System.err.println("WindowDriver.updatePixelScale.2: "+hasPixelScale[0]+", "+nativePixelScaleRaw+" (native)");
+                System.err.println("WindowDriver.updatePixelScale.2: "+hasPixelScale[0]+", "+maxPixelScaleRaw+" (max)");
             }
-            return updatePixelScale(sendEvent, true /* defer */, nativePixelScaleRaw, nativePixelScaleRaw);
+            return updatePixelScale(sendEvent, true /* defer */, false /*offthread */, maxPixelScaleRaw, maxPixelScaleRaw);
         } else {
             return false;
         }
     }
 
     /** Called from native code */
-    protected void updatePixelScale(final boolean defer, final float newPixelScaleRaw, final float nativePixelScaleRaw) {
-        final long handle = getWindowHandle();
+    protected void updatePixelScale(final boolean defer, final float newPixelScaleRaw, final float maxPixelScaleRaw) {
         if( DEBUG_IMPLEMENTATION ) {
-            System.err.println("WindowDriver.updatePixelScale.3: "+hasPixelScale[0]+" (has) -> "+newPixelScaleRaw+" (raw), "+nativePixelScaleRaw+" (native), drop "+(0==handle));
+            System.err.println("WindowDriver.updatePixelScale.3: "+hasPixelScale[0]+" (has) -> "+newPixelScaleRaw+" (new), "+maxPixelScaleRaw+" (max), drop "+!isNativeValid());
         }
-        if( 0 != handle ) {
-            updatePixelScale(true /* sendEvent*/, defer, newPixelScaleRaw, nativePixelScaleRaw);
+        if( isNativeValid() ) {
+            updatePixelScale(true /* sendEvent*/, defer, true /*offthread */, newPixelScaleRaw, maxPixelScaleRaw);
         }
     }
 
     @Override
-    protected final void instantiationFinished() {
-        updatePixelScaleByScreenIdx(false /* sendEvent*/);
+    protected final void instantiationFinishedImpl() {
+        updatePixelScaleByDisplayID(false /* sendEvent*/);
     }
 
     @Override
     protected void setScreen(final ScreenImpl newScreen) { // never null !
         super.setScreen(newScreen);
-        updatePixelScaleByScreenIdx(false /* sendEvent*/);  // caller (reparent, ..) will send reshape event
+        updatePixelScaleByDisplayID(false /* sendEvent*/);  // caller (reparent, ..) will send reshape event
     }
 
     @Override
@@ -139,44 +144,47 @@ public class WindowDriver extends WindowImpl implements MutableSurface, DriverCl
     }
 
     @Override
-    public final void setSurfaceScale(final int[] pixelScale) {
-        SurfaceScaleUtils.validateReqPixelScale(reqPixelScale, pixelScale, DEBUG_IMPLEMENTATION ? getClass().getName() : null);
+    public final boolean setSurfaceScale(final float[] pixelScale) {
+        super.setSurfaceScale(pixelScale);
 
-        final int[] resPixelScale;
+        boolean changed = false;
         if( isNativeValid() ) {
             if( isOffscreenInstance ) {
                 final NativeWindow pWin = getParent();
                 if( pWin instanceof ScalableSurface ) {
                     final ScalableSurface sSurf = (ScalableSurface)pWin;
                     sSurf.setSurfaceScale(reqPixelScale);
-                    final int[] pPixelScale = sSurf.getCurrentSurfaceScale(new int[2]);
-                    sSurf.getNativeSurfaceScale(nativePixelScale);
-                    updatePixelScale(true /* sendEvent */, true /* defer */, pPixelScale[0], nativePixelScale[0]); // HiDPI: uniformPixelScale
+                    sSurf.getMaximumSurfaceScale(maxPixelScale);
+                    sSurf.getMinimumSurfaceScale(minPixelScale);
+                    final float[] pPixelScale = sSurf.getCurrentSurfaceScale(new float[2]);
+                    changed = updatePixelScale(true /* sendEvent */, true /* defer */, true /*offthread */, pPixelScale[0], maxPixelScale[0]); // HiDPI: uniformPixelScale
                 } else {
                     // just notify updated pixelScale if offscreen
-                    SurfaceScaleUtils.replaceAutoMaxWithPlatformMax(reqPixelScale);
-                    updatePixelScale(true /* sendEvent */, true /* defer */, reqPixelScale[0], nativePixelScale[0]); // HiDPI: uniformPixelScale
+                    changed = updatePixelScale(true /* sendEvent */, true /* defer */, true /*offthread */, reqPixelScale[0], maxPixelScale[0]); // HiDPI: uniformPixelScale
                 }
             } else {
-                // set pixelScale in native code, will issue an update PixelScale
-                OSXUtil.RunOnMainThread(true, false, new Runnable() {
-                    @Override
-                    public void run() {
-                        setPixelScale0(getWindowHandle(), surfaceHandle, reqPixelScale[0]); // HiDPI: uniformPixelScale
-                    }
-                } );
+                // set pixelScale in native code, will issue an update updatePixelScale(..)
+                // hence we pre-query whether pixel-scale will change, without affecting current state 'hasPixelScale'!
+                final float[] _hasPixelScale = new float[2];
+                System.arraycopy(hasPixelScale, 0, _hasPixelScale, 0, 2);
+                if( SurfaceScaleUtils.setNewPixelScale(_hasPixelScale, _hasPixelScale, reqPixelScale, minPixelScale, maxPixelScale, DEBUG_IMPLEMENTATION ? getClass().getName() : null) ) {
+                    OSXUtil.RunOnMainThread(true, false, new Runnable() {
+                        @Override
+                        public void run() {
+                            setPixelScale0(getWindowHandle(), surfaceHandle, _hasPixelScale[0]); // HiDPI: uniformPixelScale
+                        }
+                    } );
+                    changed = true;
+                }
             }
-            resPixelScale = hasPixelScale;
-        } else {
-            hasPixelScale[0] = ScalableSurface.IDENTITY_PIXELSCALE;
-            hasPixelScale[1] = ScalableSurface.IDENTITY_PIXELSCALE;
-            resPixelScale = reqPixelScale;
         }
         if( DEBUG_IMPLEMENTATION ) {
-            System.err.println("WindowDriver.setPixelScale: "+pixelScale[0]+"x"+pixelScale[1]+" (req) -> "+
-                                reqPixelScale[0]+"x"+reqPixelScale[1]+" (validated) -> "+
-                                resPixelScale[0]+"x"+resPixelScale[1]+" (result) - realized "+isNativeValid());
+            System.err.println("WindowDriver.setPixelScale: min["+minPixelScale[0]+", "+minPixelScale[1]+"], max["+
+                                maxPixelScale[0]+", "+maxPixelScale[1]+"], req["+
+                                reqPixelScale[0]+", "+reqPixelScale[1]+"] -> result["+
+                                hasPixelScale[0]+", "+hasPixelScale[1]+"] - changed "+changed+", realized "+isNativeValid());
         }
+        return changed;
     }
 
     @Override
@@ -187,8 +195,8 @@ public class WindowDriver extends WindowImpl implements MutableSurface, DriverCl
             throw new NativeWindowException("Error choosing GraphicsConfiguration creating window: "+this);
         }
         setGraphicsConfiguration(cfg);
-        reconfigureWindowImpl(getX(), getY(), getWidth(), getHeight(), getReconfigureFlags(FLAG_CHANGE_VISIBILITY, true));
-        if (0 == getWindowHandle()) {
+        reconfigureWindowImpl(getX(), getY(), getWidth(), getHeight(), getReconfigureMask(CHANGE_MASK_VISIBILITY, true));
+        if ( !isNativeValid() ) {
             throw new NativeWindowException("Error creating window");
         }
     }
@@ -260,7 +268,7 @@ public class WindowDriver extends WindowImpl implements MutableSurface, DriverCl
             System.err.println("MacWindow.setSurfaceHandle(): 0x"+Long.toHexString(surfaceHandle));
         }
         sscSurfaceHandle = surfaceHandle;
-        if (isNativeValid()) {
+        if ( isNativeValid() ) {
             if (0 != sscSurfaceHandle) {
                 OSXUtil.RunOnMainThread(false, false, new Runnable() {
                         @Override
@@ -332,15 +340,20 @@ public class WindowDriver extends WindowImpl implements MutableSurface, DriverCl
         if( 0 != handle && !isOffscreenInstance ) {
             final NativeWindow parent = getParent();
             final boolean useParent = useParent(parent);
-            final int pX=parent.getX(), pY=parent.getY();
-            final Point p0S = getLocationOnScreenImpl(x, y, parent, useParent);
+            final Point p0S;
+            if( useParent ) {
+                p0S = getLocationOnScreenByParent(x, y, parent);
+            } else {
+                p0S = (Point) getLocationOnScreen0(handle, x, y);
+            }
             if(DEBUG_IMPLEMENTATION) {
+                final int pX=parent.getX(), pY=parent.getY();
                 System.err.println("MacWindow: updatePosition() parent["+useParent+" "+pX+"/"+pY+"] "+x+"/"+y+" ->  "+x+"/"+y+" rel-client-pos, "+p0S+" screen-client-pos");
             }
             OSXUtil.RunOnMainThread(false, false, new Runnable() {
                     @Override
                     public void run() {
-                        setWindowClientTopLeftPoint0(handle, p0S.getX(), p0S.getY(), isVisible());
+                        setWindowClientTopLeftPoint0(getWindowHandle(), p0S.getX(), p0S.getY(), isVisible());
                     } } );
             // no native event (fullscreen, some reparenting)
             positionChanged(true, x, y);
@@ -348,64 +361,67 @@ public class WindowDriver extends WindowImpl implements MutableSurface, DriverCl
     }
 
     @Override
-    protected void sizeChanged(final boolean defer, final int newWidth, final int newHeight, final boolean force) {
-        final long handle = getWindowHandle();
-        if( 0 != handle && !isOffscreenInstance ) {
-            final NativeWindow parent = getParent();
-            final boolean useParent = useParent(parent);
-            if( useParent && ( getWidth() != newWidth || getHeight() != newHeight ) ) {
-                final int x=getX(), y=getY();
-                final Point p0S = getLocationOnScreenImpl(x, y, parent, useParent);
-                if(DEBUG_IMPLEMENTATION) {
-                    System.err.println("MacWindow: sizeChanged() parent["+useParent+" "+x+"/"+y+"] "+getX()+"/"+getY()+" "+newWidth+"x"+newHeight+" ->  "+p0S+" screen-client-pos");
-                }
-                OSXUtil.RunOnMainThread(false, false, new Runnable() {
-                        @Override
-                        public void run() {
-                            setWindowClientTopLeftPoint0(getWindowHandle(), p0S.getX(), p0S.getY(), isVisible());
-                        } } );
-            }
-        }
-        super.sizeChanged(defer, newWidth, newHeight, force);
+    protected final int getSupportedReconfigMaskImpl() {
+        return minimumReconfigStateMask |
+               STATE_MASK_CHILDWIN |
+               STATE_MASK_UNDECORATED |
+               STATE_MASK_ALWAYSONTOP |
+               STATE_MASK_ALWAYSONBOTTOM |
+               STATE_MASK_STICKY |
+               STATE_MASK_RESIZABLE |
+               STATE_MASK_MAXIMIZED_VERT |
+               STATE_MASK_MAXIMIZED_HORZ |
+               STATE_MASK_POINTERVISIBLE |
+               STATE_MASK_POINTERCONFINED;
     }
 
     @Override
-    protected boolean reconfigureWindowImpl(int x, int y, final int width, final int height, final int flags) {
+    protected boolean reconfigureWindowImpl(int _x, int _y, int _width, int _height, final int flags) {
         final boolean _isOffscreenInstance = isOffscreenInstance(this, this.getParent());
         isOffscreenInstance = 0 != sscSurfaceHandle || _isOffscreenInstance;
         final PointImmutable pClientLevelOnSreen;
         if( isOffscreenInstance ) {
-            x = 0; y = 0;
+            _x = 0; _y = 0;
             pClientLevelOnSreen = new Point(0, 0);
         } else  {
             final NativeWindow parent = getParent();
-            final boolean useParent = useParent(parent);
-            if( useParent ) {
-                pClientLevelOnSreen = getLocationOnScreenImpl(x, y, parent, useParent);
+            if( useParent(parent) ) {
+                pClientLevelOnSreen = getLocationOnScreenByParent(_x, _y, parent);
             } else {
-                pClientLevelOnSreen = new Point(x, y);
+                if( 0 != ( ( CHANGE_MASK_MAXIMIZED_HORZ | CHANGE_MASK_MAXIMIZED_VERT ) & flags ) ) {
+                    final int[] posSize = { _x, _y, _width, _height };
+                    reconfigMaximizedManual(flags, posSize, getInsets());
+                    _x = posSize[0];
+                    _y = posSize[1];
+                    _width = posSize[2];
+                    _height = posSize[3];
+                }
+                pClientLevelOnSreen = new Point(_x, _y);
             }
         }
+        final int x=_x, y=_y;
+        final int width=_width, height=_height;
 
-        final boolean setVisible = 0 != ( FLAG_IS_VISIBLE & flags);
         final boolean hasFocus = hasFocus();
 
         if(DEBUG_IMPLEMENTATION) {
             final AbstractGraphicsConfiguration cWinCfg = this.getGraphicsConfiguration();
             final NativeWindow pWin = getParent();
             final AbstractGraphicsConfiguration pWinCfg = null != pWin ? pWin.getGraphicsConfiguration() : null;
-            System.err.println("MacWindow reconfig.0: "+x+"/"+y+" -> clientPos "+pClientLevelOnSreen+" - "+width+"x"+height+
+            System.err.println("MacWindow reconfig.0: "+x+"/"+y+" -> clientPosOnScreen "+pClientLevelOnSreen+" - "+width+"x"+height+
+                               ", "+getReconfigStateMaskString(flags)+
                                ",\n\t parent type "+(null != pWin ? pWin.getClass().getName() : null)+
                                ",\n\t   this-chosenCaps "+(null != cWinCfg ? cWinCfg.getChosenCapabilities() : null)+
                                ",\n\t parent-chosenCaps "+(null != pWinCfg ? pWinCfg.getChosenCapabilities() : null)+
                                ", isOffscreenInstance(sscSurfaceHandle "+toHexString(sscSurfaceHandle)+
                                ", ioi: "+_isOffscreenInstance+
-                               ") -> "+isOffscreenInstance+
-                               "\n\t, "+getReconfigureFlagsAsString(null, flags)+", setVisible "+setVisible+", hasFocus "+hasFocus);
+                               ") -> "+isOffscreenInstance);
             // Thread.dumpStack();
         }
 
-        if( 0 != ( FLAG_CHANGE_VISIBILITY & flags) && !setVisible ) {
+        if( 0 != ( CHANGE_MASK_VISIBILITY & flags) &&
+            0 == ( STATE_MASK_VISIBLE & flags) )
+        {
             if ( !isOffscreenInstance ) {
                 OSXUtil.RunOnMainThread(false, false, new Runnable() {
                         @Override
@@ -417,38 +433,51 @@ public class WindowDriver extends WindowImpl implements MutableSurface, DriverCl
                 visibleChanged(true, false);
             }
         }
-        if( 0 == getWindowHandle() && setVisible ||
-            0 != ( FLAG_CHANGE_DECORATION & flags) ||
-            0 != ( FLAG_CHANGE_PARENTING & flags) ||
-            0 != ( FLAG_CHANGE_FULLSCREEN & flags) ) {
+        final long oldWindowHandle = getWindowHandle();
+        if( ( 0 == oldWindowHandle && 0 != ( STATE_MASK_VISIBLE & flags) ) ||
+            0 != ( CHANGE_MASK_PARENTING & flags)  ||
+            0 != ( CHANGE_MASK_DECORATION & flags) ||
+            0 != ( CHANGE_MASK_ALWAYSONTOP & flags) ||
+            0 != ( CHANGE_MASK_ALWAYSONBOTTOM & flags) ||
+            0 != ( CHANGE_MASK_RESIZABLE & flags)  ||
+            0 != ( CHANGE_MASK_FULLSCREEN & flags) ) {
             if(isOffscreenInstance) {
-                createWindow(true, 0 != getWindowHandle(), pClientLevelOnSreen, 64, 64, false, setVisible, false);
+                createWindow(true, 0 != oldWindowHandle, pClientLevelOnSreen, 64, 64, flags);
             } else {
-                createWindow(false, 0 != getWindowHandle(), pClientLevelOnSreen, width, height,
-                                    0 != ( FLAG_IS_FULLSCREEN & flags), setVisible, 0 != ( FLAG_IS_ALWAYSONTOP & flags));
+                createWindow(false, 0 != oldWindowHandle, pClientLevelOnSreen, width, height, flags);
             }
             // no native event (fullscreen, some reparenting)
-            positionChanged(false,  x, y);
             updatePixelScaleByWindowHandle(false /* sendEvent */);
-            super.sizeChanged(false, width, height, true);
-            visibleChanged(false, setVisible);
+            if( isOffscreenInstance) {
+                super.sizeChanged(false, width, height, true);
+                positionChanged(false,  x, y);
+            } else {
+                updateSizePosInsets0(getWindowHandle(), false);
+            }
+            visibleChanged(false, 0 != ( STATE_MASK_VISIBLE & flags));
             if( hasFocus ) {
                 requestFocusImpl(true);
             }
-        } else {
+        } else if( 0 != oldWindowHandle ) {
             if( width>0 && height>0 ) {
                 if( !isOffscreenInstance ) {
-                    OSXUtil.RunOnMainThread(false, false, new Runnable() {
+                    OSXUtil.RunOnMainThread(true, false, new Runnable() {
                             @Override
                             public void run() {
-                                setWindowClientTopLeftPointAndSize0(getWindowHandle(), pClientLevelOnSreen.getX(), pClientLevelOnSreen.getY(), width, height, setVisible);
+                                setWindowClientTopLeftPointAndSize0(oldWindowHandle,
+                                        pClientLevelOnSreen.getX(), pClientLevelOnSreen.getY(),
+                                        width, height, 0 != ( STATE_MASK_VISIBLE & flags));
                             } } );
-                } // else offscreen size is realized via recreation
-                // no native event (fullscreen, some reparenting)
-                positionChanged(true,  x, y);
-                super.sizeChanged(true, width, height, false);
+                    updateSizePosInsets0(oldWindowHandle, false);
+                } else { // else offscreen size is realized via recreation
+                    // no native event (fullscreen, some reparenting)
+                    super.sizeChanged(false, width, height, false);
+                    positionChanged(false,  x, y);
+                }
             }
-            if( 0 != ( FLAG_CHANGE_VISIBILITY & flags) && setVisible ) {
+            if( 0 != ( CHANGE_MASK_VISIBILITY & flags) &&
+                0 != ( STATE_MASK_VISIBLE & flags) )
+            {
                 if( !isOffscreenInstance ) {
                     OSXUtil.RunOnMainThread(false, false, new Runnable() {
                             @Override
@@ -460,12 +489,11 @@ public class WindowDriver extends WindowImpl implements MutableSurface, DriverCl
                     visibleChanged(true, true);
                 }
             }
-            if( !isOffscreenInstance ) {
-                setAlwaysOnTop0(getWindowHandle(), 0 != ( FLAG_IS_ALWAYSONTOP & flags));
-            }
+        } else {
+            throw new InternalError("Null windowHandle but no re-creation triggered, check visibility: "+getStateMaskString());
         }
         if(DEBUG_IMPLEMENTATION) {
-            System.err.println("MacWindow reconfig.X: clientPos "+pClientLevelOnSreen+", "+width+"x"+height+" -> clientPos "+getLocationOnScreenImpl(0, 0)+", insets: "+getInsets());
+            System.err.println("MacWindow reconfig.X: "+getLocationOnScreenImpl(0, 0)+" "+getWidth()+"x"+getHeight()+", insets "+getInsets()+", "+getStateMaskString());
         }
         return true;
     }
@@ -473,52 +501,104 @@ public class WindowDriver extends WindowImpl implements MutableSurface, DriverCl
     @Override
     protected Point getLocationOnScreenImpl(final int x, final int y) {
         final NativeWindow parent = getParent();
-        final boolean useParent = useParent(parent);
-        return getLocationOnScreenImpl(x, y, parent, useParent);
+        if( useParent(parent) ) {
+            return getLocationOnScreenByParent(x, y, parent);
+        } else {
+            final long windowHandle = getWindowHandle();
+            if( !isOffscreenInstance && 0 != windowHandle ) {
+                return (Point) getLocationOnScreen0(windowHandle, x, y);
+            } else {
+                return new Point(x, y);
+            }
+        }
     }
 
-    private Point getLocationOnScreenImpl(final int x, final int y, final NativeWindow parent, final boolean useParent) {
-        if( !useParent && !isOffscreenInstance && 0 != surfaceHandle) {
-            return OSXUtil.GetLocationOnScreen(surfaceHandle, x, y);
-        }
-
-        final Point p = new Point(x, y);
-        if( useParent ) {
-            p.translate( parent.getLocationOnScreen(null) );
-        }
-        return p;
-    }
-
-    @Override
-    protected void updateInsetsImpl(final Insets insets) {
-        // nop - using event driven insetsChange(..)
+    private Point getLocationOnScreenByParent(final int x, final int y, final NativeWindow parent) {
+        return new Point(x, y).translate( parent.getLocationOnScreen(null) );
     }
 
     /** Callback for native screen position change event of the client area. */
     protected void screenPositionChanged(final boolean defer, final int newX, final int newY) {
         // passed coordinates are in screen position of the client area
-        if(getWindowHandle()!=0) {
+        if( isNativeValid() ) {
             final NativeWindow parent = getParent();
-            if( null == parent || isOffscreenInstance ) {
+            if( !useParent(parent) || isOffscreenInstance ) {
                 if(DEBUG_IMPLEMENTATION) {
                     System.err.println("MacWindow.positionChanged.0 (Screen Pos - TOP): ("+getThreadName()+"): (defer: "+defer+") "+getX()+"/"+getY()+" -> "+newX+"/"+newY);
                 }
                 positionChanged(defer, newX, newY);
             } else {
-                // screen position -> rel child window position
-                final Point absPos = new Point(newX, newY);
-                final Point parentOnScreen = parent.getLocationOnScreen(null);
-                absPos.translate( parentOnScreen.scale(-1, -1) );
-                if(DEBUG_IMPLEMENTATION) {
-                    System.err.println("MacWindow.positionChanged.1 (Screen Pos - CHILD): ("+getThreadName()+"): (defer: "+defer+") "+getX()+"/"+getY()+" -> absPos "+newX+"/"+newY+", parentOnScreen "+parentOnScreen+" -> "+absPos);
+                final Runnable action = new Runnable() {
+                    public void run() {
+                        // screen position -> rel child window position
+                        final Point absPos = new Point(newX, newY);
+                        final Point parentOnScreen = parent.getLocationOnScreen(null);
+                        absPos.translate( parentOnScreen.scale(-1, -1) );
+                        if(DEBUG_IMPLEMENTATION) {
+                            System.err.println("MacWindow.positionChanged.1 (Screen Pos - CHILD): ("+getThreadName()+"): (defer: "+defer+") "+getX()+"/"+getY()+" -> absPos "+newX+"/"+newY+", parentOnScreen "+parentOnScreen+" -> "+absPos);
+                        }
+                        positionChanged(false, absPos.getX(), absPos.getY());
+                    } };
+                if( defer ) {
+                    new InterruptSource.Thread(null, action).start();
+                } else {
+                    action.run();
                 }
-                positionChanged(defer, absPos.getX(), absPos.getY());
+
             }
         } else if(DEBUG_IMPLEMENTATION) {
             System.err.println("MacWindow.positionChanged.2 (Screen Pos - IGN): ("+getThreadName()+"): (defer: "+defer+") "+getX()+"/"+getY()+" -> "+newX+"/"+newY);
         }
     }
 
+    @Override
+    protected void sizeChanged(final boolean defer, final int newWidth, final int newHeight, final boolean force) {
+        if(force || getWidth() != newWidth || getHeight() != newHeight) {
+            if( isNativeValid() && !isOffscreenInstance ) {
+                final NativeWindow parent = getParent();
+                final boolean useParent = useParent(parent);
+                if( useParent ) {
+                    final int x=getX(), y=getY();
+                    final Point p0S = getLocationOnScreenByParent(x, y, parent);
+                    if(DEBUG_IMPLEMENTATION) {
+                        System.err.println("MacWindow: sizeChanged() parent["+useParent+" "+x+"/"+y+"] "+getX()+"/"+getY()+" "+newWidth+"x"+newHeight+" ->  "+p0S+" screen-client-pos");
+                    }
+                    OSXUtil.RunOnMainThread(false, false, new Runnable() {
+                        @Override
+                        public void run() {
+                            setWindowClientTopLeftPoint0(getWindowHandle(), p0S.getX(), p0S.getY(), isVisible());
+                        } } );
+                }
+            }
+            superSizeChangedOffThread(defer, newWidth, newHeight, force);
+        }
+    }
+    private void superSizeChangedOffThread(final boolean defer, final int newWidth, final int newHeight, final boolean force) {
+        if( defer ) {
+            new InterruptSource.Thread() {
+                public void run() {
+                    WindowDriver.super.sizeChanged(false /* defer */, newWidth, newHeight, force);
+                } }.start();
+        } else {
+            WindowDriver.super.sizeChanged(false /* defer */, newWidth, newHeight, force);
+        }
+    }
+
+    //
+    // Accumulated actions
+    //
+
+    /** Triggered by implementation's WM events to update the client-area position, size and insets. */
+    protected void sizeScreenPosInsetsChanged(final boolean defer,
+                                              final int newX, final int newY,
+                                              final int newWidth, final int newHeight,
+                                              final int left, final int right, final int top, final int bottom,
+                                              final boolean force) {
+        sizeChanged(defer, newWidth, newHeight, force);
+        screenPositionChanged(defer, newX, newY);
+        insetsChanged(defer, left, right, top, bottom);
+    }
+
     @Override
     protected void setPointerIconImpl(final PointerIconImpl pi) {
         if( !isOffscreenInstance ) {
@@ -556,14 +636,18 @@ public class WindowDriver extends WindowImpl implements MutableSurface, DriverCl
     @Override
     protected void warpPointerImpl(final int x, final int y) {
         if( !isOffscreenInstance ) {
-            warpPointer0(getWindowHandle(), x / getPixelScaleX(), y / getPixelScaleY());
+            warpPointer0(getWindowHandle(),
+                         SurfaceScaleUtils.scaleInv(x, getPixelScaleX()),
+                         SurfaceScaleUtils.scaleInv(y, getPixelScaleY()));
         } // else may need offscreen solution ? FIXME
     }
 
     @Override
     protected final void doMouseEvent(final boolean enqueue, final boolean wait, final short eventType, final int modifiers,
                                       final int x, final int y, final short button, final float[] rotationXYZ, final float rotationScale) {
-        super.doMouseEvent(enqueue, wait, eventType, modifiers, x * getPixelScaleX(), y * getPixelScaleY(), button, rotationXYZ, rotationScale);
+        super.doMouseEvent(enqueue, wait, eventType, modifiers,
+                           SurfaceScaleUtils.scale(x, getPixelScaleX()),
+                           SurfaceScaleUtils.scale(y, getPixelScaleY()), button, rotationXYZ, rotationScale);
     }
 
     @Override
@@ -617,22 +701,29 @@ public class WindowDriver extends WindowImpl implements MutableSurface, DriverCl
         super.enqueueKeyEvent(wait, eventType, modifiers, keyCode, keySym, keyChar);
     }
 
+    protected int getDisplayID() {
+        if( !isOffscreenInstance ) {
+            return getDisplayID0(getWindowHandle());
+        }
+        return 0;
+    }
+
     //----------------------------------------------------------------------
     // Internals only
     //
 
     private void createWindow(final boolean offscreenInstance, final boolean recreate,
                               final PointImmutable pS, final int width, final int height,
-                              final boolean fullscreen, final boolean visible, final boolean alwaysOnTop) {
-
+                              final int flags)
+    {
         final long parentWinHandle = getParentWindowHandle();
         final long preWinHandle = getWindowHandle();
 
         if(DEBUG_IMPLEMENTATION) {
             System.err.println("MacWindow.createWindow on thread "+Thread.currentThread().getName()+
                                ": offscreen "+offscreenInstance+", recreate "+recreate+
-                               ", pS "+pS+", "+width+"x"+height+", fullscreen "+fullscreen+", visible "+visible+
-                               ", alwaysOnTop "+alwaysOnTop+", preWinHandle "+toHexString(preWinHandle)+", parentWin "+toHexString(parentWinHandle)+
+                               ", pS "+pS+", "+width+"x"+height+", state "+getReconfigStateMaskString(flags)+
+                               ", preWinHandle "+toHexString(preWinHandle)+", parentWin "+toHexString(parentWinHandle)+
                                ", surfaceHandle "+toHexString(surfaceHandle));
             // Thread.dumpStack();
         }
@@ -659,9 +750,22 @@ public class WindowDriver extends WindowImpl implements MutableSurface, DriverCl
                 }
             }
 
-            final long newWin = createWindow0( pS.getX(), pS.getY(), width, height, fullscreen,
-                                               ( isUndecorated() || offscreenInstance ) ? NSBorderlessWindowMask :
-                                               NSTitledWindowMask|NSClosableWindowMask|NSMiniaturizableWindowMask|NSResizableWindowMask,
+            final int windowStyle;
+            {
+                int ws = 0;
+                if( 0 != ( STATE_MASK_UNDECORATED & flags) || offscreenInstance ) {
+                    ws = NSBorderlessWindowMask;
+                } else {
+                    ws = NSTitledWindowMask|NSClosableWindowMask|NSMiniaturizableWindowMask;
+                    if( 0 != ( STATE_MASK_RESIZABLE & flags) ) {
+                        ws |= NSResizableWindowMask;
+                    }
+                }
+                windowStyle = ws;
+            }
+            final long newWin = createWindow0( pS.getX(), pS.getY(), width, height,
+                                               0 != ( STATE_MASK_FULLSCREEN & flags),
+                                               windowStyle,
                                                NSBackingStoreBuffered, surfaceHandle);
             if ( newWin == 0 ) {
                 throw new NativeWindowException("Could not create native window "+Thread.currentThread().getName()+" "+this);
@@ -674,12 +778,15 @@ public class WindowDriver extends WindowImpl implements MutableSurface, DriverCl
                     @Override
                     public void run() {
                         initWindow0( parentWinHandle, newWin, pS.getX(), pS.getY(), width, height, reqPixelScale[0] /* HiDPI uniformPixelScale */,
-                                     isOpaque, visible && !offscreenInstance, surfaceHandle);
+                                     isOpaque,
+                                     !offscreenInstance && 0 != ( STATE_MASK_ALWAYSONTOP & flags),
+                                     !offscreenInstance && 0 != ( STATE_MASK_ALWAYSONBOTTOM & flags),
+                                     !offscreenInstance && 0 != ( STATE_MASK_VISIBLE & flags),
+                                     surfaceHandle);
                         if( offscreenInstance ) {
                             orderOut0(0!=parentWinHandle ? parentWinHandle : newWin);
                         } else {
                             setTitle0(newWin, getTitle());
-                            setAlwaysOnTop0(getWindowHandle(), alwaysOnTop);
                         }
                     } });
         } catch (final Exception ie) {
@@ -692,7 +799,9 @@ public class WindowDriver extends WindowImpl implements MutableSurface, DriverCl
     private native long createWindow0(int x, int y, int w, int h, boolean fullscreen, int windowStyle, int backingStoreType, long view);
     /** Must be called on Main-Thread */
     private native void initWindow0(long parentWindow, long window, int x, int y, int w, int h, float reqPixelScale,
-                                    boolean opaque, boolean visible, long view);
+                                    boolean opaque, boolean atop, boolean abottom, boolean visible, long view);
+
+    private native int getDisplayID0(long window);
     private native void setPixelScale0(long window, long view, float reqPixelScale);
     private native boolean lockSurface0(long window, long view);
     private native boolean unlockSurface0(long window, long view);
@@ -715,8 +824,8 @@ public class WindowDriver extends WindowImpl implements MutableSurface, DriverCl
     private native void setWindowClientTopLeftPointAndSize0(long window, int x, int y, int w, int h, boolean display);
     /** Must be called on Main-Thread */
     private native void setWindowClientTopLeftPoint0(long window, int x, int y, boolean display);
-    /** Must be called on Main-Thread */
-    private native void setAlwaysOnTop0(long window, boolean atop);
+    /** Triggers {@link #sizeScreenPosInsetsChanged(boolean, int, int, int, int, int, int, int, int, boolean)} */
+    private native void updateSizePosInsets0(long window, boolean defer);
     private static native Object getLocationOnScreen0(long windowHandle, int src_x, int src_y);
     private static native void setPointerIcon0(long windowHandle, long handle);
     private static native void setPointerVisible0(long windowHandle, boolean hasFocus, boolean visible);
