@@ -41,13 +41,12 @@ import jogamp.nativewindow.windows.GDIUtil;
 import jogamp.newt.PointerIconImpl;
 import jogamp.newt.WindowImpl;
 
-import javax.media.nativewindow.AbstractGraphicsConfiguration;
-import javax.media.nativewindow.GraphicsConfigurationFactory;
-import javax.media.nativewindow.NativeWindowException;
-import javax.media.nativewindow.VisualIDHolder;
-import javax.media.nativewindow.util.Insets;
-import javax.media.nativewindow.util.InsetsImmutable;
-import javax.media.nativewindow.util.Point;
+import com.jogamp.nativewindow.AbstractGraphicsConfiguration;
+import com.jogamp.nativewindow.GraphicsConfigurationFactory;
+import com.jogamp.nativewindow.NativeWindowException;
+import com.jogamp.nativewindow.VisualIDHolder;
+import com.jogamp.nativewindow.util.InsetsImmutable;
+import com.jogamp.nativewindow.util.Point;
 
 import com.jogamp.common.os.Platform;
 import com.jogamp.common.util.VersionNumber;
@@ -141,18 +140,34 @@ public class WindowDriver extends WindowImpl {
         }
         setGraphicsConfiguration(cfg);
         final VersionNumber winVer = Platform.getOSVersionNumber();
-        final int flags = getReconfigureFlags(0, true) &
-                          ( FLAG_IS_ALWAYSONTOP | FLAG_IS_UNDECORATED ) ;
+        int flags = getReconfigureMask(0, true) & STATE_MASK_CREATENATIVE;
+        int maxCount = 0;
+        if( 0 != ( STATE_MASK_MAXIMIZED_HORZ & flags ) ) {
+            flags |= CHANGE_MASK_MAXIMIZED_HORZ;
+            maxCount++;
+        }
+        if( 0 != ( STATE_MASK_MAXIMIZED_VERT & flags ) ) {
+            flags |= CHANGE_MASK_MAXIMIZED_VERT;
+            maxCount++;
+        }
         final long _windowHandle = CreateWindow0(DisplayDriver.getHInstance(), display.getWindowClassName(), display.getWindowClassName(),
                                                  winVer.getMajor(), winVer.getMinor(),
                                                  getParentWindowHandle(),
-                                                 getX(), getY(), getWidth(), getHeight(), autoPosition(), flags);
+                                                 getX(), getY(), getWidth(), getHeight(), flags);
         if ( 0 == _windowHandle ) {
             throw new NativeWindowException("Error creating window");
         }
+        if( !cfg.getChosenCapabilities().isBackgroundOpaque() ) {
+            GDIUtil.DwmSetupTranslucency(_windowHandle, true);
+        }
+        InitWindow0(_windowHandle, flags);
         setWindowHandle(_windowHandle);
         windowHandleClose = _windowHandle;
 
+        if( 0 == ( STATE_MASK_CHILDWIN & flags ) && 1 == maxCount ) {
+            reconfigureWindowImpl(getX(), getY(), getWidth(), getHeight(), flags);
+        }
+
         if(DEBUG_IMPLEMENTATION) {
             final Exception e = new Exception("Info: Window new window handle "+Thread.currentThread().getName()+
                                         " (Parent HWND "+toHexString(getParentWindowHandle())+
@@ -188,30 +203,58 @@ public class WindowDriver extends WindowImpl {
         hdc_old = 0;
     }
 
+    @Override
+    protected final int getSupportedReconfigMaskImpl() {
+        return minimumReconfigStateMask |
+               STATE_MASK_CHILDWIN |
+               STATE_MASK_UNDECORATED |
+               STATE_MASK_ALWAYSONTOP |
+               STATE_MASK_ALWAYSONBOTTOM |
+               // STATE_MASK_STICKY |
+               STATE_MASK_RESIZABLE |
+               STATE_MASK_MAXIMIZED_VERT |
+               STATE_MASK_MAXIMIZED_HORZ |
+               STATE_MASK_POINTERVISIBLE |
+               STATE_MASK_POINTERCONFINED;
+    }
+
     @Override
     protected boolean reconfigureWindowImpl(int x, int y, int width, int height, final int flags) {
         if(DEBUG_IMPLEMENTATION) {
-            System.err.println("WindowsWindow reconfig: "+x+"/"+y+" "+width+"x"+height+", "+
-                               getReconfigureFlagsAsString(null, flags));
+            System.err.println("WindowsWindow reconfig.0: "+x+"/"+y+" "+width+"x"+height+
+                               ", "+getReconfigStateMaskString(flags));
+        }
+        final InsetsImmutable insets = getInsets();
+
+        if( 0 == ( STATE_MASK_CHILDWIN & flags ) &&
+            0 != ( ( CHANGE_MASK_MAXIMIZED_HORZ | CHANGE_MASK_MAXIMIZED_VERT ) & flags ) ) {
+            final int[] posSize = { x, y, width, height };
+            if( ( 0 != ( STATE_MASK_MAXIMIZED_HORZ & flags ) ) == ( 0 != ( STATE_MASK_MAXIMIZED_VERT & flags ) ) ) {
+                resetMaximizedManual(posSize); // reset before native maximize/reset
+            } else {
+                reconfigMaximizedManual(flags, posSize, insets);
+            }
+            x = posSize[0];
+            y = posSize[1];
+            width = posSize[2];
+            height = posSize[3];
         }
 
-        if(0 == ( FLAG_IS_UNDECORATED & flags)) {
-            final InsetsImmutable i = getInsets();
-
-            // client position -> top-level window position
-            x -= i.getLeftWidth() ;
-            y -= i.getTopHeight() ;
-
-            if(0<width && 0<height) {
-                // client size -> top-level window size
-                width += i.getTotalWidth();
-                height += i.getTotalHeight();
-            }
+        final boolean changeDecoration = 0 != ( CHANGE_MASK_DECORATION & flags);
+        final boolean isTranslucent = !getChosenCapabilities().isBackgroundOpaque();
+        if( changeDecoration && isTranslucent ) {
+            GDIUtil.DwmSetupTranslucency(getWindowHandle(), false);
         }
         reconfigureWindow0( getParentWindowHandle(), getWindowHandle(), x, y, width, height, flags);
+        if( changeDecoration && isTranslucent ) {
+            GDIUtil.DwmSetupTranslucency(getWindowHandle(), true);
+        }
 
-        if( 0 != ( FLAG_CHANGE_VISIBILITY & flags) ) {
-            visibleChanged(false, 0 != ( FLAG_IS_VISIBLE & flags));
+        if( 0 != ( CHANGE_MASK_VISIBILITY & flags) ) {
+            visibleChanged(false, 0 != ( STATE_MASK_VISIBLE & flags));
+        }
+        if(DEBUG_IMPLEMENTATION) {
+            System.err.println("WindowsWindow reconfig.X: "+getX()+"/"+getY()+" "+getWidth()+"x"+getHeight()+", "+getStateMaskString());
         }
         return true;
     }
@@ -277,11 +320,6 @@ public class WindowDriver extends WindowImpl {
         return GDIUtil.GetRelativeLocation( getWindowHandle(), 0 /*root win*/, x, y);
     }
 
-    @Override
-    protected void updateInsetsImpl(final Insets insets) {
-        // nop - using event driven insetsChange(..)
-    }
-
     //
     // PointerEvent Handling
     //
@@ -370,7 +408,8 @@ public class WindowDriver extends WindowImpl {
     protected static native boolean initIDs0(long hInstance);
 
     private native long CreateWindow0(long hInstance, String wndClassName, String wndName, int winMajor, int winMinor,
-                                      long parentWindowHandle, int x, int y, int width, int height, boolean autoPosition, int flags);
+                                      long parentWindowHandle, int x, int y, int width, int height, int flags);
+    private native void InitWindow0(long windowHandle, int flags);
     private native long MonitorFromWindow0(long windowHandle);
     private native void reconfigureWindow0(long parentWindowHandle, long windowHandle,
                                            int x, int y, int width, int height, int flags);
