@@ -48,7 +48,7 @@
     #include "timespec.h"
 #endif
 
-static const char * const ClazzNamePoint = "javax/media/nativewindow/util/Point";
+static const char * const ClazzNamePoint = "com/jogamp/nativewindow/util/Point";
 static const char * const ClazzAnyCstrName = "<init>";
 static const char * const ClazzNamePointCstrSignature = "(II)V";
 static jclass pointClz = NULL;
@@ -366,19 +366,15 @@ JNIEXPORT void JNICALL Java_jogamp_newt_driver_macosx_DisplayDriver_destroyPoint
     [pool release];
 }
 
-NSScreen * NewtScreen_getNSScreenByIndex(int screen_idx, BOOL cap) {
+#if 0
+static NSScreen * NewtScreen_getNSScreenByIndex(int screen_idx) {
     NSArray *screens = [NSScreen screens];
     if( screen_idx<0 || screen_idx>=[screens count] ) {
-        if( cap ) {
-            screen_idx=0;
-        } else {
-            return NULL;
-        }
+        return NULL;
     }
     return (NSScreen *) [screens objectAtIndex: screen_idx];
 }
-
-NSScreen * NewtScreen_getNSScreenByCoord(int x, int y) {
+static NSScreen * NewtScreen_getNSScreenByCoord(int x, int y) {
     NSArray *screens = [NSScreen screens];
     int i;
     for(i=[screens count]-1; i>=0; i--) {
@@ -391,9 +387,9 @@ NSScreen * NewtScreen_getNSScreenByCoord(int x, int y) {
             return screen;
         }
     }
-    return (NSScreen *) [screens objectAtIndex: 0];
+    return NULL;
 }
-
+#endif
 static void NewtScreen_dump() {
 #ifdef VERBOSE_ON
     NSArray *screens = [NSScreen screens];
@@ -420,12 +416,27 @@ NS_ENDHANDLER
 
 
 CGDirectDisplayID NewtScreen_getCGDirectDisplayIDByNSScreen(NSScreen *screen) {
-    // Mind: typedef uint32_t CGDirectDisplayID; - however, we assume it's 64bit on 64bit ?!
+    if( NULL == screen ) {
+        return (CGDirectDisplayID)0;
+    }
+    // Mind: typedef uint32_t CGDirectDisplayID;
     NSDictionary * dict = [screen deviceDescription];
     NSNumber * val = (NSNumber *) [dict objectForKey: @"NSScreenNumber"];
     // [NSNumber integerValue] returns NSInteger which is 32 or 64 bit native size
     return (CGDirectDisplayID) [val integerValue];
 }
+static NSScreen * NewtScreen_getNSScreenByCGDirectDisplayID(CGDirectDisplayID displayID) {
+    NSArray *screens = [NSScreen screens];
+    int i;
+    for(i=[screens count]-1; i>=0; i--) {
+        NSScreen * screen = (NSScreen *) [screens objectAtIndex: i];
+        CGDirectDisplayID dID = NewtScreen_getCGDirectDisplayIDByNSScreen(screen);
+        if( dID == displayID ) {
+            return screen;
+        }
+    }
+    return NULL;
+}
 
 /**
  * Only in >= 10.6:
@@ -453,16 +464,28 @@ static long GetDictionaryLong(CFDictionaryRef theDict, const void* key)
 
 /*
  * Class:     jogamp_newt_driver_macosx_ScreenDriver
- * Method:    getMonitorCount0
+ * Method:    getMonitorDeviceIds0
  * Signature: ()I
  */
-JNIEXPORT jint JNICALL Java_jogamp_newt_driver_macosx_ScreenDriver_getMonitorCount0
+JNIEXPORT jintArray JNICALL Java_jogamp_newt_driver_macosx_ScreenDriver_getMonitorDeviceIds0
   (JNIEnv *env, jobject obj)
 {
     NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
     NSArray *screens = [NSScreen screens];
+    int count = [screens count];
+    int32_t displayIDs[count];
+    int i;
+    for(i=0; i<count; i++) {
+        NSScreen * screen = (NSScreen *) [screens objectAtIndex: i];
+        displayIDs[i] = (int32_t)NewtScreen_getCGDirectDisplayIDByNSScreen(screen);
+    }
+    jintArray properties = (*env)->NewIntArray(env, count);
+    if (properties == NULL) {
+        NewtCommon_throwNewRuntimeException(env, "Could not allocate int array of size %d", count);
+    }
+    (*env)->SetIntArrayRegion(env, properties, 0, count, displayIDs);
     [pool release];
-    return (jint) [screens count];
+    return properties;
 }
 
 /*
@@ -471,7 +494,7 @@ JNIEXPORT jint JNICALL Java_jogamp_newt_driver_macosx_ScreenDriver_getMonitorCou
  * Signature: (I)[I
  */
 JNIEXPORT jintArray JNICALL Java_jogamp_newt_driver_macosx_ScreenDriver_getMonitorProps0
-  (JNIEnv *env, jobject obj, jint crt_idx)
+  (JNIEnv *env, jobject obj, jint crt_id)
 {
     NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
 
@@ -485,28 +508,31 @@ JNIEXPORT jintArray JNICALL Java_jogamp_newt_driver_macosx_ScreenDriver_getMonit
     timespec_now(&t1); timespec_subtract(&td, &t1, &t0); td_ms = timespec_milliseconds(&td);
     fprintf(stderr, "MacScreen_getMonitorProps0.1: %ld ms\n", td_ms); fflush(NULL);
 #endif
-    NSScreen *screen = NewtScreen_getNSScreenByIndex((int)crt_idx, false);
+    CGDirectDisplayID displayID = (CGDirectDisplayID)crt_id;
+    NSScreen *screen =  NewtScreen_getNSScreenByCGDirectDisplayID(displayID);
     if( NULL == screen ) {
         [pool release];
         return NULL;
     }
-    CGDirectDisplayID display = NewtScreen_getCGDirectDisplayIDByNSScreen(screen);
+    CGDirectDisplayID mainDisplayID = CGMainDisplayID();
+    BOOL isPrimary = mainDisplayID == displayID;
 #ifdef DBG_PERF
     timespec_now(&t1); timespec_subtract(&td, &t1, &t0); td_ms = timespec_milliseconds(&td);
     fprintf(stderr, "MacScreen_getMonitorProps0.2: %ld ms\n", td_ms); fflush(NULL);
 #endif
 
-    CGSize sizeMM = CGDisplayScreenSize(display);
+    CGSize sizeMM = CGDisplayScreenSize(displayID);
 #ifdef DBG_PERF
     timespec_now(&t1); timespec_subtract(&td, &t1, &t0); td_ms = timespec_milliseconds(&td);
     fprintf(stderr, "MacScreen_getMonitorProps0.3: %ld ms\n", td_ms); fflush(NULL);
 #endif
 
-    CGRect dBounds = CGDisplayBounds (display); // origin top-left
+    CGRect dBounds = CGDisplayBounds (displayID); // origin top-left
 #ifdef VERBOSE_ON
-    BOOL usesGL = CGDisplayUsesOpenGLAcceleration(display);
+    BOOL usesGL = CGDisplayUsesOpenGLAcceleration(displayID);
     NSRect sFrame = [screen frame]; // origin bottom-left
-    DBG_PRINT( "getMonitorProps0: scrn %d, top-left displayBounds[%d/%d %dx%d], bottom-left screenFrame[%d/%d %dx%d], usesGL %d\n", (int)crt_idx, 
+    DBG_PRINT( "getMonitorProps0: crt_id 0x%X (prim %d), top-left displayBounds[%d/%d %dx%d], bottom-left screenFrame[%d/%d %dx%d], usesGL %d\n", 
+                 (int)crt_id, isPrimary,
                  (int)dBounds.origin.x, (int)dBounds.origin.y, (int)dBounds.size.width, (int)dBounds.size.height,
                  (int)sFrame.origin.x, (int)sFrame.origin.y, (int)sFrame.size.width, (int)sFrame.size.height,
                  (int)usesGL);
@@ -516,7 +542,9 @@ JNIEXPORT jintArray JNICALL Java_jogamp_newt_driver_macosx_ScreenDriver_getMonit
     jint prop[ propCount ];
     int offset = 0;
     prop[offset++] = propCount;
-    prop[offset++] = crt_idx;
+    prop[offset++] = crt_id;
+    prop[offset++] = 0; // isClone
+    prop[offset++] = isPrimary ? 1 : 0; // isPrimary
     prop[offset++] = (jint) sizeMM.width;
     prop[offset++] = (jint) sizeMM.height;
     prop[offset++] = (jint) dBounds.origin.x;    // rotated viewport x      (pixel units, will be fixed in java code)
@@ -545,11 +573,12 @@ JNIEXPORT jintArray JNICALL Java_jogamp_newt_driver_macosx_ScreenDriver_getMonit
  * Signature: (II)[I
  */
 JNIEXPORT jintArray JNICALL Java_jogamp_newt_driver_macosx_ScreenDriver_getMonitorMode0
-  (JNIEnv *env, jobject obj, jint crt_idx, jint mode_idx)
+  (JNIEnv *env, jobject obj, jint crt_id, jint mode_idx)
 {
     NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
 
-    NSScreen *screen = NewtScreen_getNSScreenByIndex((int)crt_idx, false);
+    CGDirectDisplayID displayID = (CGDirectDisplayID)crt_id;
+    NSScreen *screen =  NewtScreen_getNSScreenByCGDirectDisplayID(displayID);
     if( NULL == screen ) {
         [pool release];
         return NULL;
@@ -561,21 +590,19 @@ NS_DURING
 NS_HANDLER
 NS_ENDHANDLER
 
-    CGDirectDisplayID display = NewtScreen_getCGDirectDisplayIDByNSScreen(screen);
-
-    CFArrayRef availableModes = CGDisplayAvailableModes(display);
+    CFArrayRef availableModes = CGDisplayAvailableModes(displayID);
     CFIndex numberOfAvailableModes = CFArrayGetCount(availableModes);
     CFIndex numberOfAvailableModesRots = ROTMODES_PER_REALMODE * numberOfAvailableModes; 
     CFDictionaryRef mode = NULL;
-    int currentCCWRot = (int)CGDisplayRotation(display);
+    int currentCCWRot = (int)CGDisplayRotation(displayID);
     jint ccwRot = 0;
     int nativeId = 0;
 
 #ifdef VERBOSE_ON
     if(0 >= mode_idx) {
         // only for current mode (-1) and first mode (scanning)
-        DBG_PRINT( "getScreenMode0: scrn %d (s %p, d %p, pscale %lf), mode %d, avail: %d/%d, current rot %d ccw\n",  
-            (int)crt_idx, screen, (void*)(intptr_t)display, pixelScale, (int)mode_idx, (int)numberOfAvailableModes, (int)numberOfAvailableModesRots, currentCCWRot);
+        DBG_PRINT( "getScreenMode0: crtID 0x%X (s %p, pscale %lf), mode %d, avail: %d/%d, current rot %d ccw\n",  
+            (uint32_t)displayID, screen, pixelScale, (int)mode_idx, (int)numberOfAvailableModes, (int)numberOfAvailableModesRots, currentCCWRot);
     }
 #endif
 
@@ -592,7 +619,7 @@ NS_ENDHANDLER
         mode = (CFDictionaryRef)CFArrayGetValueAtIndex(availableModes, nativeId);
     } else {
         // current mode
-        mode = CGDisplayCurrentMode(display);
+        mode = CGDisplayCurrentMode(displayID);
         ccwRot = currentCCWRot;
         CFRange range = CFRangeMake (0, numberOfAvailableModes);
         nativeId = CFArrayGetFirstIndexOfValue(availableModes, range, (CFDictionaryRef)mode);
@@ -650,26 +677,25 @@ NS_ENDHANDLER
  * Signature: (III)Z
  */
 JNIEXPORT jboolean JNICALL Java_jogamp_newt_driver_macosx_ScreenDriver_setMonitorMode0
-  (JNIEnv *env, jobject object, jint crt_idx, jint nativeId, jint ccwRot)
+  (JNIEnv *env, jobject object, jint crt_id, jint nativeId, jint ccwRot)
 {
     jboolean res = JNI_TRUE;
     NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
 
-    NSScreen *screen = NewtScreen_getNSScreenByIndex((int)crt_idx, false);
+    CGDirectDisplayID displayID = (CGDirectDisplayID)crt_id;
+    NSScreen *screen =  NewtScreen_getNSScreenByCGDirectDisplayID(displayID);
     if( NULL == screen ) {
         [pool release];
         return JNI_FALSE;
     }
-    CGDirectDisplayID display = NewtScreen_getCGDirectDisplayIDByNSScreen(screen);
-
-    CFArrayRef availableModes = CGDisplayAvailableModes(display);
+    CFArrayRef availableModes = CGDisplayAvailableModes(displayID);
     CFIndex numberOfAvailableModes = CFArrayGetCount(availableModes);
 #ifdef VERBOSE_ON
     CFIndex numberOfAvailableModesRots = ROTMODES_PER_REALMODE * numberOfAvailableModes;
 #endif
 
-    DBG_PRINT( "setScreenMode0: scrn %d (%p, %p), nativeID %d, rot %d ccw, avail: %d/%d\n",  
-        (int)crt_idx, screen, (void*)(intptr_t)display, (int)nativeId, ccwRot, (int)numberOfAvailableModes, (int)numberOfAvailableModesRots);
+    DBG_PRINT( "setScreenMode0: crtID 0x%X (%p), nativeID %d, rot %d ccw, avail: %d/%d\n",  
+        (uint32_t)displayID, screen, (int)nativeId, ccwRot, (int)numberOfAvailableModes, (int)numberOfAvailableModesRots);
 
     CFDictionaryRef mode = NULL;
 
@@ -687,7 +713,7 @@ JNIEXPORT jboolean JNICALL Java_jogamp_newt_driver_macosx_ScreenDriver_setMonito
     }
 
     if( NULL != mode ) {
-        CGError err = CGDisplaySwitchToMode(display, mode);
+        CGError err = CGDisplaySwitchToMode(displayID, mode);
         if(kCGErrorSuccess != err) {
             DBG_PRINT( "setScreenMode0: SetMode failed: %d\n", (int)err);
             res = JNI_FALSE;
@@ -805,6 +831,19 @@ JNIEXPORT jlong JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_createWindow
     return (jlong) ((intptr_t) myWindow);
 }
 
+JNIEXPORT jint JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_getDisplayID0(JNIEnv *env, jobject jthis, jlong window) {
+    NewtMacWindow* myWindow = (NewtMacWindow*) ((intptr_t) window);
+    if( NULL == myWindow ) {
+        DBG_PRINT( "getDisplayID0 - NULL NEWT win - abort\n");
+        return 0;
+    }
+    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
+    NSScreen *screen =  [myWindow screen];
+    int32_t displayID = (int32_t)NewtScreen_getCGDirectDisplayIDByNSScreen(screen);
+    [pool release];
+    return (jint) displayID;
+}
+
 /**
  * Method is called on Main-Thread, hence no special invocation required inside method.
  *
@@ -814,16 +853,16 @@ JNIEXPORT jlong JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_createWindow
  */
 JNIEXPORT void JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_initWindow0
   (JNIEnv *env, jobject jthis, jlong parent, jlong window, jint x, jint y, jint w, jint h, jfloat reqPixelScale,
-   jboolean opaque, jboolean visible, jlong jview)
+   jboolean opaque, jboolean atop, jboolean abottom, jboolean visible, jlong jview)
 {
     NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
     NewtMacWindow* myWindow = (NewtMacWindow*) ((intptr_t) window);
     NewtView* myView = (NewtView*) (intptr_t) jview ;
     BOOL fullscreen = myWindow->isFullscreenWindow;
 
-    DBG_PRINT( "initWindow0 - %p (this), %p (parent), %p (window), %d/%d %dx%d, reqPixScale %f, opaque %d, fs %d, visible %d, view %p (START)\n",
+    DBG_PRINT( "initWindow0 - %p (this), %p (parent), %p (window), %d/%d %dx%d, reqPixScale %f, opaque %d, atop %d, abottom %d, fs %d, visible %d, view %p (START)\n",
         (void*)(intptr_t)jthis, (void*)(intptr_t)parent, myWindow, (int)x, (int)y, (int)w, (int)h, (float)reqPixelScale,
-        (int) opaque, (int)fullscreen, (int)visible, myView);
+        (int) opaque, (int)atop, (int)abottom, (int)fullscreen, (int)visible, myView);
 
 NS_DURING
     // HiDPI scaling: Setup - Available >= 10.7
@@ -886,6 +925,7 @@ NS_ENDHANDLER
         [myWindow setOpaque: NO];
         [myWindow setBackgroundColor: [NSColor clearColor]];
     }
+    [myWindow setAlwaysOn: atop bottom:abottom];
 
     // specify we want mouse-moved events
     [myWindow setAcceptsMouseMovedEvents:YES];
@@ -966,10 +1006,13 @@ NS_DURING
          * <https://developer.apple.com/library/mac/documentation/graphicsimaging/Conceptual/QuartzDisplayServicesConceptual/Articles/DisplayCapture.html>
          *
         NSScreen *myScreen =  NewtScreen_getNSScreenByCoord(x, y);
-        if ( [myView respondsToSelector:@selector(enterFullScreenMode:withOptions:)] ) {
-            // Available >= 10.5 - Makes the menubar disapear
-            [myView enterFullScreenMode: myScreen withOptions:NULL];
-        } */
+        if( NULL != myScreen ) {
+            if ( [myView respondsToSelector:@selector(enterFullScreenMode:withOptions:)] ) {
+                // Available >= 10.5 - Makes the menubar disapear
+                [myView enterFullScreenMode: myScreen withOptions:NULL];
+            } 
+        }
+        */
         if( myWindow->hasPresentationSwitch ) {
             DBG_PRINT( "initWindow0.%d - %p view %p, setPresentationOptions 0x%X\n", 
                 dbgIdx++, myWindow, myView, (int)myWindow->fullscreenPresentationOptions);
@@ -997,15 +1040,19 @@ NS_ENDHANDLER
 JNIEXPORT void JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_setPixelScale0
   (JNIEnv *env, jobject jthis, jlong window, jlong view, jfloat reqPixelScale)
 {
-    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
     NewtMacWindow* myWindow = (NewtMacWindow*) ((intptr_t) window);
+    if( NULL == myWindow ) {
+        DBG_PRINT( "setPixelScale0 - NULL NEWT win - abort\n");
+        return;
+    }
+    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
     NewtView* myView = (NewtView*) (intptr_t) view ;
 #ifdef VERBOSE_ON
     int dbgIdx = 1;
 #endif
-
     DBG_PRINT( "setPixelScale0 - %p (this), %p (window), view %p, reqPixScale %f (START)\n",
         (void*)(intptr_t)jthis, myWindow, myView, (float)reqPixelScale);
+    (void)myWindow;
 
 NS_DURING
     // HiDPI scaling: Setup - Available >= 10.7
@@ -1157,8 +1204,12 @@ JNIEXPORT jboolean JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_unlockSur
 JNIEXPORT void JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_requestFocus0
   (JNIEnv *env, jobject window, jlong w, jboolean force)
 {
-    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
     NSWindow* mWin = (NSWindow*) ((intptr_t) w);
+    if( NULL == mWin ) {
+        DBG_PRINT( "requestFocus - NULL NEWT win - abort\n");
+        return;
+    }
+    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
 #ifdef VERBOSE_ON
     BOOL hasFocus = [mWin isKeyWindow];
 #endif
@@ -1182,12 +1233,16 @@ JNIEXPORT void JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_requestFocus0
 JNIEXPORT void JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_resignFocus0
   (JNIEnv *env, jobject window, jlong w)
 {
-    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
     NSWindow* mWin = (NSWindow*) ((intptr_t) w);
+    if( NULL == mWin ) {
+        DBG_PRINT( "resignFocus0 - NULL NEWT win - abort\n");
+        return;
+    }
+    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
     NSWindow* pWin = [mWin parentWindow];
     BOOL hasFocus = [mWin isKeyWindow];
 
-    DBG_PRINT( "requestFocusParent0 - window: %p, parent %p, hasFocus %d (START)\n", mWin, pWin, hasFocus );
+    DBG_PRINT( "resignFocus0 - window: %p, parent %p, hasFocus %d (START)\n", mWin, pWin, hasFocus );
     if( hasFocus ) {
         if(NULL != pWin) {
             // [mWin makeFirstResponder: pWin];
@@ -1196,7 +1251,7 @@ JNIEXPORT void JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_resignFocus0
             [pWin resignKeyWindow];
         }
     }
-    DBG_PRINT( "requestFocusParent0 - window: %p (END)\n", mWin);
+    DBG_PRINT( "resignFocus0 - window: %p (END)\n", mWin);
 
     [pool release];
 }
@@ -1209,8 +1264,12 @@ JNIEXPORT void JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_resignFocus0
 JNIEXPORT void JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_orderFront0
   (JNIEnv *env, jobject unused, jlong window)
 {
-    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
     NSWindow* mWin = (NSWindow*) ((intptr_t) window);
+    if( NULL == mWin ) {
+        DBG_PRINT( "orderFront0 - NULL NEWT win - abort\n");
+        return;
+    }
+    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
     NSWindow* pWin = [mWin parentWindow];
 
     DBG_PRINT( "orderFront0 - window: (parent %p) %p visible %d (START)\n", pWin, mWin, [mWin isVisible]);
@@ -1234,8 +1293,12 @@ JNIEXPORT void JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_orderFront0
 JNIEXPORT void JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_orderOut0
   (JNIEnv *env, jobject unused, jlong window)
 {
-    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
     NSWindow* mWin = (NSWindow*) ((intptr_t) window);
+    if( NULL == mWin ) {
+        DBG_PRINT( "orderOut0 - NULL NEWT win - abort\n");
+        return;
+    }
+    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
     NSWindow* pWin = [mWin parentWindow];
 
     DBG_PRINT( "orderOut0 - window: (parent %p) %p visible %d (START)\n", pWin, mWin, [mWin isVisible]);
@@ -1259,8 +1322,12 @@ JNIEXPORT void JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_orderOut0
 JNIEXPORT void JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_setTitle0
   (JNIEnv *env, jobject unused, jlong window, jstring title)
 {
-    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
     NSWindow* win = (NSWindow*) ((intptr_t) window);
+    if( NULL == win ) {
+        DBG_PRINT( "setTitle0 - NULL NEWT win - abort\n");
+        return;
+    }
+    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
 
     DBG_PRINT( "setTitle0 - window: %p (START)\n", win);
 
@@ -1334,6 +1401,26 @@ JNIEXPORT void JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_changeContent
     [pool release];
 }
 
+/*
+ * Class:     jogamp_newt_driver_macosx_WindowDriver
+ * Method:    updateSizePosInsets0
+ * Signature: (JZ)V
+ */
+JNIEXPORT void JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_updateSizePosInsets0
+  (JNIEnv *env, jobject jthis, jlong window, jboolean defer)
+{
+    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
+    NewtMacWindow* mWin = (NewtMacWindow*) ((intptr_t) window);
+
+    DBG_PRINT( "updateSizePosInsets - window: %p, defer %d (START)\n", mWin, (int)defer);
+
+    [mWin updateSizePosInsets: env jwin:jthis defer:defer];
+
+    DBG_PRINT( "updateSizePosInsets - window: %p, defer %d (END)\n", mWin, (int)defer);
+
+    [pool release];
+}
+
 /*
  * Class:     jogamp_newt_driver_macosx_WindowDriver
  * Method:    setWindowClientTopLeftPointAndSize0
@@ -1362,8 +1449,12 @@ JNIEXPORT void JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_setWindowClie
 JNIEXPORT void JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_setWindowClientTopLeftPoint0
   (JNIEnv *env, jobject unused, jlong window, jint x, jint y, jboolean display)
 {
-    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
     NewtMacWindow* mWin = (NewtMacWindow*) ((intptr_t) window);
+    if( NULL == mWin ) {
+        DBG_PRINT( "setWindowClientTopLeftPoint - NULL NEWT win - abort\n");
+        return;
+    }
+    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
 
     DBG_PRINT( "setWindowClientTopLeftPoint - window: %p (START)\n", mWin);
 
@@ -1374,39 +1465,19 @@ JNIEXPORT void JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_setWindowClie
     [pool release];
 }
 
-/*
- * Class:     jogamp_newt_driver_macosx_WindowDriver
- * Method:    setAlwaysOnTop0
- * Signature: (JZ)V
- */
-JNIEXPORT void JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_setAlwaysOnTop0
-  (JNIEnv *env, jobject unused, jlong window, jboolean atop)
-{
-    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
-    NSWindow* win = (NSWindow*) ((intptr_t) window);
-
-    DBG_PRINT( "setAlwaysOnTop0 - window: %p, atop %d (START)\n", win, (int)atop);
-
-    if(atop) {
-        [win setLevel:NSFloatingWindowLevel];
-    } else {
-        [win setLevel:NSNormalWindowLevel];
-    }
-
-    DBG_PRINT( "setAlwaysOnTop0 - window: %p, atop %d (END)\n", win, (int)atop);
-
-    [pool release];
-}
-
 /*
  * Class:     jogamp_newt_driver_macosx_WindowDriver
  * Method:    getLocationOnScreen0
- * Signature: (JII)Ljavax/media/nativewindow/util/Point;
+ * Signature: (JII)Lcom/jogamp/nativewindow/util/Point;
  */
 JNIEXPORT jobject JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_getLocationOnScreen0
   (JNIEnv *env, jclass unused, jlong win, jint src_x, jint src_y)
 {
     NewtMacWindow *mWin = (NewtMacWindow*) (intptr_t) win;
+    if( NULL == mWin ) {
+        DBG_PRINT( "getLocationOnScreen0 - NULL NEWT win - abort\n");
+        return NULL;
+    }
     if( ![mWin isKindOfClass:[NewtMacWindow class]] ) {
         NewtCommon_throwNewRuntimeException(env, "Not a NewtMacWindow %p", mWin);
         return NULL;
@@ -1418,12 +1489,16 @@ JNIEXPORT jobject JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_getLocatio
 JNIEXPORT void JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_setPointerIcon0
   (JNIEnv *env, jobject unused, jlong window, jlong handle)
 {
+    NewtMacWindow *mWin = (NewtMacWindow*) (intptr_t) window;
+    if( NULL == mWin ) {
+        DBG_PRINT( "setPointerIcon0 - NULL NEWT win - abort\n");
+        return;
+    }
     NSCursor *c = (NSCursor*) (intptr_t) handle ;
     if ( NULL != c && NO == [c isKindOfClass:[NSCursor class]] ) {
         NewtCommon_throwNewRuntimeException(env, "Not a NSCursor %p", c);
         return;
     }
-    NewtMacWindow *mWin = (NewtMacWindow*) (intptr_t) window;
     if( ! [mWin isKindOfClass:[NewtMacWindow class]] ) {
         NewtCommon_throwNewRuntimeException(env, "Not a NewtMacWindow %p", mWin);
         return;
@@ -1447,6 +1522,10 @@ JNIEXPORT void JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_setPointerVis
   (JNIEnv *env, jclass clazz, jlong window, jboolean hasFocus, jboolean mouseVisible)
 {
     NewtMacWindow *mWin = (NewtMacWindow*) ((intptr_t) window);
+    if( NULL == mWin ) {
+        DBG_PRINT( "setPointerVisible0 - NULL NEWT win - abort\n");
+        return;
+    }
     if( ! [mWin isKindOfClass:[NewtMacWindow class]] ) {
         NewtCommon_throwNewRuntimeException(env, "Not a NewtMacWindow %p", mWin);
         return;
@@ -1471,6 +1550,10 @@ JNIEXPORT void JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_confinePointe
   (JNIEnv *env, jclass clazz, jlong window, jboolean confine)
 {
     NewtMacWindow *mWin = (NewtMacWindow*) ((intptr_t) window);
+    if( NULL == mWin ) {
+        DBG_PRINT( "confinePointer0 - NULL NEWT win - abort\n");
+        return;
+    }
     if( ! [mWin isKindOfClass:[NewtMacWindow class]] ) {
         NewtCommon_throwNewRuntimeException(env, "Not a NewtMacWindow %p", mWin);
         return;
@@ -1494,6 +1577,10 @@ JNIEXPORT void JNICALL Java_jogamp_newt_driver_macosx_WindowDriver_warpPointer0
   (JNIEnv *env, jclass clazz, jlong window, jint x, jint y)
 {
     NewtMacWindow *mWin = (NewtMacWindow*) ((intptr_t) window);
+    if( NULL == mWin ) {
+        DBG_PRINT( "warpPointer0 - NULL NEWT win - abort\n");
+        return;
+    }
     if( ! [mWin isKindOfClass:[NewtMacWindow class]] ) {
         NewtCommon_throwNewRuntimeException(env, "Not a NewtMacWindow %p", mWin);
         return;
