@@ -54,7 +54,7 @@ static jfloat GetDelta(NSEvent *event, jint javaMods[]) {
         deltaX = CGEventGetIntegerValueField(cgEvent, kCGScrollWheelEventPointDeltaAxis2);
         deltaY = CGEventGetIntegerValueField(cgEvent, kCGScrollWheelEventPointDeltaAxis1);
         // fprintf(stderr, "WHEEL/PAD: %lf/%lf - 0x%X\n", (double)deltaX, (double)deltaY, javaMods[0]);
-        if( fabsf(deltaX) > fabsf(deltaY) ) {
+        if( fabs(deltaX) > fabs(deltaY) ) {
             javaMods[0] |= EVENT_SHIFT_MASK;
             delta = deltaX;
         } else {
@@ -179,9 +179,10 @@ static jmethodID requestFocusID = NULL;
 
 static jmethodID insetsChangedID   = NULL;
 static jmethodID sizeChangedID     = NULL;
+static jmethodID sizeScreenPosInsetsChangedID = NULL;
 static jmethodID updatePixelScaleID = NULL;
 static jmethodID visibleChangedID = NULL;
-static jmethodID positionChangedID = NULL;
+static jmethodID screenPositionChangedID = NULL;
 static jmethodID focusChangedID    = NULL;
 static jmethodID windowDestroyNotifyID = NULL;
 static jmethodID windowRepaintID = NULL;
@@ -649,31 +650,32 @@ static jmethodID windowRepaintID = NULL;
 
     // convert to 1-based button number (or use zero if no button is involved)
     // TODO: detect mouse button when mouse wheel scrolled  
-    jshort javaButtonNum = 0;
+    jshort javaButtonNum;
     jfloat scrollDeltaY = 0.0f;
     switch ([event type]) {
-    case NSScrollWheel: {
-        scrollDeltaY = GetDelta(event, javaMods);
-        javaButtonNum = 1;
-        break;
+        case NSScrollWheel:
+            scrollDeltaY = GetDelta(event, javaMods);
+            javaButtonNum = 1;
+            break;
+        case NSLeftMouseDown:
+        case NSLeftMouseUp:
+        case NSLeftMouseDragged:
+            javaButtonNum = 1;
+            break;
+        case NSRightMouseDown:
+        case NSRightMouseUp:
+        case NSRightMouseDragged:
+            javaButtonNum = 3;
+            break;
+        case NSOtherMouseDown:
+        case NSOtherMouseUp:
+        case NSOtherMouseDragged:
+            javaButtonNum = 2;
+            break;
+        default:
+            javaButtonNum = 0;
+            break;
     }
-    case NSLeftMouseDown:
-    case NSLeftMouseUp:
-    case NSLeftMouseDragged:
-        javaButtonNum = 1;
-        break;
-    case NSRightMouseDown:
-    case NSRightMouseUp:
-    case NSRightMouseDragged:
-        javaButtonNum = 3;
-        break;
-    case NSOtherMouseDown:
-    case NSOtherMouseUp:
-    case NSOtherMouseDragged:
-        javaButtonNum = 2;
-        break;
-    }
-
     if (evType == EVENT_MOUSE_WHEEL_MOVED && scrollDeltaY == 0) {
         // ignore 0 increment wheel scroll events
         return;
@@ -784,11 +786,21 @@ static jmethodID windowRepaintID = NULL;
     [super viewDidChangeBackingProperties];
 
     // HiDPI scaling
-    BOOL useHiDPI = [self wantsBestResolutionOpenGLSurface];
-    CGFloat pixelScaleNative = [[self window] backingScaleFactor];
-    CGFloat pixelScaleUse = useHiDPI ? pixelScaleNative : 1.0;
-    DBG_PRINT("viewDidChangeBackingProperties: PixelScale: HiDPI %d, native %f -> use %f\n", useHiDPI, (float)pixelScaleNative, (float)pixelScaleUse);
-    [[self layer] setContentsScale: pixelScaleUse];
+    BOOL useHiDPI = false;
+    CGFloat maxPixelScale = 1.0;
+    CGFloat winPixelScale = 1.0;
+    NSWindow* window = [self window];
+    NSScreen* screen = [window screen];
+NS_DURING
+    maxPixelScale = [screen backingScaleFactor];
+    useHiDPI = [self wantsBestResolutionOpenGLSurface];
+    if( useHiDPI ) {
+        winPixelScale = [window backingScaleFactor];
+    }
+NS_HANDLER
+NS_ENDHANDLER
+    DBG_PRINT("viewDidChangeBackingProperties: PixelScale: HiDPI %d, max %f, window %f\n", useHiDPI, (float)maxPixelScale, (float)winPixelScale);
+    [[self layer] setContentsScale: winPixelScale];
 
     if (javaWindowObject == NULL) {
         DBG_PRINT("viewDidChangeBackingProperties: null javaWindowObject\n");
@@ -801,7 +813,7 @@ static jmethodID windowRepaintID = NULL;
         return;
     }
 
-    (*env)->CallVoidMethod(env, javaWindowObject, updatePixelScaleID, JNI_TRUE, (jfloat)pixelScaleUse, (jfloat)pixelScaleNative); // defer 
+    (*env)->CallVoidMethod(env, javaWindowObject, updatePixelScaleID, JNI_TRUE, (jfloat)winPixelScale, (jfloat)maxPixelScale); // defer 
 
     // detaching thread not required - daemon
     // NewtCommon_ReleaseJNIEnv(shallBeDetached);
@@ -820,13 +832,15 @@ static jmethodID windowRepaintID = NULL;
     updatePixelScaleID = (*env)->GetMethodID(env, clazz, "updatePixelScale", "(ZFF)V");
     visibleChangedID = (*env)->GetMethodID(env, clazz, "visibleChanged", "(ZZ)V");
     insetsChangedID = (*env)->GetMethodID(env, clazz, "insetsChanged", "(ZIIII)V");
-    positionChangedID = (*env)->GetMethodID(env, clazz, "screenPositionChanged", "(ZII)V");
+    sizeScreenPosInsetsChangedID = (*env)->GetMethodID(env, clazz, "sizeScreenPosInsetsChanged", "(ZIIIIIIIIZ)V");
+    screenPositionChangedID = (*env)->GetMethodID(env, clazz, "screenPositionChanged", "(ZII)V");
     focusChangedID = (*env)->GetMethodID(env, clazz, "focusChanged", "(ZZ)V");
     windowDestroyNotifyID = (*env)->GetMethodID(env, clazz, "windowDestroyNotify", "(Z)Z");
     windowRepaintID = (*env)->GetMethodID(env, clazz, "windowRepaint", "(ZIIII)V");
     requestFocusID = (*env)->GetMethodID(env, clazz, "requestFocus", "(Z)V");
-    if (enqueueMouseEventID && enqueueKeyEventID && sizeChangedID && updatePixelScaleID && visibleChangedID && insetsChangedID &&
-        positionChangedID && focusChangedID && windowDestroyNotifyID && requestFocusID && windowRepaintID)
+    if (enqueueMouseEventID && enqueueKeyEventID && sizeChangedID && updatePixelScaleID && visibleChangedID && 
+        insetsChangedID && sizeScreenPosInsetsChangedID &&
+        screenPositionChangedID && focusChangedID && windowDestroyNotifyID && requestFocusID && windowRepaintID)
     {
         CKCH_CreateDictionaries();
         return YES;
@@ -918,6 +932,20 @@ static jmethodID windowRepaintID = NULL;
     return realized;
 }
 
+- (void) setAlwaysOn: (BOOL)top bottom:(BOOL)bottom
+{
+    if( top ) {
+        DBG_PRINT( "*************** setAlwaysOn -> top\n");
+        [self setLevel: kCGMaximumWindowLevel];
+    } else if ( bottom ) {
+        DBG_PRINT( "*************** setAlwaysOn -> bottom\n");
+        [self setLevel: kCGDesktopIconWindowLevel]; // w/ input
+    } else {
+        DBG_PRINT( "*************** setAlwaysOn -> normal\n");
+        [self setLevel:NSNormalWindowLevel];
+    }
+}
+
 - (void) updateInsets: (JNIEnv*) env jwin: (jobject) javaWin
 {
     NSRect frameRect = [self frame];
@@ -938,6 +966,32 @@ static jmethodID windowRepaintID = NULL;
     }
 }
 
+- (void) updateSizePosInsets: (JNIEnv*) env jwin: (jobject) javaWin defer: (jboolean)defer
+{
+    // update insets on every window resize for lack of better hook place
+    [self updateInsets: NULL jwin:NULL];
+
+    NSRect frameRect = [self frame];
+    NSRect contentRect = [self contentRectForFrameRect: frameRect];
+
+    DBG_PRINT( "updateSize: [ w %d, h %d ]\n", (jint) contentRect.size.width, (jint) contentRect.size.height);
+
+    NSPoint p0 = { 0, 0 };
+    p0 = [self getLocationOnScreen: p0];
+
+    DBG_PRINT( "updatePos: [ x %d, y %d ]\n", (jint) p0.x, (jint) p0.y);
+
+    if( NULL != env && NULL != javaWin ) {
+        (*env)->CallVoidMethod(env, javaWin, sizeScreenPosInsetsChangedID, defer,
+                               (jint) p0.x, (jint) p0.y,
+                               (jint) contentRect.size.width, (jint) contentRect.size.height,
+                               cachedInsets[0], cachedInsets[1], cachedInsets[2], cachedInsets[3],
+                               JNI_FALSE // force
+                              );
+    }
+}
+
+
 - (void) attachToParent: (NSWindow*) parent
 {
     DBG_PRINT( "attachToParent.1\n");
@@ -1188,15 +1242,7 @@ static jmethodID windowRepaintID = NULL;
         javaWindowObject = [newtView getJavaWindowObject];
     }
     if( NULL != javaWindowObject ) {
-        // update insets on every window resize for lack of better hook place
-        [self updateInsets: env jwin:javaWindowObject];
-
-        NSRect frameRect = [self frame];
-        NSRect contentRect = [self contentRectForFrameRect: frameRect];
-
-        (*env)->CallVoidMethod(env, javaWindowObject, sizeChangedID, JNI_TRUE, // defer 
-                               (jint) contentRect.size.width,
-                               (jint) contentRect.size.height, JNI_FALSE);
+        [self updateSizePosInsets: env jwin: javaWindowObject defer:JNI_TRUE];
     }
     // detaching thread not required - daemon
     // NewtCommon_ReleaseJNIEnv(shallBeDetached);
@@ -1222,7 +1268,8 @@ static jmethodID windowRepaintID = NULL;
 
     NSPoint p0 = { 0, 0 };
     p0 = [self getLocationOnScreen: p0];
-    (*env)->CallVoidMethod(env, javaWindowObject, positionChangedID, JNI_FALSE, (jint) p0.x, (jint) p0.y);
+    DBG_PRINT( "windowDidMove: [ x %d, y %d ]\n", (jint) p0.x, (jint) p0.y);
+    (*env)->CallVoidMethod(env, javaWindowObject, screenPositionChangedID, JNI_TRUE, (jint) p0.x, (jint) p0.y);
 
     // detaching thread not required - daemon
     // NewtCommon_ReleaseJNIEnv(shallBeDetached);
