@@ -29,16 +29,17 @@ package com.jogamp.opengl.test.junit.graph;
 
 import java.io.File;
 import java.io.IOException;
+import java.net.URL;
 
-import javax.media.opengl.GL;
-import javax.media.opengl.GL2ES2;
-import javax.media.opengl.GLAnimatorControl;
-import javax.media.opengl.GLAutoDrawable;
-import javax.media.opengl.GLCapabilities;
-import javax.media.opengl.GLCapabilitiesImmutable;
-import javax.media.opengl.GLException;
-import javax.media.opengl.GLProfile;
-import javax.media.opengl.GLRunnable;
+import com.jogamp.opengl.GL;
+import com.jogamp.opengl.GL2ES2;
+import com.jogamp.opengl.GLAnimatorControl;
+import com.jogamp.opengl.GLAutoDrawable;
+import com.jogamp.opengl.GLCapabilities;
+import com.jogamp.opengl.GLCapabilitiesImmutable;
+import com.jogamp.opengl.GLException;
+import com.jogamp.opengl.GLProfile;
+import com.jogamp.opengl.GLRunnable;
 
 import org.junit.Assert;
 import org.junit.FixMethodOrder;
@@ -53,6 +54,7 @@ import com.jogamp.graph.curve.opengl.RenderState;
 import com.jogamp.graph.font.Font;
 import com.jogamp.graph.font.FontFactory;
 import com.jogamp.graph.geom.SVertex;
+import com.jogamp.junit.util.JunitTracer;
 import com.jogamp.newt.Window;
 import com.jogamp.newt.opengl.GLWindow;
 import com.jogamp.opengl.test.junit.util.MiscUtils;
@@ -72,9 +74,10 @@ public class TestTextRendererNEWT00 extends UITestCase {
     static int GraphVBAASamples = 0;
     static int GraphMSAASamples = 0;
     static boolean ManualTest = false;
-    static int SwapInterval = 0;
+    static int SwapInterval = 1;
 
     static String fontFileName = null;
+    static URL fontURL = null;
     static int fontSet = 0;
     static int fontFamily = 0;
     static int fontStylebits = 0;
@@ -92,6 +95,9 @@ public class TestTextRendererNEWT00 extends UITestCase {
             if(args[i].equals("-time")) {
                 i++;
                 Duration = atoi(args[i]);
+            } else if(args[i].equals("-fontURL")) {
+                i++;
+                fontURL = new URL(args[i]);
             } else if(args[i].equals("-fontFile")) {
                 i++;
                 fontFileName = args[i];
@@ -208,7 +214,7 @@ public class TestTextRendererNEWT00 extends UITestCase {
         window.display();
         System.err.println("Chosen: "+window.getChosenGLCapabilities());
         if( WaitStartEnd ) {
-            UITestCase.waitForKey("Start");
+            JunitTracer.waitForKey("Start");
         }
 
         final RenderState rs = RenderState.createRenderState(SVertex.factory());
@@ -247,7 +253,7 @@ public class TestTextRendererNEWT00 extends UITestCase {
         });
         anim.stop();
         if( WaitStartEnd ) {
-            UITestCase.waitForKey("Stop");
+            JunitTracer.waitForKey("Stop");
         }
         destroyWindow(window);
     }
@@ -279,7 +285,15 @@ public class TestTextRendererNEWT00 extends UITestCase {
 
             regionFPS = GLRegion.create(renderModes, null);
             regionFPSAnim = GLRegion.create(renderModes, null);
-            if( null != fontFileName ) {
+            if( null != fontURL ) {
+                Font _font = null;
+                try {
+                    _font = FontFactory.get(fontURL.openStream(), true);
+                } catch (final IOException e) {
+                    e.printStackTrace();
+                }
+                font = _font;
+            } else if( null != fontFileName ) {
                 Font _font = null;
                 try {
                     _font = FontFactory.get(getClass(), fontFileName, false);
