@@ -29,12 +29,12 @@ package com.jogamp.opengl.test.junit.graph.demos;
 
 import java.io.IOException;
 
-import javax.media.opengl.GL;
-import javax.media.opengl.GL2ES2;
-import javax.media.opengl.GLAnimatorControl;
-import javax.media.opengl.GLAutoDrawable;
-import javax.media.opengl.GLException;
-import javax.media.opengl.fixedfunc.GLMatrixFunc;
+import com.jogamp.opengl.GL;
+import com.jogamp.opengl.GL2ES2;
+import com.jogamp.opengl.GLAnimatorControl;
+import com.jogamp.opengl.GLAutoDrawable;
+import com.jogamp.opengl.GLException;
+import com.jogamp.opengl.fixedfunc.GLMatrixFunc;
 
 import com.jogamp.graph.curve.Region;
 import com.jogamp.graph.curve.opengl.GLRegion;
@@ -165,9 +165,14 @@ public abstract class GPUTextRendererListenerBase01 extends GPURendererListenerB
         super.init(drawable);
         final Object upObj = drawable.getUpstreamWidget();
         if( upObj instanceof Window ) {
-            final float[] pixelsPerMM = new float[2];
-            ((Window)upObj).getPixelsPerMM(pixelsPerMM);
-            dpiH = pixelsPerMM[1]*25.4f;
+            final Window window = (Window) upObj;
+            final float[] sDPI = window.getPixelsPerMM(new float[2]);
+            sDPI[0] *= 25.4f;
+            sDPI[1] *= 25.4f;
+            dpiH = sDPI[1];
+            System.err.println("Using screen DPI of "+dpiH);
+        } else {
+            System.err.println("Using default DPI of "+dpiH);
         }
         fontNameBox = font.getMetricBounds(fontName, font.getPixelSize(fontSizeFName, dpiH));
         switchHeadBox();
