@@ -2,21 +2,21 @@ package com.jogamp.opengl.test.junit.graph.demos;
 
 import java.io.File;
 import java.io.IOException;
-import java.net.URI;
 import java.net.URISyntaxException;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
 
-import javax.media.nativewindow.util.Dimension;
-import javax.media.opengl.GL;
-import javax.media.opengl.GL2ES2;
-import javax.media.opengl.GLAnimatorControl;
-import javax.media.opengl.GLAutoDrawable;
-import javax.media.opengl.GLEventListener;
-import javax.media.opengl.GLPipelineFactory;
-import javax.media.opengl.GLRunnable;
+import com.jogamp.opengl.GL;
+import com.jogamp.opengl.GL2ES2;
+import com.jogamp.opengl.GLAnimatorControl;
+import com.jogamp.opengl.GLAutoDrawable;
+import com.jogamp.opengl.GLEventListener;
+import com.jogamp.opengl.GLPipelineFactory;
+import com.jogamp.opengl.GLRunnable;
 
+import com.jogamp.common.net.Uri;
+import com.jogamp.common.util.InterruptSource;
 import com.jogamp.graph.curve.Region;
 import com.jogamp.graph.curve.opengl.RegionRenderer;
 import com.jogamp.graph.curve.opengl.RenderState;
@@ -157,8 +157,7 @@ public class GPUUISceneGLListener0A implements GLEventListener {
         try {
             font = FontFactory.get(FontFactory.UBUNTU).getDefault();
         } catch (final IOException ioe) {
-            System.err.println("Caught: "+ioe.getMessage());
-            ioe.printStackTrace();
+            throw new RuntimeException(ioe);
         }
         sceneUIController = new SceneUIController(sceneDist, zNear, zFar);
         screenshot = new GLReadBufferUtil(false, false);
@@ -356,7 +355,7 @@ public class GPUUISceneGLListener0A implements GLEventListener {
         button.addMouseListener(new UIShape.MouseGestureAdapter() {
             @Override
             public void mouseClicked(final MouseEvent e) {
-                new Thread() {
+                new InterruptSource.Thread() {
                     public void run() {
                         if( null != cDrawable ) {
                             final GLAnimatorControl actrl = cDrawable.getAnimator();
@@ -523,6 +522,7 @@ public class GPUUISceneGLListener0A implements GLEventListener {
             mPlayer.setTextureUnit(texUnitMediaPlayer);
             final MediaPlayerButton mPlayerButton = new MediaPlayerButton(renderer.getRenderState().getVertexFactory(), renderModes,
                                                                           button2XSize, button2YSize, mPlayer);
+            mPlayerButton.setVerbose(true);
             mPlayerButton.addDefaultEventListener();
             mPlayerButton.translate(xStartRight, yStartTop - diffY*1, 0f);
             mPlayerButton.setToggleable(true);
@@ -536,7 +536,7 @@ public class GPUUISceneGLListener0A implements GLEventListener {
                 } } );
             buttons.add(mPlayerButton);
             try {
-                final URI streamLoc = new URI("http://archive.org/download/BigBuckBunny_328/BigBuckBunny_512kb.mp4");
+                final Uri streamLoc = Uri.cast("http://archive.org/download/BigBuckBunny_328/BigBuckBunny_512kb.mp4");
                 mPlayer.initStream(streamLoc, GLMediaPlayer.STREAM_ID_AUTO, GLMediaPlayer.STREAM_ID_AUTO, GLMediaPlayer.TEXTURE_COUNT_DEFAULT);
             } catch (final URISyntaxException e1) {
                 e1.printStackTrace();
@@ -667,14 +667,17 @@ public class GPUUISceneGLListener0A implements GLEventListener {
         if( upObj instanceof Window ) {
             final Window upWin = (Window)upObj;
             final MonitorDevice mm = upWin.getMainMonitor();
-            final float[] monitorPixelsPerMM = mm.getPixelsPerMM(new float[2]);
-            final float monitorDpiH = monitorPixelsPerMM[1]*25.4f;
-            final float[] surfacePixelsPerMM = upWin.getPixelsPerMM(new float[2]);
-            dpiH = surfacePixelsPerMM[1]*25.4f;
+            final float[] monitorDPI = mm.getPixelsPerMM(new float[2]);
+            monitorDPI[0] *= 25.4f;
+            monitorDPI[1] *= 25.4f;
+            final float[] sDPI = upWin.getPixelsPerMM(new float[2]);
+            sDPI[0] *= 25.4f;
+            sDPI[1] *= 25.4f;
+            dpiH = sDPI[1];
             System.err.println("Monitor detected: "+mm);
-            System.err.println("Monitor dpi: "+monitorDpiH);
-            System.err.println("Surface scale: native "+new Dimension(upWin.getNativeSurfaceScale(new int[2]))+", current "+new Dimension(upWin.getCurrentSurfaceScale(new int[2])));
-            System.err.println("Surface dpi: "+dpiH);
+            System.err.println("Monitor dpi: "+monitorDPI[0]+" x "+monitorDPI[1]);
+            System.err.println("Surface scale: native "+Arrays.toString(upWin.getMaximumSurfaceScale(new float[2]))+", current "+Arrays.toString(upWin.getCurrentSurfaceScale(new float[2])));
+            System.err.println("Surface dpi "+sDPI[0]+" x "+sDPI[1]);
         } else {
             System.err.println("Using default DPI of "+dpiH);
         }
@@ -695,17 +698,16 @@ public class GPUUISceneGLListener0A implements GLEventListener {
         cDrawable = drawable;
         GL2ES2 gl = drawable.getGL().getGL2ES2();
         if(debug) {
-            gl = gl.getContext().setGL( GLPipelineFactory.create("javax.media.opengl.Debug", null, gl, null) ).getGL2ES2();
+            gl = gl.getContext().setGL( GLPipelineFactory.create("com.jogamp.opengl.Debug", null, gl, null) ).getGL2ES2();
         }
         if(trace) {
-            gl = gl.getContext().setGL( GLPipelineFactory.create("javax.media.opengl.Trace", null, gl, new Object[] { System.err } ) ).getGL2ES2();
+            gl = gl.getContext().setGL( GLPipelineFactory.create("com.jogamp.opengl.Trace", null, gl, new Object[] { System.err } ) ).getGL2ES2();
         }
 
         try {
             font = FontFactory.get(fontSet).getDefault();
         } catch (final IOException ioe) {
-            System.err.println("Caught: "+ioe.getMessage());
-            ioe.printStackTrace();
+            throw new RuntimeException(ioe);
         }
 
         renderer = RegionRenderer.create(rs, RegionRenderer.defaultBlendEnable, RegionRenderer.defaultBlendDisable);
