@@ -29,8 +29,8 @@ package com.jogamp.opengl.test.junit.graph.demos.ui;
 
 import java.util.ArrayList;
 
-import javax.media.nativewindow.NativeWindowException;
-import javax.media.opengl.GL2ES2;
+import com.jogamp.nativewindow.NativeWindowException;
+import com.jogamp.opengl.GL2ES2;
 
 import com.jogamp.graph.curve.OutlineShape;
 import com.jogamp.graph.curve.Region;
