@@ -43,19 +43,19 @@ import com.jogamp.opengl.util.glsl.ShaderCode;
 import com.jogamp.opengl.util.glsl.ShaderProgram;
 import com.jogamp.opengl.util.glsl.ShaderState;
 import com.jogamp.opengl.util.stereo.EyeParameter;
-import com.jogamp.opengl.util.stereo.EyePose;
+import com.jogamp.opengl.util.stereo.ViewerPose;
 import com.jogamp.opengl.util.stereo.StereoGLEventListener;
 
 import java.nio.FloatBuffer;
 
-import javax.media.nativewindow.NativeWindow;
-import javax.media.opengl.GL;
-import javax.media.opengl.GL2ES2;
-import javax.media.opengl.GLAnimatorControl;
-import javax.media.opengl.GLAutoDrawable;
-import javax.media.opengl.GLProfile;
-import javax.media.opengl.GLUniformData;
-import javax.media.opengl.fixedfunc.GLMatrixFunc;
+import com.jogamp.nativewindow.NativeWindow;
+import com.jogamp.opengl.GL;
+import com.jogamp.opengl.GL2ES2;
+import com.jogamp.opengl.GLAnimatorControl;
+import com.jogamp.opengl.GLAutoDrawable;
+import com.jogamp.opengl.GLProfile;
+import com.jogamp.opengl.GLUniformData;
+import com.jogamp.opengl.fixedfunc.GLMatrixFunc;
 
 /**
  * GearsES2.java <BR>
@@ -319,9 +319,7 @@ public class GearsES2 implements StereoGLEventListener, TileRendererBase.TileRen
     public void reshape(final GLAutoDrawable glad, final int x, final int y, final int width, final int height) {
         if( !isInit ) { return; }
         final GL2ES2 gl = glad.getGL().getGL2ES2();
-        if(-1 != swapInterval) {
-            gl.setSwapInterval(swapInterval);
-        }
+        gl.setSwapInterval(swapInterval);
         reshapeImpl(gl, x, y, width, height, width, height);
     }
 
@@ -407,9 +405,11 @@ public class GearsES2 implements StereoGLEventListener, TileRendererBase.TileRen
     private final float[] vec3Tmp2 = new float[3];
     private final float[] vec3Tmp3 = new float[3];
 
+    private static final float[] vec3ScalePos = new float[] { 20f, 20f, 20f };
+
     @Override
     public void reshapeForEye(final GLAutoDrawable drawable, final int x, final int y, final int width, final int height,
-                              final EyeParameter eyeParam, final EyePose eyePose) {
+                              final EyeParameter eyeParam, final ViewerPose viewerPose) {
         final GL2ES2 gl = drawable.getGL().getGL2ES2();
         pmvMatrix.glMatrixMode(GLMatrixFunc.GL_PROJECTION);
         final float[] mat4Projection = FloatUtil.makePerspective(mat4Tmp1, 0, true, eyeParam.fovhv, zNear, zFar);
@@ -426,10 +426,12 @@ public class GearsES2 implements StereoGLEventListener, TileRendererBase.TileRen
         final Quaternion rollPitchYaw = new Quaternion();
         // private final float eyeYaw = FloatUtil.PI; // 180 degrees in radians
         // rollPitchYaw.rotateByAngleY(eyeYaw);
-        final float[] shiftedEyePos = rollPitchYaw.rotateVector(vec3Tmp1, 0, eyePose.position, 0);
+        // final float[] shiftedEyePos = rollPitchYaw.rotateVector(vec3Tmp1, 0, viewerPose.position, 0);
+        final float[] shiftedEyePos = VectorUtil.copyVec3(vec3Tmp1, 0, viewerPose.position, 0);
+        VectorUtil.scaleVec3(shiftedEyePos, shiftedEyePos, vec3ScalePos); // amplify viewerPose position
         VectorUtil.addVec3(shiftedEyePos, shiftedEyePos, eyeParam.positionOffset);
 
-        rollPitchYaw.mult(eyePose.orientation);
+        rollPitchYaw.mult(viewerPose.orientation);
         final float[] up = rollPitchYaw.rotateVector(vec3Tmp2, 0, VectorUtil.VEC3_UNIT_Y, 0);
         final float[] forward = rollPitchYaw.rotateVector(vec3Tmp3, 0, VectorUtil.VEC3_UNIT_Z_NEG, 0);
         final float[] center = VectorUtil.addVec3(forward, shiftedEyePos, forward);
@@ -530,9 +532,9 @@ public class GearsES2 implements StereoGLEventListener, TileRendererBase.TileRen
             // Special handling for the case where the GLJPanel is translucent
             // and wants to be composited with other Java 2D content
             if (GLProfile.isAWTAvailable() &&
-                (drawable instanceof javax.media.opengl.awt.GLJPanel) &&
-                !((javax.media.opengl.awt.GLJPanel) drawable).isOpaque() &&
-                ((javax.media.opengl.awt.GLJPanel) drawable).shouldPreserveColorBufferIfTranslucent()) {
+                (drawable instanceof com.jogamp.opengl.awt.GLJPanel) &&
+                !((com.jogamp.opengl.awt.GLJPanel) drawable).isOpaque() &&
+                ((com.jogamp.opengl.awt.GLJPanel) drawable).shouldPreserveColorBufferIfTranslucent()) {
               gl.glClear(GL.GL_DEPTH_BUFFER_BIT);
             } else {
               gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
@@ -583,12 +585,6 @@ public class GearsES2 implements StereoGLEventListener, TileRendererBase.TileRen
         return "GearsES2[obj "+sid()+" isInit "+isInit+", usesShared "+usesSharedGears+", 1 "+gear1+", 2 "+gear2+", 3 "+gear3+", sharedGears "+sharedGears+"]";
     }
 
-    boolean confinedFixedCenter = false;
-
-    public void setConfinedFixedCenter(final boolean v) {
-        confinedFixedCenter = v;
-    }
-
     class GearsKeyAdapter extends KeyAdapter {
         public void keyPressed(final KeyEvent e) {
             final int kc = e.getKeyCode();
@@ -665,8 +661,8 @@ public class GearsES2 implements StereoGLEventListener, TileRendererBase.TileRen
         }
 
         private void navigate(final MouseEvent e) {
-            int x = e.getX();
-            int y = e.getY();
+            final int x = e.getX();
+            final int y = e.getY();
 
             int width, height;
             final Object source = e.getSource();
@@ -690,11 +686,6 @@ public class GearsES2 implements StereoGLEventListener, TileRendererBase.TileRen
             final float thetaX = 360.0f * ( (float)(prevMouseY-y)/(float)height);
             view_rotx += thetaX;
             view_roty += thetaY;
-            if(e.isConfined() && confinedFixedCenter && null!=window) {
-                x=window.getSurfaceWidth()/2;
-                y=window.getSurfaceHeight()/2;
-                window.warpPointer(x, y);
-            }
             prevMouseX = x;
             prevMouseY = y;
             // System.err.println("rotXY.1: "+view_rotx+"/"+view_roty+", source "+e);
