@@ -30,23 +30,24 @@ package com.jogamp.opengl.test.junit.jogl.demos.es2.av;
 
 import java.io.File;
 import java.io.IOException;
-import java.net.URI;
 import java.net.URISyntaxException;
 
-import javax.media.opengl.GL;
-import javax.media.opengl.GL2ES2;
-import javax.media.opengl.GLAnimatorControl;
-import javax.media.opengl.GLAutoDrawable;
-import javax.media.opengl.GLCapabilities;
-import javax.media.opengl.GLEventListener;
-import javax.media.opengl.GLException;
-import javax.media.opengl.GLProfile;
+import com.jogamp.opengl.GL;
+import com.jogamp.opengl.GL2ES2;
+import com.jogamp.opengl.GLAnimatorControl;
+import com.jogamp.opengl.GLAutoDrawable;
+import com.jogamp.opengl.GLCapabilities;
+import com.jogamp.opengl.GLEventListener;
+import com.jogamp.opengl.GLException;
+import com.jogamp.opengl.GLProfile;
 
-import com.jogamp.common.util.IOUtil;
+import com.jogamp.common.net.Uri;
+import com.jogamp.common.util.InterruptSource;
 import com.jogamp.graph.curve.Region;
 import com.jogamp.graph.curve.opengl.GLRegion;
 import com.jogamp.graph.curve.opengl.RegionRenderer;
 import com.jogamp.graph.font.Font;
+import com.jogamp.junit.util.JunitTracer;
 import com.jogamp.newt.Window;
 import com.jogamp.newt.event.KeyAdapter;
 import com.jogamp.newt.event.KeyEvent;
@@ -58,7 +59,6 @@ import com.jogamp.opengl.JoglVersion;
 import com.jogamp.opengl.test.junit.graph.TextRendererGLELBase;
 import com.jogamp.opengl.test.junit.jogl.demos.es2.TextureSequenceCubeES2;
 import com.jogamp.opengl.test.junit.util.MiscUtils;
-import com.jogamp.opengl.test.junit.util.UITestCase;
 import com.jogamp.opengl.util.Animator;
 import com.jogamp.opengl.util.av.GLMediaPlayer;
 import com.jogamp.opengl.util.av.GLMediaPlayer.GLMediaEventListener;
@@ -76,18 +76,18 @@ public class MovieCube implements GLEventListener {
     private TextureSequenceCubeES2 cube=null;
     private GLMediaPlayer mPlayer=null;
     private int swapInterval = 1;
-    private int swapIntervalSet = -1;
+    private boolean swapIntervalSet = true;
     private long lastPerfPos = 0;
     private volatile boolean resetGLState = false;
 
     /** Blender's Big Buck Bunny: 24f 416p H.264,  AAC 48000 Hz, 2 ch, mpeg stream. */
-    public static final URI defURI;
+    public static final Uri defURI;
     static {
-        URI _defURI = null;
+        Uri _defURI = null;
         try {
             // Blender's Big Buck Bunny Trailer: 24f 640p VP8, Vorbis 44100Hz mono, WebM/Matroska Stream.
             // _defURI = new URI("http://video.webmfiles.org/big-buck-bunny_trailer.webm");
-            _defURI = new URI("http://archive.org/download/BigBuckBunny_328/BigBuckBunny_512kb.mp4");
+            _defURI = Uri.cast("http://archive.org/download/BigBuckBunny_328/BigBuckBunny_512kb.mp4");
         } catch (final URISyntaxException e) {
             e.printStackTrace();
         }
@@ -117,7 +117,7 @@ public class MovieCube implements GLEventListener {
                     resetGLState();
                 }
                 if( 0 != ( GLMediaEventListener.EVENT_CHANGE_EOS & event_mask ) ) {
-                    new Thread() {
+                    new InterruptSource.Thread() {
                         public void run() {
                             // loop for-ever ..
                             mPlayer.seek(0);
@@ -151,7 +151,7 @@ public class MovieCube implements GLEventListener {
         mPlayer = GLMediaPlayerFactory.createDefault();
     }
 
-    public void initStream(final URI streamLoc, final int vid, final int aid, final int textureCount) {
+    public void initStream(final Uri streamLoc, final int vid, final int aid, final int textureCount) {
         mPlayer.initStream(streamLoc, vid, aid, textureCount);
         System.out.println("pC.1b "+mPlayer);
     }
@@ -247,12 +247,12 @@ public class MovieCube implements GLEventListener {
             final String text1 = String.format("%0"+ptsPrec+"f/%0"+ptsPrec+"f s, %s (%01.2fx, vol %01.2f), a %01.2f, fps %02.1f -> %02.1f / %02.1f, v-sync %d",
                     pts, mPlayer.getDuration() / 1000f,
                     mPlayer.getState().toString().toLowerCase(), mPlayer.getPlaySpeed(), mPlayer.getAudioVolume(),
-                    aspect, mPlayer.getFramerate(), lfps, tfps, swapIntervalSet);
+                    aspect, mPlayer.getFramerate(), lfps, tfps, swapInterval);
             final String text2 = String.format("audio: id %d, kbps %d, codec %s",
                     mPlayer.getAID(), mPlayer.getAudioBitrate()/1000, mPlayer.getAudioCodec());
             final String text3 = String.format("video: id %d, kbps %d, codec %s",
                     mPlayer.getVID(), mPlayer.getVideoBitrate()/1000, mPlayer.getVideoCodec());
-            final String text4 = mPlayer.getURI().getRawPath();
+            final String text4 = mPlayer.getUri().path.decode();
             if( displayOSD && null != renderer ) {
                 gl.glClearColor(1.0f, 1.0f, 1.0f, 0.0f);
                 if( null != regionFPS ) {
@@ -279,10 +279,13 @@ public class MovieCube implements GLEventListener {
             int pts1 = 0;
             switch(e.getKeySymbol()) {
                 case KeyEvent.VK_V: {
-                    switch(swapIntervalSet) {
-                        case 0: swapInterval = 1; break;
-                        default: swapInterval = 0; break;
+                    switch(swapInterval) {
+                        case  0: swapInterval = -1; break;
+                        case -1: swapInterval =  1; break;
+                        case  1: swapInterval =  0; break;
+                        default: swapInterval =  1; break;
                     }
+                    swapIntervalSet = true;
                     break;
                 }
                 case KeyEvent.VK_O:          displayOSD = !displayOSD; break;
@@ -363,7 +366,7 @@ public class MovieCube implements GLEventListener {
         cube = new TextureSequenceCubeES2(mPlayer, false, zoom0, rotx, roty);
 
         if(waitForKey) {
-            UITestCase.waitForKey("Init>");
+            JunitTracer.waitForKey("Init>");
         }
 
         if( GLMediaPlayer.State.Initialized == mPlayer.getState() ) {
@@ -434,12 +437,14 @@ public class MovieCube implements GLEventListener {
 
     @Override
     public void display(final GLAutoDrawable drawable) {
-        if(-1 != swapInterval) {
+        if( swapIntervalSet ) {
             final GL2ES2 gl = drawable.getGL().getGL2ES2();
-            gl.setSwapInterval(swapInterval); // in case switching the drawable (impl. may bound attribute there)
+            final int _swapInterval = swapInterval;
+            gl.setSwapInterval(_swapInterval); // in case switching the drawable (impl. may bound attribute there)
             drawable.getAnimator().resetFPSCounter();
-            swapIntervalSet = swapInterval;
-            swapInterval = -1;
+            swapInterval = gl.getSwapInterval();
+            System.err.println("Swap Interval: "+_swapInterval+" -> "+swapInterval);
+            swapIntervalSet = false;
         }
         if(null == mPlayer) { return; }
 
@@ -517,11 +522,11 @@ public class MovieCube implements GLEventListener {
             }
             origSize = _origSize;
         }
-        final URI streamLoc;
+        final Uri streamLoc;
         if( null != url_s ) {
-            streamLoc = new URI(url_s);
+            streamLoc = Uri.cast( url_s );
         } else if( null != file_s ) {
-            streamLoc = IOUtil.toURISimple(new File(file_s));
+            streamLoc = Uri.valueOf(new File(file_s));
         } else {
             streamLoc = defURI;
         }
@@ -595,7 +600,7 @@ public class MovieCube implements GLEventListener {
                     if( null != se ) {
                         se.printStackTrace();
                     }
-                    new Thread() {
+                    new InterruptSource.Thread() {
                         public void run() {
                             window.destroy();
                         } }.start();
