@@ -30,28 +30,29 @@ package com.jogamp.opengl.test.junit.jogl.demos.es2.av;
 
 import java.io.File;
 import java.io.IOException;
-import java.net.URI;
 import java.net.URISyntaxException;
 import java.nio.FloatBuffer;
 
-import javax.media.opengl.GL;
-import javax.media.opengl.GL2ES2;
-import javax.media.opengl.GLAnimatorControl;
-import javax.media.opengl.GLAutoDrawable;
-import javax.media.opengl.GLCapabilities;
-import javax.media.opengl.GLES2;
-import javax.media.opengl.GLEventListener;
-import javax.media.opengl.GLException;
-import javax.media.opengl.GLProfile;
-import javax.media.opengl.GLUniformData;
-import javax.media.opengl.fixedfunc.GLMatrixFunc;
+import com.jogamp.opengl.GL;
+import com.jogamp.opengl.GL2ES2;
+import com.jogamp.opengl.GLAnimatorControl;
+import com.jogamp.opengl.GLAutoDrawable;
+import com.jogamp.opengl.GLCapabilities;
+import com.jogamp.opengl.GLES2;
+import com.jogamp.opengl.GLEventListener;
+import com.jogamp.opengl.GLException;
+import com.jogamp.opengl.GLProfile;
+import com.jogamp.opengl.GLUniformData;
+import com.jogamp.opengl.fixedfunc.GLMatrixFunc;
 
+import com.jogamp.common.net.Uri;
 import com.jogamp.common.os.Platform;
-import com.jogamp.common.util.IOUtil;
+import com.jogamp.common.util.InterruptSource;
 import com.jogamp.graph.curve.Region;
 import com.jogamp.graph.curve.opengl.GLRegion;
 import com.jogamp.graph.curve.opengl.RegionRenderer;
 import com.jogamp.graph.font.Font;
+import com.jogamp.junit.util.JunitTracer;
 import com.jogamp.newt.Window;
 import com.jogamp.newt.event.KeyAdapter;
 import com.jogamp.newt.event.KeyEvent;
@@ -66,7 +67,6 @@ import com.jogamp.opengl.GLExtensions;
 import com.jogamp.opengl.JoglVersion;
 import com.jogamp.opengl.test.junit.graph.TextRendererGLELBase;
 import com.jogamp.opengl.test.junit.util.MiscUtils;
-import com.jogamp.opengl.test.junit.util.UITestCase;
 import com.jogamp.opengl.util.Animator;
 import com.jogamp.opengl.util.GLArrayDataServer;
 import com.jogamp.opengl.util.PMVMatrix;
@@ -106,7 +106,7 @@ public class MovieSimple implements GLEventListener {
     private int effects = EFFECT_NORMAL;
     private float alpha = 1.0f;
     private int swapInterval = 1;
-    private int swapIntervalSet = -1;
+    private boolean swapIntervalSet = true;
 
     private GLMediaPlayer mPlayer;
     private final boolean mPlayerShared;
@@ -122,13 +122,13 @@ public class MovieSimple implements GLEventListener {
     private static final String myTextureLookupName = "myTexture2D";
 
     /** Blender's Big Buck Bunny: 24f 416p H.264,  AAC 48000 Hz, 2 ch, mpeg stream. */
-    public static final URI defURI;
+    public static final Uri defURI;
     static {
-        URI _defURI = null;
+        Uri _defURI = null;
         try {
             // Blender's Big Buck Bunny Trailer: 24f 640p VP8, Vorbis 44100Hz mono, WebM/Matroska Stream.
             // _defURI = new URI("http://video.webmfiles.org/big-buck-bunny_trailer.webm");
-            _defURI = new URI("http://archive.org/download/BigBuckBunny_328/BigBuckBunny_512kb.mp4");
+            _defURI = Uri.cast("http://archive.org/download/BigBuckBunny_328/BigBuckBunny_512kb.mp4");
         } catch (final URISyntaxException e) {
             e.printStackTrace();
         }
@@ -186,7 +186,7 @@ public class MovieSimple implements GLEventListener {
             final float aspect = (float)mPlayer.getWidth() / (float)mPlayer.getHeight();
 
             final String ptsPrec = null != regionFPS ? "3.1" : "3.0";
-            final String text1 = String.format("%0"+ptsPrec+"f/%0"+ptsPrec+"f s, %s (%01.2fx, vol %01.2f), a %01.2f, fps %02.1f -> %02.1f / %02.1f, v-sync %d",
+            final String text1 = String.format("%0"+ptsPrec+"f/%0"+ptsPrec+"f s, %s (%01.2fx, vol %01.2f), a %01.2f, fps %02.1f -> %02.1f / %02.1f, v-sync %b",
                     pts, mPlayer.getDuration() / 1000f,
                     mPlayer.getState().toString().toLowerCase(), mPlayer.getPlaySpeed(), mPlayer.getAudioVolume(),
                     aspect, mPlayer.getFramerate(), lfps, tfps, swapIntervalSet);
@@ -194,7 +194,7 @@ public class MovieSimple implements GLEventListener {
                     mPlayer.getAID(), mPlayer.getAudioBitrate()/1000, mPlayer.getAudioCodec());
             final String text3 = String.format("video: id %d, kbps %d, codec %s",
                     mPlayer.getVID(), mPlayer.getVideoBitrate()/1000, mPlayer.getVideoCodec());
-            final String text4 = mPlayer.getURI().getRawPath();
+            final String text4 = mPlayer.getUri().path.decode();
             if( displayOSD && null != renderer ) {
                 // We share ClearColor w/ MovieSimple's init !
                 final float pixelSize = font.getPixelSize(fontSize, dpiH);
@@ -266,10 +266,13 @@ public class MovieSimple implements GLEventListener {
             int pts1 = 0;
             switch(e.getKeySymbol()) {
                 case KeyEvent.VK_V: {
-                    switch(swapIntervalSet) {
-                        case 0: swapInterval = 1; break;
-                        default: swapInterval = 0; break;
+                    switch(swapInterval) {
+                        case  0: swapInterval = -1; break;
+                        case -1: swapInterval =  1; break;
+                        case  1: swapInterval =  0; break;
+                        default: swapInterval =  1; break;
                     }
+                    swapIntervalSet = true;
                     break;
                 }
                 case KeyEvent.VK_O:          displayOSD = !displayOSD; break;
@@ -353,7 +356,7 @@ public class MovieSimple implements GLEventListener {
                     resetGLState();
                 }
                 if( 0 != ( GLMediaEventListener.EVENT_CHANGE_EOS & event_mask ) ) {
-                    new Thread() {
+                    new InterruptSource.Thread() {
                         public void run() {
                             // loop for-ever ..
                             mPlayer.seek(0);
@@ -388,7 +391,7 @@ public class MovieSimple implements GLEventListener {
         System.out.println("pC.1a shared "+mPlayerShared+", "+mPlayer);
     }
 
-    public void initStream(final URI streamLoc, final int vid, final int aid, final int textureCount) {
+    public void initStream(final Uri streamLoc, final int vid, final int aid, final int textureCount) {
         mPlayer.initStream(streamLoc, vid, aid, textureCount);
         System.out.println("pC.1b "+mPlayer);
     }
@@ -484,7 +487,7 @@ public class MovieSimple implements GLEventListener {
                            ", "+drawable.getClass().getName()+", "+drawable);
 
         if(waitForKey) {
-            UITestCase.waitForKey("Init>");
+            JunitTracer.waitForKey("Init>");
         }
         final Texture tex;
         try {
@@ -737,11 +740,13 @@ public class MovieSimple implements GLEventListener {
     @Override
     public void display(final GLAutoDrawable drawable) {
         final GL2ES2 gl = drawable.getGL().getGL2ES2();
-        if(-1 != swapInterval) {
-            gl.setSwapInterval(swapInterval); // in case switching the drawable (impl. may bound attribute there)
+        if( swapIntervalSet ) {
+            final int _swapInterval = swapInterval;
+            gl.setSwapInterval(_swapInterval); // in case switching the drawable (impl. may bound attribute there)
             drawable.getAnimator().resetFPSCounter();
-            swapIntervalSet = swapInterval;
-            swapInterval = -1;
+            swapInterval = gl.getSwapInterval();
+            System.err.println("Swap Interval: "+_swapInterval+" -> "+swapInterval);
+            swapIntervalSet = false;
         }
         if(null == mPlayer) { return; }
 
@@ -803,7 +808,7 @@ public class MovieSimple implements GLEventListener {
 
     static class MyGLMediaEventListener implements GLMediaEventListener {
             void destroyWindow(final Window window) {
-                new Thread() {
+                new InterruptSource.Thread() {
                     public void run() {
                         window.destroy();
                     } }.start();
@@ -837,7 +842,7 @@ public class MovieSimple implements GLEventListener {
                     /**
                      * Kick off player w/o GLEventListener, i.e. for audio only.
                      *
-                        new Thread() {
+                        new InterruptSource.Thread() {
                             public void run() {
                                 try {
                                     mp.initGL(null);
@@ -869,7 +874,7 @@ public class MovieSimple implements GLEventListener {
                     } else {
                         System.err.println("MovieSimple State: EOS");
                         if( loopEOS ) {
-                            new Thread() {
+                            new InterruptSource.Thread() {
                                 public void run() {
                                     mp.setPlaySpeed(1f);
                                     mp.seek(0);
@@ -988,14 +993,14 @@ public class MovieSimple implements GLEventListener {
             }
             origSize = _origSize;
         }
-        final URI streamLoc0;
+        final Uri streamLoc0;
         if( null != urls_s[0] ) {
-            streamLoc0 = new URI(urls_s[0]);
+            streamLoc0 = Uri.cast( urls_s[0] );
         } else if( null != file_s1 ) {
             final File movieFile = new File(file_s1);
-            streamLoc0 = movieFile.toURI();
+            streamLoc0 = Uri.valueOf(movieFile);
         } else if( null != file_s2 ) {
-            streamLoc0 = IOUtil.toURISimple(new File(file_s2));
+            streamLoc0 = Uri.valueOf(new File(file_s2));
         } else {
             streamLoc0 = defURI;
         }
@@ -1049,12 +1054,12 @@ public class MovieSimple implements GLEventListener {
             windows[i].setVisible(true);
             anim.add(windows[i]);
 
-            final URI streamLocN;
+            final Uri streamLocN;
             if( 0 == i ) {
                 streamLocN = streamLoc0;
             } else {
                 if( null != urls_s[i] ) {
-                    streamLocN = new URI(urls_s[i]);
+                    streamLocN = Uri.cast(urls_s[i]);
                 } else {
                     streamLocN = defURI;
                 }
