@@ -35,16 +35,21 @@ import java.awt.event.ComponentEvent;
 import java.awt.event.ComponentListener;
 import java.lang.reflect.InvocationTargetException;
 
-import javax.media.nativewindow.ScalableSurface;
-import javax.media.opengl.GLAutoDrawable;
-import javax.media.opengl.GLCapabilities;
-import javax.media.opengl.GLCapabilitiesImmutable;
-import javax.media.opengl.GLEventListener;
-import javax.media.opengl.GLProfile;
-import javax.media.opengl.awt.GLJPanel;
+import com.jogamp.nativewindow.ScalableSurface;
+import com.jogamp.nativewindow.util.Rectangle;
+import com.jogamp.nativewindow.util.RectangleImmutable;
+import com.jogamp.opengl.GLAutoDrawable;
+import com.jogamp.opengl.GLCapabilities;
+import com.jogamp.opengl.GLCapabilitiesImmutable;
+import com.jogamp.opengl.GLEventListener;
+import com.jogamp.opengl.GLProfile;
+import com.jogamp.opengl.awt.GLJPanel;
+
 import javax.swing.JFrame;
 import javax.swing.SwingUtilities;
 
+import jogamp.newt.awt.NewtFactoryAWT;
+
 import org.junit.AfterClass;
 import org.junit.Assert;
 import org.junit.Assume;
@@ -53,11 +58,14 @@ import org.junit.Test;
 import org.junit.FixMethodOrder;
 import org.junit.runners.MethodSorters;
 
+import com.jogamp.newt.MonitorDevice;
+import com.jogamp.newt.Screen;
 import com.jogamp.newt.event.KeyEvent;
 import com.jogamp.newt.event.TraceKeyAdapter;
 import com.jogamp.newt.event.TraceWindowAdapter;
 import com.jogamp.newt.event.awt.AWTKeyAdapter;
 import com.jogamp.newt.event.awt.AWTWindowAdapter;
+import com.jogamp.opengl.test.junit.jogl.demos.GLClearOnInitReshape;
 import com.jogamp.opengl.test.junit.jogl.demos.es2.GearsES2;
 import com.jogamp.opengl.test.junit.jogl.demos.gl2.Gears;
 import com.jogamp.opengl.test.junit.util.AWTRobotUtil;
@@ -70,17 +78,20 @@ import com.jogamp.opengl.util.FPSAnimator;
 public class TestGearsES2GLJPanelAWT extends UITestCase {
     static Dimension wsize, rwsize=null;
     static boolean forceES2 = false;
+    static boolean forceES3 = false;
     static boolean forceGL3 = false;
     static boolean forceGLFFP = false;
+    static int demoType = 1;
     static boolean shallUsePBuffer = false;
     static boolean shallUseBitmap = false;
     static boolean useMSAA = false;
+    static int msaaNumSamples = 4;
     static int swapInterval = 0;
     static boolean useAnimator = true;
     static boolean manualTest = false;
     static boolean skipGLOrientationVerticalFlip = false;
     static int xpos = 10, ypos = 10;
-    static int[] reqSurfacePixelScale = new int[] { ScalableSurface.AUTOMAX_PIXELSCALE, ScalableSurface.AUTOMAX_PIXELSCALE };
+    static float[] reqSurfacePixelScale = new float[] { ScalableSurface.AUTOMAX_PIXELSCALE, ScalableSurface.AUTOMAX_PIXELSCALE };
 
     @BeforeClass
     public static void initClass() {
@@ -111,15 +122,41 @@ public class TestGearsES2GLJPanelAWT extends UITestCase {
     private void setTitle(final JFrame frame, final GLJPanel glc, final GLCapabilitiesImmutable caps) {
         final String capsA = caps.isBackgroundOpaque() ? "opaque" : "transl";
         final java.awt.Rectangle b = glc.getBounds();
-        frame.setTitle("GLJPanel["+capsA+"], swapI "+swapInterval+", win: ["+b.x+"/"+b.y+" "+b.width+"x"+b.height+"], pix: "+glc.getSurfaceWidth()+"x"+glc.getSurfaceHeight());
+
+        final float[] minSurfacePixelScale = glc.getMinimumSurfaceScale(new float[2]);
+        final float[] maxSurfacePixelScale = glc.getMaximumSurfaceScale(new float[2]);
+        final float[] reqSurfacePixelScale = glc.getRequestedSurfaceScale(new float[2]);
+        final float[] hasSurfacePixelScale = glc.getCurrentSurfaceScale(new float[2]);
+        frame.setTitle("GLJPanel["+capsA+"], swapI "+swapInterval+", win: ["+b.x+"/"+b.y+" "+b.width+"x"+b.height+"], pix: "+glc.getSurfaceWidth()+"x"+glc.getSurfaceHeight()+
+                ", scale[min "+minSurfacePixelScale[0]+"x"+minSurfacePixelScale[1]+", max "+
+                maxSurfacePixelScale[0]+"x"+maxSurfacePixelScale[1]+", req "+
+                reqSurfacePixelScale[0]+"x"+reqSurfacePixelScale[1]+" -> has "+
+                hasSurfacePixelScale[0]+"x"+hasSurfacePixelScale[1]+"]");
     }
 
-    protected void runTestGL(final GLCapabilities caps)
+    protected GLEventListener createDemo(final GLCapabilities caps) {
+        final GLEventListener demo;
+        if( 1 == demoType ) {
+            if( caps.isBitmap() || caps.getGLProfile().isGL2() ) {
+                final Gears gears = new Gears(swapInterval);
+                gears.setFlipVerticalInGLOrientation(skipGLOrientationVerticalFlip);
+                demo = gears;
+            } else {
+                final GearsES2 gears = new GearsES2(swapInterval);
+                gears.setFlipVerticalInGLOrientation(skipGLOrientationVerticalFlip);
+                demo = gears;
+            }
+        } else if( 0 == demoType ) {
+            demo = new GLClearOnInitReshape();
+        } else {
+            demo = null;
+        }
+        return demo;
+    }
+
+    protected GLJPanel newGLJPanel(final JFrame frame, final GLCapabilities caps, final FPSAnimator animator, final SnapshotGLEventListener snap)
             throws AWTException, InterruptedException, InvocationTargetException
     {
-        final JFrame frame = new JFrame("Swing GLJPanel");
-        Assert.assertNotNull(frame);
-
         final GLJPanel glJPanel = new GLJPanel(caps);
         Assert.assertNotNull(glJPanel);
         glJPanel.setSkipGLOrientationVerticalFlip(skipGLOrientationVerticalFlip);
@@ -127,18 +164,15 @@ public class TestGearsES2GLJPanelAWT extends UITestCase {
         glJPanel.setPreferredSize(wsize);
         glJPanel.setSize(wsize);
         glJPanel.setSurfaceScale(reqSurfacePixelScale);
-        final int[] valReqSurfacePixelScale = glJPanel.getRequestedSurfaceScale(new int[2]);
-        if( caps.isBitmap() || caps.getGLProfile().isGL2() ) {
-            final Gears gears = new Gears(swapInterval);
-            gears.setFlipVerticalInGLOrientation(skipGLOrientationVerticalFlip);
-            glJPanel.addGLEventListener(gears);
-        } else {
-            final GearsES2 gears = new GearsES2(swapInterval);
-            gears.setFlipVerticalInGLOrientation(skipGLOrientationVerticalFlip);
-            glJPanel.addGLEventListener(gears);
+        {
+            final GLEventListener demo = createDemo(caps);
+            if( null != demo ) {
+                glJPanel.addGLEventListener(demo);
+            }
+        }
+        if( null != snap ) {
+            glJPanel.addGLEventListener(snap);
         }
-        final SnapshotGLEventListener snap = new SnapshotGLEventListener();
-        glJPanel.addGLEventListener(snap);
         glJPanel.addGLEventListener(new GLEventListener() {
             @Override
             public void init(final GLAutoDrawable drawable) { }
@@ -153,7 +187,6 @@ public class TestGearsES2GLJPanelAWT extends UITestCase {
 
         });
         setTitle(frame, glJPanel, caps);
-        frame.setLocation(xpos, ypos);
 
         frame.addComponentListener(new ComponentListener() {
             @Override
@@ -173,61 +206,154 @@ public class TestGearsES2GLJPanelAWT extends UITestCase {
             public void componentHidden(final ComponentEvent e) { }
         });
 
-        final FPSAnimator animator = useAnimator ? new FPSAnimator(glJPanel, 60) : null;
+        if( SwingUtilities.isEventDispatchThread() ) {
+            frame.getContentPane().add(glJPanel, BorderLayout.CENTER);
+            frame.getContentPane().validate();
+            frame.pack();
+            frame.setVisible(true);
+        } else {
+            SwingUtilities.invokeAndWait(new Runnable() {
+                    public void run() {
+                        frame.getContentPane().add(glJPanel, BorderLayout.CENTER);
+                        frame.getContentPane().validate();
+                        frame.pack();
+                        frame.setVisible(true);
+                    } } ) ;
+            Assert.assertEquals(true,  AWTRobotUtil.waitForVisible(frame, true));
+            Assert.assertEquals(true,  AWTRobotUtil.waitForRealized(glJPanel, true));
 
-        SwingUtilities.invokeAndWait(new Runnable() {
-                public void run() {
-                    frame.getContentPane().add(glJPanel, BorderLayout.CENTER);
-                    frame.getContentPane().validate();
-                    frame.pack();
-                    frame.setVisible(true);
-                } } ) ;
-        Assert.assertEquals(true,  AWTRobotUtil.waitForVisible(frame, true));
-        Assert.assertEquals(true,  AWTRobotUtil.waitForRealized(glJPanel, true));
+            final float[] minSurfacePixelScale = glJPanel.getMinimumSurfaceScale(new float[2]);
+            final float[] maxSurfacePixelScale = glJPanel.getMaximumSurfaceScale(new float[2]);
+            final float[] valReqSurfacePixelScale = glJPanel.getRequestedSurfaceScale(new float[2]);
+            final float[] hasSurfacePixelScale = glJPanel.getCurrentSurfaceScale(new float[2]);
+            System.err.println("HiDPI PixelScale: min "+
+                               minSurfacePixelScale[0]+"x"+minSurfacePixelScale[1]+", max "+
+                               maxSurfacePixelScale[0]+"x"+maxSurfacePixelScale[1]+", req "+
+                               reqSurfacePixelScale[0]+"x"+reqSurfacePixelScale[1]+" -> val "+
+                               valReqSurfacePixelScale[0]+"x"+valReqSurfacePixelScale[1]+" -> has "+
+                               hasSurfacePixelScale[0]+"x"+hasSurfacePixelScale[1]);
+            setTitle(frame, glJPanel, caps);
+        }
 
-        final int[] hasSurfacePixelScale0 = glJPanel.getNativeSurface().convertToPixelUnits(new int[] { 1, 1 });
-        final int[] hasSurfacePixelScale1 = glJPanel.getCurrentSurfaceScale(new int[2]);
-        System.err.println("HiDPI PixelScale: "+reqSurfacePixelScale[0]+"x"+reqSurfacePixelScale[1]+" (req) -> "+
-                           valReqSurfacePixelScale[0]+"x"+valReqSurfacePixelScale[1]+" (val) -> "+
-                           hasSurfacePixelScale1[0]+"x"+hasSurfacePixelScale1[1]+" (has)");
-        setTitle(frame, glJPanel, caps);
-        Assert.assertArrayEquals(hasSurfacePixelScale0, hasSurfacePixelScale1);
-
-        if( useAnimator ) {
+        if( null != animator ) {
+            animator.add(glJPanel);
             animator.setUpdateFPSFrames(60, System.err);
+        }
+        return glJPanel;
+    }
+
+    protected void destroy(final JFrame frame, final GLJPanel glJPanel) {
+        try {
+            if( SwingUtilities.isEventDispatchThread() ) {
+                if( null != frame ) {
+                    frame.setVisible(false);
+                    if( null != glJPanel ) {
+                        frame.getContentPane().remove(glJPanel);
+                    }
+                    frame.remove(glJPanel);
+                }
+                if( null != glJPanel ) {
+                    glJPanel.destroy();
+                }
+                if( null != frame ) {
+                    frame.dispose();
+                }
+            } else {
+                SwingUtilities.invokeAndWait(new Runnable() {
+                        public void run() {
+                            if( null != frame ) {
+                                frame.setVisible(false);
+                                if( null != glJPanel ) {
+                                    frame.getContentPane().remove(glJPanel);
+                                }
+                                frame.remove(glJPanel);
+                            }
+                            if( null != glJPanel ) {
+                                glJPanel.destroy();
+                            }
+                            if( null != frame ) {
+                                frame.dispose();
+                            }
+                        } } );
+            }
+        } catch (final Exception e) {
+            e.printStackTrace();
+        }
+    }
+
+    protected void runTestGL(final GLCapabilities caps)
+            throws AWTException, InterruptedException, InvocationTargetException
+    {
+        final JFrame frame = new JFrame("Swing GLJPanel");
+        frame.setLocation(xpos, ypos);
+        Assert.assertNotNull(frame);
+
+        final FPSAnimator animator = useAnimator ? new FPSAnimator(60) : null;
+        final SnapshotGLEventListener snap = new SnapshotGLEventListener();
+        final GLJPanel glJPanel = newGLJPanel(frame, caps, animator, snap);
+        if( null != animator ) {
             animator.start();
             Assert.assertEquals(true, animator.isAnimating());
         }
+        final Screen screen = NewtFactoryAWT.createScreen(glJPanel, true);
+        screen.addReference(); // initial native creation - keep alive!
+        System.err.println("GetPixelScale: AWT -> Screen: "+screen);
 
         final QuitAdapter quitAdapter = new QuitAdapter();
         new AWTKeyAdapter(new TraceKeyAdapter(quitAdapter), glJPanel).addTo(glJPanel);
         new AWTWindowAdapter(new TraceWindowAdapter(quitAdapter), glJPanel).addTo(frame);
 
+        final JFrame[] frame2 = { null };
+        final GLJPanel[] glJPanel2 = { null };
+
         final com.jogamp.newt.event.KeyListener kl = new com.jogamp.newt.event.KeyAdapter() {
             @Override
             public void keyPressed(final KeyEvent e) {
                 if( e.isAutoRepeat() ) {
                     return;
                 }
-                if(e.getKeyChar()=='x') {
-                    final int[] hadSurfacePixelScale = glJPanel.getCurrentSurfaceScale(new int[2]);
-                    final int[] reqSurfacePixelScale;
-                    if( hadSurfacePixelScale[0] == ScalableSurface.IDENTITY_PIXELSCALE ) {
-                        reqSurfacePixelScale = new int[] { ScalableSurface.AUTOMAX_PIXELSCALE, ScalableSurface.AUTOMAX_PIXELSCALE };
+                if( e.getKeySymbol() == KeyEvent.VK_P ) {
+                    System.err.println();
+                    {
+                        // Just for manual validation!
+                        final java.awt.Point los = glJPanel.getLocationOnScreen();
+                        final RectangleImmutable r = new Rectangle(los.x, los.y, glJPanel.getWidth(), glJPanel.getHeight());
+                        System.err.printf("GetPixelScale: Panel Bounds: %s window-units%n", r.toString());
+                        System.err.printf("GetPixelScale: Panel Resolution: %d x %d pixel-units%n", glJPanel.getSurfaceWidth(), glJPanel.getSurfaceHeight());
+                    }
+                    final MonitorDevice monitor = NewtFactoryAWT.getMonitorDevice(screen, glJPanel);
+                    System.err.printf("GetPixelScale: %s%n", monitor.toString());
+                    final float[] pixelPerMM;
+                    final boolean cached;
+                    if( e.isShiftDown() ) {
+                        // SHIFT: query current mode!
+                        pixelPerMM = monitor.getPixelsPerMM(monitor.queryCurrentMode(), new float[2]);
+                        cached = false;
                     } else {
-                        reqSurfacePixelScale = new int[] { ScalableSurface.IDENTITY_PIXELSCALE, ScalableSurface.IDENTITY_PIXELSCALE };
+                        // Default: Use cached mode!
+                        pixelPerMM = monitor.getPixelsPerMM(new float[2]);
+                        cached = true;
+                    }
+                    System.err.println("  pixel/mm ["+pixelPerMM[0]+", "+pixelPerMM[1]+"], cached-mode "+cached);
+                    System.err.println("  pixel/in ["+pixelPerMM[0]*25.4f+", "+pixelPerMM[1]*25.4f+"], cached-mode "+cached);
+                    System.err.println();
+                } else if(e.getKeyChar()=='x') {
+                    final float[] hadSurfacePixelScale = glJPanel.getCurrentSurfaceScale(new float[2]);
+                    final float[] reqSurfacePixelScale;
+                    if( hadSurfacePixelScale[0] == ScalableSurface.IDENTITY_PIXELSCALE ) {
+                        reqSurfacePixelScale = new float[] { ScalableSurface.AUTOMAX_PIXELSCALE, ScalableSurface.AUTOMAX_PIXELSCALE };
+                    } else {
+                        reqSurfacePixelScale = new float[] { ScalableSurface.IDENTITY_PIXELSCALE, ScalableSurface.IDENTITY_PIXELSCALE };
                     }
                     System.err.println("[set PixelScale pre]: had "+hadSurfacePixelScale[0]+"x"+hadSurfacePixelScale[1]+" -> req "+reqSurfacePixelScale[0]+"x"+reqSurfacePixelScale[1]);
                     glJPanel.setSurfaceScale(reqSurfacePixelScale);
-                    final int[] valReqSurfacePixelScale = glJPanel.getRequestedSurfaceScale(new int[2]);
-                    final int[] hasSurfacePixelScale0 = glJPanel.getNativeSurface().convertToPixelUnits(new int[] { 1, 1 });
-                    final int[] hasSurfacePixelScale1 = glJPanel.getCurrentSurfaceScale(new int[2]);
+                    final float[] valReqSurfacePixelScale = glJPanel.getRequestedSurfaceScale(new float[2]);
+                    final float[] hasSurfacePixelScale1 = glJPanel.getCurrentSurfaceScale(new float[2]);
                     System.err.println("[set PixelScale post]: "+hadSurfacePixelScale[0]+"x"+hadSurfacePixelScale[1]+" (had) -> "+
                                        reqSurfacePixelScale[0]+"x"+reqSurfacePixelScale[1]+" (req) -> "+
                                        valReqSurfacePixelScale[0]+"x"+valReqSurfacePixelScale[1]+" (val) -> "+
                                        hasSurfacePixelScale1[0]+"x"+hasSurfacePixelScale1[1]+" (has)");
                     setTitle(frame, glJPanel, caps);
-                    Assert.assertArrayEquals(hasSurfacePixelScale0, hasSurfacePixelScale1);
                 } else if(e.getKeyChar()=='m') {
                     final GLCapabilitiesImmutable capsPre = glJPanel.getChosenGLCapabilities();
                     final GLCapabilities capsNew = new GLCapabilities(capsPre.getGLProfile());
@@ -247,6 +373,33 @@ public class TestGearsES2GLJPanelAWT extends UITestCase {
                     glJPanel.setRequestedGLCapabilities(capsNew);
                     System.err.println("XXX-A2: "+animator.toString());
                     System.err.println("XXX: "+glJPanel.toString());
+                } else if(e.getKeyChar()=='n') {
+                    System.err.println("XXX: frame2: "+frame2[0]);
+                    if( null != frame2[0] ) {
+                        System.err.println("XXX: frame2.isShowing: "+frame2[0].isShowing());
+                    }
+                    System.err.println("XXX: glJPanel2: "+glJPanel2[0]);
+                    if( null != frame2[0] && frame2[0].isShowing() ) {
+                        destroy(frame2[0], glJPanel2[0]);
+                        frame2[0] = null;
+                        glJPanel2[0] = null;
+                    } else {
+                        frame2[0] = new JFrame("GLJPanel2");
+                        frame2[0].setLocation(frame.getX()+frame.getWidth()+64, frame.getY());
+                        final FPSAnimator animator2 = useAnimator ? new FPSAnimator(60) : null;
+                        if( null != animator2 ) {
+                            animator2.start();
+                        }
+                        final SnapshotGLEventListener snap2 = new SnapshotGLEventListener();
+                        try {
+                            glJPanel2[0] = newGLJPanel(frame2[0], caps, animator2, snap2);
+                        } catch (final Exception e2) {
+                            e2.printStackTrace();
+                            destroy(frame2[0], glJPanel2[0]);
+                            frame2[0] = null;
+                            glJPanel2[0] = null;
+                        }
+                    }
                 }
             } };
         new AWTKeyAdapter(kl, glJPanel).addTo(glJPanel);
@@ -285,14 +438,12 @@ public class TestGearsES2GLJPanelAWT extends UITestCase {
         } else {
             Assert.assertNull(animator);
         }
-        SwingUtilities.invokeAndWait(new Runnable() {
-                public void run() {
-                    frame.setVisible(false);
-                    frame.getContentPane().remove(glJPanel);
-                    frame.remove(glJPanel);
-                    glJPanel.destroy();
-                    frame.dispose();
-                } } );
+
+        screen.removeReference(); // final native destroy
+        destroy(frame, glJPanel);
+        if( null != frame2[0] ) {
+            destroy(frame2[0], glJPanel2[0]);
+        }
     }
 
     @Test
@@ -302,16 +453,18 @@ public class TestGearsES2GLJPanelAWT extends UITestCase {
         final GLProfile glp;
         if(forceGL3) {
             glp = GLProfile.get(GLProfile.GL3);
+        } else if(forceES3) {
+            glp = GLProfile.get(GLProfile.GLES3);
         } else if(forceES2) {
             glp = GLProfile.get(GLProfile.GLES2);
         } else if(forceGLFFP) {
             glp = GLProfile.getMaxFixedFunc(true);
         } else {
-            glp = GLProfile.getGL2ES2();
+            glp = GLProfile.getDefault();
         }
         final GLCapabilities caps = new GLCapabilities( glp );
         if(useMSAA) {
-            caps.setNumSamples(4);
+            caps.setNumSamples(msaaNumSamples);
             caps.setSampleBuffers(true);
         }
         if(shallUsePBuffer) {
@@ -406,7 +559,24 @@ public class TestGearsES2GLJPanelAWT extends UITestCase {
     }
 
     @Test
-    public void test30_GL3()
+    public void test30_GLES3()
+            throws AWTException, InterruptedException, InvocationTargetException
+    {
+        if( manualTest ) {
+            return;
+        }
+
+        if( !GLProfile.isAvailable(GLProfile.GLES3) ) {
+            System.err.println("GLES3 n/a");
+            return;
+        }
+        final GLProfile glp = GLProfile.get(GLProfile.GLES3);
+        final GLCapabilities caps = new GLCapabilities( glp );
+        runTestGL(caps);
+    }
+
+    @Test
+    public void test40_GL3()
             throws AWTException, InterruptedException, InvocationTargetException
     {
         if( manualTest ) {
@@ -447,6 +617,8 @@ public class TestGearsES2GLJPanelAWT extends UITestCase {
                 duration = MiscUtils.atol(args[i], duration);
             } else if(args[i].equals("-es2")) {
                 forceES2 = true;
+            } else if(args[i].equals("-es3")) {
+                forceES3 = true;
             } else if(args[i].equals("-gl3")) {
                 forceGL3 = true;
             } else if(args[i].equals("-glFFP")) {
@@ -471,7 +643,7 @@ public class TestGearsES2GLJPanelAWT extends UITestCase {
                 rh = MiscUtils.atoi(args[i], rh);
             } else if(args[i].equals("-pixelScale")) {
                 i++;
-                final int pS = MiscUtils.atoi(args[i], reqSurfacePixelScale[0]);
+                final float pS = MiscUtils.atof(args[i], reqSurfacePixelScale[0]);
                 reqSurfacePixelScale[0] = pS;
                 reqSurfacePixelScale[1] = pS;
             } else if(args[i].equals("-userVFlip")) {
@@ -480,7 +652,9 @@ public class TestGearsES2GLJPanelAWT extends UITestCase {
                 i++;
                 swapInterval = MiscUtils.atoi(args[i], swapInterval);
             } else if(args[i].equals("-msaa")) {
+                i++;
                 useMSAA = true;
+                msaaNumSamples = MiscUtils.atoi(args[i], msaaNumSamples);
             } else if(args[i].equals("-noanim")) {
                 useAnimator  = false;
             } else if(args[i].equals("-pbuffer")) {
@@ -489,6 +663,9 @@ public class TestGearsES2GLJPanelAWT extends UITestCase {
                 shallUseBitmap = true;
             } else if(args[i].equals("-manual")) {
                 manualTest = true;
+            } else if(args[i].equals("-demo")) {
+                i++;
+                demoType = MiscUtils.atoi(args[i], 0);
             }
         }
         wsize = new Dimension(w, h);
@@ -503,11 +680,12 @@ public class TestGearsES2GLJPanelAWT extends UITestCase {
         System.err.println("forceES2 "+forceES2);
         System.err.println("forceGL3 "+forceGL3);
         System.err.println("forceGLFFP "+forceGLFFP);
-        System.err.println("useMSAA "+useMSAA);
+        System.err.println("useMSAA "+useMSAA+", msaaNumSamples "+msaaNumSamples);
         System.err.println("useAnimator "+useAnimator);
         System.err.println("shallUsePBuffer "+shallUsePBuffer);
         System.err.println("shallUseBitmap "+shallUseBitmap);
         System.err.println("manualTest "+manualTest);
+        System.err.println("demoType "+demoType);
 
         org.junit.runner.JUnitCore.main(TestGearsES2GLJPanelAWT.class.getName());
     }
