@@ -30,48 +30,42 @@ package com.jogamp.opengl.test.junit.jogl.demos.es2.newt;
 
 import java.io.IOException;
 import java.lang.reflect.InvocationTargetException;
-import java.net.URLConnection;
 
-import com.jogamp.common.util.IOUtil;
 import com.jogamp.junit.util.JunitTracer;
 import com.jogamp.newt.Display;
-import com.jogamp.newt.Display.PointerIcon;
 import com.jogamp.newt.NewtFactory;
 import com.jogamp.newt.Screen;
 import com.jogamp.newt.Window;
+import com.jogamp.newt.event.WindowEvent;
 import com.jogamp.newt.event.KeyAdapter;
 import com.jogamp.newt.event.KeyEvent;
-import com.jogamp.newt.event.MouseAdapter;
-import com.jogamp.newt.event.MouseEvent;
-import com.jogamp.newt.event.WindowEvent;
+import com.jogamp.newt.event.TraceMouseAdapter;
 import com.jogamp.newt.event.WindowAdapter;
 import com.jogamp.newt.opengl.GLWindow;
+import com.jogamp.newt.opengl.util.NEWTDemoListener;
 import com.jogamp.newt.util.EDTUtil;
 import com.jogamp.opengl.test.junit.util.AWTRobotUtil;
 import com.jogamp.opengl.test.junit.util.MiscUtils;
 import com.jogamp.opengl.test.junit.util.UITestCase;
-import com.jogamp.opengl.test.junit.util.QuitAdapter;
 import com.jogamp.opengl.util.Animator;
 import com.jogamp.opengl.util.AnimatorBase;
-import com.jogamp.opengl.util.Gamma;
-import com.jogamp.opengl.util.PNGPixelRect;
+import com.jogamp.opengl.test.junit.jogl.demos.GLClearOnInitReshape;
 import com.jogamp.opengl.test.junit.jogl.demos.es2.GearsES2;
-
-import javax.media.nativewindow.NativeWindowFactory;
-import javax.media.nativewindow.ScalableSurface;
-import javax.media.nativewindow.util.Dimension;
-import javax.media.nativewindow.util.Point;
-import javax.media.nativewindow.util.PointImmutable;
-import javax.media.nativewindow.util.DimensionImmutable;
-import javax.media.opengl.GLAnimatorControl;
-import javax.media.opengl.GLAutoDrawable;
-import javax.media.opengl.GLCapabilities;
-import javax.media.opengl.GLCapabilitiesImmutable;
-import javax.media.opengl.GLEventListener;
-import javax.media.opengl.GLProfile;
+import com.jogamp.opengl.test.junit.jogl.demos.es2.LineSquareXDemoES2;
+import com.jogamp.nativewindow.NativeWindowFactory;
+import com.jogamp.nativewindow.ScalableSurface;
+import com.jogamp.nativewindow.util.Dimension;
+import com.jogamp.nativewindow.util.Point;
+import com.jogamp.nativewindow.util.PointImmutable;
+import com.jogamp.nativewindow.util.DimensionImmutable;
+import com.jogamp.opengl.GLAnimatorControl;
+import com.jogamp.opengl.GLAutoDrawable;
+import com.jogamp.opengl.GLCapabilities;
+import com.jogamp.opengl.GLCapabilitiesImmutable;
+import com.jogamp.opengl.GLEventListener;
+import com.jogamp.opengl.GLProfile;
 
 import jogamp.newt.DefaultEDTUtil;
-import jogamp.newt.driver.PNGIcon;
 
 import org.junit.Assert;
 import org.junit.BeforeClass;
@@ -85,13 +79,18 @@ public class TestGearsES2NEWT extends UITestCase {
     static int screenIdx = 0;
     static PointImmutable wpos;
     static DimensionImmutable wsize, rwsize=null;
-    static int[] reqSurfacePixelScale = new int[] { ScalableSurface.AUTOMAX_PIXELSCALE, ScalableSurface.AUTOMAX_PIXELSCALE };
+    static float[] reqSurfacePixelScale = new float[] { ScalableSurface.AUTOMAX_PIXELSCALE, ScalableSurface.AUTOMAX_PIXELSCALE };
 
     static long duration = 500; // ms
     static boolean opaque = true;
     static int forceAlpha = -1;
     static boolean undecorated = false;
     static boolean alwaysOnTop = false;
+    static boolean alwaysOnBottom = false;
+    static boolean resizable = true;
+    static boolean sticky = false;
+    static boolean max_vert= false;
+    static boolean max_horz= false;
     static boolean fullscreen = false;
     static int swapInterval = 1;
     static boolean waitForKey = false;
@@ -105,6 +104,8 @@ public class TestGearsES2NEWT extends UITestCase {
     static boolean forceES3 = false;
     static boolean forceGL3 = false;
     static boolean forceGL2 = false;
+    static int demoType = 1;
+    static boolean traceMouse = false;
     static boolean manualTest = false;
     static boolean exclusiveContext = false;
     static boolean useAnimator = true;
@@ -123,13 +124,6 @@ public class TestGearsES2NEWT extends UITestCase {
     public static void releaseClass() {
     }
 
-    private void setTitle(final Window win, final GLCapabilitiesImmutable caps) {
-        final String capsA = caps.isBackgroundOpaque() ? "opaque" : "transl";
-        final float[] sDPI = win.getPixelsPerMM(new float[2]);
-        sDPI[0] *= 25.4f;
-        sDPI[1] *= 25.4f;
-        win.setTitle("GLWindow["+capsA+"], swapI "+swapInterval+", win: "+win.getBounds()+", pix: "+win.getSurfaceWidth()+"x"+win.getSurfaceHeight()+", sDPI "+sDPI[0]+" x "+sDPI[1]);
-    }
     protected void runTestGL(final GLCapabilitiesImmutable caps, final boolean undecorated) throws InterruptedException {
         System.err.println("requested: vsync "+swapInterval+", "+caps);
         final Display dpy = NewtFactory.createDisplay(null);
@@ -137,21 +131,38 @@ public class TestGearsES2NEWT extends UITestCase {
         final GLWindow glWindow = GLWindow.create(screen, caps);
         Assert.assertNotNull(glWindow);
         glWindow.setSurfaceScale(reqSurfacePixelScale);
-        final int[] valReqSurfacePixelScale = glWindow.getRequestedSurfaceScale(new int[2]);
+        final float[] valReqSurfacePixelScale = glWindow.getRequestedSurfaceScale(new float[2]);
         glWindow.setSize(wsize.getWidth(), wsize.getHeight());
         if(null != wpos) {
             glWindow.setPosition(wpos.getX(), wpos.getY());
         }
         glWindow.setUndecorated(undecorated);
         glWindow.setAlwaysOnTop(alwaysOnTop);
+        glWindow.setAlwaysOnBottom(alwaysOnBottom);
+        glWindow.setResizable(resizable);
+        glWindow.setSticky(sticky);
+        glWindow.setMaximized(max_horz, max_vert);
         glWindow.setFullscreen(fullscreen);
         glWindow.setPointerVisible(mouseVisible);
         glWindow.confinePointer(mouseConfined);
 
-        final GearsES2 demo = new GearsES2(swapInterval);
-        demo.setUseMappedBuffers(useMappedBuffers);
-        demo.setValidateBuffers(true);
-        glWindow.addGLEventListener(demo);
+        final GLEventListener demo;
+        if( 2 == demoType ) {
+            final LineSquareXDemoES2 demo2 = new LineSquareXDemoES2(false);
+            demo = demo2;
+        } else if( 1 == demoType ) {
+            final GearsES2 gearsES2 = new GearsES2(swapInterval);
+            gearsES2.setUseMappedBuffers(useMappedBuffers);
+            gearsES2.setValidateBuffers(true);
+            demo = gearsES2;
+        } else if( 0 == demoType ) {
+            demo = new GLClearOnInitReshape();
+        } else {
+            demo = null;
+        }
+        if( null != demo ) {
+            glWindow.addGLEventListener(demo);
+        }
 
         final SnapshotGLEventListener snap = new SnapshotGLEventListener();
         glWindow.addGLEventListener(snap);
@@ -162,7 +173,7 @@ public class TestGearsES2NEWT extends UITestCase {
                 public void display(final GLAutoDrawable drawable) {
                     final GLAnimatorControl  actrl = drawable.getAnimator();
                     if(waitForKey && actrl.getTotalFPSFrames() == 60*3) {
-                        UITestCase.waitForKey("3s mark");
+                        JunitTracer.waitForKey("3s mark");
                         actrl.resetFPSCounter();
                         waitForKey = false;
                     }
@@ -178,249 +189,51 @@ public class TestGearsES2NEWT extends UITestCase {
             animator.setExclusiveContext(exclusiveContext);
         }
 
-        final QuitAdapter quitAdapter = new QuitAdapter();
-        //glWindow.addKeyListener(new TraceKeyAdapter(quitAdapter));
-        //glWindow.addWindowListener(new TraceWindowAdapter(quitAdapter));
-        glWindow.addKeyListener(quitAdapter);
-        glWindow.addWindowListener(quitAdapter);
-
         glWindow.addWindowListener(new WindowAdapter() {
             public void windowResized(final WindowEvent e) {
-                System.err.println("window resized: "+glWindow.getX()+"/"+glWindow.getY()+" "+glWindow.getSurfaceWidth()+"x"+glWindow.getSurfaceHeight());
-                setTitle(glWindow, caps);
+                System.err.println("window resized: "+glWindow.getBounds()+" "+glWindow.getSurfaceWidth()+"x"+glWindow.getSurfaceHeight());
+                NEWTDemoListener.setTitle(glWindow);
             }
             public void windowMoved(final WindowEvent e) {
-                System.err.println("window moved:   "+glWindow.getX()+"/"+glWindow.getY()+" "+glWindow.getSurfaceWidth()+"x"+glWindow.getSurfaceHeight());
-                setTitle(glWindow, caps);
+                System.err.println("window moved:   "+glWindow.getBounds()+" "+glWindow.getSurfaceWidth()+"x"+glWindow.getSurfaceHeight());
+                NEWTDemoListener.setTitle(glWindow);
             }
         });
 
-        final PointerIcon[] pointerIcons = { null, null, null, null, null };
-        {
-            final Display disp = glWindow.getScreen().getDisplay();
-            disp.createNative();
-            int idx = 0;
-            {
-                PointerIcon _pointerIcon = null;
-                final IOUtil.ClassResources res = new IOUtil.ClassResources(glWindow.getClass(), new String[] { "newt/data/cross-grey-alpha-16x16.png" } );
-                try {
-                    _pointerIcon = disp.createPointerIcon(res, 8, 8);
-                    System.err.printf("Create PointerIcon #%02d: %s%n", idx, _pointerIcon.toString());
-                } catch (final Exception e) {
-                    e.printStackTrace();
-                }
-                pointerIcons[idx] = _pointerIcon;
-            }
-            idx++;
-            {
-                PointerIcon _pointerIcon = null;
-                final IOUtil.ClassResources res = new IOUtil.ClassResources(glWindow.getClass(), new String[] { "newt/data/pointer-grey-alpha-16x24.png" } );
-                try {
-                    _pointerIcon = disp.createPointerIcon(res, 0, 0);
-                    System.err.printf("Create PointerIcon #%02d: %s%n", idx, _pointerIcon.toString());
-                } catch (final Exception e) {
-                    e.printStackTrace();
-                }
-                pointerIcons[idx] = _pointerIcon;
-            }
-            idx++;
-            {
-                PointerIcon _pointerIcon = null;
-                final IOUtil.ClassResources res = new IOUtil.ClassResources(glWindow.getClass(), new String[] { "arrow-red-alpha-64x64.png" } );
-                try {
-                    _pointerIcon = disp.createPointerIcon(res, 0, 0);
-                    System.err.printf("Create PointerIcon #%02d: %s%n", idx, _pointerIcon.toString());
-                } catch (final Exception e) {
-                    e.printStackTrace();
-                }
-                pointerIcons[idx] = _pointerIcon;
-            }
-            idx++;
-            {
-                PointerIcon _pointerIcon = null;
-                final IOUtil.ClassResources res = new IOUtil.ClassResources(glWindow.getClass(), new String[] { "arrow-blue-alpha-64x64.png" } );
-                try {
-                    _pointerIcon = disp.createPointerIcon(res, 0, 0);
-                    System.err.printf("Create PointerIcon #%02d: %s%n", idx, _pointerIcon.toString());
-                } catch (final Exception e) {
-                    e.printStackTrace();
-                }
-                pointerIcons[idx] = _pointerIcon;
-            }
-            idx++;
-            if( PNGIcon.isAvailable() ) {
-                PointerIcon _pointerIcon = null;
-                final IOUtil.ClassResources res = new IOUtil.ClassResources(glWindow.getClass(), new String[] { "jogamp-pointer-64x64.png" } );
-                try {
-                    final URLConnection urlConn = res.resolve(0);
-                    final PNGPixelRect image = PNGPixelRect.read(urlConn.getInputStream(), null, false /* directBuffer */, 0 /* destMinStrideInBytes */, false /* destIsGLOriented */);
-                    System.err.printf("Create PointerIcon #%02d: %s%n", idx, image.toString());
-                    _pointerIcon = disp.createPointerIcon(image, 32, 0);
-                    System.err.printf("Create PointerIcon #%02d: %s%n", idx, _pointerIcon.toString());
-                } catch (final Exception e) {
-                    e.printStackTrace();
-                }
-                pointerIcons[idx] = _pointerIcon;
-            }
-            idx++;
-        }
-        if( setPointerIcon ) {
-            glWindow.setPointerIcon(pointerIcons[0]);
-            System.err.println("Set PointerIcon: "+glWindow.getPointerIcon());
-        }
+        final GLWindow[] glWindow2 = { null };
 
+        final NEWTDemoListener newtDemoListener = new NEWTDemoListener(glWindow);
+        newtDemoListener.quitAdapterEnable(true);
+        glWindow.addKeyListener(newtDemoListener);
+        if( traceMouse ) {
+            glWindow.addMouseListener(new TraceMouseAdapter());
+        }
+        glWindow.addMouseListener(newtDemoListener);
+        glWindow.addWindowListener(newtDemoListener);
         glWindow.addKeyListener(new KeyAdapter() {
-            int pointerIconIdx = 0;
-            float gamma = 1f;
-            float brightness = 0f;
-            float contrast = 1f;
-
             @Override
             public void keyPressed(final KeyEvent e) {
                 if( e.isAutoRepeat() ) {
                     return;
                 }
-                if(e.getKeyChar()=='f') {
-                    new Thread() {
-                        public void run() {
-                            final Thread t = glWindow.setExclusiveContextThread(null);
-                            System.err.println("[set fullscreen  pre]: "+glWindow.getX()+"/"+glWindow.getY()+" "+glWindow.getSurfaceWidth()+"x"+glWindow.getSurfaceHeight()+", f "+glWindow.isFullscreen()+", a "+glWindow.isAlwaysOnTop()+", "+glWindow.getInsets());
-                            if( glWindow.isFullscreen() ) {
-                                glWindow.setFullscreen( false );
-                            } else {
-                                if( e.isAltDown() ) {
-                                    glWindow.setFullscreen( null );
-                                } else {
-                                    glWindow.setFullscreen( true );
-                                }
-                            }
-                            System.err.println("[set fullscreen post]: "+glWindow.getX()+"/"+glWindow.getY()+" "+glWindow.getSurfaceWidth()+"x"+glWindow.getSurfaceHeight()+", f "+glWindow.isFullscreen()+", a "+glWindow.isAlwaysOnTop()+", "+glWindow.getInsets());
-                            glWindow.setExclusiveContextThread(t);
-                    } }.start();
-                } else if( e.getKeySymbol()== KeyEvent.VK_G ) {
-                    new Thread() {
-                        public void run() {
-                            final float newGamma = gamma + ( e.isShiftDown() ? -0.1f : 0.1f );
-                            System.err.println("[set gamma]: "+gamma+" -> "+newGamma);
-                            if( Gamma.setDisplayGamma(glWindow, newGamma, brightness, contrast) ) {
-                                gamma = newGamma;
-                            }
-                    } }.start();
-                } else if(e.getKeyChar()=='a') {
-                    new Thread() {
-                        public void run() {
-                            final Thread t = glWindow.setExclusiveContextThread(null);
-                            System.err.println("[set alwaysontop pre]: "+glWindow.getX()+"/"+glWindow.getY()+" "+glWindow.getSurfaceWidth()+"x"+glWindow.getSurfaceHeight()+", f "+glWindow.isFullscreen()+", a "+glWindow.isAlwaysOnTop()+", "+glWindow.getInsets());
-                            glWindow.setAlwaysOnTop(!glWindow.isAlwaysOnTop());
-                            System.err.println("[set alwaysontop post]: "+glWindow.getX()+"/"+glWindow.getY()+" "+glWindow.getSurfaceWidth()+"x"+glWindow.getSurfaceHeight()+", f "+glWindow.isFullscreen()+", a "+glWindow.isAlwaysOnTop()+", "+glWindow.getInsets());
-                            glWindow.setExclusiveContextThread(t);
-                    } }.start();
-                } else if(e.getKeyChar()=='d') {
-                    new Thread() {
-                        public void run() {
-                            final Thread t = glWindow.setExclusiveContextThread(null);
-                            // while( null != glWindow.getExclusiveContextThread() ) ;
-                            System.err.println("[set undecorated  pre]: "+glWindow.getX()+"/"+glWindow.getY()+" "+glWindow.getSurfaceWidth()+"x"+glWindow.getSurfaceHeight()+", d "+glWindow.isUndecorated()+", "+glWindow.getInsets());
-                            glWindow.setUndecorated(!glWindow.isUndecorated());
-                            System.err.println("[set undecorated post]: "+glWindow.getX()+"/"+glWindow.getY()+" "+glWindow.getSurfaceWidth()+"x"+glWindow.getSurfaceHeight()+", d "+glWindow.isUndecorated()+", "+glWindow.getInsets());
-                            glWindow.setExclusiveContextThread(t);
-                    } }.start();
-                } else if(e.getKeyChar()=='s') {
-                    new Thread() {
-                        public void run() {
-                            final Thread t = glWindow.setExclusiveContextThread(null);
-                            System.err.println("[set position  pre]: "+glWindow.getX()+"/"+glWindow.getY()+" "+glWindow.getSurfaceWidth()+"x"+glWindow.getSurfaceHeight()+", "+glWindow.getInsets());
-                            glWindow.setPosition(100, 100);
-                            System.err.println("[set position post]: "+glWindow.getX()+"/"+glWindow.getY()+" "+glWindow.getSurfaceWidth()+"x"+glWindow.getSurfaceHeight()+", "+glWindow.getInsets());
-                            glWindow.setExclusiveContextThread(t);
-                    } }.start();
-                } else if(e.getKeyChar()=='c') {
-                    new Thread() {
-                        public void run() {
-                            final Thread t = glWindow.setExclusiveContextThread(null);
-                            System.err.println("[set pointer-icon pre]");
-                            final PointerIcon currentPI = glWindow.getPointerIcon();
-                            final PointerIcon newPI;
-                            if( pointerIconIdx >= pointerIcons.length ) {
-                                newPI=null;
-                                pointerIconIdx=0;
-                            } else {
-                                newPI=pointerIcons[pointerIconIdx++];
-                            }
-                            glWindow.setPointerIcon( newPI );
-                            System.err.println("[set pointer-icon post] "+currentPI+" -> "+glWindow.getPointerIcon());
-                            glWindow.setExclusiveContextThread(t);
-                    } }.start();
-                } else if(e.getKeyChar()=='i') {
-                    new Thread() {
-                        public void run() {
-                            final Thread t = glWindow.setExclusiveContextThread(null);
-                            System.err.println("[set mouse visible pre]: "+glWindow.isPointerVisible());
-                            glWindow.setPointerVisible(!glWindow.isPointerVisible());
-                            System.err.println("[set mouse visible post]: "+glWindow.isPointerVisible());
-                            glWindow.setExclusiveContextThread(t);
-                    } }.start();
-                } else if(e.getKeyChar()=='j') {
-                    new Thread() {
-                        public void run() {
-                            final Thread t = glWindow.setExclusiveContextThread(null);
-                            System.err.println("[set mouse confined pre]: "+glWindow.isPointerConfined());
-                            glWindow.confinePointer(!glWindow.isPointerConfined());
-                            System.err.println("[set mouse confined post]: "+glWindow.isPointerConfined());
-                            if(!glWindow.isPointerConfined()) {
-                                demo.setConfinedFixedCenter(false);
-                            }
-                            glWindow.setExclusiveContextThread(t);
-                    } }.start();
-                } else if(e.getKeyChar()=='J') {
-                    new Thread() {
-                        public void run() {
-                            final Thread t = glWindow.setExclusiveContextThread(null);
-                            System.err.println("[set mouse confined pre]: "+glWindow.isPointerConfined());
-                            glWindow.confinePointer(!glWindow.isPointerConfined());
-                            System.err.println("[set mouse confined post]: "+glWindow.isPointerConfined());
-                            demo.setConfinedFixedCenter(glWindow.isPointerConfined());
-                            glWindow.setExclusiveContextThread(t);
-                    } }.start();
-                } else if(e.getKeyChar()=='w') {
-                    new Thread() {
-                        public void run() {
-                            final Thread t = glWindow.setExclusiveContextThread(null);
-                            System.err.println("[set mouse pos pre]");
-                            glWindow.warpPointer(glWindow.getSurfaceWidth()/2, glWindow.getSurfaceHeight()/2);
-                            System.err.println("[set mouse pos post]");
-                            glWindow.setExclusiveContextThread(t);
-                    } }.start();
-                } else if(e.getKeyChar()=='x') {
-                    final int[] hadSurfacePixelScale = glWindow.getCurrentSurfaceScale(new int[2]);
-                    final int[] reqSurfacePixelScale;
-                    if( hadSurfacePixelScale[0] == ScalableSurface.IDENTITY_PIXELSCALE ) {
-                        reqSurfacePixelScale = new int[] { ScalableSurface.AUTOMAX_PIXELSCALE, ScalableSurface.AUTOMAX_PIXELSCALE };
+                if(e.getKeyChar()=='n') {
+                    if( null != glWindow2[0] && glWindow2[0].isNativeValid() ) {
+                        glWindow2[0].destroy();
+                        glWindow2[0] = null;
                     } else {
-                        reqSurfacePixelScale = new int[] { ScalableSurface.IDENTITY_PIXELSCALE, ScalableSurface.IDENTITY_PIXELSCALE };
+                        glWindow2[0] = GLWindow.create(screen, caps);
+                        glWindow2[0].setTitle("GLWindow2");
+                        glWindow2[0].setPosition(glWindow.getX()+glWindow.getWidth()+64, glWindow.getY());
+                        glWindow2[0].setSize(glWindow.getWidth(), glWindow.getHeight());
+                        glWindow2[0].addGLEventListener(new LineSquareXDemoES2(false));
+                        final Animator animator2 = useAnimator ? new Animator(glWindow2[0]) : null;
+                        if( null != animator2 ) {
+                            animator2.start();
+                        }
+                        glWindow2[0].setVisible(true);
                     }
-                    System.err.println("[set PixelScale pre]: had "+hadSurfacePixelScale[0]+"x"+hadSurfacePixelScale[1]+" -> req "+reqSurfacePixelScale[0]+"x"+reqSurfacePixelScale[1]);
-                    glWindow.setSurfaceScale(reqSurfacePixelScale);
-                    final int[] valReqSurfacePixelScale = glWindow.getRequestedSurfaceScale(new int[2]);
-                    final int[] hasSurfacePixelScale0 = glWindow.convertToPixelUnits(new int[] { 1, 1 });
-                    final int[] hasSurfacePixelScale1 = glWindow.getCurrentSurfaceScale(new int[2]);
-                    System.err.println("[set PixelScale post]: "+hadSurfacePixelScale[0]+"x"+hadSurfacePixelScale[1]+" (had) -> "+
-                                       reqSurfacePixelScale[0]+"x"+reqSurfacePixelScale[1]+" (req) -> "+
-                                       valReqSurfacePixelScale[0]+"x"+valReqSurfacePixelScale[1]+" (val) -> "+
-                                       hasSurfacePixelScale1[0]+"x"+hasSurfacePixelScale1[1]+" (has)");
-                    setTitle(glWindow, caps);
-                    Assert.assertArrayEquals(hasSurfacePixelScale0, hasSurfacePixelScale1);
                 }
-            }
-        });
-        glWindow.addMouseListener(new MouseAdapter() {
-            public void mouseClicked(final MouseEvent e) {
-                if(e.getClickCount() == 2 && e.getPointerCount() == 1) {
-                    glWindow.setFullscreen(!glWindow.isFullscreen());
-                    System.err.println("setFullscreen: "+glWindow.isFullscreen());
-                }
-            }
-         });
+            } } );
 
         if( useAnimator ) {
             animator.add(glWindow);
@@ -454,7 +267,7 @@ public class TestGearsES2NEWT extends UITestCase {
                                     final EDTUtil edt = ((Window)upstream).getScreen().getDisplay().getEDTUtil();
                                     System.err.println("EDT invokeAndWaitError: edt type "+edt.getClass().getName());
                                     if( edt instanceof DefaultEDTUtil ) {
-                                        quitAdapter.doQuit();
+                                        newtDemoListener.doQuit();
                                         ((DefaultEDTUtil)edt).invokeAndWaitError(new Runnable() {
                                             public void run() {
                                                 throw new RuntimeException("XXX Should never ever be seen! - "+Thread.currentThread());
@@ -478,17 +291,17 @@ public class TestGearsES2NEWT extends UITestCase {
             animator.setUpdateFPSFrames(60, showFPS ? System.err : null);
         }
 
+        System.err.println("Window Current State   : "+glWindow.getStateMaskString());
+        System.err.println("Window Supported States: "+glWindow.getSupportedStateMaskString());
         System.err.println("NW chosen: "+glWindow.getDelegatedWindow().getChosenCapabilities());
         System.err.println("GL chosen: "+glWindow.getChosenCapabilities());
         System.err.println("window pos/siz: "+glWindow.getX()+"/"+glWindow.getY()+" "+glWindow.getSurfaceWidth()+"x"+glWindow.getSurfaceHeight()+", "+glWindow.getInsets());
 
-        final int[] hasSurfacePixelScale0 = glWindow.convertToPixelUnits(new int[] { 1, 1 });
-        final int[] hasSurfacePixelScale1 = glWindow.getCurrentSurfaceScale(new int[2]);
+        final float[] hasSurfacePixelScale1 = glWindow.getCurrentSurfaceScale(new float[2]);
         System.err.println("HiDPI PixelScale: "+reqSurfacePixelScale[0]+"x"+reqSurfacePixelScale[1]+" (req) -> "+
                            valReqSurfacePixelScale[0]+"x"+valReqSurfacePixelScale[1]+" (val) -> "+
                            hasSurfacePixelScale1[0]+"x"+hasSurfacePixelScale1[1]+" (has)");
-        setTitle(glWindow, caps);
-        Assert.assertArrayEquals(hasSurfacePixelScale0, hasSurfacePixelScale1);
+        NEWTDemoListener.setTitle(glWindow);
 
         snap.setMakeSnapshot();
 
@@ -502,7 +315,7 @@ public class TestGearsES2NEWT extends UITestCase {
 
         final long t0 = System.currentTimeMillis();
         long t1 = t0;
-        while(!quitAdapter.shouldQuit() && t1-t0<duration) {
+        while(!newtDemoListener.shouldQuit() && t1-t0<duration) {
             Thread.sleep(100);
             t1 = System.currentTimeMillis();
             if( SysExit.testError == sysExit || SysExit.testExit == sysExit || SysExit.testEDTError == sysExit) {
@@ -517,7 +330,7 @@ public class TestGearsES2NEWT extends UITestCase {
                         final EDTUtil edt = glWindow.getScreen().getDisplay().getEDTUtil();
                         System.err.println("EDT invokeAndWaitError: edt type "+edt.getClass().getName());
                         if( edt instanceof DefaultEDTUtil ) {
-                            quitAdapter.doQuit();
+                            newtDemoListener.doQuit();
                             ((DefaultEDTUtil)edt).invokeAndWaitError(new Runnable() {
                                 public void run() {
                                     throw new RuntimeException("XXX Should never ever be seen!");
@@ -537,6 +350,10 @@ public class TestGearsES2NEWT extends UITestCase {
         }
         Assert.assertEquals(null, glWindow.getExclusiveContextThread());
         glWindow.destroy();
+        if( null != glWindow2[0] && glWindow2[0].isNativeValid() ) {
+            glWindow2[0].destroy();
+            glWindow2[0] = null;
+        }
         if( NativeWindowFactory.isAWTAvailable() ) {
             Assert.assertEquals(true,  AWTRobotUtil.waitForRealized(glWindow, false));
         }
@@ -624,6 +441,16 @@ public class TestGearsES2NEWT extends UITestCase {
                 undecorated = true;
             } else if(args[i].equals("-atop")) {
                 alwaysOnTop = true;
+            } else if(args[i].equals("-abottom")) {
+                alwaysOnBottom = true;
+            } else if(args[i].equals("-noresize")) {
+                resizable = false;
+            } else if(args[i].equals("-sticky")) {
+                sticky = true;
+            } else if(args[i].equals("-maxv")) {
+                max_vert = true;
+            } else if(args[i].equals("-maxh")) {
+                max_horz = true;
             } else if(args[i].equals("-fullscreen")) {
                 fullscreen = true;
             } else if(args[i].equals("-vsync")) {
@@ -669,7 +496,7 @@ public class TestGearsES2NEWT extends UITestCase {
                 usePos = true;
             } else if(args[i].equals("-pixelScale")) {
                 i++;
-                final int pS = MiscUtils.atoi(args[i], reqSurfacePixelScale[0]);
+                final float pS = MiscUtils.atof(args[i], reqSurfacePixelScale[0]);
                 reqSurfacePixelScale[0] = pS;
                 reqSurfacePixelScale[1] = pS;
             } else if(args[i].equals("-rwidth")) {
@@ -691,6 +518,11 @@ public class TestGearsES2NEWT extends UITestCase {
                 sysExit = SysExit.valueOf(args[i]);
             } else if(args[i].equals("-manual")) {
                 manualTest = true;
+            } else if(args[i].equals("-demo")) {
+                i++;
+                demoType = MiscUtils.atoi(args[i], 0);
+            } else if(args[i].equals("-traceMouse")) {
+                traceMouse = true;
             }
         }
         wsize = new Dimension(w, h);
@@ -709,6 +541,11 @@ public class TestGearsES2NEWT extends UITestCase {
         System.err.println("forceAlpha "+forceAlpha);
         System.err.println("undecorated "+undecorated);
         System.err.println("atop "+alwaysOnTop);
+        System.err.println("abottom "+alwaysOnBottom);
+        System.err.println("resizable "+resizable);
+        System.err.println("sticky "+sticky);
+        System.err.println("max_vert "+max_vert);
+        System.err.println("max_horz "+max_horz);
         System.err.println("fullscreen "+fullscreen);
         System.err.println("mouseVisible "+mouseVisible);
         System.err.println("mouseConfined "+mouseConfined);
@@ -724,9 +561,11 @@ public class TestGearsES2NEWT extends UITestCase {
         System.err.println("useAnimator "+useAnimator);
         System.err.println("sysExitWithin "+sysExit);
         System.err.println("mappedBuffers "+useMappedBuffers);
+        System.err.println("demoType "+demoType);
+        System.err.println("traceMouse "+traceMouse);
 
         if(waitForKey) {
-            UITestCase.waitForKey("Start");
+            JunitTracer.waitForKey("Start");
         }
         org.junit.runner.JUnitCore.main(TestGearsES2NEWT.class.getName());
     }
