@@ -28,22 +28,26 @@
 package com.jogamp.opengl.test.junit.jogl.stereo;
 
 import java.io.File;
-import java.net.URI;
 import java.net.URISyntaxException;
 import java.util.Arrays;
 
-import javax.media.nativewindow.util.DimensionImmutable;
-import javax.media.nativewindow.util.PointImmutable;
-import javax.media.opengl.GL;
-import javax.media.opengl.GLCapabilities;
-import javax.media.opengl.GLProfile;
+import com.jogamp.nativewindow.util.DimensionImmutable;
+import com.jogamp.nativewindow.util.PointImmutable;
+import com.jogamp.opengl.GL;
+import com.jogamp.opengl.GLAutoDrawable;
+import com.jogamp.opengl.GLCapabilities;
+import com.jogamp.opengl.GLProfile;
+import com.jogamp.opengl.GLRunnable;
 
 import jogamp.opengl.util.stereo.GenericStereoDevice;
 
-import com.jogamp.common.util.IOUtil;
+import com.jogamp.common.net.Uri;
+import com.jogamp.newt.MonitorDevice;
+import com.jogamp.newt.Screen;
 import com.jogamp.newt.event.KeyAdapter;
 import com.jogamp.newt.event.KeyEvent;
 import com.jogamp.newt.opengl.GLWindow;
+import com.jogamp.newt.opengl.util.stereo.StereoDeviceUtil;
 import com.jogamp.opengl.math.FovHVHalves;
 import com.jogamp.opengl.test.junit.jogl.demos.es2.GearsES2;
 import com.jogamp.opengl.test.junit.jogl.demos.es2.av.MovieSBSStereo;
@@ -180,14 +184,14 @@ public class StereoDemo01 {
         }
         final StereoGLEventListener upstream;
         final MovieSBSStereo movieSimple;
-        final URI movieURI;
+        final Uri movieURI;
         if( null != useFilmFile ) {
             movieSimple = new MovieSBSStereo();
-            movieURI = IOUtil.toURISimple(new File(useFilmFile));
+            movieURI = Uri.valueOf(new File(useFilmFile));
             upstream = movieSimple;
         } else if( null != useFilmURI ) {
             movieSimple = new MovieSBSStereo();
-            movieURI = new URI(useFilmURI);
+            movieURI = Uri.cast(useFilmURI);
             upstream = movieSimple;
         } else {
             final GearsES2 demo = new GearsES2(0);
@@ -205,7 +209,7 @@ public class StereoDemo01 {
     }
 
     public void doIt(final StereoDeviceFactory.DeviceType deviceType, final int deviceIndex, final int posx, final int posy,
-                     final StereoGLEventListener upstream, final MovieSBSStereo movieSimple, final URI movieURI,
+                     final StereoGLEventListener upstream, final MovieSBSStereo movieSimple, final Uri movieURI,
                      final boolean biLinear, final int numSamples, final boolean useSingleFBO,
                      final boolean useRecommendedDistortionBits, final boolean useVignette, final boolean useChromatic, final boolean useTimewarp,
                      final boolean useAutoSwap, final boolean useAnimator, final boolean exclusiveContext) throws InterruptedException {
@@ -242,23 +246,29 @@ public class StereoDemo01 {
         }
         System.err.println("StereoDevice "+stereoDevice);
 
+        //
+        //
+        //
+        final PointImmutable devicePos = stereoDevice.getPosition();
+        final DimensionImmutable deviceRes = stereoDevice.getSurfaceSize();
+        System.err.println("Device Res "+deviceRes+", reqRotation "+stereoDevice.getRequiredRotation());
+        System.err.println("Device Pos "+devicePos);
+
+        final MonitorDevice monitor = StereoDeviceUtil.getMonitorDevice(stereoDevice, true);
+        final Screen screen = monitor.getScreen();
+
         // Start the sensor which provides the Rift’s pose and motion.
-        if( !stereoDevice.startSensors(true) ) {
+        if( !stereoDevice.startSensors(stereoDevice.getSupportedSensorBits(), 0) ) {
             System.err.println("Could not start sensors on device "+deviceIndex);
         }
 
-        //
-        //
-        //
         final GLCapabilities caps = new GLCapabilities(GLProfile.getMaxProgrammable(true /* favorHardwareRasterizer */));
-        final GLWindow window = GLWindow.create(caps);
+        final GLWindow window = GLWindow.create(screen, caps);
 
-        final PointImmutable devicePos = stereoDevice.getPosition();
-        final DimensionImmutable deviceRes = stereoDevice.getSurfaceSize();
         if( useStereoScreen ) {
             window.setPosition(devicePos.getX(), devicePos.getY());
         }
-        window.setSurfaceSize(deviceRes.getWidth(), deviceRes.getHeight()); // might be not correct ..
+        window.setSurfaceSize(deviceRes.getWidth(), deviceRes.getHeight());
         window.setAutoSwapBufferMode(useAutoSwap);
         window.setUndecorated(true);
 
@@ -326,8 +336,30 @@ public class StereoDemo01 {
                     return;
                 }
                 switch(e.getKeySymbol()) {
+                    case KeyEvent.VK_O: {
+                        window.invoke(false, new GLRunnable() {
+                            @Override
+                            public boolean run(final GLAutoDrawable drawable) {
+                                stereoDevice.resetLocationSensorOrigin();
+                                return true;
+                            } });
+                        break;
+                    }
+                    case KeyEvent.VK_P: {
+                        window.invoke(false, new GLRunnable() {
+                            @Override
+                            public boolean run(final GLAutoDrawable drawable) {
+                                System.err.println(stereoDeviceRenderer.getLastViewerPose());
+                                return true;
+                            } });
+                        break;
+                    }
                     case KeyEvent.VK_R: {
-                        stereoDevice.startSensors(!stereoDevice.getSensorsStarted());
+                        if( stereoDevice.getSensorsStarted() ) {
+                            stereoDevice.stopSensors();
+                        } else {
+                            stereoDevice.startSensors(stereoDevice.getSupportedSensorBits(), 0);
+                        }
                         break;
                     }
                 }
@@ -344,6 +376,9 @@ public class StereoDemo01 {
         System.err.println("Window.0.windowSize : "+window.getWidth()+" x "+window.getHeight());
         System.err.println("Window.0.surfaceSize: "+window.getSurfaceWidth()+" x "+window.getSurfaceHeight());
         window.setSurfaceSize(deviceRes.getWidth(), deviceRes.getHeight());
+        if( useStereoScreen ) {
+            window.setPosition(devicePos.getX(), devicePos.getY());
+        }
         System.err.println("Window.1.windowSize : "+window.getWidth()+" x "+window.getHeight());
         System.err.println("Window.1.surfaceSize: "+window.getSurfaceWidth()+" x "+window.getSurfaceHeight());
 
@@ -362,6 +397,8 @@ public class StereoDemo01 {
             animator.stop();
         }
         window.destroy();
+        screen.removeReference(); // StereoDeviceUtil.getMonitorDevice(stereoDevice, true);
         stereoDevice.dispose();
+        stereoDeviceFactory.shutdown();
     }
 }
