@@ -29,8 +29,8 @@
 package com.jogamp.opengl.test.junit.newt.mm;
 
 import java.io.IOException;
-import javax.media.opengl.GLCapabilities;
-import javax.media.opengl.GLProfile;
+import com.jogamp.opengl.GLCapabilities;
+import com.jogamp.opengl.GLProfile;
 
 import com.jogamp.opengl.util.Animator;
 
@@ -54,8 +54,8 @@ import com.jogamp.opengl.test.junit.util.AWTRobotUtil;
 import com.jogamp.opengl.test.junit.util.UITestCase;
 
 import java.util.List;
-import javax.media.nativewindow.util.Dimension;
-import javax.media.nativewindow.util.RectangleImmutable;
+import com.jogamp.nativewindow.util.Dimension;
+import com.jogamp.nativewindow.util.RectangleImmutable;
 
 /**
  * Demonstrates fullscreen without MonitorMode change
