@@ -36,16 +36,19 @@ import java.lang.reflect.InvocationTargetException;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.awt.AWTException;
 import java.awt.EventQueue;
+import java.awt.MouseInfo;
+import java.awt.Point;
 import java.awt.Robot;
 
-import javax.media.nativewindow.NativeWindow;
-import javax.media.nativewindow.NativeWindowFactory;
-import javax.media.opengl.GLAutoDrawable;
-import javax.media.opengl.GLContext;
-import javax.media.opengl.GLDrawable;
+import com.jogamp.nativewindow.NativeWindow;
+import com.jogamp.nativewindow.NativeWindowFactory;
+import com.jogamp.opengl.GLAutoDrawable;
+import com.jogamp.opengl.GLContext;
+import com.jogamp.opengl.GLDrawable;
 
 import org.junit.Assert;
 
+import com.jogamp.common.ExceptionUtils;
 import com.jogamp.common.util.awt.AWTEDTExecutor;
 import com.jogamp.newt.event.WindowEvent;
 
@@ -135,9 +138,9 @@ public class AWTRobotUtil {
     private static int[] getCenterLocationNEWT(final com.jogamp.newt.Window win, final boolean onTitleBarIfWindow)
         throws InterruptedException, InvocationTargetException {
 
-        final javax.media.nativewindow.util.Point p0 = win.getLocationOnScreen(null);
+        final com.jogamp.nativewindow.util.Point p0 = win.getLocationOnScreen(null);
         if( onTitleBarIfWindow ) {
-            final javax.media.nativewindow.util.InsetsImmutable insets = win.getInsets();
+            final com.jogamp.nativewindow.util.InsetsImmutable insets = win.getInsets();
             p0.translate(win.getWidth()/2, insets.getTopHeight()/2);
         } else {
             p0.translate(win.getWidth()/2, win.getHeight()/2);
@@ -172,7 +175,7 @@ public class AWTRobotUtil {
     }
     private static int[] getClientLocationNEWT(final com.jogamp.newt.Window win, final int x, final int y)
         throws InterruptedException, InvocationTargetException {
-        final javax.media.nativewindow.util.Point p0 = win.getLocationOnScreen(null);
+        final com.jogamp.nativewindow.util.Point p0 = win.getLocationOnScreen(null);
         return new int[] { p0.getX(), p0.getY() };
     }
     private static int[] getClientLocationAWT(final java.awt.Component comp, final int x, final int y)
@@ -229,7 +232,7 @@ public class AWTRobotUtil {
             System.err.println("*** AWTRobotUtil.toFrontAndRequestFocus() UI failure");
             System.err.println("*** window: "+window);
             System.err.println("*** window.hasFocus(): "+window.hasFocus());
-            Thread.dumpStack();
+            ExceptionUtils.dumpStack(System.err);
         }
         return success;
     }
@@ -442,7 +445,7 @@ public class AWTRobotUtil {
             }
             System.err.println("*** gain: "+gain);
             System.err.println("*** lost: "+lost);
-            Thread.dumpStack();
+            ExceptionUtils.dumpStack(System.err);
         }
         Assert.assertTrue("Did not gain focus", hasFocus);
     }
@@ -578,6 +581,25 @@ public class AWTRobotUtil {
         Assert.assertEquals("Wrong key count", typeCount, counter.getCount()-c0);
     }
 
+    public static void mouseMove(final Robot robot, final Point destination, final int iter, final int delay) {
+        final Point origin = MouseInfo.getPointerInfo().getLocation();
+        for (int i = 1; i <= iter; i++) {
+            final float alpha = i / (float) iter;
+            robot.mouseMove((int) (origin.x * (1 - alpha) + destination.x * alpha),
+                            (int) (origin.y * (1 - alpha) + destination.y * alpha));
+            robot.delay(delay);
+        }
+    }
+    public static void mouseClick(final Robot robot, final Point pos, final int moveIter, final int moveDelay, final int actionDelay) {
+        robot.delay(actionDelay);
+        mouseMove(robot, pos, moveIter, moveDelay);
+
+        robot.delay(actionDelay);
+        robot.mousePress(java.awt.event.InputEvent.BUTTON1_MASK);
+        robot.mouseRelease(java.awt.event.InputEvent.BUTTON1_MASK);
+        robot.delay(actionDelay);
+    }
+
     static int mouseClick(final int i, final Robot robot, final int mouseButton,
                           final Object obj, final InputEventCountAdapter counter) throws InterruptedException, AWTException, InvocationTargetException
     {
